/*
xste is copyright Jan Panteltje 2004 and later.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "xste.h"

FD_xste *fdui;


struct animation_template *lookup_animation_template(char *name)
{
struct animation_template *pa;

/* argument check */
if(! name) return 0;

for(pa = animation_templatetab[0]; pa != 0; pa = pa -> nxtentr)
	{
	if(strcmp(pa -> name, name) == 0) return(pa);
	}
return 0; /* not found */
} /* end function lookup_animation_template */


struct animation_template *install_animation_template_at_end_of_list(char *name)
{
struct animation_template *plast, *pnew;
struct animation_template *lookup_animation_template();

if(debug_flag)
	{
	fprintf(stdout,\
	"install_animation_template_at_end_of_list(): arg name=%s\n", name);
	}

/* argument check */
if(! name) return 0;

pnew = lookup_animation_template(name);
if(pnew)
	{
	return(pnew);/* already there */
	}

/* create new structure */
pnew = (struct animation_template *) calloc(1, sizeof(*pnew) );
if(! pnew) return 0;
pnew -> name = strsave(name);
if(! pnew -> name) return 0;

/* get previous structure */
plast = animation_templatetab[1]; /* end list */

/* set new structure pointers */
pnew -> nxtentr = 0; /* new points to zero (is end) */
pnew -> prventr = plast; /* point to previous entry, or 0 if first entry */

/* set previous structure pointers */
if( !animation_templatetab[0] ) animation_templatetab[0] = pnew; /* first element in list */
else plast -> nxtentr = pnew;

/* set array end pointer */
animation_templatetab[1] = pnew;

return(pnew);/* pointer to new structure */
}/* end function install_animation_template_at_end_of_list */


int delete_animation_template(char *name)/* delete entry from double linked list */
{
struct animation_template *pa, *pprev, *pdel, *pnext;

if(debug_flag)
	{
	fprintf(stdout, "delete_animation_template(): arg name=%s\n", name);
	}

/* argument check */
if(! name) return 0;

pa = animation_templatetab[0];
while(1)
	{
	/* if end list, return not found */
	if(! pa) return 0;

	/* test for match in name */
	if(strcmp(name, pa -> name) != 0) /* no match */
		{
		/* point to next element in list */
		pa = pa -> nxtentr;

		/* loop for next element in list */
		continue;
		}

	/* we now know which struture to delete */
	pdel = pa;

	/* get previous and next structure */
	pnext = pa -> nxtentr;
	pprev = pa -> prventr;

	/* set pointers for previous structure */
	/* if first one, modify animation_templatetab[0] */
	if(pprev == 0) animation_templatetab[0] = pnext;
	else pprev -> nxtentr = pnext;

	/* set pointers for next structure */
	/* if last one, modify animation_templatetab[1] */
	if(pnext == 0) animation_templatetab[1] = pprev;
	else pnext -> prventr = pprev;
	
	/* delete structure */	
	free(pdel -> name);
	free(pdel -> fontname);

	free(pdel); /* free structure */

	/* return OK deleted */
	return 1;
	}/* end for all structures */

//fprintf(stdout, "WAS END ERROR\n");

return 0;
}/* end function delete_animation_template */


int delete_all_animation_templates()/* delete all entries from table */
{
struct animation_template *pa;

if(debug_flag)
	{
	fprintf(stdout, "delete_all_animation_templates() arg none\n");
	}

while(1)
	{	
	pa = animation_templatetab[0];
	if(! pa) break;

	animation_templatetab[0] = pa -> nxtentr;
	free(pa -> name);
	free(pa -> fontname);

	free(pa);/* free structure */
	}/* end while all structures */

animation_templatetab[1] = 0;
return 1;
}/* end function delete_all_animation_templates */


int sort_animation_templates()
/* sorts the double linked list for zpos
doing some sort of bubble sort.
*/
{
struct animation_template *pa;
struct animation_template *pb;
int swap_flag;

while(1)/* go through list again and again */
	{
	if(debug_flag)
		{
		fprintf(stdout, "CHECKING LIST\n");
		}/* end if debug_flag */
		
	swap_flag = 0;
	for(pa = animation_templatetab[0]; pa != 0; pa = pa -> nxtentr)
		{
		if(debug_flag)
			{
			fprintf(stdout, "sort_animation_templates(): sorting %s pa=%p\n",\
			pa -> name, pa);		
			}	
	
		pb = pa -> prventr;
		if(debug_flag)
			{
			fprintf(stdout, "pb=pa->prventr=%p\n", pb);
			}
		
		if(pb)
			{
			/* compare */
//			if(strcmp(pa -> name, pb -> name) < 0)
			if(pa -> zpos < pb -> zpos)
				{
				swap_flag = swap_animation_template_position(pa, pb);
				/* indicate position was swapped */
				if(debug_flag)
					{
					fprintf(stdout, "swap_flag=%d\n", swap_flag);
					fprintf(stdout,\
					"AFTER SWAP pa->prventr=%p pa->nxtentr=%p\n\
					pb->prventr=%p pb-nxtentrr=%p\n",\
					pa -> prventr, pa -> nxtentr,\
					pb -> prventr, pb -> nxtentr);		
					}										
				}/* end if strcmp < 0 */
			}/* end if pb */
		}/* end for all entries */

	/* if no more swapping took place, ready, list is sorted */
	if(! swap_flag) break;
	}/* end while go through list again and again */

return 1;
} /* end function sort_animation_templates */


int sort_animation_templates_for_pzpos()
/* sorts the double linked list for pzpos
doing some sort of bubble sort.
*/
{
struct animation_template *pa;
struct animation_template *pb;
int swap_flag;

while(1)/* go through list again and again */
	{
	if(debug_flag)
		{
		fprintf(stdout, "CHECKING LIST\n");
		}/* end if debug_flag */
		
	swap_flag = 0;
	for(pa = animation_templatetab[0]; pa != 0; pa = pa -> nxtentr)
		{
		if(debug_flag)
			{
			fprintf(stdout, "sort_animation_templates(): sorting %s pa=%p\n",\
			pa -> name, pa);		
			}	
	
		pb = pa -> prventr;
		if(debug_flag)
			{
			fprintf(stdout, "pb=pa->prventr=%p\n", pb);
			}
		
		if(pb)
			{
			/* compare */
			if(pa -> pzpos < pb -> pzpos)
				{
				swap_flag = swap_animation_template_position(pa, pb);
				/* indicate position was swapped */
				if(debug_flag)
					{
					fprintf(stdout, "swap_flag=%d\n", swap_flag);
					fprintf(stdout,\
					"AFTER SWAP pa->prventr=%p pa->nxtentr=%p\n\
					pb->prventr=%p pb-nxtentrr=%p\n",\
					pa -> prventr, pa -> nxtentr,\
					pb -> prventr, pb -> nxtentr);		
					}										
				}/* end if strcmp < 0 */
			}/* end if pb */
		}/* end for all entries */

	/* if no more swapping took place, ready, list is sorted */
	if(! swap_flag) break;
	}/* end while go through list again and again */

return 1;
} /* end function sort_animation_templates_for_pzpos */


int load_animation_templates()
{
int a;
FILE *animation_templates_dat_file;
char temp[READSIZE];
char pathfilename[TEMP_SIZE];
struct animation_template *pa = 0;

char name[TEMP_SIZE];
int status;
int browser_line;
char fontname[4096];
int fontsize;
int iso;
double outline;
double blur;

double xpos;
double dxpos;
double ypos;
double dypos;
double zpos;
double dzpos;
double espace;
double despace;  
double vespace;
double color;  
double dcolor; 
double saturation;
double dsaturation;
double contrast;
double dcontrast;
double transparency;
double dtransparency;
double heading;      
double dheading;
double speed;
double dspeed;

int arg_count;

if(debug_flag)
	{
	fprintf(stdout, "load_animation_templates() arg none\n");
	}
	
delete_all_animation_templates();

sprintf(pathfilename, "%s/%s/%s/animation_templates.dat",\
home_dir, selected_data_directory, selected_project);

animation_templates_dat_file = fopen(pathfilename, "r");
if(! animation_templates_dat_file)
	{
	if(debug_flag)
		{
		fprintf(stdout, "could not load file %s\n", pathfilename);
		}

	return 0;
	} 

while(1)
	{
	a = readline(animation_templates_dat_file, temp);
	if(a == EOF) break;/* last line, or file format error */

	arg_count = sscanf(temp,\
	"%s %d %d %s %d %d %lf %lf  %lf %lf %lf %lf %lf %lf %lf %lf %lf %lf %lf %lf %lf %lf %lf %lf %lf %lf %lf %lf %lf",\
	name,\
	&status,\
	&browser_line,\
	fontname,\
	&fontsize,\
	&iso,\
	&outline,\
	&blur,\
	&xpos,\
	&dxpos,\
	&ypos,\
	&dypos,\
	&zpos,\
	&dzpos,
	&espace,\
	&despace,\
	&vespace,\
	&color,\
	&dcolor,\
	&saturation,\
	&dsaturation,\
	&contrast,\
	&dcontrast,\
	&transparency,\
	&dtransparency,\
	&heading,\
	&dheading,\
	&speed,\
	&dspeed\
	);

	if(arg_count != 29)
		{
		fl_show_alert("load_animation_templates", temp, "file format error", 0);

		fclose(animation_templates_dat_file);
		
		return 0;
		}

	/* create a structure entry */
	pa = install_animation_template_at_end_of_list(name);
	if(! pa)
		{
		if(debug_flag)
			{
			fprintf(stdout,\
			"load_animation_templates(): cannot install animation_template %s\n", temp);
			}
		break;
		}

	pa -> status = status;
	pa -> browser_line = browser_line;

	pa -> fontname = strsave(fontname);
	if(! pa -> fontname)
		{
		fprintf(stderr, "load_animation_templates(): could not strsave fontname\n");

		return 0;
		}

	pa -> fontsize = fontsize; 
	pa -> iso = iso;
	pa -> outline = outline;
	pa -> blur = blur;
	pa -> xpos = xpos;
	pa -> dxpos = dxpos;
	pa -> ypos = ypos;
	pa -> dypos = dypos;
	pa -> zpos = zpos;
	pa -> dzpos = dzpos;
	pa -> espace = espace;
	pa -> despace = despace; 
	pa -> vespace = vespace;
	pa -> color = color;
	pa -> dcolor = dcolor;
	pa -> saturation = saturation;
	pa -> dsaturation = dsaturation;
	pa -> contrast = contrast;
	pa -> dcontrast = dcontrast;
	pa -> transparency = transparency;
	pa -> dtransparency = dtransparency;
	pa -> heading = heading;
	pa -> dheading = dheading;
	pa -> speed = speed;
	pa -> dspeed = dspeed;

	}/* end while all lines in animation_templates.dat */

return 1;
} /* end function load_animation_templates */


int save_animation_templates()
{
FILE *animation_templates_dat_file;
char pathfilename[TEMP_SIZE];
char pathfilename2[TEMP_SIZE];
struct animation_template *pa;

if(debug_flag)
	{
	fprintf(stdout, "save_animation_templates() arg none\n");
	}

sprintf(pathfilename, "%s/%s/%s/animation_templates.tmp",\
home_dir, selected_data_directory, selected_project);

animation_templates_dat_file = fopen(pathfilename, "w");
if(! animation_templates_dat_file)
	{
	if(debug_flag)
		{
		fprintf(stdout,\
		"could not open file %s for write\n", pathfilename);
		}
	return 0;
	} 

/* save list of animation_templates */
for(pa = animation_templatetab[0]; pa != 0; pa = pa -> nxtentr)	
	{
	fprintf(animation_templates_dat_file,\
	"%s %d %d %s %d %d %.2f %.2f  %.2f %.2f %.2f %.2f %.2f %.2f %.2f %.2f %.2f %.2f %.2f %.2f %.2f %.2f %.2f %.2f %.2f %.2f %.2f %.2f %.2f\n",\
	pa -> name,\
	pa -> status,\
	pa -> browser_line,\
	pa -> fontname,\
	pa -> fontsize,\
	pa -> iso,\
	pa -> outline,\
	pa -> blur,\
	pa -> xpos,\
	pa -> dxpos,\
	pa -> ypos,\
	pa -> dypos,\
	pa -> zpos,\
	pa -> dzpos,\
	pa -> espace,\
	pa -> despace,\
	pa -> vespace,\
	pa -> color,\
	pa -> dcolor,\
	pa -> saturation,\
	pa -> dsaturation,\
	pa -> contrast,\
	pa -> dcontrast,\
	pa -> transparency,\
	pa -> dtransparency,\
	pa -> heading,\
	pa -> dheading,\
	pa -> speed,\
	pa -> dspeed\
	);

	}/* end while all elements in list */

fclose(animation_templates_dat_file);

/* set some path file names */
sprintf(pathfilename, "%s/%s/%s/animation_templates.dat",\
home_dir, selected_data_directory, selected_project);

sprintf(pathfilename2, "%s/%s/%s/animation_templates.dat~",\
home_dir, selected_data_directory, selected_project);

/* unlink the old .dat~ */
unlink(pathfilename2);

/* rename .dat to .dat~ */
if( rename(pathfilename, pathfilename2) == -1)
	{
	if(debug_flag)
		{
		fprintf(stdout, "save_animation_templates(); rename %s into %s failed\n",\
		pathfilename, pathfilename2);
		}

	/* this will fail first time, if no animation_templates.dat! so no exit */
//	return 0;
	}

/* rename .tmp to .dat */
sprintf(pathfilename2, "%s/%s/%s/animation_templates.tmp",
home_dir, selected_data_directory, selected_project);

if( rename(pathfilename2, pathfilename) == -1)
	{
	if(debug_flag)
		{
		fprintf(stdout, "save_animation_templates(); rename %s into %s failed\n",\
		pathfilename, pathfilename2);
		}
	return 0;
	}

return 1; /* ok */
} /* end function save_animation_templates */


int show_animation_template_form()
{

selected_animation_template = animation_template_line_to_form(selected_animation_template_browser_line);

show_animation_templates(SHOW_UNREAD_AT_TOP);

if(!selected_animation_template) clear_animation_template_form();
else fill_in_animation_template_form(selected_animation_template);

fl_set_input(fdui -> animation_template_form_input_field, "");

//fl_show_form(fdui -> animation_template_form, FL_PLACE_CENTER, FL_NOBORDER, "");
fl_show_form(fdui -> animation_template_form, FL_PLACE_CENTER, FL_TRANSIENT, "");

animation_template_form_shown_flag = 1;

return 1;
} /* end function show_animation_template_form */


int add_animation_template(char *name)
{
int illegal;
struct animation_template *pa;
char temp[TEMP_SIZE];

if(debug_flag)
	{
	fprintf(stdout, "add_animation_template(): arg name=%s\n", name);
	}

/* argument check */
if(! name) return 0;
illegal = 0;
if(strlen(name) == 0) illegal = 1;
if(strstr(name, "/") != 0) illegal = 1;
if(strstr(name, " ") != 0) illegal = 1;
if(strstr(name, "*") != 0) illegal = 1;
if(strstr(name, ".dat") != 0) illegal = 1;
if(strstr(name, ".dat~") != 0) illegal = 1;
if(strstr(name, ".wav") != 0) illegal = 1;
if(strstr(name, ".ppml") != 0) illegal = 1;
if(strstr(name, ".bak") != 0) illegal = 1;
if(illegal)
	{
	fl_show_alert("Illegal name", name, "command cancelled", 0);
	return 0;
	}

pa = lookup_animation_template(name);
if(pa)
	{
	fl_show_alert(\
	"animation_template name exists", name, "command cancelled", 0);

	if(debug_flag)
		{
		fprintf(stdout, "add_animation_template(): %s exists, returning\n",\
		name);
		}
	return 0;/* do not select */
	}

sprintf(temp,\
"animation_template %s does not exist\nCreate entry?",\
name);
if(! fl_show_question(temp, 1) )/* 1 = mouse on yes */
	{
	return 0;
	}

pa = install_animation_template_at_end_of_list(name);
if(! pa)
	{
	fl_show_alert(\
	"add_animation_template():",\
	"cannot install animation_template", \
	"command cancelled", 0);
	return 0;
	}

return 1;
}/* end function add_animation_template */



int show_animation_templates(int position)
{
char temp[TEMP_SIZE];
struct animation_template *pa;
char formatstr[20];
int browser_line;
int browser_topline;
int first_new_animation_template_line;
int find_first_new_animation_template_flag;

int maxline;

if(debug_flag)
	{
	fprintf(stdout, "show_animation_templates(): arg position=%d\n", position);
	}

sort_animation_templates();

/* remember vertical slider position */
browser_topline = fl_get_browser_topline(fdui -> animation_template_form_browser);

fl_freeze_form(fdui -> animation_template_form);

fl_clear_browser(fdui -> animation_template_form_browser);

first_new_animation_template_line = -1;/* keep gcc -Wall from complaining */
find_first_new_animation_template_flag = 1;
browser_line = 1;
for(pa = animation_templatetab[0]; pa != 0; pa = pa -> nxtentr)	
	{
	if(position == SHOW_UNREAD_AT_TOP)
		{
		/* want to show the first unaccessed at top of display */
		if(find_first_new_animation_template_flag)
			{
			if(pa -> status == 1)
				{
				first_new_animation_template_line = browser_line;
				find_first_new_animation_template_flag = 0;
				}
			}
		}
	
	strcpy(formatstr, "@f");
	if(pa -> status == 0) strcat(formatstr, "@C0");/* black */
	
	sprintf(temp, "%s %s",\
	formatstr, pa -> name);

	fl_add_browser_line(fdui -> animation_template_form_browser, temp);
	pa -> browser_line = browser_line;/* first one is 1 */
	browser_line++;
	}/* end for all elements in list */
	
/* re adjust browser for same position of top line (vertical slider) */	
if(position == SHOW_UNREAD_AT_TOP)
	{
	fl_set_browser_topline(fdui -> animation_template_form_browser, first_new_animation_template_line);
	}
if(position == SHOW_BOTTOM)
	{
	/* make the status of the last animation_template visible */
	maxline = fl_get_browser_maxline(fdui -> animation_template_form_browser);
	fl_set_browser_topline(fdui -> animation_template_form_browser, maxline);
	}
/*
if(position == SHOW_TOP)
	{
	fl_set_browser_topline(fdui -> animation_template_form_browser, 1);
	}
*/
if(position == SHOW_SAME_POSITION)
	{
	fl_set_browser_topline(fdui -> animation_template_form_browser, browser_topline);
	}	

fl_unfreeze_form(fdui -> animation_template_form);

if(selected_animation_template)
	{
	fl_select_browser_line(\
		fdui -> animation_template_form_browser, selected_animation_template_browser_line);
	}

return 1;
}/* end function show_animation_templates */


char *line_to_animation_template(int line)
{
struct animation_template *pa;

if(debug_flag)
	{
	fprintf(stdout, "line_to_animation_template(): arg line=%d\n", line);
	}

/* argument check */
if(line < 0) return 0;

for(pa = animation_templatetab[0]; pa != 0; pa = pa -> nxtentr)	
	{
	if(pa -> browser_line == line)
		{
		return(pa -> name);
		}
	}

return 0;
}/* end function line_to_animation_template_id */


int animation_template_to_line(char *name, int *line)
{
struct animation_template *pa;

if(debug_flag)
	{
	fprintf(stdout, "animation_template_to_line(): arg name=%s\n", name);
	}

/* argument check */
if(! name) return 0;

pa = lookup_animation_template(name);
if(! pa) return 0;

*line = pa -> browser_line;
return 1;
}/* end function animation_template_to_line */


int swap_animation_template_position(struct animation_template *ptop, struct animation_template *pbottom)
{
struct animation_template *punder;
struct animation_template *pabove;

if(debug_flag)
	{
	fprintf(stdout,\
	"swap_animation_template_position(): swapping top=%p bottom=%p\n", ptop, pbottom);
	}

/* argument check */
if(! ptop) return 0;
if(! pbottom) return 0;

/* get one below the bottom */
punder = pbottom -> prventr;/* could be zero if first entry */
if(debug_flag)
	{
	fprintf(stdout,\
	"swap_animation_template_position(): punder=%p\n", punder);
	}

/* get the one above the top */
pabove = ptop -> nxtentr;/* could be zero if last entry */
if(debug_flag)
	{
	fprintf(stdout,\
	"swap_animation_template_position(): pabove=%p\n", pabove);
	}

/* the next pointer in punder (or animation_templatetab[0]) must now point to ptop */
if(! punder)
	{
	animation_templatetab[0] = ptop; 
	}
else
	{
	punder -> nxtentr = ptop;
	}

/* the prev pointer in in ptop must now point to punder */
ptop -> prventr = punder;/* could be zero if first entry */

/* the next pointer in ptop must now point to pbottom */
ptop -> nxtentr = pbottom;

/* the next pointer in pbottom must now point to pabove */
pbottom -> nxtentr = pabove;

/* mark last one in animation_templatetab */
if(! pabove)
	{
	animation_templatetab[1] = pbottom;
	}
else
	{
	/* the prev pointer in pabove must now point to pbottom */
	pabove -> prventr = pbottom;
	}

/* the prev pointer in pbottom must now point to ptop */
pbottom -> prventr = ptop;

/* return swapped */
return 1;
}/* end function swap_animation_template_position */


struct animation_template *animation_template_line_to_form(int line)
{
struct animation_template *pa;

if(debug_flag)
	{
	fprintf(stdout, "animation_template_line_to_form(): arg line=%d\n", line);
	}

for(pa = animation_templatetab[0]; pa != 0; pa = pa -> nxtentr)
	{
	if(pa -> browser_line == line) return pa;

	} /* end for all entries in list */

return 0;
} /* end function animation_template_line_to_form */


int fill_in_animation_template_form(struct animation_template *pa)
{
int a;
double da;
char temp[80];

if(debug_flag)
	{
	fprintf(stdout, "fill_in_animation_template_form(): arg pa=%p\n", pa);
	}

/* set buttons */

if(pa -> status & ANIMATION_TEMPLATE_CENTER) a = 1;
else a = 0;
fl_set_button(fdui -> animation_template_form_center_button, a);

if(pa -> status & ANIMATION_TEMPLATE_REFORMAT) a = 1;
else a = 0;
fl_set_button(fdui -> animation_template_form_reformat_button, a);

/* set output fields */

fl_set_object_label(fdui -> animation_template_form_fontname_output_field, pa -> fontname);

/* set input fields */
sprintf(temp, "%d", pa -> fontsize);
fl_set_input(fdui -> animation_template_form_character_point_size_input_field, temp);

sprintf(temp, "%d", pa -> iso);
fl_set_input(fdui -> animation_template_form_character_style_input_field, temp);

sprintf(temp, "%.2f", pa -> outline);
fl_set_input(fdui -> animation_template_form_outline_thickness_input_field, temp);

sprintf(temp, "%.2f", pa -> blur);
fl_set_input(fdui -> animation_template_form_blur_radius_input_field, temp);

fl_set_slider_value(fdui -> animation_template_form_espace_slider, pa -> espace);

if(pa -> espace < 1.0)\
	fl_set_object_color(fdui -> animation_template_form_espace_slider, FL_DARKORANGE, FL_DARKORANGE);
else if(pa -> espace > 1.0)\
	fl_set_object_color(fdui -> animation_template_form_espace_slider, FL_YELLOW, FL_YELLOW);
else fl_set_object_color(fdui -> animation_template_form_espace_slider, FL_COL1, FL_COL1);
fl_redraw_object(fdui -> animation_template_form_espace_slider);

fl_set_slider_value(fdui -> animation_template_form_despace_slider, pa -> despace);
if(pa -> despace == 0.0)\
	fl_set_object_color(fdui -> animation_template_form_despace_slider, FL_COL1, FL_COL1);
else fl_set_object_color(fdui -> animation_template_form_despace_slider, FL_DARKORANGE, FL_DARKORANGE);
fl_redraw_object(fdui -> animation_template_form_despace_slider);

fl_set_slider_value(fdui -> animation_template_form_contrast_slider, pa -> contrast);
fl_set_slider_value(fdui -> animation_template_form_dcontrast_slider, pa -> dcontrast);
if(pa -> dcontrast == 0.0)\
	fl_set_object_color(fdui -> animation_template_form_dcontrast_slider, FL_COL1, FL_COL1);
else fl_set_object_color(fdui -> animation_template_form_dcontrast_slider, FL_DARKORANGE, FL_DARKORANGE);
fl_redraw_object(fdui -> animation_template_form_dcontrast_slider);                     

fl_set_slider_value(fdui -> animation_template_form_transparency_slider, pa -> transparency);
fl_set_slider_value(fdui -> animation_template_form_dtransparency_slider, pa -> dtransparency);
if(pa -> dtransparency == 0.0)\
	fl_set_object_color(fdui -> animation_template_form_dtransparency_slider, FL_COL1, FL_COL1);
else fl_set_object_color(fdui -> animation_template_form_dtransparency_slider, FL_DARKORANGE, FL_DARKORANGE);
fl_redraw_object(fdui -> animation_template_form_dtransparency_slider);                     

fl_set_slider_value(fdui -> animation_template_form_saturation_slider, pa -> saturation);
fl_set_slider_value(fdui -> animation_template_form_dsaturation_slider, pa -> dsaturation);
if(pa -> dsaturation == 0.0)\
	fl_set_object_color(fdui -> animation_template_form_dsaturation_slider, FL_COL1, FL_COL1);
else fl_set_object_color(fdui -> animation_template_form_dsaturation_slider, FL_DARKORANGE, FL_DARKORANGE);
fl_redraw_object(fdui -> animation_template_form_dsaturation_slider);

fl_set_slider_value(fdui -> animation_template_form_speed_slider, pa -> speed);
fl_set_slider_value(fdui -> animation_template_form_dspeed_slider, pa -> dspeed);
if(pa -> dspeed == 0.0)\
	fl_set_object_color(fdui -> animation_template_form_dspeed_slider, FL_COL1, FL_COL1);
else fl_set_object_color(fdui -> animation_template_form_dspeed_slider, FL_DARKORANGE, FL_DARKORANGE);
fl_redraw_object(fdui -> animation_template_form_dspeed_slider);

sprintf(temp, "%d", (int)pa -> vespace);
fl_set_input(fdui -> animation_template_form_vespace_input_field, temp);

/* set color dial */
fl_set_dial_value(fdui -> animation_template_form_color_circle_dial, pa -> color);
set_template_dial_color(pa -> color, pa -> saturation);

sprintf(temp, "%.2f", pa -> color);
fl_set_object_label(fdui -> animation_template_form_color_degrees_output_field, temp);

fl_set_slider_value(fdui -> animation_template_form_dcolor_slider, pa -> dcolor);
if(pa -> dcolor == 0.0)\
	fl_set_object_color(fdui -> animation_template_form_dcolor_slider, FL_COL1, FL_COL1);
else fl_set_object_color(fdui -> animation_template_form_dcolor_slider, FL_DARKORANGE, FL_DARKORANGE);
fl_redraw_object(fdui -> animation_template_form_dcolor_slider);

/* heading */

/* flip the dial increment direction */
da = 360.0 - pa -> heading;

da += 180.0;

if(da > 360.0) da -= 360.0;

fl_set_dial_value(fdui -> animation_template_form_heading_dial, da);

sprintf(temp, "%03d", (int) pa -> heading);
fl_set_object_label(fdui -> animation_template_form_heading_degrees_output_field, temp);

fl_set_slider_value(fdui -> animation_template_form_dheading_slider, pa -> dheading);
if(pa -> dheading == 0.0)\
	fl_set_object_color(fdui -> animation_template_form_dheading_slider, FL_COL1, FL_COL1);
else fl_set_object_color(fdui -> animation_template_form_dheading_slider, FL_DARKORANGE, FL_DARKORANGE);
fl_redraw_object(fdui -> animation_template_form_dheading_slider);

/* x y z */
fl_set_positioner_xvalue(fdui -> animation_template_form_x_y_positioner, pa -> xpos);
fl_set_positioner_yvalue(fdui -> animation_template_form_x_y_positioner, v_size - pa -> ypos);

fl_set_slider_value(fdui -> animation_template_form_xpos_slider, pa -> xpos);
fl_set_slider_value(fdui -> animation_template_form_ypos_slider, pa -> ypos);

sprintf(temp, "%03d,%03d",\
(int)selected_animation_template -> xpos, (int)selected_animation_template -> ypos);
fl_set_object_label(fdui -> animation_template_form_x_y_position_output_field, temp);

fl_set_slider_value(fdui -> animation_template_form_dxpos_slider, pa -> dxpos);
if(pa -> dxpos == 0.0)\
	fl_set_object_color(fdui -> animation_template_form_dxpos_slider, FL_COL1, FL_COL1);
else fl_set_object_color(fdui -> animation_template_form_dxpos_slider, FL_DARKORANGE, FL_DARKORANGE);
fl_redraw_object(fdui -> animation_template_form_dxpos_slider);

fl_set_slider_value(fdui -> animation_template_form_dypos_slider, pa -> dypos);
if(pa -> dypos == 0.0)\
	fl_set_object_color(fdui -> animation_template_form_dypos_slider, FL_COL1, FL_COL1);
else fl_set_object_color(fdui -> animation_template_form_dypos_slider, FL_DARKORANGE, FL_DARKORANGE);
fl_redraw_object(fdui -> animation_template_form_dypos_slider);

fl_set_slider_value(fdui -> animation_template_form_zpos_slider, pa -> zpos);
fl_set_slider_value(fdui -> animation_template_form_dzpos_slider, pa -> dzpos);
if(pa -> dzpos == 0.0)\
	fl_set_object_color(fdui -> animation_template_form_dzpos_slider, FL_COL1, FL_COL1);
else fl_set_object_color(fdui -> animation_template_form_dzpos_slider, FL_DARKORANGE, FL_DARKORANGE);
fl_redraw_object(fdui -> animation_template_form_dzpos_slider);

/* vespace */

sprintf(temp, "%d", (int)pa -> vespace);
fl_set_input(fdui -> animation_template_form_vespace_input_field, temp);

/* select heading dial + speed and dspaeed and dheading sliders, or dxpos + dypos sliders */
if(pa -> status & ANIMATION_TEMPLATE_SHOW_X_Y_DELTA)
	{
	select_heading_or_x_y(1);
	fl_set_button(fdui -> animation_template_form_heading_or_x_y_select_button, 1);
	}
else
	{
	select_heading_or_x_y(0);
	fl_set_button(fdui -> animation_template_form_heading_or_x_y_select_button, 0);
	}

return 1;
} /* end function fill_in_animation_template_form */


int clear_animation_template_form()
{
char temp[80];

if(debug_flag)
	{
	fprintf(stdout, "clear_animation_template_form(): arg none\n");
	}

fl_set_object_label(fdui -> animation_template_form_fontname_output_field, "");
fl_set_input(fdui -> animation_template_form_character_point_size_input_field, "");
fl_set_input(fdui -> animation_template_form_character_style_input_field, "");
fl_set_input(fdui -> animation_template_form_outline_thickness_input_field, "");
fl_set_input(fdui -> animation_template_form_blur_radius_input_field, "");

fl_set_slider_value(fdui -> animation_template_form_espace_slider, 1.0);
fl_set_object_color(fdui -> animation_template_form_espace_slider, FL_COL1, FL_COL1);
fl_redraw_object(fdui -> animation_template_form_espace_slider);

fl_set_slider_value(fdui -> animation_template_form_despace_slider, 0.0);
fl_set_object_color(fdui -> animation_template_form_despace_slider, FL_COL1, FL_COL1);
fl_redraw_object(fdui -> animation_template_form_despace_slider);

fl_set_positioner_xbounds(fdui -> animation_template_form_x_y_positioner, 0, h_size);
fl_set_positioner_ybounds(fdui -> animation_template_form_x_y_positioner, 0, v_size);
fl_set_positioner_xvalue(fdui -> animation_template_form_x_y_positioner, h_size / 2);
fl_set_positioner_yvalue(fdui -> animation_template_form_x_y_positioner, v_size / 2);

if(selected_animation_template)
	{
	sprintf(temp, "%03d,%03d",\
	(int)selected_animation_template -> xpos, (int)selected_animation_template -> ypos);
	}
else strcpy(temp, "");
fl_set_object_label(fdui -> animation_template_form_x_y_position_output_field, temp);

fl_set_slider_value(fdui -> animation_template_form_zpos_slider, 0.0);
fl_set_slider_value(fdui -> animation_template_form_dzpos_slider, 0.0);
fl_set_object_color(fdui -> animation_template_form_dzpos_slider, FL_COL1, FL_COL1);   
fl_redraw_object(fdui -> animation_template_form_dzpos_slider);                        
 
fl_set_dial_value(fdui -> animation_template_form_color_circle_dial, 0.0);
fl_set_slider_value(fdui -> animation_template_form_dcolor_slider, 0.0);
fl_set_object_color(fdui -> animation_template_form_dcolor_slider, FL_COL1, FL_COL1); 
fl_redraw_object(fdui -> animation_template_form_dcolor_slider);                      

fl_set_slider_value(fdui -> animation_template_form_contrast_slider, 0.0);
fl_set_slider_value(fdui -> animation_template_form_dcontrast_slider, 0.0);
fl_set_object_color(fdui -> animation_template_form_dcontrast_slider, FL_COL1, FL_COL1);  
fl_redraw_object(fdui -> animation_template_form_dcontrast_slider);                       

fl_set_slider_value(fdui -> animation_template_form_saturation_slider, 0.0);
fl_set_slider_value(fdui -> animation_template_form_dsaturation_slider, 0.0);
fl_set_object_color(fdui -> animation_template_form_dsaturation_slider, FL_COL1, FL_COL1);
fl_redraw_object(fdui -> animation_template_form_dsaturation_slider);                     

fl_set_slider_value(fdui -> animation_template_form_transparency_slider, 0.0);
fl_set_slider_value(fdui -> animation_template_form_dtransparency_slider, 0.0);
fl_set_object_color(fdui -> animation_template_form_dtransparency_slider, FL_COL1, FL_COL1);
fl_redraw_object(fdui -> animation_template_form_dtransparency_slider);                     

fl_set_slider_value(fdui -> animation_template_form_speed_slider, 0.0);
fl_set_slider_value(fdui -> animation_template_form_dspeed_slider, 0.0);
fl_set_object_color(fdui -> animation_template_form_dspeed_slider, FL_COL1, FL_COL1);  
fl_redraw_object(fdui -> animation_template_form_dspeed_slider);                       

fl_set_dial_value(fdui -> animation_template_form_heading_dial, 0.0);
fl_set_slider_value(fdui -> animation_template_form_dheading_slider, 0.0);
fl_set_object_color(fdui -> animation_template_form_dheading_slider, FL_COL1, FL_COL1);
fl_redraw_object(fdui -> animation_template_form_dheading_slider);

set_template_dial_color(0.0, 0.0);

fl_set_button(fdui -> animation_template_form_center_button, 0);
fl_set_button(fdui -> animation_template_form_reformat_button, 0);

fl_set_input(fdui -> animation_template_form_vespace_input_field, "");

select_heading_or_x_y(0);
fl_set_button(fdui -> animation_template_form_heading_or_x_y_select_button, 0);

return 1;
} /* end function clear_animation_template_form */


int set_template_dial_color(double degrees, double saturation)
{
int y, u, v;
int cy, cr, cg, cb;
double du, dv;
int red, green, blue;

if(debug_flag)
	{
	fprintf(stdout,\
	"set_template_dial_color(): arg degrees=%.2f saturation=%.2f\n",\
	degrees, saturation);
	}

saturation /= 100.0;

du = sin(degrees * M_PI / 180.0);
dv = cos(degrees * M_PI / 180.0);

u = du * 255.0 * saturation;
v = dv * 255.0 * saturation;

/* yuv to rgb */
/* lot of white */
y = 200;
cy = y - 16;
cy  *= 76310;

cr = 104635 * v;
cg = -25690 * u + -53294 * v;
cb = 132278 * u;

red = LIMIT(cr + cy);
green = LIMIT(cg + cy);
blue =  LIMIT(cb + cy);

if(debug_flag)
	{
	fprintf(stdout, "set_template_dial_color(): red=%d green=%d blue=%d\n", red, green, blue);
	}

fl_mapcolor(FL_FREE_COL1 + 24, red, green, blue);

fl_set_object_color(\
	fdui -> animation_template_form_color_circle_dial, FL_FREE_COL1 + 24, FL_FREE_COL1 + 24);

fl_redraw_object(fdui -> animation_template_form_color_circle_dial);

return 1;
} /* end function set_template_dial_color */


int select_heading_or_x_y(int x_y)
{
if(x_y)
	{
	fl_hide_object(fdui -> animation_template_form_heading_dial);
	fl_hide_object(fdui -> animation_template_form_heading_degrees_output_field);
	fl_hide_object(fdui -> animation_template_form_speed_slider);
	fl_hide_object(fdui -> animation_template_form_dspeed_slider);
	fl_hide_object(fdui -> animation_template_form_dspeed_slider_reset_button);
	fl_hide_object(fdui -> animation_template_form_dheading_slider);
	fl_hide_object(fdui -> animation_template_form_dheading_slider_reset_button);
	
	fl_show_object(fdui -> animation_template_form_xpos_slider);
	fl_show_object(fdui -> animation_template_form_ypos_slider);
	fl_show_object(fdui -> animation_template_form_dxpos_slider);
	fl_show_object(fdui -> animation_template_form_dxpos_slider_reset_button);
	fl_show_object(fdui -> animation_template_form_dypos_slider);
	fl_show_object(fdui -> animation_template_form_dypos_slider_reset_button);
	}
else
	{
	fl_hide_object(fdui -> animation_template_form_xpos_slider);
	fl_hide_object(fdui -> animation_template_form_ypos_slider);
	fl_hide_object(fdui -> animation_template_form_dxpos_slider);
	fl_hide_object(fdui -> animation_template_form_dxpos_slider_reset_button);
	fl_hide_object(fdui -> animation_template_form_dypos_slider);
	fl_hide_object(fdui -> animation_template_form_dypos_slider_reset_button);

	fl_show_object(fdui -> animation_template_form_heading_dial);
	fl_show_object(fdui -> animation_template_form_heading_degrees_output_field);
	fl_show_object(fdui -> animation_template_form_speed_slider);
	fl_show_object(fdui -> animation_template_form_dspeed_slider);
	fl_show_object(fdui -> animation_template_form_dspeed_slider_reset_button);
	fl_show_object(fdui -> animation_template_form_dheading_slider);
	fl_show_object(fdui -> animation_template_form_dheading_slider_reset_button);
	}

return 1;
} /* end function select_heading_or_x_y */


int create_subtitle_animation_template()
{
struct animation_template *pa;

if(debug_flag)
	{
	fprintf(stderr, "create_subtitle_animation_template(): arg none\n");
	}

/* there should be at least a 'subtitle' object, so we can set its parameters */

pa = lookup_animation_template("subtitle");
if(! pa)
	{
	pa = install_animation_template_at_end_of_list("subtitle");
	if(! pa)
		{
		fl_show_alert("load_animation_templates():", "could not create default subtitle object", "", 0 );

		return 0;
		}

	selected_animation_template = pa;

	/* set some defaults */
	animation_template_form_default_button_cb(0, 0);

	fill_in_animation_template_form(selected_animation_template);
	} /* end if subtitle entry did not exists */

return 1;
} /* end function create_subtitle_animation_template */


