/*
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "xste.h"

FD_xste *fdui;

int forked_flag;


int play_audio_raw(unsigned long start_sample, unsigned long end_sample)
{
int a, c, i, j, k;
char temp[4096];
pid_t pid;
char execv_args[51][1024];
char *flip[51]; /* this way I do not have to free anything arguments to ececlv are copied */
char helper_program[512];
char helper_flags[1024];

if(debug_flag)
	{
	fprintf(stdout,\
	"play_audio_raw(): arg start_sample=%lu end_sample=%lu\n",\
	start_sample, end_sample);
	}   

/* argument check */
if(end_sample <= start_sample) return 0;

if(forked_flag) return 1;/* was already started */
	
audio_sample_speed = RAW_SAMPLE_SPEED;

strcpy(helper_program, "xste_audio"); /* program name */

sprintf(helper_flags, "%s/%s/%s/sound.raw %s %lu %lu %d %s",\
home_dir, selected_data_directory, selected_project,\
selected_device,\
start_sample, end_sample, audio_sample_speed,\
home_dir);

/* fill in the multidimensional argument array for execv() */
/* the first one is the helper program its self */
strcpy(execv_args[0], helper_program);
k = 0;
i = 1;
while(1)
	{
	if(helper_flags[k] == ' ')
		{
		k++;
		continue;
		}
	j = 0;
	while(1)
		{
		c = helper_flags[k];
		if(c == ' ') c = 0;
		execv_args[i][j] = c; 
		if(c == 0) break;
		j++;
		k++;
		}
	i++;
	if(helper_flags[k] == 0) break;
	}
execv_args[i][0] = 0;

/*
the reason I am using char *flip[] is that I cannot assign a NULL to
execv_args[i].
The reason I copy the (temporary stack variable) execv_args[][] into a *ptr[]
is that, by not using strsave(), I do not have to free later.
ececlv() will copy its arguments, flip[] may be destroyed after the call to
execlv().
*/
temp[0] = 0;
i = 0;
while(1)
	{
	flip[i] = execv_args[i];	
	if(execv_args[i][0] == 0)
		{
		flip[i] = temp;
		flip[i + 1] = NULL;
		break;
		}
	i++;
	}

if(debug_flag)
	{
	for(i = 0; execv_args[i][0] != 0; i++)
		{
		fprintf(stdout, "i=%d execv_args[i]=%s flip[i]=%s\n",\
		i, execv_args[i], flip[i]);
		}
	}

/* report to user */
if(debug_flag)
	{
	fprintf(stdout, "Starting helper program %s %s\n",\
	helper_program, temp);
	}

/* fork */
pid = fork();
if(pid == 0)/* in child */
	{
	/* 
	start the helper program
	NOTE: helper program  must be in the path or in the current directory.
	*/

	/* a NULL pointer is required as the last argument */
	a = execvp(helper_program, flip );
	if(a < 0)
		{
		if(debug_flag)
			{
			fprintf(stdout,\
			"\nCannot start helper program execvp failed: %s %s errno=%d",\
			helper_program, temp, errno);
			}
		return 0;
		}
	}
else if(pid < 0)/* fork failed */
	{
	fl_show_alert("Helper program fork failed", "", "", 0);
	return 0;
	}
else/* parent */
	{	
	}
//forked_flag = 1;

return 1;
}/* end function play_audio_raw */


int play_audio_wav(unsigned long start_sample, unsigned long end_sample)
{
int a, c, i, j, k;
char temp[4096];
pid_t pid;
char execv_args[51][1024];
char *flip[51];/* this way I do not have to free anything  arguments to ececlv are copied */
char helper_program[512];
char helper_flags[1024];

if(debug_flag)
	{
	fprintf(stdout,\
	"play_audio_wav(): arg start_sample=%lu end_sample=%lu\n\
	audio_sample_speed=%d\n",\
	start_sample, end_sample, audio_sample_speed);
	}   

/* argument check */
if(end_sample <= start_sample) return 0;

if(forked_flag) return 1;/* was already started */
	
strcpy(helper_program, "xste_audio"); /* program name */

sprintf(helper_flags, "%s/%s/%s/sound.wav %s %lu %lu %d %s",\
home_dir, selected_data_directory, selected_project,\
selected_device,\
start_sample, end_sample, audio_sample_speed,\
home_dir);

/* fill in the multidimensional argument array for execv() */
/* the first one is the helper program its self */
strcpy(execv_args[0], helper_program);
k = 0;
i = 1;
while(1)
	{
	if(helper_flags[k] == ' ')
		{
		k++;
		continue;
		}
	j = 0;
	while(1)
		{
		c = helper_flags[k];
		if(c == ' ') c = 0;
		execv_args[i][j] = c; 
		if(c == 0) break;
		j++;
		k++;
		}
	i++;
	if(helper_flags[k] == 0) break;
	}
execv_args[i][0] = 0;

/*
the reason I am using char *flip[] is that I cannot assign a NULL to
execv_args[i].
The reason I copy the (temporary stack variable) execv_args[][] into a *ptr[]
is that, by not using strsave(), I do not have to free later.
ececlv() will copy its arguments, flip[] may be destroyed after the call to
execlv().
*/
temp[0] = 0;
i = 0;
while(1)
	{
	flip[i] = execv_args[i];	
	if(execv_args[i][0] == 0)
		{
		flip[i] = temp;
		flip[i + 1] = NULL;
		break;
		}
	i++;
	}

if(debug_flag)
	{
	for(i = 0; execv_args[i][0] != 0; i++)
		{
		fprintf(stdout, "i=%d execv_args[i]=%s flip[i]=%s\n",\
		i, execv_args[i], flip[i]);
		}
	}

/* report to user */
if(debug_flag)
	{
	fprintf(stdout, "Starting helper program %s %s\n",\
	helper_program, temp);
	}

/* fork */
pid = fork();
if(pid == 0)/* in child */
	{
	/* 
	start the helper program
	NOTE: helper program  must be in the path or in the current directory.
	*/

	/* a NULL pointer is required as the last argument */
	a = execvp(helper_program, flip );
	if(a < 0)
		{
		if(debug_flag)
			{
			fprintf(stdout,\
			"\nCannot start helper program execvp failed: %s %s errno=%d",\
			helper_program, temp, errno);
			}
		return 0;
		}
	}
else if(pid < 0)/* fork failed */
	{
	fl_show_alert("Helper program fork failed", "", "", 0);
	return 0;
	}
else/* parent */
	{	
	}
//forked_flag = 1;

return 1;
}/* end function play_audio_wav */


int play_audio_mp3(int start_frame)
{
int a, c, i, j, k;
char temp[4096];
pid_t pid;
char execv_args[51][1024];
char *flip[51];/* this way I do not have to free anything  arguments to ececlv are copied */
char helper_program[512];
char helper_flags[1024];
double dsf;

if(debug_flag)
	{
	fprintf(stdout,\
	"play_audio_mp3(): arg start_frame=%d\n", start_frame);
	}   

if(forked_flag) return 1;/* was already started */
	
//printf("start_frame=%d\n", start_frame);
//dsf = 6.650310375 * start_frame; // calculated
dsf = 6.650470375 * start_frame; // experimental
//printf("dsf=%.3f\n", dsf);

dsf -= 8.4;
//printf("dsf=%.3f\n", dsf);

start_frame = (int) dsf; /* TESTING */
//printf("start_frame=%d\n", start_frame);

strcpy(helper_program, "mpg123"); /* program name */

sprintf(helper_flags, "-r44100 -k%d %s/%s/%s/sound.mp3",\
start_frame, home_dir, selected_data_directory, selected_project);

if(debug_flag)
	{
	printf("play_audio_mp3(): helper_flags=%s\n", helper_flags);
	}

/* fill in the multidimensional argument array for execv() */
/* the first one is the helper program its self */
strcpy(execv_args[0], helper_program);
k = 0;
i = 1;
while(1)
	{
	if(helper_flags[k] == ' ')
		{
		k++;
		continue;
		}
	j = 0;
	while(1)
		{
		c = helper_flags[k];
		if(c == ' ') c = 0;
		execv_args[i][j] = c; 
		if(c == 0) break;
		j++;
		k++;
		}
	i++;
	if(helper_flags[k] == 0) break;
	}
execv_args[i][0] = 0;

/*
the reason I am using char *flip[] is that I cannot assign a NULL to
execv_args[i].
The reason I copy the (temporary stack variable) execv_args[][] into a *ptr[]
is that, by not using strsave(), I do not have to free later.
ececlv() will copy its arguments, flip[] may be destroyed after the call to
execlv().
*/
temp[0] = 0;
i = 0;
while(1)
	{
	flip[i] = execv_args[i];	
	if(execv_args[i][0] == 0)
		{
		flip[i] = temp;
		flip[i + 1] = NULL;
		break;
		}
	i++;
	}

if(debug_flag)
	{
	for(i = 0; execv_args[i][0] != 0; i++)
		{
		fprintf(stdout, "i=%d execv_args[i]=%s flip[i]=%s\n",\
		i, execv_args[i], flip[i]);
		}
	}

/* report to user */
if(debug_flag)
	{
	fprintf(stdout, "Starting helper program %s %s\n",\
	helper_program, temp);
	}

/* fork */
pid = fork();
if(pid == 0)/* in child */
	{
	/* 
	start the helper program
	NOTE: helper program  must be in the path or in the current directory.
	*/

	/* a NULL pointer is required as the last argument */
	a = execvp(helper_program, flip );
	if(a < 0)
		{
		if(debug_flag)
			{
			fprintf(stdout,\
			"\nCannot start helper program execvp failed: %s %s errno=%d",\
			helper_program, temp, errno);
			}
		return 0;
		}
	}
else if(pid < 0)/* fork failed */
	{
	fl_show_alert("Helper program fork failed", "", "", 0);
	return 0;
	}
else/* parent */
	{	
	}
//forked_flag = 1;

return 1;
}/* end function play_audio_mp3 */


