/*
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "xste.h"

FD_xste *fdui;

#define BUFFER_SIZE	65536

/*
Find start of signals after pauses in audio.raw file 8 bits 11025 Hz mono.
*/
int find_start_points_in_raw_file(\
	char *filename, char *device, int window, int m_secs)
{
int a, i;
unsigned char d;
char temp[1024];
int filefd;
int pause_cnt;
int top;
int bottom;
int pause_mode;
unsigned long sample;
char *buffer;
int entries;
int max_pause;
int start_frame;
int end_frame;
unsigned long start_sample;
unsigned long end_sample;

if(debug_flag)
	{
	fprintf(stdout, "find_start_points(): arg\n\
	filename=%s device=%s window=%d m_secs=%d\n",\
	filename, device, window, m_secs);
	}

if ((filefd = open (filename, O_RDONLY, 0)) == -1)
	{
	fl_show_alert(\
	"find_start_points(): could not open file", filename, "aborted", 1);

	return 0;
	}

buffer = malloc(BUFFER_SIZE);
if(! buffer)
	{
	fl_show_alert("find_start_points()", "Could allocate buffer", "", 1);

	return 0;
	}

delete_all_frames();

top = 127 + (window / 2);
bottom = 127 - (window / 2);

#define PAUSE_PRESENT	0
#define PAUSE_END		1

max_pause = m_secs * 11;
entries = 0;
pause_cnt = 0;
pause_mode = PAUSE_PRESENT;
sample = 0;
while(1)
	{
	errno = 0;
	a = read (filefd, buffer, BUFFER_SIZE);
	if(a < 0)
		{
		if(errno == EAGAIN) continue;
		else if(errno == EINTR) continue;	
		else
			{
			fl_show_alert("find_start_points():", "read failed", "aborted", 1);

			close(filefd);

			return 0;
			}
		}
	if(a == 0)
		{
		close(filefd);

		return 1;
		}

	for(i = 0; i < a; i++)
		{
		sample++;

		d = buffer[i];

		/* test for no signal */
		if( (d < top) && (d > bottom) ) /* no signal */
			{
			pause_cnt++;

			/* if no signal for max_pause samples it is a pause */
			if(pause_cnt > max_pause)
				{
				pause_mode = PAUSE_PRESENT;
//fprintf(stdout, "PAUSE PRESENT\n");
				}
			}
		else /* signal */
			{
			/*
			if a pause was present, and now signal, we have the start
			of a new piece of text or music.
			*/
			if(pause_mode == PAUSE_PRESENT)
				{
//fprintf(stdout, "NEW WORD\n");
				/* new start of audio detected */
				
				/* look up start frame from current sample */	
				if(! get_next_frame_and_sample(\
				sample, &start_frame, &start_sample) )
					{
					close(filefd);

					sprintf(temp, "%lu", sample);
					fl_show_alert(\
					"find_start_points():",\
					"Could not look up sample",\
					temp,\
					1);
	
					return 0;
					}

				/* since no text, length of this entry is zero */
				end_frame = start_frame;

				end_sample = start_sample;					

				/* add entry to frame list */
				strcpy(temp, "");
				if(! add_frame(temp,\
				start_frame, end_frame, 0) ) /* text_hold_flag is zero */
					{
					close(filefd);

					sprintf(temp, "%d", start_frame);
					fl_show_alert(\
					"find_start_points_in_raw_file():",\
					"Could not add frame",\
					temp,\
					1);

					return 0;
					}

				entries++;
				}

			/* reset the pause mode to no pause */
			pause_mode = PAUSE_END;
//fprintf(stdout, "PAUSE_END\n");
			/* reset the pause counter */
			pause_cnt = 0;
			}
		} /* end for i (bytes read) */

	} /* end while 1 */

/* never here */
return 1;
}/* end function find_start_points_in_raw_file */


/*
Find start of signals after pauses in audio.wav file.
*/
int find_start_points_in_wave_file(\
	char *filename, char *device, int window, int m_secs)
{
int a, c, i;
unsigned char d;
unsigned int value = 0; // -Wall
char temp[1024];
int filefd;
int pause_cnt;
int top;
int bottom;
int pause_mode;
unsigned long sample;
char *buffer;
int entries;
int max_pause;
int start_frame;
int end_frame;
unsigned long start_sample = 0; // -Wall
unsigned long end_sample = 0; // -Wall
int header_size;
wave_header *pw;
int dsp_speed;
int sample_size = 0; // -Wall
int multiplier;
int dsp_stereo = 0; // -Wall
int byte_counter;

if(debug_flag)
	{
	fprintf(stdout, "find_start_points_in_wave_file(): arg\n\
	filename=%s device=%s window=%d m_secs=%d\n",\
	filename, device, window, m_secs);
	}

header_size = sizeof(wave_header);

pw = malloc(header_size);
if(! pw)
	{
	printf("find_start_points_in_wave_file():\n\
	could not allocate memory for wave_header\n");

	return 0;
	}

if( (filefd = open (filename, O_RDONLY, 0)) == -1)
	{
	fl_show_alert(\
	"find_start_points_in_wave_file():\n\
	could not open file", filename, "aborted", 1);

	return 0;
	}

/* read in header_size (44 bytes) wave header */
c = header_size;
while(1)
	{
	a = read(filefd, pw, c);

	if(a < 0)
		{
		perror("find_start_points_in_wave_file(): read header");

		return 0;
		}

	if(a == 0) break;
	c -= a;	
	} /* end while read header */

dsp_speed = pw -> sample_fq;
sample_size *= 8 * pw -> byte_p_spl;

//multiplier = pw -> modus * pw -> byte_p_spl;
multiplier = pw -> byte_p_spl;
multiplier *= pw -> sample_fq / RAW_SAMPLE_SPEED;

start_sample *= multiplier;
end_sample *= multiplier;
start_sample += header_size;
end_sample += header_size;

if(debug_flag)
	{
	printf(\
	"find_start_points_in_wave_file():\n\
	dsp_speed=%d dsp_stereo=%d sample_size=%d multiplier=%d\n\
	start_sample=%lu end_sample=%lu\n",\
	dsp_speed, dsp_stereo, sample_size, multiplier,\
	start_sample, end_sample);
	}

buffer = malloc(BUFFER_SIZE);
if(! buffer)
	{
	fl_show_alert(\
	"find_start_points_in_wave_file()", "Could allocate buffer", "", 1);

	return 0;
	}

delete_all_frames();

top = 127 + (window / 2);
bottom = 127 - (window / 2);

#define PAUSE_PRESENT	0
#define PAUSE_END		1

/* skip the header */
a = lseek(filefd, header_size, SEEK_SET);
if(a == -1)
	{
	close(filefd);

	fprintf(stdout,\
	"find_start_points_in_wave_file():\n\
	could not seek to sample %d, aborted\n",\
	header_size);

	return 0;
	}

max_pause = m_secs * 11;
entries = 0;
pause_cnt = 0;
pause_mode = PAUSE_PRESENT;
sample = 0;
byte_counter = 0;
while(1)
	{
	errno = 0;
	a = read(filefd, buffer, BUFFER_SIZE);
	if(a < 0)
		{
		if(errno == EAGAIN) continue;
		else if(errno == EINTR) continue;	
		else
			{
			fl_show_alert(\
			"find_start_points_in_wave_file():", "read failed", "aborted", 1);

			close(filefd);

			return 0;
			}
		}
	if(a == 0)
		{
		close(filefd);

		return 1;
		}

	for(i = 0; i < a; i++)
		{
		sample++;

		d = buffer[i];
		
		/*
		if mono 8 bits, read each byte
		if mono 16 bits, combine each 2 bytes
		if stereo 8 bit, read each other byte
		if stereo 16 bits, combine 2 bytes, then scip 3 bytes
		*/
		if(pw -> modus == 1) // mono
			{
			if(pw -> byte_p_spl == 1) // 8 bits mono
				{
				/* use each byte */
				value = d;				

				byte_counter = 0; 
				} /* end if 8 bits mono */
			else if(pw -> byte_p_spl == 2) // 16 bits mono
				{
				if(byte_counter == 0)
					{
					/* ignore low byte */
					byte_counter++;
					continue;
					}			
				else if(byte_counter == 1)
					{
					/* use high byte */
					value = d;
					
					byte_counter = 0;
					}
				} /* end if 16 bits mono */
			} /* end if mono */
		else if(pw -> modus == 2) // stereo
			{
			if(pw -> byte_p_spl == 2) // 8 bits stereo
				{
				if(byte_counter == 0)
					{
					/* use channel 1 each byte */
					value = d;
					byte_counter++;
					}
				else if(byte_counter == 1)
					{
					/* ignore channel 2 */
					byte_counter = 0;
					continue;
					}
				} /* end if 8 bits stereo */
			else if(pw -> byte_p_spl == 4) // 16 bits stereo
				{
				if(byte_counter == 0)
					{
					/* ignore channel 1 low byte */
					byte_counter++;
					continue;
					}
				else if(byte_counter == 1)
					{
					/* use channel 1 high byte */
					value = d;
					byte_counter++;
					}
				else if(byte_counter == 2)
					{
					/* ignore channel 2 low byte */
					byte_counter++;
					continue;
					}
				else if(byte_counter == 3)
					{
					/* ignore channel 2 high byte */
					byte_counter = 0;
					continue;
					}
				} /* end if 16 bits stereo */
			} /* end if stereo */

		/* only using high byte, msb is sign bit */
		if(value > 127) value -= 256;
		value += 128;

		/* test for no signal */
		if( (value < top) && (value > bottom) ) /* no signal */
			{
			pause_cnt++;

			/* if no signal for max_pause samples it is a pause */
			if(pause_cnt > (max_pause * multiplier) )
				{
				pause_mode = PAUSE_PRESENT;
//fprintf(stdout, "PAUSE PRESENT\n");
				}
			}
		else /* signal */
			{
			/*
			if a pause was present, and now signal, we have the start
			of a new piece of text or music.
			*/
			if(pause_mode == PAUSE_PRESENT)
				{
//fprintf(stdout, "NEW WORD\n");
				/* new start of audio detected */
				
				/* look up start frame from current sample */	
				if(! get_next_frame_and_sample(\
				sample / multiplier, &start_frame, &start_sample) )
					{
					close(filefd);

					sprintf(temp, "%lu", sample);
					fl_show_alert(\
					"find_start_points_in_wave_file():",\
					"Could not look up sample",\
					temp,\
					1);
	
					return 0;
					}

				/* since no text, length of this entry is zero */
				end_frame = start_frame;

				end_sample = start_sample;					

				/* add entry to frame list */
				strcpy(temp, "");
				if(! add_frame(temp,\
				start_frame, end_frame, 0) ) /* text_hold_flag is zero */
					{
					close(filefd);

					sprintf(temp, "%d", start_frame);
					fl_show_alert(\
					"find_start_points_in_wave_file():",\
					"Could not add frame",\
					temp,\
					1);

					return 0;
					}

				entries++;
				}

			/* reset the pause mode to no pause */
			pause_mode = PAUSE_END;
//fprintf(stdout, "PAUSE_END\n");

			/* reset the pause counter */
			pause_cnt = 0;
			} /* end if signal */

		} /* end for i (bytes read) */

	} /* end while 1 */

/* never here */
return 1;
}/* end function find_start_points_in_wave_file */


