/*
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "xste.h"

extern FD_xste *fdui;
extern FD_xste *fd_main_form;


int create_bitmap_file(struct frame *pa)
{
int a, c, i, j;
int l, h, hh;
char *tptr, *cptr;
int x, y;
char temp[4096];
FILE *fptr;
int text_start, max_width, line_len;
uint8_t *py;
int color;
int file_size;
int image_size;
int bitmap_byte = 0; // -Wall
int odd_byte;
int error;
int padding;
int bits;

if(debug_flag)
	{
	printf("create_bitmap_file(): arg pa=%p\n", pa);
	}

/* argument check */
if(! pa) return 0;

/* set all of drawing area to background */
for(i = 0; i < (720 * 576 * 3); i += 3)
	{
	image_buffer[i] = pa -> background;
	}

/* some defaults */
pa -> width = 0;
pa -> height = 0;
pa -> xpos = 0;  
pa -> ypos = 0;
/* end defaults */

line_h_start = h_factor * (double)h_size;
line_h_end = (double)h_size - (double)line_h_start;

window_bottom = v_size - (v_factor * (double)v_size);

/*
Set hor_position for start text in all lines to zero,
center_text() may overrule this if center_flag.
*/
for(i = 0; i < MAX_SCREEN_LINES; i++)
	{
	screen_text[i][0] = 0;
	screen_start[i] = 0; // pixels from left used to center text
	}

/* reformat text inserting (multiple) '/n' if too long */ 
tptr = p_reformat_text(pa, line_h_end - line_h_start);
if(! tptr)
	{
	printf("create_bitmap_file(): could not reformat text=%s\n", pa -> text);
	
	/* return error */
	return 0;
	} /* end if reformat text failed */		

/* center text */
if(pa -> status & CENTER_TEXT) p_center_text(tptr, pa);

/* text to array screen_text[] */
cptr = tptr;
screen_lines = 1; /* at least one */
while(1) /* all chars in tptr */
	{
	i = 0;
	while(1) /* all chars in a line */
		{
		if(*cptr == '\n')
			{
			/* scip the '\n' */
			cptr++;
				
			/* force string termination */
			screen_text[screen_lines - 1][i] = 0;

			/* point to next screen line */
			screen_lines++;
			break;
			}

		/* copy character */
		screen_text[screen_lines - 1][i] = *cptr;	

		/* test for end of string tptr */
		if(*cptr == 0) break;

		/* point to next char in screen_lines[][] */
		i++;

		/* point to next char in tptr */
		cptr++;
		} /* end while all characters in line terminated with LF */

	/* ready if end of tptr */
	if(*cptr == 0) break;
				
	} /* end while all lines in tptr */
free(tptr);

/* some limit */
if(screen_lines > MAX_SCREEN_LINES) screen_lines = MAX_SCREEN_LINES;

line_height = pa -> pfd -> height + pa -> vespace;
window_top = window_bottom - (screen_lines * line_height) + pa -> vespace;

if(debug_flag)
	{
	printf("line_height=%d\n", line_height);
	printf("screen_lines=%d\n", screen_lines);
	printf("line_h_start=%d line_h_end=%d\n",\
	line_h_start, line_h_end);
	printf("window_bottom=%d window_top=%d\n",\
	window_bottom, window_top);
	}

// Maybe no warn / scip here, input test was already done, this is fuzzy? 
if(window_top < 0 - line_height)
	{
	fl_show_alert("Image for frame", pa -> name, "does not fit.\nSkipped.", 0);
	
	return 0;
	}

/* print lines of text on screen in right position */
text_start = INT_MAX;
max_width = 0;
for(i = 0; i < screen_lines; i++)
	{
	/* convert any special characters */
/*
	j = 0;
	while(1)
		{
		c = screen_text[i][j];
		character_lookup(c, &b);
		screen_text[i][j] = b;

		if(c == 0) break;
		j++;
		}
*/

	/* get text length */
	line_len = 0;
	j = 0;
	while(1)
		{
		c = screen_text[i][j];
		line_len += get_h_pixels(c, pa);

		if(c == 0) break;
		j++;
		}

	x = screen_start[i];

	if(x < text_start) text_start = x;

	if(line_len > max_width) max_width = line_len;

	y = window_top + (i * line_height);

	if(debug_flag)
		{
		printf(\
		"screen_start[%d]=%d window_bottom=%d window_top=%d\n\
line_height=%d x=%d y=%d\n\
text=%s\n",\
		i, screen_start[i], window_bottom, window_top,\
		line_height, x, y,\
		screen_text[i]);
		}
/*
	int add_text_x(\
	int x, int y, char *text, struct frame *pa, int u, int v, double contrast, double transparency)
*/
	add_text(x, y, screen_text[i], pa, 0, 0, 100.0, 0.0);
 
	} /* end for all screen_lines */

pa -> ypos = window_top;

/* 
text_start is the start of the most left character start in a (multiline)
text.
max_width is the length of the text in pixels.
*/

pa -> height = screen_lines * line_height - pa -> vespace; // + 9;
pa -> xpos = text_start;

/* add some space for outline */
pa -> width = max_width + 6;

/* CVD needs even width */
/* if width is not multiple of 8, make it */
a = pa -> width % 8;
if(a) pa -> width += 8 - a; 

/* CVD needs even height, if height is not multiple of 4, make it */
a = pa -> height % 4;
if(a) pa -> height += 4 - a;

if(debug_flag)
	{
	printf("create_bitmap_file():\n\
	frame=%s\n\
	text_start=%d max_width=%d\n\
	pa>xpos=%d pa->ypos=%d\n\
	pa->width=%d pa->height=%d\n\
	pa->text=%s\n",\
	pa -> name,\
	text_start, max_width,\
	pa -> xpos, pa -> ypos,\
	pa -> width, pa -> height,\
	pa -> text);
	}

/* return error if not usable overlay */
error = 0;

/* no pointer to text */
if(! pa -> text) error = 1;

/* no text */
if(pa -> text[0] == 0) error = 1;

/* no width */
if(pa -> width <= 0) error = 1;

/* no height */
if(pa -> height <= 0) error = 1;

/* negative xpos */
if(pa -> xpos < 0) error = 1;

/* negatve ypos */
if(pa -> ypos < 0) error = 1;

/* reformatter failed */
if(max_width == 0) error = 1;

/* no space left to print, reformatter failed margin? */
if(pa -> xpos >= (h_size / 2) ) error = 1;

/* falls of right hand side of screen */
if( (pa -> width + pa -> xpos) >= h_size) error = 1;

/* falls of bottom of screen */
if(pa -> height >= v_size ) error = 1;
if(error)
	{
	if(debug_flag)
		{
		printf("create_bitmap_file(): ERROR\n");
		}

	/* return error */
	return 0;
	} /* end if error */

/* 
lets write our own bitmap, ppmtobmp does not fill in the complete colormap,
the h and v size, the ColorsImportant field, and the bitmap size.
*/

/* open the bmp file for write */
if(pa -> status & TEST_BITMAP)
	{
	sprintf(temp, "%s/%s/%s/test.bmp",\
	home_dir, selected_data_directory, selected_project);
	}
else
	{
	sprintf(temp, "%s/%s/%s/frame%04d.bmp",\
	home_dir, selected_data_directory, selected_project, atoi(pa -> name) );
	}

fptr = fopen(temp, "w");
if(! fptr)
	{
	printf(\
	"create_bitmap_file(): could not open file %s for write, aborting\n",\
	temp);

	exit(1);
	} 

/*
600 x 120 origial .bmp from iauthor.
0000:0000 42 4d 16 8d 00 00 00 00 00 00 76 00 00 00 28 00 BM........v...(.
0000:0010 00 00 58 02 00 00 78 00 00 00 01 00 04 00 00 00 ..X...x.........
0000:0020 00 00 a0 8c 00 00 00 00 00 00 00 00 00 00 10 00 ...............
0000:0030 00 00 00 00 00 00 00 00 00 00 ff ff ff 00 00 00 .............
0000:0040 80 00 00 80 00 00 00 00 48 0a e6 3c f5 bf 00 00 ........H.<..
0000:0050 7f 10 84 c2 64 7a ff ff c7 3d 98 c2 f7 3e d7 16 ...dz=.>.
0000:0060 ff ff c7 3d 00 00 b0 80 02 00 6f 00 b2 c2 d0 10 =.....o..
0000:0070 af 17 00 00 00 00 00 00 00 00 00 00 00 00 00 00 ...............
0000:0080 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 ................
*/

/* print header, all low byte first!  */

/* type */
fprintf(fptr, "%c%c", 0x42, 0x4d);

/* file_size */
file_size = (pa -> width * pa -> height / 2) + 118;
h = file_size / 256;
l = file_size - (256 * h);
fprintf(fptr, "%c%c", l, h);

if(debug_flag)
	{
	printf("file_size=%d l=%d h=%d to file=%02x %02x\n",\
	file_size, l, h, l, h);
	}

/* 2 x reserved */
fprintf(fptr, "%c%c%c%c%c%c", 0, 0, 0,  0, 0, 0);

/* offset to bitmap data */
fprintf(fptr, "%c%c%c%c", 0x76, 0, 0, 0); // 118

/* BitmapInfoHeader starts here, its size */
fprintf(fptr, "%c%c%c%c", 0x28, 0, 0, 0);

/* 
picture width, always < 65535 here (SVHS standard), so highest byte is zero.
*/
h = pa -> width / 256;
l = pa -> width - (h * 256);
fprintf(fptr, "%c%c%c%c", l, h, 0, 0);

if(debug_flag)
	{
	printf("pa->width=%d l=%d h=%d to file=%02x %02x %02x %02x\n",\
	pa -> width, l, h, l, h, 0, 0);
	}

/* picture height, always < 65535! so highest byte is zero */
h = pa -> height / 256;
l = pa -> height - (h * 256);
fprintf(fptr, "%c%c%c%c", l, h, 0, 0);

if(debug_flag)
	{
	printf("pa->height=%d l=%d h=%d to file=%02x %02x %02x %02x\n",\
	pa -> height, l, h, l, h, 0, 0);
	}

/* planes , one plane */
fprintf(fptr, "%c%c", 1, 0);

/* bit count, 4 bpp */
fprintf(fptr, "%c%c", 4, 0);

/* compression, no compression */
fprintf(fptr, "%c%c%c%c", 0, 0, 0, 0);

/* size image */
image_size = pa -> width * pa -> height / 2;
hh = image_size / 65536;
h = (image_size - (65536 * hh) ) / 256;
l = image_size - (65536 * hh) - (256 * h);
fprintf(fptr, "%c%c%c%c", l, h, hh, 0);

if(debug_flag)
	{
	printf("image_size=%d l=%d h=%d hh=%d to file=%02x %02x %02x %02x\n",\
	image_size, l, h, hh, l, h, hh, 0);
	}

/* XPelsPerMeter */
fprintf(fptr, "%c%c%c%c", 0, 0, 0, 0);

/* YPelsPerMeter */
fprintf(fptr, "%c%c%c%c", 0, 0, 0, 0);

/* ColorsUsed */
fprintf(fptr, "%c%c%c%c", 0x10, 0, 0, 0);

/* ColorsImportant */
fprintf(fptr, "%c%c%c%c", 0x00, 0, 0, 0);

for(i = 0; i < 16; i++)
	{
	/* bgr to rgb */
	fprintf(fptr, "%c%c%c%c", xste_rgb_palette[i][2], xste_rgb_palette[i][1], xste_rgb_palette[i][0], 0);
	}

/* write bitmap data */
for(y = pa -> height; y > 0; y--)
	{
	/* start line of pixels in main screen */
	py = \
	image_buffer + \
	(3 * \
	( ( (pa -> ypos - 1 + y) * h_size ) + \
	pa -> xpos\
	)\
	);

	/* get a line from buffer start, to file in RGB */
	/*
	from BMP.txt from the www:
	If necessary, a scan line must be zero-padded to end on a 32-bit
	boundary.
	However, segment boundaries can appear anywhere in the bitmap. 

	So, 32 bits is 4 bytes!!! zero padding.
	*/

	odd_byte = 0;
	bits = 0;
	for(x = 0; x < pa -> width ; x++)
		{
		/*
		We have 3 bytes per pixel in the buffer,
		we want to send 4 bits per pixel, so 2 pixels is one byte.

		For every 6 bytes in the buffer we write one byte!
		*/

		color = *py;

		/* always start line on byte boundary */
		/* does win bmp have low byte first in the bitmap ????? */

		/* if low nibble, remember */
		if(! odd_byte)
			{
			/* data to upper 4 bits */
			bitmap_byte = color << 4;
			}
		else
			{
			/* combine */
			bitmap_byte |= color;

			/* to bmp file */

			fprintf(fptr, "%c", bitmap_byte);

			bits += 8;
			}

		/* point to next pixel in source */
		py += 3;

		/* reverse odd byte */
		odd_byte = 1 - odd_byte;

		} /* end for all x */

	/* Test if we need to pad, need to be at 32 bits boundary. */
	padding = bits % 32;

	if(! padding) continue;

	/* output an extra nibble */
	a = 32 - padding;
	for(i = 0; i < a / 8; i++)
		{
		fprintf(fptr, "%c", 0);
		}		

	} /* end for y (all lines) */

fclose(fptr);

return 1;
} /* end function create_bitmap_file*/


int add_text(\
int x, int y,\
char *text,\
struct frame *pa,\
int u, int v,\
double contrast, double transparency)
{
int a;
char *ptr;

if(debug_flag)
	{
	printf("add_text(): x=%d y=%d text=%s\n\
	pa=%p u=%d v=%d contrast=%.2f transparency=%.2f\n",\
	x, y, text, pa , u, v, contrast, transparency);
	} 

ptr = text;
while(*ptr)
	{
	/* convert to signed */
	a = *ptr;
	if(*ptr < 0) a += 256;

	if(a == ' ')
		{
		/* want to print background only here, not '_' */
		print_char(x, y, a, pa, u, v, contrast, transparency, pa -> pfd, 1);
		}
	else
		{
		print_char(x, y, a, pa, u, v, contrast, transparency, pa -> pfd, 0);
		}

	x += pa -> pfd -> width[a] + pa -> pfd -> charspace;

	x += pa -> espace;

	ptr++;
	}

return 1;
} /* end function add_text */


int print_char(\
int x, int y, int c,\
struct frame *pa,\
int u, int v,\
double contrast, double transparency, font_desc_t *pfd, int is_space)
{
if(debug_flag)
	{
	printf("subtiter(): print_char(): arg\n\
	x=%d y=%d c=%d pa=%p u=%d v=%d contrast=%.2f transparency=%.2f pfd=%p is_space=%d",\
	x, y, c, pa, u, v, contrast, transparency, pfd, is_space);
	}

draw_alpha(\
	x,\
	y,\
	pa,\
	pfd->width[c],\
	pfd->pic_a[pa -> symbols]->h,\
	pfd->pic_b[pa -> symbols]->bmp + pfd->start[c],\
	pfd->pic_a[pa -> symbols]->bmp + pfd->start[c],\
	pfd->pic_a[pa -> symbols]->w,\
	u, v, contrast, transparency, is_space);

return 1;
} /* end function print_char */


void draw_alpha(\
	int x0, int y0,\
	struct frame *pa,\
	int w, int h,\
	uint8_t *src, uint8_t *srca, int stride,\
	int u, int v, double contrast, double transparency, int is_space)
{
if(debug_flag)
	{
	printf(\
	"draw_alpha(): x0=%d y0=%d pa=%p w=%d h=%d src=%p srca=%p stride=%d is_space=%d\n",\
	x0, y0, pa, w, h, src, srca, stride, is_space);
	}

draw_alpha_rgb24(\
	w, h, pa, src, srca, stride, image_buffer + 3 * (y0 * h_size + x0), 3 * h_size, is_space);
} /* end function draw_alpha */


void draw_alpha_rgb24(\
int w, int h,\
struct frame *pa,\
unsigned char* src, unsigned char *srca, int srcstride,\
unsigned char* dstbase, int dststride, int is_space)
{
int y;
int fg_red, fg_green, fg_blue;
int bg_red, bg_green, bg_blue;
double dmulto, dmulti;

if(debug_flag)
	{
	printf("draw_alpha_rgb24(): dstbase=%p p=%p using draw_mode=%d is_space=%d\n",\
	dstbase, pa, draw_mode, is_space);
	}

fg_red = 255;
fg_green = 255;
fg_blue = 255;

bg_red = 0;
bg_green = 0;
bg_blue = 0;

for(y = 0; y < h; y++)
	{
	register unsigned char *dst = dstbase;
	register int x;

	/* clip top */
	if(y >= v_size) continue;

	for(x = 0; x < w; x++)
		{
		/* clip right side */
		if(x >= h_size) continue;

		/*
		srca outline
		src white part of character.
		*/
		switch(draw_mode)
			{
			case DRAW_MODE_COLOR:
				if(srca[x])
					{
					/* BGR */
					dst[0] = ((dst[0] * fg_blue) >> 8) + src[x];
					dst[1] = ((dst[1] * fg_green) >> 8) + src[x];
					dst[2] = ((dst[2] * fg_red) >> 8) + src[x];
					}
				else
					{
					dst[0] = ((dst[0] * bg_blue) >> 8) + src[x];
					dst[1] = ((dst[1] * bg_green) >> 8) + src[x];
					dst[2] = ((dst[2] * bg_red) >> 8) + src[x];
					}
				break;
			case DRAW_MODE_BW_FAST:
				if(srca[x])
					{
					dst[0] = dst[1] = dst[2] = src[x];
					}
				break;
			case DRAW_MODE_BW:	
				if(srca[x])
					{
					dst[0] = ((dst[0] * srca[x]) >> 8) + src[x];
					dst[1] = ((dst[1] * srca[x]) >> 8) + src[x];
					dst[2] = ((dst[2] * srca[x]) >> 8) + src[x];
					}
				break;
			case DRAW_MODE_OGT:
				/* character present */
				/*
				using some code to have 4 parameters for the bitmap routine,
				so no longer anything to do with rgb.
				*/
				if(srca[x] && !is_space)
					{
					/* get multiplier as double */
					dmulto = (double) srca[x] / 256.0;

					/* multiplier to range 0 to 1.0 */
					dmulti = (double) src[x] / 256.0;

					if(pa -> status & DVD_ANTI_ALIAS)
						{
						/* multiplier to range 0 to 1.0 */
						dmulti = (double) src[x] / 256.0;
						}
					else
						{
						if(src[x]) dmulti = 1.0;
						else dmulti = 0.0;
						}

					if(dmulti > .5)
						{
						/* white part */
						/* if transparent display background */
						if(pa -> pattern_contrast != 0) dst[0] = pa -> pattern;
						}
					else
						{	
						if(pa -> status & DVD_ANTI_ALIAS) /* use emphasis2 for anti-alias */
							{	
							/* use outline2 for anti aliasing, set to grey 50 % */
							if( (dmulti > 0) && (dmulti < .5) )
								{
								if(pa -> emphasis2_contrast != 0) dst[0] = pa -> emphasis2;
								}
							else
								{
								/* border */
								/* if transparent display background */
								if(pa -> emphasis1_contrast != 0) dst[0] = pa -> emphasis1;
								}
							} /* end if DVD anti-alias */
						else
							{	
							/* border */
							/* if transparent display background */
							if(pa -> emphasis1_contrast != 0) dst[0] = pa -> emphasis1;
							}
						}
					} /* end if srca[x] && ! is_space */
				else /* emphasis2 */
					{
					if(! (pa -> status & DVD_ANTI_ALIAS) ) /* use emphasis2 for  character space */
						{
						/* if transparent display background */
						if(pa -> emphasis2_contrast != 0) dst[0] = pa -> emphasis2;
						}
					}
				break;
			default:
				return;
				break;
			} /* end switch draw_mode */

		dst += 3; // 24bpp
		} /* end for all x */

	src += srcstride;
	srca += srcstride;
	dstbase += dststride; // flipped ??
	} /* end for all y */

return;
} /* end function draw_alpha_rgb24 */

