
/* 
xste subtitle editor
xste  is copyright (C) 2004 and later <Jan Panteltje>
email: panteltje@yahoo.com

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "xste.h"

extern FD_xste *fdui;


/*
This generates a file with timecodes for dvdwizard-0.4.1
dvdwizard is copyright  Wolfgang Wershofen (mailto: itconsult at wershofen.de)
dvdwizard automatically generates a DVD with menus using dvdauthor.

The format:
   b. Specify a string with the desired timecodes in the same format, dvdauthor expects them:
      [h:][mm:]ss[.frac], where <frac> are ms, to make frame accurate specifications. This frac must be a
      multiple of 40 (1 Frame = 40 ms for PAL), e.g. dvdwizard -c 39.400,5:12.640,1:10:42.080
      starts a chapter at 39 seconds and 10 frames, 5 minutes, 12 seconds and 16 frames and the last at
      1 hour, 10 minutes, 42 seconds and 2 frames.
      dvdauthor may not be frame accurate, if the specified frame is not an I-Frame. In this case, dvdauthor
      the chapter boundary on the most recent I-frame before the specified timecode.

   c. Specify a text file containing the timecodes as described unter b. this way, you may note down your 
      chapter breakpoints in a file and pass it to dvdwizard.

The format in German:
   (b) Angabe einer Liste von Timecodes, durch Komma getrennt und
          ohne Leerzeichen dazwischen. Format der Timecodes:
          [h:][mm:]ss[.frac], wobei <frac> Millisekunden sind, um
          framegenaue Angaben machen zu kvnnen. Die Angabe mu_ f|r PAL
          ein ganzzahliges Vielfaches von 40 sein (1 Frame = 40 ms),  
          z.B.
                  dvdwizard -c 39.400,5:12.640,1:10:42.080
          erzeugt neue Kapitel nach 39 Sekunden und 10 Frames, 5
          Minuten, 12 Sekunden und 16 Frames sowie 1 Stunde, 10 
          Minuten, 42 Sekunden und 2 Frames.
          dvdauthor weicht von dieser framegenauen Angabe ab, wenn der
          Frame kein I-Frame ist. In diesem Fall verlegt dvdauthor den
          Kapitelbeginn auf den ndchstgelegenen I-Frame vor der
          gemachten Angabe.


      (c) Angabe einer Textdatei, in der die Timecodes abgelegt sind.
          Das Format in dieser Datei unterliegt den gleichen Regeln  
          wie im vorherigen Punkt beschrieben.
*/



int write_chapter_list()
{
struct frame *pa;
FILE *fptr;
char temp[4096];
int start_hour, start_minute, start_second;
double dstart_second;
long lthousands;
int start_milli_second;
char text_part[TEXT_ARRAY_SIZE];
char command_part[TEXT_ARRAY_SIZE];
int chapter_number;
char *ptr;

if(debug_flag)
	{
	printf("write_chapter_list(): arg none\n");
	}

draw_mode = DRAW_MODE_BW;

/* some defaults */
/* end defaults */

/* 40 ms per frame (of 25 Hz) */
//audio_samples_per_frame = audio_sample_speed / dmilli_seconds_per_frame;

/* had to imagine some extention name 'srt' */
sprintf(temp, "%s/%s/%s/chapters.txt",\
home_dir, selected_data_directory, selected_project);
fptr = fopen(temp, "w");
if(! fptr)
	{
	fl_show_alert("write_chapter_list", "Could not open for write", temp, 1);

	return 0;
	}

chapter_number = 0;
for(pa = frametab[0]; pa != 0; pa = pa -> nxtentr)
	{
	/*
	do not print negative frames, this results in negative times,
	not sure srt likes that.
	*/
	if( atoi(pa -> name) < 0) continue;

	/* do not print xste command lines in the form '*command' */
//	if(pa -> text[0] == '*') continue;

	/* we have this data at our disposal */ 
	//	char *name; /* this is the start frame in ASCII decimal */
	//	char *text;
	//	int end_frame; /* this is the end frame */
	//	unsigned long start_sample;
	//	unsigned long end_sample;
 
	/*
	We can time to the sound file, at audio_sample_speed samples / second,
	(actually 11025).
	Some players sync to the audio, some to the video.
	*/

	/* calculate start_time */
	lthousands = atoi(pa -> name) * dmilli_seconds_per_frame;
	start_hour = lthousands / 3600000;
	lthousands -= start_hour * 3600000; 
	start_minute = lthousands / 60000;
	lthousands -= start_minute * 60000;
	dstart_second = lthousands / 1000.0;

	start_second = lthousands / 1000; 
	start_milli_second = lthousands - (1000 * start_second);	

	/* do not print negative frames, this results in negative times. */
	if(start_hour < 0) continue;
	if(start_minute < 0) continue;
	if(dstart_second < 0.0) continue;
		
	/* split of any commands in the form of '[command] */
	if(! extract_command(pa -> text, text_part, command_part) ) return 0;

	/* skip any command that does not start with chapter */
	if(strncmp(command_part, "chapter", 7) != 0) continue;	

	if(debug_flag)
		{
		fprintf(stderr,\
		"write_chapter_list(): command_part=%s chapter_number=%d\n", command_part, chapter_number);
		}

	/* extract argument to chapter, the chapter name as how to appear below thumbnail in menu  */
	ptr = strchr(command_part, ' ');
	if(! ptr)
		{
		/* have an empty string ready */
		temp[0] = 0;	
		ptr = temp;
		} /* end if no chapter name specified */
	else
		{
		/* point past space, field may have zero length (*ptr=0) */
		ptr++;
		}

/*
	[h:][mm:]ss[.frac],
	wobei <frac> Millisekunden sind, um framegenaue Angaben machen zu kvnnen.
	Die Angabe mu_ f|r PAL  ein ganzzahliges Vielfaches von 40 sein (1 Frame = 40 ms),
	z.B.
	dvdwizard -c 39.400,5:12.640,1:10:42.080
*/

	/* print entry */
	if(chapter_number != 0)
		{
		fprintf(fptr, ",");
		}

	fprintf(fptr, "%02d:%02d:%02d.%03d%s",\
	start_hour, start_minute, start_second, start_milli_second, ptr);

	chapter_number++;
	} /* end for all frames */

/* close the file */
fclose(fptr);

return 1;
}/* end function write_chapter_list */

