#ifndef _EBU_GSI_LIST_H_
#define _EBU_GSI_LIST_H_

struct ebu_gsi
	{
	/* Byte 0-2 Code Page Number CPN */
	// 850 multilangual -> 0x38 0x35 0x30 
	char code_page_number[3 + 1];
	
	/* Byte 3-10 Disc Format Code DFC */
	// STL25.01 for 25 fps  ''S' 'T' 'L' '2' '5' '.' '0' '1'
	// STL30.01 for 30 fps
	char disk_format_code[8 + 1];

	/* Byte 11 Display Standard Code */
	// Undifined 0x20
	// Open Subtitling 0x30
	// Level 1 Teletext 0x31
	// Level 2 teletext 0x32
	char display_standard_code[1 + 1];

	/* Byte 12-13 Character Code Table number CCT */
	// Latin 0x30 0x30
	// Latin /Cyrillic	0x30 0x31
	// Latin/Arabic		0x30 x032
	// Latin/Greek		0x30 0x33
	// Latin/Hebrew		0x30 0x34
	char character_code_table_number[2 + 1];

	/* Byte 14-15 Language Code LC */
	// Dutch = 0x33 should this be 0x00 0x33 or 0x33 0x33 or 0x33 0x00 ??????
	// GAMBLE 0x33 0x00 (second byte reserved for national assignment)
	char language_code[2 + 1];

	/* Byte 16-47 Original Program Title OPT */
	char original_program_title[32 + 1];

	/* Byte 48-79 Original Episode Title OET */
	char original_episode_title[32 + 1];	

	/* Byte 80-111 Translated Program Title TPT */
	char translated_program_title[32 + 1];

	/* Byte 112-143 Translated Episode Title TET */
	char translated_episode_title[32 + 1];

	/* Byte 144-175 Tranlators Name TN */
	char translators_name[32 + 1];

	/* Byte 176-207 Translators Contact Details TCD */
	char translators_contact_details[32 + 1];

	/* Byte 208-223 Subtitle List Reference code SLR */
	// 16 character string as reference for subtitle list
	char subtitle_list_reference_code[16 + 1];

	/* Byte 224-229 Creation Date CD */
	// ISO8601 YYMMDD
	char creation_date[6 + 1];

	/* Byte 230-235 Revision Date RD */
	// ISO8601 YYMMDD
	char revision_date[6 + 1];

	/* Byte 236-237 Revision Number RN */
	// 0-99 decimal
	char revision_number[2 + 1];

	/* Byte 238-242 Total Number of Text and Timing Information (TTI) blocks TNB */
	// 0-99999 decimal
	char total_number_of_text_and_timing_information[5 + 1];

	/* Byte 243-247 Total Number of Subtitles TNS */
	// 0-99999 decimal
	char number_of_subtitles[5 + 1];

	/* Byte 248-250 Total  Number of Subtitle Groups TNG */
	// 0-255 decimal
	char total_number_of_subtitle_groups[3 + 1];

	/* Byte 251-252 Maximum Number of Displayable Characters in any text row MNC */
	// 0-99 decimal
	char maximum_displayable_characters_in_any_text_row[2 + 1];

	/* Byte 253-254 Maximum Number of Displayable Rows MNR */
	// 0-99 decimal
	char maximum_number_of_displayable_rows[2 + 1];

	/* Byte 255 Time Code: Status TCS */
	// 0x30 Not intended for use (TCP and TCF in GSI block; and TCI TCO in TTI blocks)
	// 0x31 Intended for use
	char timecode_status[1 + 1];

	/* Byte 256-263 Time Code: Start Of Program TCP */
	// hhmmssff
	char timecode_start_of_program[8 + 1];

	/* Byte 264-271 Time Code: First In-Cue TCF*/
	// hhmmssff
	char timecode_first_in_cue[8 + 1];

	/* Byte 272 Total Number of Disks TND */
	// <= 9
	char total_number_of_disks[1 + 1];

	/* Byte 273 Disk Sequence Number DSN */
	// 1-9
	char disk_sequence_number[1 + 1];

	/* Byte 274-276 Country of Origin CO */
	// NLD
	char country_of_origin[3 + 1];

	/* Byte 277-308 Publisher PUB */
	char publisher[32 + 1];

	/* Byte 309-340 Editors Name EN */
	char editors_name[32 + 1];

	/* Byte 341-372 Editor's Contact Details ECD */
	char editors_contact_details[32 + 1];

	/* Byte 373-447 Spare Bytes */
	char spare_bytes[75 + 1];

	/* Byte 448-1023 User-Define Area UDA */
	char user_defined_area[576 + 1];

	};

#endif /* _EBU_GSI_LIST_H_ */

