/*
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/



/*
Wrote this to be able to create output files for the EBU (European Broadcasting Union)
.STL subtitle data exchange format, for TELETEXT subtitles.
See tech_t3264.pdf in this distribution for a description of the format, that document
also can be found on the Internet.
JP
*/


#include "xste.h"

FD_xste *fd_main_form;
extern FD_xste *fdui;

#include "ebu_tti.h"

/* name is made up of: structure name, subtitle_group_number + subtitle_number + extension_block_number */

struct ebu_tti *lookup_ebu_tti(char *name)
{
struct ebu_tti *pa;

for(pa = ebu_ttitab[0]; pa != 0; pa = pa -> nxtentr)
	{
	if(strcmp(pa -> name, name) == 0) return(pa);
	}

return 0; /* not found */
}/* end function lookup_ebu_tti */


struct ebu_tti *install_ebu_tti_at_end_of_list(char *name)
{
struct ebu_tti *plast, *pnew;

if(debug_flag)
	{
	fprintf(stdout,\
	"install_ebu_tti_at_end_off_list(): arg name=%s\n", name);
	}

pnew = lookup_ebu_tti(name);
if(pnew) return pnew;/* already there */

/* create new structure */
pnew = (struct ebu_tti *) calloc(1, sizeof(*pnew) );
if(! pnew) return 0;
pnew -> name = strsave(name);
if(! pnew -> name) return 0;

/* get previous structure */
plast = ebu_ttitab[1]; /* end list */

/* set new structure pointers */
pnew -> nxtentr = 0; /* new points top zero (is end) */
pnew -> prventr = plast; /* point to previous entry, or 0 if first entry */

/* set previous structure pointers */
if( !ebu_ttitab[0] ) ebu_ttitab[0] = pnew; /* first element in list */
else plast -> nxtentr = pnew;

/* set array end pointer */
ebu_ttitab[1] = pnew;

return pnew; /* pointer to new structure */
} /* end function install_ebu_tti_at_end_of_list */


int delete_ebu_tti(int ebu_ttinr) /* delete entry from double linked list */
{
struct ebu_tti *pa, *pprev, *pdel, *pnext;
char name[80];

if(debug_flag)
	{
	fprintf(stdout, "delete_ebu_tti(): arg ebu_ttinr=%d\n", ebu_ttinr);
	}

sprintf(name, "%d", ebu_ttinr);
pa = ebu_ttitab[0];
while(1)
	{
	/* if end list, return not found */
	if(! pa) return 0;

	/* test for match in name */
	if(strcmp(name, pa -> name) != 0) /* no match */
		{
		/* point to next element in list */
		pa = pa -> nxtentr;

		/* loop for next element in list */
		continue;
		}

	/* we now know which struture to delete */
	pdel = pa;

	/* get previous and next structure */
	pnext = pa -> nxtentr;
	pprev = pa -> prventr;

	/* set pointers for previous structure */
	/* if first one, modify ebu_ttitab[0] */
	if(pprev == 0) ebu_ttitab[0] = pnext;
	else pprev -> nxtentr = pnext;

	/* set pointers for next structure */
	/* if last one, modify ebu_ttitab[1] */
	if(pnext == 0) ebu_ttitab[1] = pprev;
	else pnext -> prventr = pprev;
	
	/* delete structure */	
	free(pdel -> name);
	free(pdel); /* free structure */

	/* return OK deleted */
	return 1;
	} /* end for all structures */
} /* end function delete_ebu_tti */


int delete_all_ebu_ttis() /* delete all entries from table */
{
struct ebu_tti *pa;

if(debug_flag)
	{
	fprintf(stdout, "delete_all_ebu_ttis() arg none\n");
	}

while(1)
	{	
	pa = ebu_ttitab[0];
	if(! pa) break;
	ebu_ttitab[0] = pa -> nxtentr;

	free(pa -> name);
	free(pa); /* free structure */
	}/* end while all structures */

ebu_ttitab[1] = 0;
return 1;
} /* end function delete_all_ebu_ttis */


int write_ebu_stl_list()
{
int i;
struct ebu_tti *pa;
FILE *fptr;
char temp[4096];

if(debug_flag)
	{
	printf("write_ebu_stl_list(): arg none\n");
	}

draw_mode = DRAW_MODE_BW;

/* open .STL file */
sprintf(temp, "%s/%s/%s/%s.STL",\
home_dir, selected_data_directory, selected_project, selected_project);
fptr = fopen(temp, "w");
if(! fptr)
	{
	fl_show_alert(\
		"write_ebu_stl_list", "Could not open for write", temp, 1);

	return 0;
	}

/* Start General Subtitle Information block GSI. */

/* Byte 0-2 Code Page Number CPN */
// 850 multilangual -> 0x38 0x35 0x30 
fprintf(fptr, "%3s", pebu_gsi -> code_page_number);

/* Byte 3-10 Disc Format Code DFC */
// STL25.01 for 25 fps  ''S' 'T' 'L' '2' '5' '.' '0' '1'
// STL30.01 for 30 fps
fprintf(fptr, "%8s", pebu_gsi -> disk_format_code);

/* Byte 11 Display Standard Code */
// Undifined 0x20
// Open Subtitling 0x30
// Level 1 Teletext 0x31
// Level 2 teletext 0x32
fprintf(fptr, "%1s", pebu_gsi -> display_standard_code);

/* Byte 12-13 Character Code Table number CCT */
// Latin 0x30 0x30
// Latin /Cyrillic	0x30 0x31
// Latin/Arabic		0x30 x032
// Latin/Greek		0x30 0x33
// Latin/Hebrew		0x30 0x34
fprintf(fptr, "%2s", pebu_gsi -> character_code_table_number);

/* Byte 14-15 Language Code LC */
// Dutch = 0x33 should this be 0x00 0x33 or 0x33 0x33 or 0x33 0x00 ??????
// GAMBLE 0x33 0x00 (second byte reserved for national assignment)
fprintf(fptr, "%2s", pebu_gsi -> language_code);

/* Byte 16-47 Original Program Title OPT */
fprintf(fptr, "%32s", pebu_gsi -> original_program_title);

/* Byte 48-79 Original Episode Title OET */
fprintf(fptr, "%32s", pebu_gsi -> original_episode_title);

/* Byte 80-111 Translated Program Title TPT */
fprintf(fptr, "%32s", pebu_gsi -> translated_program_title);

/* Byte 112-143 Translated Episode Title TET */
fprintf(fptr, "%32s", pebu_gsi -> translated_episode_title);

/* Byte 144-175 Tranlators Name TN */
fprintf(fptr, "%32s", pebu_gsi -> translators_name);

/* Byte 176-207 Translators Contact Details TCD */
fprintf(fptr, "%32s", pebu_gsi -> translators_contact_details);

/* Byte 208-223 Subtitle List Reference code SLR */
// 16 character string as reference for subtitle list
fprintf(fptr, "%16s", pebu_gsi -> subtitle_list_reference_code);

/* Byte 224-229 Creation Date CD */
// ISO8601 YYMMDD
fprintf(fptr, "%6s", pebu_gsi -> creation_date);

/* Byte 230-235 Revision Date RD */
// ISO8601 YYMMDD
fprintf(fptr, "%6s", pebu_gsi -> revision_date);

/* Byte 236-237 Revision Number RN */
// 0-99 decimal
fprintf(fptr, "%2s", pebu_gsi -> revision_number);

/* Byte 238-242 Total Number of Text and Timing Information (TTI) blocks TNB */
// 0-9 99 99 decimal
fprintf(fptr, "%5s", pebu_gsi -> total_number_of_text_and_timing_information);

/* Byte 243-247 Total Number of Subtitles TNS */
// 0-9 99 99 decimal
fprintf(fptr, "%5s", pebu_gsi -> number_of_subtitles);

/* Byte 248-250 Total  Number of Subtitle Groups TNG */
// 0-255 decimal
fprintf(fptr, "%3s", pebu_gsi -> total_number_of_subtitle_groups);

/* Byte 251-252 Maximum Number of Displayable Characters in any text row MNC */
// 0-99 decimal
fprintf(fptr, "%2s", pebu_gsi -> maximum_displayable_characters_in_any_text_row);

/* Byte 253-254 Maximum Number of Displayable Rows MNR */
// 0-99 decimal
fprintf(fptr, "%2s", pebu_gsi -> maximum_number_of_displayable_rows);

/* Byte 255 Time Code: Status TCS */
// 0x30 Not intended for use (TCP and TCF in GSI block; and TCI TCO in TTI blocks)
// 0x31 Intended for use
fprintf(fptr, "%1s", pebu_gsi -> timecode_status);

/* Byte 256-263 Time Code: Start Of Program TCP */
// hhmmssff
fprintf(fptr, "%8s", pebu_gsi -> timecode_start_of_program);

/* Byte 264-271 Time Code: First In-Cue TCF*/
// hhmmssff
fprintf(fptr, "%8s", pebu_gsi -> timecode_first_in_cue);

/* Byte 272 Total Number of Disks TND */
// <= 9
fprintf(fptr, "%1s", pebu_gsi -> total_number_of_disks);

/* Byte 273 Disk Sequence Number DSN */
// 1-9
fprintf(fptr, "%1s", pebu_gsi -> disk_sequence_number);

/* Byte 274-276 Country of Origin CO */
// NLD
fprintf(fptr, "%3s", pebu_gsi -> country_of_origin);

/* Byte 277-308 Publisher PUB */
fprintf(fptr, "%32s", pebu_gsi -> publisher);

/* Byte 309-340 Editors Name EN */
fprintf(fptr, "%32s", pebu_gsi -> editors_name);

/* Byte 341-372 Editor's Contact Details ECD */
fprintf(fptr, "%32s", pebu_gsi -> editors_contact_details);

/* Byte 373-447 Spare Bytes */
fprintf(fptr, "%75s", pebu_gsi -> spare_bytes);

/* Byte 448-1023 User-Define Area UDA */
fprintf(fptr, "%576s", pebu_gsi -> user_defined_area);

/* end General Subtitle Information block GSI */

/* write TTI blocks */
for(pa = ebu_ttitab[0]; pa != 0; pa = pa -> nxtentr)
	{
	fprintf(fptr, "%c", pa -> subtitle_group_number);

	fprintf(fptr, "%c",	pa -> subtitle_number[0]);
	fprintf(fptr, "%c",	pa -> subtitle_number[1]);

	fprintf(fptr, "%c",	pa -> extension_block_number);

	fprintf(fptr, "%c",	pa -> cumulative_status);

	fprintf(fptr, "%c",	pa -> timecode_in[0]);
	fprintf(fptr, "%c",	pa -> timecode_in[1]);
	fprintf(fptr, "%c",	pa -> timecode_in[2]);
	fprintf(fptr, "%c",	pa -> timecode_in[3]);

	fprintf(fptr, "%c",	pa -> timecode_out[0]);
	fprintf(fptr, "%c",	pa -> timecode_out[1]);
	fprintf(fptr, "%c",	pa -> timecode_out[2]);
	fprintf(fptr, "%c",	pa -> timecode_out[3]);

	fprintf(fptr, "%c",	pa -> vertical_position);
	fprintf(fptr, "%c",	pa -> justification_code);
	fprintf(fptr, "%c",	pa -> comment_flag);

	for(i = 0; i < 112; i++)
		{
		fprintf(fptr, "%c", pa -> text_field[i]);
		}
	} /* end for all entries in list */

/* close the file */
fclose(fptr);

return 1;
}/* end function write_ebu_stl_list */


int format_and_center_text_in_arays(struct frame *pa)
{
int a, c, i, j;
char *tptr, *cptr;
int x, y;
char temp[4096];
int text_start, max_width, line_len;
int error;

if(debug_flag)
	{
	printf("format_and_center_text_in_arays(): arg pa=%p\n", pa);
	}

/* argument check */
if(! pa) return 0;

/* clear all of drawing area */
for(i = 0; i < 720 * 576 * 3; i++)
	{
	image_buffer[i] = 0;
	}

/* some defaults */
pa -> width = 0;
pa -> height = 0;
pa -> xpos = 0;  
pa -> ypos = 0;
/* end defaults */

line_h_start = h_factor * (double)h_size;
line_h_end = (double)h_size - (double)line_h_start;

window_bottom = v_size - (v_factor * (double)v_size);

/*
Set hor_position for start text in all lines to zero,
center_text() may overrule this if center_flag.
*/
for(i = 0; i < MAX_SCREEN_LINES; i++)
	{
	screen_text[i][0] = 0;
	screen_start[i] = 0; // pixels from left used to center text
	}

/* reformat text inserting (multiple) '\n' if too long */ 
tptr = p_reformat_text(pa, line_h_end - line_h_start);
if(! tptr)
	{
	printf("format_and_center_text_in_arays(): could not reformat text=%s\n", pa -> text);
	
	/* return error */
	return 0;
	} /* end if reformat text failed */		

/* center text */
if(pa -> status & CENTER_TEXT) p_center_text(tptr, pa);

/* text to array screen_text[] */
cptr = tptr;
screen_lines = 1; /* at least one */
while(1) /* all chars in tptr */
	{
	i = 0;
	while(1) /* all chars in a line */
		{
		if(*cptr == '\n')
			{
			/* scip the '\n' */
			cptr++;
				
			/* force string termination */
			screen_text[screen_lines - 1][i] = 0;

			/* point to next screen line */
			screen_lines++;
			break;
			}

		/* copy character */
		screen_text[screen_lines - 1][i] = *cptr;	

		/* test for end of string tptr */
		if(*cptr == 0) break;

		/* point to next char in screen_text[][] */
		i++;

		/* point to next char in tptr */
		cptr++;
		} /* end while all characters in line terminated with LF */

	/* ready if end of tptr */
	if(*cptr == 0) break;
				
	} /* end while all lines in tptr */
free(tptr);

/* some limit */
if(screen_lines > MAX_SCREEN_LINES) screen_lines = MAX_SCREEN_LINES;

line_height = pa -> pfd -> height;
window_top = window_bottom - (screen_lines * line_height);

if(window_top < 0) fl_show_alert("WARNING\nvertical image size in frame:", pa -> name, "to big", 0); 

if(debug_flag)
	{
	printf("format_and_center_text_in_arays(): screen_lines=%d\n", screen_lines);
	printf("format_and_center_text_in_arays(): line_h_start=%d line_h_end=%d\n",\
	line_h_start, line_h_end);
	printf("format_and_center_text_in_arays(): window_bottom=%d window_top=%d\n",\
	window_bottom, window_top);
	}

/* print lines of text on screen in right position */
text_start = INT_MAX;
max_width = 0;
for(i = 0; i < screen_lines; i++)
	{
	/* convert any special characters */
/*
	j = 0;
	while(1)
		{
		c = screen_text[i][j];
		character_lookup(c, &b);
		screen_text[i][j] = b;

		if(c == 0) break;
		j++;
		}
*/

	/* get text length */
	line_len = 0;
	j = 0;
	while(1)
		{
		c = screen_text[i][j];
		line_len += get_h_pixels(c, pa);

		if(c == 0) break;
		j++;
		}

	x = screen_start[i];

	if(x < text_start) text_start = x;

	if(line_len > max_width) max_width = line_len;

	y = window_top + (i * line_height);

	if(debug_flag)
		{
		printf(\
		"format_and_center_text_in_arays(): screen_start[%d]=%d window_bottom=%d window_top=%d\n\
line_height=%d x=%d y=%d\n\
text=%s\n",\
		i, screen_start[i], window_bottom, window_top,\
		line_height, x, y,\
		screen_text[i]);
		}

	} /* end for all screen_lines */

pa -> ypos = window_top;

/* 
text_start is the start of the most left character start in a (multiline)
text.
max_width is the length of the text in pixels.
*/

pa -> height = screen_lines * line_height + 9;
pa -> xpos = text_start;

/* add some space for outline */
pa -> width = max_width + 6;

/* CVD needs even width */
/* if width is not multiple of 8, make it */
a = pa -> width % 8;
if(a) pa -> width += 8 - a; 

/* CVD needs even height, if height is not multiple of 4, make it */
a = pa -> height % 4;
if(a) pa -> height += 4 - a;

if(debug_flag)
	{
	printf("format_and_center_text_in_arays():\n\
	frame=%s\n\
	text_start=%d max_width=%d\n\
	pa>xpos=%d pa->ypos=%d\n\
	pa->width=%d pa->height=%d\n\
	pa->text=%s\n",\
	pa -> name,\
	text_start, max_width,\
	pa -> xpos, pa -> ypos,\
	pa -> width, pa -> height,\
	pa -> text);
	}

/* return error if not usable overlay */
error = 0;

strcpy(temp, "");
/* no pointer to text */
if(! pa -> text) strcpy(temp, "pa is zero");

/* no text */
//if(pa -> text[0] == 0) strcpy(temp, "no text");

/* no width */
if(pa -> width <= 0) strcpy(temp, "width < 0");

/* no height */
if(pa -> height <= 0) strcpy(temp, "height < 0");

/* negative xpos */
if(pa -> xpos < 0) strcpy(temp, "xpos < 0");

/* negatve ypos */
if(pa -> ypos < 0) strcpy(temp, "ypos < 0");

/* reformatter failed */
//if(max_width == 0) strcpy(temp, "max_width = 0");

/* no space left to print, reformatter failed margin? */
//if(pa -> xpos >= (h_size / 2) ) strcpy(temp, "reformatter failed");

/* falls of right hand side of screen */
if( (pa -> width + pa -> xpos) > h_size)  strcpy(temp, "past right side screen");

/* falls of bottom of screen */
if(pa -> height >= v_size)
	{
	strcpy(temp, "vertical size to big");
	fprintf(stdout, "pa->height=%d v_size=%d\n", pa -> height, v_size);
	}

if(temp[0] != 0)
	{
	fprintf(stderr, "xste: format_and_center_text_in_arays(): frame=%s ERROR %s\n", pa -> name, temp);

	fl_show_alert("WARNING\nframe", pa -> name, temp, 0);

	/* return error */
	return 0;
	} /* end if error */

return 1;
} /* end format_and_center_text_in_arays */


int xste_frames_to_ebu_gsi_tti()
{
int c, i, j, k;
struct frame *pa;
struct frame *pnext, *ptemp;
char temp[4096];
int next_frame_start;
int start_hour, start_minute, start_second;
int end_hour, end_minute, end_second;
long lthousands;
int text_end_frame;
int start_frame;
int end_frame;
/* output */
int subtitle_group_number;
int subtitle_number;
int extension_block_number;
int cumulative_status;
int justification_code;
int comment_flag;
char timecode_in[4];
char timecode_out[4];
char timecode_first_in_cue[4] = {0,0,0,0}; // -Wall
int timecode_first_flag;
int vertical_position = 0; // -Wall
char ebu_lf_text[READSIZE]; /* all screen lines of a sub with LF code added  */
char text_field[112];
int pad_flag;
struct ebu_tti *pe;
int disk_space;
int tti_size;
int total_number_of_disks;
int disk_sequence_number;
int tti_blocks;
int hh, mm, ss, ff;
int start_frame_shift;

if(debug_flag)
	{
	printf("xste_frames_to_ebu_gsi_tti(): arg none\n");
	}

/* clear list */
if(! delete_all_ebu_ttis() ) return 0;

/*
frame list to structure,
get number TTI, number subs, number sub grous (1),
timecode prog start, timecode first in cue,
char code, columns, rows,
disks, disk sequence, 
These values to EBU GSI and EBU GSI form
*/

/* some start points */
subtitle_group_number = 0; 	/* only one group in xste */
subtitle_number = 0; 		/* 0-65535 */
extension_block_number = 0xff; /* for the last block, 0-0xef for count, 0xf0-0xfd reserved, 0xfe user data */
cumulative_status = 0; 		/* no cumulative subs in xste atm. */
justification_code = 2;		/* 0=control and spaces in text, 1=left justified, 2=center, 3=right justified */
comment_flag = 0;
tti_blocks = 0;

sscanf(pebu_gsi ->timecode_start_of_program, "%2d%2d%2d%2d", &hh, &mm, &ss, &ff);
/* will this work for 29.9 ? */
start_frame_shift = (dframes_per_second * ( (3600 * hh) + (60 * mm) + ss) ) + ff;

timecode_first_flag = 1;
/* add Text And Timing Information block TTI. */   
for(pa = frametab[0]; pa != 0; pa = pa -> nxtentr)
	{
	if(debug_flag)
		{
		printf(\
		"xste_frames_to_ebu_gsi_tti: frame=%d pa->text=%s\n", atoi(pa -> name), pa -> text);
		}

	/* do not print negative frames, this results in negative times. */
	if( atoi(pa -> name) < 0) continue;

	/* normal situation */
	text_end_frame = pa -> end_frame;

	/* get pointer to next structure entry */ 
	if(pa -> nxtentr) 
		{
		pnext = pa -> nxtentr;	
		
		/* find next line that is not a '*' comand line */
		while(pnext)
			{
			if(pnext -> text[0] != '*') break;

			if(pnext -> nxtentr)
				{	
				ptemp = pnext -> nxtentr;

				pnext = ptemp;
				}	
			} /* end while looking for non '*' entry */ 

		if(pnext)
			{
			/* get the start of the next frame */ 
			next_frame_start = atoi(pnext -> name);
		
			/* test if the current frame ends before the start of the next */
			if(pa -> end_frame > next_frame_start)
				{
				text_end_frame = next_frame_start;
				} /* end if frame ends before next one starts */ 
			}

		} /* end if not last entry */	
	
	/* we have this data at our disposal */ 
	//	char *name;/* this is the text start frame */
	//	char *text;
	//	int text_end_frame; /* this is the end frame */
	//	unsigned long start_sample;
	//	unsigned long end_sample;
 
	/* calculate start_time */
	lthousands = (atoi(pa -> name) + start_frame_shift) * dmilli_seconds_per_frame;
	start_hour = lthousands / 3600000;
	lthousands -= start_hour * 3600000; 
	start_minute = lthousands / 60000;
	lthousands -= start_minute * 60000;
	start_second = lthousands / 1000;
	lthousands -= start_second * 1000;
	
	start_frame = lthousands / dmilli_seconds_per_frame;

	/* calculate end_time */
	lthousands = (text_end_frame + start_frame_shift) * dmilli_seconds_per_frame;
	end_hour = lthousands / 3600000;
	lthousands -= end_hour * 3600000; 
	end_minute = lthousands / 60000;
	lthousands -= end_minute * 60000;
	end_second = lthousands / 1000;
	lthousands -= end_second * 1000;

	end_frame = lthousands / dmilli_seconds_per_frame;

	/*
	do not print negative frames, this results in negative times,
	not sure submux likes that.
	*/
	if(start_hour < 0) continue;
	if(start_minute < 0) continue;
	if(start_second < 0) continue;
	if(lthousands < 0) continue;
	
	/* do not print xste command lines in the form '*command' */
	if(pa -> text[0] == '*') continue;

	/* 
	print entry, format:
	example:

	BE CAREFUL don't make subtitles with zero or negative duration they will
	be skipped.
	*/

	/*
	fills array screen_text[], sets screen_lines (number of lines),
    fills array screen_start[i], with start pixel in line.
	sets pa -> width, pa -> height, pa -> xpos, pa -> ypos,
	from pa -> text and subtitle_current_font_descriptor.
	*/
	if(! format_and_center_text_in_arays(pa) ) continue;

	if(debug_flag)
		{
		printf("xste_frames_to_ebu_gsi_tti():\n\
		frame=%s text_end_frame=%d time_code_in=%02d:%02d:%02d:%02d  time_code_out=%02d:%02d:%02d:%02d\n",\
		pa -> name, text_end_frame,\
		start_hour, start_minute, start_second, start_frame,\
		end_hour, end_minute, end_second, end_frame);

		printf("\
		pattern=  %02d  background=%02d emphasis1=%02d emphasis2=%02d\n",\
		pa -> pattern, pa -> background, pa -> emphasis1, pa -> emphasis2);
		printf("\
		contrast0=%02d  contrast1= %02d contrast2=%02d contrast3=%02d\n",\
		pa -> pattern_contrast, pa -> background_contrast, pa -> emphasis1_contrast, pa -> emphasis2_contrast);
		}

	timecode_in[0] = start_hour;
	timecode_in[1] = start_minute;
	timecode_in[2] = start_second;
	timecode_in[3] = start_frame;

	if(timecode_first_flag)
		{
		timecode_first_in_cue[0] = timecode_in[0];  
		timecode_first_in_cue[1] = timecode_in[1];
		timecode_first_in_cue[2] = timecode_in[2];
		timecode_first_in_cue[3] = timecode_in[3];

		timecode_first_flag = 0;
		}

	timecode_out[0] = end_hour;
	timecode_out[1] = end_minute;
	timecode_out[2] = end_second;
	timecode_out[3] = end_frame;

	if(pebu_gsi -> display_standard_code[0] == 0x30) /* open */
		{
		vertical_position = pebu_gsi -> maximum_number_of_displayable_rows[0];
		}
 	else if(pebu_gsi -> display_standard_code[0] == 0x31) /* teletext level 1 */
		{
		vertical_position = 23 - screen_lines;
		}
	else if(pebu_gsi -> display_standard_code[0] == 0x32) /* teletext level 2 */
		{
		vertical_position = 23 - screen_lines;
		}

	/* character count in output array ebu_lf_text[] */
	k = 0;
	for(i = 0; i < screen_lines; i++)
		{
		if(debug_flag)
			{
			printf("\
	screen_start[%d]=%d screen_text[%d]=%s\n", i, screen_start[i], i, screen_text[i]);

			printf("xste_frames_to_ebu_gsi_tti():\n\
			frame=%s\n\
			pa -> text=%s\n\
			pa>xpos=%d pa->ypos=%d\n\
			pa->width=%d pa->height=%d\n",\
			pa -> name,\
			pa -> text,\
			pa -> xpos, pa -> ypos,\
			pa -> width, pa -> height);
			} /* end if debug_flag */

		/* for each character in a screen line */
		j = 0;
		while(1)
			{
			c = screen_text[i][j];
			if(c == 0) /* end of a subtitle line */
				{
				/* test if last subtitle line (bottom one) */
				if(i != screen_lines - 1)
					{
					/* add EBU code for LF */
					ebu_lf_text[k] = 0x8a;
					}
				else
					{
					/* string termination */
					ebu_lf_text[k] = 0;
					}
				}
			else
				{
				ebu_lf_text[k] = c;
				}

			k++;
			if(k >= READSIZE)
				{
				fprintf(stdout, "xste_frames_to_ebu_gsi_tti(): k=%d ebu_lf_text[] overflow.\n", k);

				return 0;
				}

			if(c == 0) break; 

			j++;
			} /* end while each character in a screen line */

		} /* end for all screen lines */

	if(debug_flag)
		{
		fprintf(stdout, "xste_frames_to_ebu_gsi_tti(): ebu_lf_text=%s\n", ebu_lf_text);
		}

	/*
	characters to text_field, pad with 0x8f.
	when text_filed is full, start a new one, and increment numbers and set extension_block_number
	accordingly.
	*/ 

	/* pointer in source zero */
	i = 0;

	/* for al characters in source */
	while(1)
		{
		pad_flag = 0;
		for(j = 0; j < 112; j++)
			{
			if(! pad_flag)
				{
				c = ebu_lf_text[i];
				i++;

				if(c == 0) pad_flag = 1;
				}

			if(pad_flag) text_field[j] = 0x8f;
			else text_field[j] = c;
			}

		if(debug_flag)
			{
			fprintf(stdout,\
			"xste_frames_to_ebu_gsi_tti(): subtitle_number=%d extension_block_number=%d\n",\
			subtitle_number, extension_block_number);
			
			for(k = 0; k < 112; k++)
				{
				fprintf(stdout, "%c", text_field[k]);
				}

			fprintf(stdout, "\n");
			}

		/* we now have one full text block */

		/* if any padding, this was an 'end' block */
		if(pad_flag) extension_block_number = 0xff;
		else extension_block_number++; /* 00 for first, up to 0xef */

		extension_block_number &= 0xff;

		/* create structure entry */
		sprintf(temp, "%d_%d_%d", subtitle_group_number, subtitle_number, extension_block_number);
		pe = install_ebu_tti_at_end_of_list(temp);
		if(! pe)
			{
			fprintf(stdout, "xste_frames_to_ebu_gsi_tti(): could not install ebu_tti structure entry\n");
		
			return 0;
			}

		tti_blocks++;

		/* data to structure entry */
		pe -> subtitle_group_number = subtitle_group_number;

		pe -> subtitle_number[0] = subtitle_number % 256; /* MS DOS stores ow byte first */ 
		pe -> subtitle_number[1] = subtitle_number / 256;

		pe -> extension_block_number = extension_block_number;

		pe -> cumulative_status = cumulative_status; 

		pe -> timecode_in[0] = timecode_in[0];
		pe -> timecode_in[1] = timecode_in[1];
		pe -> timecode_in[2] = timecode_in[2];
		pe -> timecode_in[3] = timecode_in[3];

		pe -> timecode_out[0] = timecode_out[0];
		pe -> timecode_out[1] = timecode_out[1];
		pe -> timecode_out[2] = timecode_out[2];
		pe -> timecode_out[3] = timecode_out[3];

		pe -> vertical_position = vertical_position;
		pe -> justification_code = justification_code;
		pe -> comment_flag = comment_flag;

		for(k = 0; k < 112; k++) pe -> text_field[k] = text_field[k];

		/* test for complete sub */
		if(pad_flag) break;

		} /* end while all blocks in a sub */

	/* here we have one complete sub */
	subtitle_number++;

	} /* end for all frames */


/* calculate */

disk_space = 1440 * 1024;
tti_size = tti_blocks * 128;

/* each disk has a 1024 byte GSI */
total_number_of_disks = (tti_size / (disk_space - 1024) ) + 1;

/* fix this for now !!! */
disk_sequence_number = 1;


/* calculated to GSI */

sprintf(temp, "%d", subtitle_number);
fl_set_input(fdui -> ebu_general_subtitle_information_form_number_of_subtitles_input, temp);

sprintf(temp, "%d", tti_blocks);
fl_set_input(fdui -> ebu_general_subtitle_information_form_tti_blocks_input, temp);

sprintf(temp, "%d", total_number_of_disks);
fl_set_input(fdui -> ebu_general_subtitle_information_form_number_of_disks_input, temp);

sprintf(temp, "%d", disk_sequence_number);
fl_set_input(fdui -> ebu_general_subtitle_information_form_disk_sequence_number_input, temp);

/* this should come from individual structure entries, but can only specify global in EBU GSI */
sprintf(temp, "%d", default_subtitle_iso_extention);
fl_set_input(fdui -> ebu_general_subtitle_information_form_character_code_input, temp);

if(dframes_per_second == 25.0) sprintf(temp, "STL25.01");
else if(dframes_per_second == 30.0) sprintf(temp, "STL30.01");
else if(dframes_per_second == 29.97) sprintf(temp, "STL30.0");
fl_set_input(fdui -> ebu_general_subtitle_information_form_disk_format_code_input, temp);

fl_call_object_callback(fdui -> ebu_general_subtitle_information_form_disk_format_code_input);


sprintf(temp, "%02d%02d%02d%02d",\
timecode_first_in_cue[0], timecode_first_in_cue[1], timecode_first_in_cue[2], timecode_first_in_cue[3]);
fl_set_input(fdui -> ebu_general_subtitle_information_form_timecode_first_cue_in_input, temp);

fl_call_object_callback(fdui -> ebu_general_subtitle_information_form_timecode_first_cue_in_input);


sprintf(temp, "%d", subtitle_group_number);
fl_set_input(fdui -> ebu_general_subtitle_information_form_subtitle_groups_input, temp);

/* override manual if we found timecode */

if(timecode_first_flag) pebu_gsi -> timecode_status[0] = '0';
else pebu_gsi -> timecode_status[0] = '1';
set_timecode_valid_button();

return 1;
} /* end function xste_frames_to_ebu_gsi_tti */



