#ifndef _EBU_TTI_LIST_H_
#define _EBU_TTI_LIST_H_

typedef short BYTE;

struct ebu_tti
{
/* the structure name, subtitle_group_number + subtitle_number + extension_block_number */
char *name;

/* Subtitle Group Number SGN */
// 1 byte
// 0-ff
int8_t subtitle_group_number;

/* Subtitle Number */
// 2 bytes
// 0-ffff
int8_t subtitle_number[2];

/* Extension Block Number SN */
// 1 byte
// 0-ef block with additional text
// f0-fd EBU reserved
// fe reserved user data
// ff last TTI block (also for 1 block subs)
int8_t extension_block_number;

/* Cumulative Status CS */	
// 1 byte
// 00-03
// 00 sub not part of cumulative
// 01 first sub of cumulative
// 02 intermediate sub in cumulative
// 03 last sub of cumulative
int8_t cumulative_status;

/* Time Code In TCI */
// 4 bytes
// hh 0-23
// mm 0-59
// ss 0-59
// ff 0-24 (0-29 if STL30.01)
int8_t timecode_in[4];

/* Time Code Out TCO */
// 4 bytes
// hh 0-23
// mm 0-59
// ss 0-59
// ff 0-24 (0-29 if STL30.01)
int8_t timecode_out[4];

/* Vertical Position VP */
// 1 byte
// for teletext 0-23
// for in vision 0-as specified in MNR field in GSI block
int8_t vertical_position;

/* Justification Code JC */
// 1 byte
// 00 unchanged presentation (control characters and spaces in text set position) 
// 01 left-justified-text (all leading -  and trailing spaces ignored) 
// 02 centered text (all leading -  and trailing spaces ignored)
// 03 right-justified text (all leading -  and trailing spaces ignored)
int8_t justification_code;

/* Comment Flag CF */
// 1 byte
// 00 TF contains subtitle data
// 01 TF contains comments not intended for transmission
int8_t comment_flag;

/* Text Field TF */
// 112 bytes
// all or some of the text and control characters to convey a subtitle.
// any valid chracter code from the CCT field of the GSI block.
// CR/LF shall be 0x8a
// text field last subtitle must terminate with 0x8f
// unused space in text field shall be set to 0x8f
int8_t text_field[112];

/* all undefied values in TTI block are reserved for future use */

struct ebu_tti *nxtentr;
struct ebu_tti *prventr;
};
struct ebu_tti *ebu_ttitab[2];


#endif /* _EBU_TTI_LIST_H_ */

