/*
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "xste.h"

FD_xste *fdui;


struct font *lookup_font(char *name)
{
struct font *pa;

for(pa = fonttab[0]; pa != 0; pa = pa -> nxtentr)
	{
	if(strcmp(pa -> name, name) == 0) return(pa);
	}

return 0; /* not found */
}/* end function lookup_font */


struct font *install_font_at_end_of_list(char *name)
{
struct font *plast, *pnew;
struct font *lookup_font();

if(debug_flag)
	{
	printf("install_font(): arg name=%s\n", name);
	}

pnew = lookup_font(name);
if(pnew)
	{
	/* free previous definition */
	return pnew;/* already there */
	}

/* create new structure */
pnew = (struct font *) calloc(1, sizeof(*pnew) );
if(! pnew) return(0);
pnew -> name = strsave(name);
if(! pnew -> name) return(0);

/* get previous structure */
plast = fonttab[1]; /* end list */

/* set new structure pointers */
pnew -> nxtentr = 0; /* new points top zero (is end) */
pnew -> prventr = plast; /* point to previous entry, or 0 if first entry */

/* set previous structure pointers */
if( !fonttab[0] ) fonttab[0] = pnew; /* first element in list */
else plast -> nxtentr = pnew;

/* set array end pointer */
fonttab[1] = pnew;

return pnew;/* pointer to new structure */
}/* end function install_font */


int delete_font(long font_id)/* delete entry from double linked list */
{
struct font *pa, *pprev, *pdel, *pnext;
char name[50];

if(debug_flag)
	{
	printf("delete_font(): arg font_id=%ld\n", font_id);
	}

sprintf(name, "%ld", font_id);
pa = fonttab[0];
while(1)
	{
	/* if end list, return not found */
	if(! pa) return(0);

	/* test for match in name */
	if(strcmp(name, pa -> name) != 0) /* no match */
		{
		/* point to next element in list */
		pa = pa -> nxtentr;

		/* loop for next element in list */
		continue;
		}

	/* we now know which struture to delete */
	pdel = pa;

	/* get previous and next structure */
	pnext = pa -> nxtentr;
	pprev = pa -> prventr;

	/* set pointers for previous structure */
	/* if first one, modify fonttab[0] */
	if(pprev == 0) fonttab[0] = pnext;
	else pprev -> nxtentr = pnext;

	/* set pointers for next structure */
	/* if last one, modify fonttab[1] */
	if(pnext == 0) fonttab[1] = pprev;
	else pnext -> prventr = pprev;
	
	/* delete structure */	
	free(pdel -> name);
	free(pdel -> iso_name);
	free(pdel); /* free structure */

	/* return OK deleted */
	return 1;
	}/* end for all structures */
}/* end function delete_font */


int delete_all_fonts()/* delete all entries from table */
{
struct font *pa;

if(debug_flag)
	{
	printf("delete_all_fonts() arg none\n");
	}

while(1)
	{	
	pa = fonttab[0];
	if(! pa) break;
	fonttab[0] = pa -> nxtentr;
	free(pa -> name);
	free(pa -> iso_name);
	free(pa);/* free structure */
	}/* end while all structures */

fonttab[1] = 0;
if(debug_flag)
	{
	printf("returning from delete_all_fonts\n");
	}

return 1;
}/* end function delete_all_fonts */


void add_xforms_font(const char *name)
{
struct font *pa, *install_font_at_end_off_list();

if(debug_flag)
	{
	printf("add_xforms_font(): arg name=%s\n", name);
	}

pa = install_font_at_end_of_list( (char *)name);
if(!pa) return;

pa -> iso_name = strsave("");
if(! pa -> name) return;

pa -> iso_extention = -1;

return;
}/* end function add_xforms_font */


int add_X_font(const char *name, char *iso_name, int iso_extention)
{
struct font *pa, *install_font_at_end_off_list();

if(debug_flag)
	{
	printf("add_X_font(): arg name=%s\n", name);
	}

pa = install_font_at_end_of_list( (char *)name);
if(!pa) return 0;

pa -> iso_name = strsave(iso_name);
if(! pa -> iso_name) return 0;

pa -> iso_extention = iso_extention;

return 1;
}/* end function add_X_font */


int show_fonts(int position)
{
char temp[TEMP_SIZE];
struct font *pa;
char formatstr[20];
int browser_line;
int browser_topline;
int first_new_font_line;
int find_first_new_font_flag;
int maxline;

if(debug_flag)
	{
	printf("show_fonts(): arg position=%d\n", position);
	}

/* remember vertical slider position */
browser_topline = fl_get_browser_topline(fdui -> fonts_form_browser);

fl_freeze_form(fdui -> fonts_form);

fl_clear_browser(fdui -> fonts_form_browser);

/* keep gcc -Wall from complaining */
first_new_font_line = -1;
find_first_new_font_flag = 1;
browser_line = 1;
for(pa = fonttab[0]; pa != 0; pa = pa -> nxtentr)	
	{
	if(position == SHOW_UNREAD_AT_TOP)
		{
		/* want to show the first unaccessed at top of display */
		if(find_first_new_font_flag)
			{
			first_new_font_line = browser_line;
			find_first_new_font_flag = 0;
			}
		}
	
//	strcpy(formatstr, "@f");
	strcpy(formatstr, "");

	/* black is default color */
	strcat(formatstr, "@C0");
	if( (browser_line - 1) == selected_articles_font)
		{
		/* blue articles fonts */
		strcat(formatstr, "@C4");
		} /* end if articles font */
	if( (browser_line - 1) == selected_global_font)
		{
		if(selected_articles_font == selected_global_font)
			{
			/* magenta if both selected */
			strcat(formatstr, "@C5");
			}
		else
			{
			/* red global fonts */
			strcat(formatstr, "@C1");
			}
		} /* end if global font */

//	strcat(formatstr, "@C1"); /* red */
//	strcat(formatstr, "@C0"); /* black */
//	strcat(formatstr, "@C18"); /* dark green */
//	strcat(formatstr, "@C4"); /* blue */
//	strcat(formatstr, "@C5"); /* magenta */
//	strcat(formatstr, "@_");/* underscore */
	
	sprintf(temp, "%s %s",\
	formatstr, pa -> name);

	fl_add_browser_line(fdui -> fonts_form_browser, temp);

	pa -> browser_line = browser_line;/* first one is 1 */
	browser_line++;
	}/* end for all elements in list */
	
/* re adjust browser for same position of top line (vertical slider) */	
if(position == SHOW_UNREAD_AT_TOP)
	{
	fl_set_browser_topline(fdui -> fonts_form_browser, first_new_font_line);
	}
if(position == SHOW_BOTTOM)
	{
	/* make the status of the last font visible */
	maxline = fl_get_browser_maxline(fdui -> fonts_form_browser);
	fl_set_browser_topline(fdui -> fonts_form_browser, maxline);
	}
if(position == SHOW_TOP)
	{
	fl_set_browser_topline(fdui -> fonts_form_browser, 1);
	}
if(position == SHOW_SAME_POSITION)
	{
	fl_set_browser_topline(fdui -> fonts_form_browser, browser_topline);
	}	

fl_unfreeze_form(fdui -> fonts_form);
return 1;
}/* end function show_fonts */


char *line_to_font(int line)
{
struct font *pa;

if(debug_flag)
	{
	printf("line_to_font(): arg line=%d\n", line);
	}

/* argument check */
if(line < 0) return 0;

for(pa = fonttab[0]; pa != 0; pa = pa -> nxtentr)	
	{
	if( (pa -> browser_line - 1) == line)
		{
		return pa -> name;
		}
	}

return 0;
}/* end function line_to_font_id */


int get_max_fonts(int *fonts)
{
int a;
struct font *pa;

if(debug_flag)
	{
	printf("get_max_fonts(): arg *fonts=%d\n", *fonts);
	}

a = 0;
for(pa = fonttab[0]; pa != 0; pa = pa -> nxtentr)
	{
	a++;
	}

*fonts = a;

if(debug_flag)
	{
	printf("get_max_fonts(): returing *fonts=%d\n", *fonts);
	}

return 1;
}/* end function get_max_fonts */


int set_requested_charset(char *iso_field)
{
struct font *pa;
int a;
int iso_number;
int iso_extention;
char *ptr;
int search_mode;
int i_number;
#define SEARCH_NO	0
#define SEARCH_UP	1
#define SEARCH_DOWN	2
#define SEARCH_ANY	3

if(debug_flag)
	{
	printf("set_requested_charset(): arg iso_field=%s\n", iso_field);
	}

if(! iso_field) return 0;

/*
ISO-8859-1
iso-8859-15
us-ascii
*/

/* point to first '-' */
ptr = strchr(iso_field, '-');
if(! ptr) return 0;

/* point to first digit or whatever */
ptr += 1;

/* should do something with 'us' too ??? */
/* if ASCII, use selected_articles_font */
if(strcasecmp(ptr, "ascii") == 0)
	{
	set_article_fonts(selected_articles_font);

	return 1;
	}

a = sscanf(ptr, "%d-%d", &iso_number, &iso_extention);
if(a != 2) return 0; 

if(debug_flag)
	{
	printf("set_requested_charset(): iso_number=%d iso_extention=%d\n",\
	iso_number, iso_extention);
	printf("selected_articles_font=%d\n", selected_articles_font);
	}

/* get selected font string (type of font) line number */

/* first find name of font */
search_mode = SEARCH_NO;

pa = fonttab[0];
while(1)
    {
	if(! pa) break;

	if(search_mode == SEARCH_ANY)
		{
		a = sscanf(pa -> iso_name + 3, "%d", &i_number);
		if(a == 1)
			{
			if(\
				(i_number == iso_number) &&\
				(pa -> iso_extention == iso_extention)\
				)
				{
				if(debug_flag)
					{
					printf("set_requested_charset():\n\
					SEARCH ANY FOUND pa -> iso_name=%s iso_extention=%d\n",\
					pa -> iso_name, pa -> iso_extention);
					}

				set_article_fonts(pa -> browser_line - 1);

				return 1;
				}
			} /* end if a == 1 */

		pa = pa -> nxtentr;
		continue;
		} /* end if search any */

	if(search_mode == SEARCH_UP)
		{
		if(pa -> iso_extention > iso_extention)
			{
			if(debug_flag)
				{
				printf("set_requested_charset(): Too big, not found\n");
				}

			break;
			}
		if(pa -> iso_extention == iso_extention)
			{
			if(debug_flag)
				{
				printf("set_requested_charset():\n\
				SEARCH_UP FOUND pa->name=%s\n",\
				pa -> name);
				}

			set_article_fonts(pa -> browser_line - 1);

			return 1;
			}

		pa = pa -> nxtentr;
		continue;
		} /* end if search up */

	if(search_mode == SEARCH_DOWN)
		{
		if(pa -> iso_extention < iso_extention)
			{
			if(debug_flag)
				{
				printf("set_requested_charset(): Too small, not found\n");
				}

			break;
			}
		if(pa -> iso_extention == iso_extention)
			{
			if(debug_flag)
				{
				printf("set_requested_charset():\n\
				SEARCH_DOWN FOUND pa->name=%s\n",\
				pa -> name);
				}

			set_article_fonts(pa -> browser_line - 1);

			return 1;
			}

		pa = pa -> prventr;
		continue;
		} /* end if search down */

	if(pa -> browser_line - 1 == selected_articles_font)
		{
		/* test if correct font already selected */

		/* xforms fonts can never match */
		if(pa -> iso_name[0] == 0)
			{
			pa = pa -> nxtentr;			
			continue;
			}		
		
		if(strncasecmp(pa -> iso_name, iso_field, 3) == 0)
			{
			a = sscanf(pa -> iso_name + 3, "%d", &i_number);
			if(a == 1)
				{
				if(i_number != iso_number)
					{
					if(debug_flag)
						{
						printf(\
						"set_requested_charset(): going SEARCH ANY\n");
						}

					/* search for any from the beginning of the list */
					search_mode = SEARCH_ANY;

// WAS MODIFIED FOR -Wall was pa = pa = fonttab[0];
 					pa = fonttab[0];
					continue;	
					} /* end if different iso_number */
				} /* end if a == 1 */
			} /* end if no iso number match */

		if(pa -> iso_extention == iso_extention)
			{
			if(debug_flag)
				{
				printf("set_requested_charset():\n\
				Correct font selected, returning pa->name=%s\n",\
				pa -> name);
				}

			set_article_fonts(pa -> browser_line - 1);

			return 1;
			}

		if(pa -> iso_extention < iso_extention)
			{
			if(debug_flag)
				{
				printf("set_requested_charset(): searching up\n");
				}
			
			search_mode = SEARCH_UP;
			
			pa = pa -> nxtentr;
			continue;
			}
		else
			{
			if(debug_flag)
				{
				printf("set_requested_charset(): searching down\n");
				}

			search_mode = SEARCH_DOWN;

			pa = pa -> prventr;
			continue;
			}
	
		} /* end if no search (initial) */

	pa = pa -> nxtentr;
    } /* end while all entries in list */

fl_show_alert(\
"No font found for requested",\
iso_field,\
"using user selected",\
0);

set_article_fonts(selected_articles_font);

/* not found */
return 0;
} /* end function set_requested_charset */


char *line_to_iso(int line, int *iso_extention)
{
struct font *pa;

if(debug_flag)
	{
	printf("line_to_iso(): arg line=%d\n", line);
	}

/* argument check */
if(line < 0) return 0;

for(pa = fonttab[0]; pa != 0; pa = pa -> nxtentr)	
	{
	if( (pa -> browser_line - 1) == line)
		{
		*iso_extention = pa -> iso_extention;
		return pa -> iso_name;
		}
	}

return 0;
}/* end function line_to_iso */



int select_a_font(int font_type, int iso_number, int iso_extention)
{
char iso_field[80];

if(debug_flag)
	{
	printf(\
	"select_a_font(): arg font_type=%d iso_number=%d iso_extention=%d\n",\
	font_type, iso_number, iso_extention);	
	}

/*
ISO-8859-1   
iso-8859-15
us-ascii         
*/

sprintf(iso_field,\
"%c%c%c-%d-%d",\
( (font_type & 0xff0000) >> 16) + 'A',\
( (font_type & 0xff00) >> 8) + 'A',\
(font_type & 0xff) + 'A',\
iso_number,\
iso_extention\
);

if(debug_flag)
	{
	printf("select_a_font(): recombined iso_field=%s\n", iso_field);
	}

if(! set_requested_charset(iso_field) ) return 0;

return 1;
} /* end function select_a_font */


int selected_font_to_vars(\
int line, int *font_type, int *iso_number, int *iso_extention)
{
int a;
char b, c, d;
struct font *pa;

if(debug_flag)
	{
	printf("selected_font_to_vars(): arg selected_font=%d\n",\
	line);
	}

/* return zero vars if not found */
*font_type = 0;
*iso_number = 0;
*iso_extention = 0;
for(pa = fonttab[0]; pa != 0; pa = pa -> nxtentr)	
	{
	if( (pa -> browser_line - 1) == line)
		{
		if(debug_flag)
			{
			printf("selected_font_to_vars(): pa -> iso_name=%s\n",\
			pa -> iso_name);
			}

		if(pa -> iso_name)
			{
			/* iso8859-15 in xlsforms format */
			a = sscanf(pa -> iso_name,\
			"%c%c%c%d",\
			&b, &c, &d, iso_number); /* last 2 already pointers */

			if(debug_flag)
				{
				printf("selected_font_to_vars(): sscanf a=%d\n", a); 
				}

			if(a == 4)
				{
				if(debug_flag)
					{	
					printf("b=%d (%c) c=%d (%c) d=%d (%c)\n",\
					b, b, c, c, d, d);
					}

				*font_type =\
				(65536 * (toupper(b) - 'A') ) +\
				(256 * (toupper(c) - 'A') ) +\
				(toupper(d) - 'A');
				
				*iso_extention = pa -> iso_extention;

				if(debug_flag)
					{
					printf("selected_font_to_vars(): returning found\n\
*font_type=%d *iso_number=%d *iso_extention=%d\n",\
					*font_type, *iso_number, *iso_extention);
					}

				return 1;
				} /* end if a is 4 */
			} /* end if name specified */
		
		} /* end if found */
	} /* end for all entries in list */

if(debug_flag)
	{
	printf("selected_font_to_vars(): returning not found all zeros\n");
	}

/* not found */
return 0;
} /* end function selected_font_to_vars */


int sort_fonts()
/*
sorts the double linked list with as criterium that the lowest ASCII goes
on top.
doing some sort of bubble sort.
*/
{
struct font *pa;
struct font *pb;
int swap_flag;
char *ptra, *ptrb;
char tempa[1024];
char tempb[1024];

while(1)/* go through list again and again */
	{
	if(debug_flag)
		{
		fprintf(stdout, "CHECKING LIST\n");
		}/* end if debug_flag */
		
	swap_flag = 0;
	for(pa = fonttab[0]; pa != 0; pa = pa -> nxtentr)
		{
		if(debug_flag)
			{
			fprintf(stdout, "sort_fonts(): sorting %s pa=%p\n",\
			pa -> name, pa);		
			}	
	
		pb = pa -> prventr;
		if(debug_flag)
			{
			fprintf(stdout, "pb=pa->prventr=%p\n", pb);
			}
		
		if(pb)
			{
			/* compare */
			if(pa -> iso_extention < pb -> iso_extention)
				{
				if(strcmp(pa -> iso_name, pb -> iso_name) == 0)
					{
					strcpy(tempa, pa -> name);
					ptra = strstr(tempa, pa -> iso_name);
					if(! ptra) return 0;
					*ptra = 0;

					strcpy(tempb, pb -> name);
					ptrb = strstr(tempb, pb -> iso_name);	
					if(! ptrb) return 0;
					*ptrb = 0;
					
					if(strcmp(tempa, tempb) == 0)
						{
						/* indicate position was swapped */
						swap_flag = swap_font_position(pa , pb);
						}			
					}
				}										
			}/* end if pb */
		}/* end for all entries */

	/* if no more swapping took place, ready, list is sorted */
	if(! swap_flag) break;
	}/* end while go through list again and again */

return 1;
}/* end function sort_fonts */


int swap_font_position(struct font *ptop, struct font *pbottom)
{
struct font *punder;
struct font *pabove;

if(debug_flag)
	{
	fprintf(stdout,\
	"swap_font_position(): swapping top=%p bottom=%p\n", ptop, pbottom);
	}

/* argument check */
if(! ptop) return 0;
if(! pbottom) return 0;

/* get one below the bottom */
punder = pbottom -> prventr;/* could be zero if first entry */
if(debug_flag)
	{
	fprintf(stdout,\
	"swap_font_position(): punder=%p\n", punder);
	}

/* get the one above the top */
pabove = ptop -> nxtentr;/* could be zero if last entry */
if(debug_flag)
	{
	fprintf(stdout,\
	"swap_font_position(): pabove=%p\n", pabove);
	}

/* the next pointer in punder (or fonttab[0]) must now point to ptop */
if(! punder)
	{
	fonttab[0] = ptop; 
	}
else
	{
	punder -> nxtentr = ptop;
	}

/* the prev pointer in in ptop must now point to punder */
ptop -> prventr = punder;/* could be zero if first entry */

/* the next pointer in ptop must now point to pbottom */
ptop -> nxtentr = pbottom;

/* the next pointer in pbottom must now point to pabove */
pbottom -> nxtentr = pabove;

/* mark last one in fonttab */
if(! pabove)
	{
	fonttab[1] = pbottom;
	}
else
	{
	/* the prev pointer in pabove must now point to pbottom */
	pabove -> prventr = pbottom;
	}

/* the prev pointer in pbottom must now point to ptop */
pbottom -> prventr = ptop;

/* return swapped */
return 1;
}/* end function swap_font_position */


/*
get_character_width(int character)
{
char *ptr;
Font font_id;
Display *dpy;

dpy = XtDisplay(app_shell);

printf("get_character_width(): used_font=%d\n", used_font);

ptr = line_to_font(used_font);
if(! ptr) return 0;

printf("get_character_width(): ptr=%s\n", ptr);

font_id = XLoadFont(":0:0", ptr);

printf("get_character_width(): font_id=%p\n", font_id);



return 1;
}
*/

/* end function get_character_width */
 
