/*
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/


#include "xste.h"

extern FD_xste *fdui;

int last_found_line;


struct frame *lookup_frame(char *name)
{
struct frame *pa;

for(pa = frametab[0]; pa != 0; pa = pa -> nxtentr)
	{
	if(strcmp(pa -> name, name) == 0) return(pa);
	}

return 0;/*not found*/
}/* end function lookup_frame */


struct frame *install_frame_at_end_of_list(char *name)
{
struct frame *plast, *pnew;

if(debug_flag)
	{
	fprintf(stdout,\
	"install_frame_at_end_off_list(): arg name=%s\n", name);
	}

pnew = lookup_frame(name);
if(pnew) return(pnew);/* already there */

/* create new structure */
pnew = (struct frame *) calloc(1, sizeof(*pnew) );
if(! pnew) return 0;
pnew -> name = strsave(name);
if(! pnew -> name) return 0;

/* get previous structure */
plast = frametab[1]; /* end list */

/* set new structure pointers */
pnew -> nxtentr = 0; /* new points top zero (is end) */
pnew -> prventr = plast; /* point to previous entry, or 0 if first entry */

/* set previuos structure pointers */
if( !frametab[0] ) frametab[0] = pnew; /* first element in list */
else plast -> nxtentr = pnew;

/* set array end pointer */
frametab[1] = pnew;

return(pnew);/* pointer to new structure */
}/* end function install_frame */


int delete_frame(int framenr)/* delete entry from double linked list */
{
struct frame *pa, *pprev, *pdel, *pnext;
char name[80];

if(debug_flag)
	{
	fprintf(stdout, "delete_frame(): arg framenr=%d\n", framenr);
	}

sprintf(name, "%d", framenr);
pa = frametab[0];
while(1)
	{
	/* if end list, return not found */
	if(! pa) return 0;

	/* test for match in name */
	if(strcmp(name, pa -> name) != 0) /* no match */
		{
		/* point to next element in list */
		pa = pa -> nxtentr;

		/* loop for next element in list */
		continue;
		}

	/* we now know which struture to delete */
	pdel = pa;

	/* get previous and next structure */
	pnext = pa -> nxtentr;
	pprev = pa -> prventr;

	/* set previous entry for preview display to signal it no longer exists */
//	real_pa = 0;

	/* set pointers for previous structure */
	/* if first one, modify frametab[0] */
	if(pprev == 0) frametab[0] = pnext;
	else pprev -> nxtentr = pnext;

	/* set pointers for next structure */
	/* if last one, modify frametab[1] */
	if(pnext == 0) frametab[1] = pprev;
	else pnext -> prventr = pprev;
	
	/* delete structure */	
	/* do not delete pfd! it points to data that may be in use by an other frame (same font) */
	free(pdel -> fontname);
	free(pdel -> text);
	free(pdel -> name);
	free(pdel); /* free structure */

	/* return OK deleted */
	return 1;
	}/* end for all structures */
}/* end function delete_frame */


int delete_all_frames()/* delete all entries from table */
{
struct frame *pa;

if(debug_flag)
	{
	fprintf(stdout, "delete_all_frames() arg none\n");
	}

while(1)
	{	
	pa = frametab[0];
	if(! pa) break;
	frametab[0] = pa -> nxtentr;

	/* do not delete pfd! it points to data that may be in use by an other frame (same font) */
	free(pa -> fontname);
	free(pa -> text);
	free(pa -> name);
	free(pa);/* free structure */
	}/* end while all structures */

frametab[1] = 0;

/* set previous entry for preview display to signal it no longer exists */
//real_pa = 0;

return 1;
}/* end function delete_all_frames */


int load_frames()
{
int a;
FILE *fileptr;
char temp[READSIZE];
char pathfilename[TEMP_SIZE];
struct frame *pa;
extern char *strsave();
char name[TEMP_SIZE];
char temp2[1024];
char temp3[1024];
int browser_line; /* used to lookup name (= id) if clicked in display,
						not saved on disk, assigned each time list
						is displayed */
int end_frame;
char text[READSIZE];
int status;
int line;
int previous_start_frame;

int pattern;
int background;
int emphasis1;
int emphasis2;

int pattern_contrast;
int background_contrast;
int emphasis1_contrast;
int emphasis2_contrast;

char fontname[4096];
int symbols;
int espace;
int vespace;
int fontsize;
int iso_extension;
double outline_thickness;
double blur_radius;   
font_desc_t *pfd;

int arg_count;

if(debug_flag)
	{
	fprintf(stdout, "load_frames() arg none\n");
	}
	
delete_all_frames();

delete_all_subtitle_fontnames();

sprintf(pathfilename, "%s/%s/%s/%s",\
home_dir, selected_data_directory, selected_project, data_file_name);
fileptr = fopen(pathfilename, "r");
if(! fileptr)
	{
	fl_show_alert("could not open file", pathfilename, "for read", 1);

	if(debug_flag)
		{
		fprintf(stdout, "could not load file %s\n", pathfilename);
		}
	return 0;
	} 

/* for all lines in data_file_name */
line = 0;
previous_start_frame = -1;
while(1)
	{
	a = readline(fileptr, temp);
	if(a == EOF) return 1;/* empty file or last line */

	arg_count = sscanf(temp,\
	"%s %d %d %d  %d %d %d %d  %d %d %d %d  %d %d  %lf %lf  %s  %d %d %d",\
	name,\
	&end_frame,\
	&browser_line,\
	&status,\
	&pattern,\
	&background,\
	&emphasis1,\
	&emphasis2,\
	&pattern_contrast,\
	&background_contrast,\
	&emphasis1_contrast,\
	&emphasis2_contrast,\
	&fontsize,\
	&iso_extension,\
	&outline_thickness,\
	&blur_radius,\
	fontname,\
	&symbols,\
	&espace,\
	&vespace);

	if(debug_flag)
		{
		/* arg_count should be 17 for xste-3.0, and 4 for earlier versions */
		fprintf(stdout, "arg_count=%d\n", arg_count);
		}

	/* increment line counter for error reporting */
	line ++;

	/* test if import from earlier version then xste-3.0 */
	if(arg_count == 4)
		{
		/* will not be set perhaps */
//		strcpy(fontname, default_subtitle_font_name);
		strcpy(fontname, "arial.ttf");

		symbols = 0;

		status = 0;
		status |= CENTER_TEXT;
		status |= DVD_ANTI_ALIAS;
		status &= ~NO_KERNING;
		
		fontsize = 32;	
		iso_extension = 15;
		outline_thickness = 1.0;
		blur_radius = 0.1;
		
		background = 0;
		pattern = 1;
		emphasis1 = 2;
		emphasis2 = 15;

		background_contrast = 0;
		pattern_contrast = 15;
		emphasis1_contrast = 15;
		emphasis2_contrast = 15;
		}

	pfd = add_font(fontname, symbols, fontsize, iso_extension, outline_thickness, blur_radius,\
	espace, vespace);
	if(! pfd)
		{
		fl_show_alert("could not add font for frame", name, fontname, 0);

		return 0;
		}

	/* font pointer now in pfd */
	
	/* consistence / incremental sequence check audio samples */
	if(line > 2)
		{
		/* test if the start_frame is lower then in the previous line */
		if( atoi(name) <= previous_start_frame)
			{
			sprintf(temp2,\
			"%s line %d start_frame=%d",\
			data_file_name, line - 2, previous_start_frame);

			sprintf(temp3,\
			"%s line %d start_frame=%d",\
			data_file_name, line , atoi(name) ); 

			fl_show_alert(\
				"CONSISTENCY ERROR FIX IT", temp2, temp3, 1);

			/*
			load the file anyways, allow user to correct with the
			'edit frame list button'
			*/
	
			}
		} /* end consistency check */

	a = readline(fileptr, text);
	if(a == EOF) return 0;/* last line, or file format error */

	/* increment line counter for error reporting */
	line ++;

	/* remember start_frame for next time here */
	previous_start_frame = atoi(name);

	/* create a structure entry */
	pa = install_frame_at_end_of_list(name);
	if(! pa)
		{
		if(debug_flag)
			{
			fprintf(stdout,\
			"load_frames: cannot install frame %s\n", temp);
			}
		return 0;
		}

	pa -> end_frame = end_frame;

	pa -> text = strsave(text);
	if(! pa -> text) return 0;

	pa -> status = status;

	pa -> browser_line = browser_line;
  
	pa -> pattern = pattern;
	pa -> background = background;
	pa -> emphasis1 = emphasis1;
	pa -> emphasis2 = emphasis2;

	pa -> pattern_contrast = pattern_contrast;
	pa -> background_contrast = background_contrast;
	pa -> emphasis1_contrast = emphasis1_contrast;
	pa -> emphasis2_contrast = emphasis2_contrast;

	pa -> symbols = symbols;

	pa -> espace = espace;
	pa -> vespace = vespace;

	pa -> fontsize = fontsize;
	pa -> iso_extension = iso_extension;

	pa -> outline_thickness = outline_thickness;   
	pa -> blur_radius = blur_radius;

	pa -> pfd = pfd;

	pa -> fontname = strsave(fontname);
	if(! pa -> fontname)
		{
		fprintf(stdout, "load_frames(): could not allocate space for pa -> fontname\n");

		return 0;
		} 

	}/* end while all lines in data_file_name */

return 1;
}/* end function load frames */


int save_frames()
{
FILE *fileptr;
char pathfilename[TEMP_SIZE];
char pathfilename2[TEMP_SIZE];
struct frame *pa;
int max_frame;

if(debug_flag)
	{
	fprintf(stdout, "save_frames() arg none\n");
	}

sprintf(pathfilename, "%s/%s/%s/%s.tmp",\
home_dir, selected_data_directory, selected_project, data_file_name);
fileptr = fopen(pathfilename, "w");
if(! fileptr)
	{
	fl_show_alert("could not open file", pathfilename, "for write", 1);

	if(debug_flag)
		{
		fprintf(stdout,\
		"could not open file %s for write\n", pathfilename);
		}
	return 0;
	} 

max_frame = 0;
for(pa = frametab[0]; pa != 0; pa = pa -> nxtentr)	
	{
	fprintf(fileptr, "%s %d %d %d  %d %d %d %d  %d %d %d %d  %d %d  %.4f %.4f  %s  %d %d %d\n",\
	pa -> name,\
	pa -> end_frame,\
	pa -> browser_line,\
	pa -> status,\
	pa -> pattern,\
	pa -> background,\
	pa -> emphasis1,\
	pa -> emphasis2,\
	pa -> pattern_contrast,\
	pa -> background_contrast,\
	pa -> emphasis1_contrast,\
	pa -> emphasis2_contrast,\
	pa -> fontsize,\
	pa -> iso_extension,\
	pa -> outline_thickness,\
	pa -> blur_radius,\
	pa -> fontname,\
	pa -> symbols,\
	pa -> espace,\
	pa -> vespace \
	);

	fprintf(fileptr, "%s\n", pa -> text);
	} /* end while all elements in list */

fclose(fileptr);

if(debug_flag)
	{
	fprintf(stdout, "save_frames(): backing up %s\n", data_file_name);
	}

/* set some path file names */
sprintf(pathfilename, "%s/%s/%s/%s",\
home_dir, selected_data_directory, selected_project, data_file_name);

sprintf(pathfilename2, "%s/%s/%s/%s~",\
home_dir, selected_data_directory, selected_project, data_file_name);

/* unlink the old .dat~ */
unlink(pathfilename2);

/* rename .dat to .dat~ */
if( rename(pathfilename, pathfilename2) == -1)
	{
	if(debug_flag)
		{
		fprintf(stdout, "save_frames(); rename %s into %s failed\n",\
		pathfilename, pathfilename2);
		}
	}

/* rename .tmp to .dat */
sprintf(pathfilename2, "%s/%s/%s/%s.tmp",\
home_dir, selected_data_directory, selected_project, data_file_name);
if( rename(pathfilename2, pathfilename) == -1)
	{
	fl_show_alert("could not rename", pathfilename2, pathfilename, 1);

	if(debug_flag)
		{
		fprintf(stdout, "save_frames(); rename %s into %s failed\n",\
		pathfilename, pathfilename2);
		}
	return 0;
	}

return 1;/* ok */
}/* end function save_frames */


int show_frame_form()
{
int line;

if(debug_flag)
	{
	fprintf(stdout, "show_frame_form(): arg none\n");
	}

if(! load_frames() )
	{
	/*
	exit here, but if not exit user can fix using 'edit frame list'.
	*/

	}

/* in case no text, avoid NULL pointers */
fl_set_input(fdui -> main_form_text_input_field, "");

fl_set_input(fdui -> main_form_frame_input_field, "");

fl_set_input(fdui -> main_form_current_frame_input_field, "");

show_frames(SHOW_TOP);

frame_to_line(selected_start_frame, &line);
fl_select_browser_line(fdui -> main_form_browser, line);

if(! set_force_duration_button(selected_start_frame) ) return 0;

/* for the MEM button */
current_frame = selected_start_frame;

return 1;
}/* end function show_frame_form */


int show_frames(int position)
{
char temp[TEMP_SIZE];
struct frame *pa;
char formatstr[20];
int browser_line;
int browser_topline;
char *strsave();
int first_new_frame_line;
int find_first_new_frame_flag;
int maxline;
int next_start_frame;
struct frame *pnext;
int text_lines;

if(debug_flag)
	{
	fprintf(stdout, "show_frames(): arg position=%d\n", position);
	}

/* remember vertical slider position */
browser_topline = fl_get_browser_topline(fdui -> main_form_browser);

fl_freeze_form(fdui -> main_form);

fl_clear_browser(fdui -> main_form_browser);

first_new_frame_line = -1;/* keep gcc -Wall from complaining */
find_first_new_frame_flag = 1;
browser_line = 1;
for(pa = frametab[0]; pa != 0; pa = pa -> nxtentr)	
	{
	/*
	test if space for entry, so the end_frame of this one should be
	lower then the start_frame of the next one.
	*/
	pa -> status &= 0xff;
	if(pa -> nxtentr)
		{
		pnext = pa -> nxtentr;
		next_start_frame = atoi(pnext -> name);

		if(pa -> end_frame > next_start_frame)
			{
			pa -> status |= NO_SPACE;
			}
		else pa -> status &= 255 - NO_SPACE;
		} /* end if not last entry */
	
//	fixed spacing
//	strcpy(formatstr, "@f");
	strcpy(formatstr, "");

	/* black is default color */
	strcat(formatstr, "@C0"); /* black */

	/* returns at least 1 */
	if(! calculate_text_lines(pa, &text_lines) )
		{
		fl_show_alert("could not calculate number of text lines for frame", pa -> name, "ignoring", 0);

//		return 0;	
		}
	
	/* set status */
	/* text_lines cannot be 0 here */
	if(text_lines >= WARN_NUMBER_LINES) pa -> status |= TOO_LONG;
	else pa -> status &= ~TOO_LONG;

	/* colors */

//	strcat(formatstr, "@C17"); /* dark grey */
//	strcat(formatstr, "@C18"); /* dark green */
//	strcat(formatstr, "@C20"); /* dark violet */
//	strcat(formatstr, "@C22"); /* dark brown */
//	strcat(formatstr, "@C29"); /* dark blue */

	/* general indication */
	if(text_lines == 2) strcat(formatstr, "@C27"); /* violet */

	/* warning only if requested */
	if(show_time_overlap_flag)
		{
		if(pa -> status & NO_SPACE)
			{
			strcat(formatstr, "@C18"); /* dark green */

//			if(text_lines == 1) strcat(formatstr, "@C17"); /* dark grey */ 
//			if(text_lines == 2) strcat(formatstr, "@C10"); /* dark violet */
			}
		}
	
	/* warning */
	if(pa -> status & TOO_LONG) strcat(formatstr, "@C1"); /* red */

	/* special condition */
	if(pa -> status & TXT_HOLD) strcat(formatstr, "@C24"); /* orange */

	/* fatal error */
	if(pa -> status & RLE_ERROR) strcat(formatstr, "@C8"); /* dark orange */
	
//	if(highlight_frames(pa) ) strcat(formatstr, "@_");/* underscore */

	/* end colors */

	sprintf(temp, "%s %d %s",\
		formatstr, atoi(pa -> name), pa -> text);
	
	fl_add_browser_line(fdui -> main_form_browser, temp);
	pa -> browser_line = browser_line;/* first one is 1 */

	browser_line++;
	}/* end for all elements in list */

/* re adjust browser for same position of top line (vertical slider) */	
if(position == SHOW_UNREAD_AT_TOP)
	{
	fl_set_browser_topline(fdui -> main_form_browser, first_new_frame_line);
	}
if(position == SHOW_BOTTOM)
	{
	/* make the status of the last frame visible */
	maxline = fl_get_browser_maxline(fdui -> main_form_browser);
	fl_set_browser_topline(fdui -> main_form_browser, maxline);
	}
if(position == SHOW_TOP)
	{
	fl_set_browser_topline(fdui -> main_form_browser, 1);
	}
if(position == SHOW_SAME_POSITION)
	{
	fl_set_browser_topline(fdui -> main_form_browser, browser_topline);
	}	
fl_unfreeze_form(fdui -> main_form);

return 1;
}/* end function show_frames */


int line_to_frame(int line, int *start_frame, int *end_frame)
{
struct frame *pa;

if(debug_flag)
	{
	fprintf(stdout, "line_to_frame(): arg line=%d\n", line);
	}

for(pa = frametab[0]; pa != 0; pa = pa -> nxtentr)	
	{
	if(pa -> browser_line == line)
		{
		*start_frame = atoi(pa -> name);
		*end_frame = pa -> end_frame;
		return 1;
		}
	}

/* no save_frames, nothing was changed */
return 0;
}/* end function line_to_frame */


int line_to_next_frame(int line, int *start_frame, int *end_frame)
{
struct frame *pa;

if(debug_flag)
	{
	fprintf(stdout, "line_to_next_frame(): arg line=%d\n", line);
	}

for(pa = frametab[0]; pa != 0; pa = pa -> nxtentr)	
	{
	if(pa -> browser_line >= line)
		{
		*start_frame = atoi(pa -> name);
		*end_frame = pa -> end_frame;
		return 1;
		}
	}

/* no save_frames, nothing was changed */
return 0;
}/* end function line_to_next_frame */


int frame_to_line(int framenr, int *line)
{
struct frame *pa;
char name[50];

if(debug_flag)
	{
	fprintf(stdout, "frame_to_line(): arg frame=%d\n", framenr);
	}

sprintf(name, "%d", framenr);
pa = lookup_frame(name);
if(! pa) return 0;

*line = pa -> browser_line;
return 1;
}/* end function frame_to_line */


int frame_to_next_line(int framenr, int *line)
{
struct frame *pa;

if(debug_flag)
	{
	fprintf(stdout, "frame_to_next_line(): arg frame=%d\n", framenr);
	}

for(pa = frametab[0]; pa != 0; pa = pa -> nxtentr)
	{
	if(atoi(pa -> name) >= framenr)
		{
		*line = pa -> browser_line;

		return 1;
		}
	} /* end for all entries in frame list */

return 1;
}/* end function frame_to_next_line */


int add_frame(char *text,\
	int start_framenr, int end_framenr, int text_hold_flag)
{
struct frame *pa;
char name[TEMP_SIZE];
font_desc_t *pfd;

if(debug_flag)
	{
	fprintf(stdout,\
	"add_frame(): arg text=%s\n\
	start_framenr=%d end_framenr=%d\n\
	text_hold_flag=%d\n",\
	text, start_framenr, end_framenr,\
	text_hold_flag);
	}

/* argument check */
if(! text) return 0;

sprintf(name, "%d", start_framenr);
pa = install_frame_at_end_of_list(name);
if(! pa) return 0;

pa -> end_frame = end_framenr;
pa -> text = strsave(text);
if(! pa -> text) return 0;

pa -> status = 0;
pa -> status |= NEW_ENTRY;
pa -> status |= CENTER_TEXT;

if(text_hold_flag) pa -> status |= TXT_HOLD;

/* font stuff, added in version 3.0 */

/* use default font */
pa -> background = default_subtitle_background_color;
pa -> pattern = default_subtitle_body_color;
pa -> emphasis1 = default_subtitle_outline1_color;
pa -> emphasis2 = default_subtitle_outline2_color;
    
pa -> background_contrast = default_subtitle_background_contrast;
pa -> pattern_contrast = default_subtitle_body_contrast;
pa -> emphasis1_contrast = default_subtitle_outline1_contrast;
pa -> emphasis2_contrast = default_subtitle_outline2_contrast;

pa -> fontname = strsave(default_subtitle_font_name);
if(! pa -> fontname)
	{
	fprintf(stderr, "xste: add_frame(): could not allocate space for pa -> fontname\n");
	
	return 0;
	}

pa -> fontsize = default_subtitle_font_size;
pa -> iso_extension = default_subtitle_iso_extention;
pa -> outline_thickness = default_subtitle_outline_thickness;
pa -> blur_radius = default_subtitle_blur_radius;
  
pfd = add_font(\
	default_subtitle_font_name,\
	default_subtitle_font_symbols,\
	default_subtitle_font_size, default_subtitle_iso_extention,\
	default_subtitle_outline_thickness, default_subtitle_blur_radius,\
	default_subtitle_extra_character_space, default_subtitle_extra_character_vspace);
	if(! pfd)
		{
		fprintf(stderr,\
		"xste: add_frame(): could not add font for frame %d\n", start_framenr);

		return 0;
		}

pa -> pfd = pfd;

/* 
called from AUTO and done after all entries have been added for speed 
*/
//save_frames():

return 1;
}/* end function add_frame */


void replace(char *text, char *string, char *replace, char *result)
{
while(1)
	{
	if( (char *)strstr(text, string) == text)
		{
		*result = 0;
		strcat(result, replace);
		text += strlen(string);
		result += strlen(replace);
		}
	*result = *text;
	if(*text == 0)break;
	result++;
	text++;
	}
}/* end function replace */


int set_text_input(int start_framenr)
{
struct frame *pa;
char temp[1024];

sprintf(temp, "%d", start_framenr);
pa = lookup_frame(temp);
if(! pa) return 0;

fl_set_input(fdui -> main_form_text_input_field, pa -> text);

return 1;
}/* end function set_text_input */


int search_and_select_next_frame()
{
struct frame *pa;
char *keyword;
int case_sensitive_flag;
int found;

if(debug_flag)
	{
	fprintf(stdout, "search_and_select_next_frame(): arg none\n");
	}

keyword = (char *)fl_get_input(fdui -> main_form_search_input_field);
if(! keyword) return 0;

case_sensitive_flag =\
fl_get_button(fdui -> main_form_search_case_button);

found = 0;
/* pa points to next entry */
for(pa = frametab[0]; pa != 0; pa = pa -> nxtentr)
	{
	/* search from last position (set to zero if keyword changes */
	if(pa -> browser_line <= last_found_line) continue;
	
	if(search_in_for(pa -> name, keyword, case_sensitive_flag) )
		{
		found = 1;
		}

	if(search_in_for(pa -> text, keyword, case_sensitive_flag) )
		{
		found = 1;
		}

	if(found)
		{
		free(selected_text);
		selected_text = strsave(pa -> name);

		/*
		maintain visible indication that line is selected with single
		click
		*/
		fl_select_browser_line(fdui -> main_form_browser, pa -> browser_line);

		/* show the entry */
		fl_set_browser_topline(fdui -> main_form_browser, pa -> browser_line);

		/* select the entry */
		fl_set_input(fdui -> main_form_text_input_field, pa -> text);

		/* remember where we are */
		last_found_line = pa -> browser_line;
		
		selected_start_frame = atoi(pa -> name);

		set_frame_input(selected_start_frame);

		selected_end_frame = pa -> end_frame;

		/* return found */
		return 1;
		}/* end if found */
	}/* end for all frame's */

if(last_found_line) fl_show_alert(keyword, "end of list", "", 0);
else fl_show_alert(keyword, "not found", "", 0);

last_found_line = 0;

/* return not found */
return 0;
}/* end function search_and_select_frame */


int main_form_reset_last_found_line()
{
last_found_line = 0;

return 1;
}/* end function url_form_reset_last_found_line */


int set_frame_status(char *frame, int status)
{
struct frame *pa;

if(debug_flag)
	{
	fprintf(stdout,\
	"set_frame_status(): arg frame=%s status=%d\n", frame, status);
	}

/* argument check */
if(! frame) return 0;

pa = lookup_frame(frame);
if(! pa) return 0;

pa -> status = status;

return 1;
}/* end function set_frame_status */
 

int get_frame_status(char *frame, int *status)
{
struct frame *pa;

if(debug_flag)
	{
	fprintf(stdout,\
	"get_frame_status(): arg frame=%s\n", frame);
	}

/* argument check */
if(! frame) return 0;

pa = lookup_frame(frame);
if(! pa) return 0;

*status = pa -> status;

return 1;
}/* end function get_frame_status */
 

int search_in_for(char *database, char *data, int case_sensitive)
{
/* This function returns 1 if all words in data are present in database */
char *ptra, *ptrb, *ptrc;
int match_failed_flag;
int last_string_flag;
char *strsave();
char *strstr_case_insensitve();

if(debug_flag)
	{
	fprintf(stdout,\
	"search_in_for(): arg database=%s data=%s case_sensitive=%d\n",\
	database, data, case_sensitive);
	}

/* argument check */
if(! database) return 0;
if(! data) return 0;
if( (case_sensitive != 0) && (case_sensitive != 1) ) return 0;

/*
Test if some data,
data may consist of one or more words separated by spaces
*/
if(! data[0]) return 0;/* no data */

/* test if data present in database */
/* for all entries in data */
ptra = data;
ptrb = strsave(data);/* create space */
match_failed_flag = 0;
last_string_flag = 0;
while(1)
	{
	ptrc = ptrb;/* point to start available space */
	while(1)/* string in data to ptrb */
		{
		*ptrc = *ptra;
		if(! *ptra) break;/* end data */
		if(*ptra == ' ')/* end of string in data */
			{
			*ptrc = 0;
			ptra++;/* scip the space */
			break;
			}
		ptra++;
		ptrc++;
		}/* end while all characters in string */
	/* test if string present in database */
	if(case_sensitive)
		{
		if(! strstr(database, ptrb) )
			{
			match_failed_flag = 1;
			break;
			}
		}
	else
		{
		if(! strstr_case_insensitive(database, ptrb) )
			{
			match_failed_flag = 1;
			break;
			}
		}/* end if not case sensitive */
	if(! *ptra) break;/* end filter */
	}/* end while all strings in filter */	
free(ptrb);/* release space */

if(match_failed_flag) return 0;
return 1;
} /* end function search_in_for */


char *strstr_case_insensitive(char *s, char *t)
{
int i, j, k;
for(i = 0; s[i] != '\0'; i++)
	{
	for(j = i, k = 0; t[k] != '\0' &&  toupper(s[j]) == toupper(t[k]);\
	j++, k++)
	;
	if(t[k] == '\0') return(s + i);
	}
return 0;
}/* end function strstr_case_insensitive */


struct frame *install_frame_in_position(char *name, int *new_flag)
{
struct frame *pa, *pprev, *plast, *plastmatch, *pnew, *pnext;
struct frame *lookup_frame();
int have_bigger_flag;

if(debug_flag)
	{
	fprintf(stdout,\
	"install_frame_in_position():\nname=%s\n",\
	name);
	}
	
/* argument check */
if(! name) return 0;

plastmatch = 0;/* for gcc -Wall */

*new_flag = 0;

/*
Test if frame number already exists, if so, overwrite,
If frame number > then the one in the list, look for the next,
if bigger insert before the bigger one, else at end of list.
This quarantees the frames are always sequential. 
*/
have_bigger_flag = 0;
for(pa = frametab[0]; pa != 0; pa = pa -> nxtentr)
	{
	if(atoi(pa -> name) == atoi(name) )
		{
		/* already there and leave status as is */
		
		return(pa); 
		}
	
	if(have_bigger_flag)
		{
		/* test if to far */
		if(atoi(name) < atoi(pa -> name) ) break;
		} /* end if smaller */

	if(atoi(name) > atoi(pa -> name) )
		{
		have_bigger_flag = 1;
		} /* end if bigger */

	/* save last position */
	plastmatch = pa;
	} /* end for all entries in list */

if(debug_flag)
	{
	fprintf(stdout,\
	"ouside for plastmatch=%p\n", plastmatch);
	}

/* plastmatch now points to insert point, or zero if not found, */

/* create new structure */
pnew = (struct frame *) calloc(1, sizeof(*pnew) );
if(! pnew) return 0;
pnew -> name = strsave(name);
if(! pnew -> name) return 0;

/* flag new structure entry was created */ 
*new_flag = 1;

/* add to end list if no match */
if(! plastmatch)
	{
	/* get previous structure */
	plast = frametab[1]; /* end list */

	/* set new structure pointers */
	pnew -> nxtentr = 0; /* new points top zero (is end) */
	pnew -> prventr = plast; /* point to previous entry, or 0 if first entry */

	/* set previous structure pointers */
	if( !frametab[0] ) frametab[0] = pnew; /* first element in list */
	else plast -> nxtentr = pnew;

	/* set array end pointer */
	frametab[1] = pnew;

	/* return pointer to new structure */
	return pnew;
	}/* end if ! plastmatch */

/* plastmatch now holds insert position, insert frame */
/* insert the structure AFTER plastmatch */
		
/* get previous and next structure */
pnext = plastmatch -> nxtentr;
pprev = plastmatch;

/* set pointers for new struture */
pnew -> nxtentr = pnext;
pnew -> prventr = pprev;

/* set pointers for next structure */
if(pnext == 0) frametab[1] = pnew;
else pnext -> prventr = pnew;		

/* set pointer for previous structure */
if(pprev == 0) frametab[0] = pnew;
else pprev -> nxtentr = pnew;
		
/* mark it as a new entry */
pnew -> status = NEW_ENTRY;

/* return pointer to new structure */
return pnew;
}/* end function install_frame_in_position */


struct frame *add_frame_in_position(char *text,\
	int start_framenr, int end_framenr,\
	int text_hold_flag, int *new_frame_flag)
{
char temp[512];
struct frame *pa;

if(debug_flag)
	{
	fprintf(stdout,\
	"add_frame_in_position(): arg text=%s\n\
	start_framenr=%d end_framenr=%d\n\
	text_hold_flag=%d\n",\
	text, start_framenr, end_framenr,\
	text_hold_flag);
	}

sprintf(temp, "%d", start_framenr);

/* sets status to new entry if found not to exist */
pa = install_frame_in_position(temp, new_frame_flag); /* new_frame_flag already a pointer */

if(! pa) return 0;

pa -> text = strsave(text);
if (! pa -> text) return 0;

if(text_hold_flag == 0)
	{
	pa -> status &= (65535 - TXT_HOLD);
	pa -> end_frame = end_framenr;
	}
else
	{
	pa -> status |= TXT_HOLD;
	pa -> end_frame = INT_MAX; /* maximum frames */
	}

/* 
sort the list, cause if item < first entry was entered it
would be at the end
*/
sort_frames();

return pa;
}/* end function add_frame_in_position */


int text_to_frames(char *text, int *frames)
{
/*
calculates how many frames are needed to view text.
*/
int c;
char *ptr;
int words;
char wordlen;
int longest_word_length;
int m_seconds;
int complex_word_count;

if(debug_flag)
	{
	fprintf(stdout, "text_to_frames(): arg text=%s\n", text);
	}

/* count words */
ptr = text;
complex_word_count = 0;
longest_word_length = 0;
words = 0;
wordlen = 0;
while(1)
	{
	c = *ptr;
	if(! c) break;

	if(c == ' ')
		{
		words++;

		if(wordlen > longest_word_length) longest_word_length = wordlen;

		if(wordlen > 10) complex_word_count++;
		wordlen = 0;
		}
	else 
		{
		if(words == 0) words = 1;

		wordlen++;
		}

	ptr++;
	} /* end while count words */

/*
There are now 3 quantities:
words					(the number of words in text)
longest_word_length		the number of characters in the longest word in text.
complex_word_count		the number of words with more then 10 letters. 

This should allow a simple estimate how long the text should be on the
screen.
*/

/*
Words can be maximum 20 for 20 single letter words (separated by spaces)
word complexity can maximum be 40 for a 40 letter word
*/

/*
Let's say on average 3 a 4 seconds for a line with 14 words, so
3500 / 14 milli seconds per word, is 250 mS per word.
*/

/* calculate milli seconds for words */
m_seconds = words * 250;

/*
Let's say, that words longer then 10 characters add to complexity,
these may take up to half a second per word more, but not likely
*/
m_seconds += (complex_word_count * 500);

/*
Let's say (very) long words need extra more reading time,
20 mS per character 
*/

// ATTENTION FIX WAS  + so statement had no effect!!! 
m_seconds += (longest_word_length * 20);

m_seconds *= text_duration_multiplier;

/* calculate frames, 1 frame is 40 mS if 25.0 frames / second */
*frames = m_seconds / dmilli_seconds_per_frame;

if(debug_flag)
    {
    fprintf(stdout,\
	"text_to_frames(): arg words=%d\n\
	longest_word_length=%d\n\
	complex_word_count=%d\n\
	m_seconds=%d\n\
	*frames=%d\n",\
	words,\
	longest_word_length,\
	complex_word_count,\
	m_seconds,\
	*frames);
    }

return 1;
} /* end function text_to_frames */


int recalculate_end_frames()
{
int a;
struct frame *pa;

if(debug_flag)
	{
	fprintf(stdout, "recalculate_end_frames(): arg none\n");
	}

for(pa = frametab[0]; pa != 0; pa = pa -> nxtentr)
	{
	/* get length frame depending on text */                 
	if(! text_to_frames(pa -> text, &a) ) return 0;

	if(pa -> status & TXT_HOLD)
		{
		pa -> end_frame = INT_MAX; /* max frames */
		}
	else
		{
		pa -> end_frame = atoi(pa -> name) + a;
		}

	} /* end for frames */

return 1;
} /* end function recalculate_end_frames */


int find_next_too_long_line()
{
struct frame *pa;
int found;
static int last_found_too_long_line;
int too_long_flag;
int text_lines;

if(debug_flag)
    {
    fprintf(stdout, "find_next_too_long_line(): arg none\n");
    }

found = 0;
/* pa points to next entry */
for(pa = frametab[0]; pa != 0; pa = pa -> nxtentr)
	{
	/* search from last position (set to zero if keyword changes */
	if(pa -> browser_line <= last_found_too_long_line) continue;

	too_long_flag = 0;
	if(output_format == OUTPUT_FORMAT_SUBMUX_DVD)
		{
		text_lines = 0;
		if(! calculate_text_lines(pa, &text_lines) )
			{
			if(debug_flag)
				{
				fprintf(stdout, "find_next_too_long_line(): calculate_text_lines failed\n");
				}

			fl_show_alert("Could not calculate text lines for", pa -> name, "skipping", 0);
	
//			return 0;
			}
			
		if(text_lines >= WARN_NUMBER_LINES)
			{
			pa -> status |= TOO_LONG;
			}
		else
			{
			pa -> status &= ~TOO_LONG;
			}
		} /* end if submux_dvd output */

	if(pa -> status & TOO_LONG)
		{
		/*
		maintain visible indication that line is selected with single
		click
		*/
		fl_select_browser_line(fdui -> main_form_browser, pa -> browser_line);

		/* show the entry */
		fl_set_browser_topline(fdui -> main_form_browser, pa -> browser_line);

		/* select the entry */
		fl_set_input(fdui -> main_form_text_input_field, pa -> text);

		/* remember where we are */
		last_found_too_long_line = pa -> browser_line;
		
		selected_start_frame = atoi(pa -> name);

		set_frame_input(selected_start_frame);	

		selected_end_frame = pa -> end_frame;

		/* return found */
		return 1;
		}/* end if found */
	}/* end for all frame's */

if(last_found_too_long_line)
	{
	fl_show_alert("end of list", "", "", 0);
	last_found_too_long_line = 0;
	}
else fl_show_alert("No too long lines found", "", "", 0);

last_found_line = 0;

/* return not found */
return 0;
} /* end function find_next_too_long_line */


int sort_frames()
/*
sorts the double linked list with criterium set by  mode.
doing some sort of bubble sort.
*/
{
struct frame *pa;
struct frame *pb;
int swap_flag;

while(1)/* go through list again and again */
	{
	swap_flag = 0;
	for(pa = frametab[0]; pa != 0; pa = pa -> nxtentr)
		{
		pb = pa -> prventr;
		if(pb)
			{
			if(atol(pa -> name) < atol(pb -> name) )
				{
				/* indicate position was swapped */
				swap_flag = swap_frame_position(pa , pb);
				}/* end if condition */
			}/* end if pb */
		}/* end for all entries */

	/* if no more swapping took place, ready, list is sorted */
	if(! swap_flag) break;
	}/* end while go through list again and again */

return 1;
}/* end function sort_frames */


int swap_frame_position(struct frame *ptop, struct frame *pbottom)
{
struct frame *punder;
struct frame *pabove;

if(debug_flag)
	{
	fprintf(stdout,\
	"swap_position(): swapping top=%p bottom=%p\n", ptop, pbottom);
	}

/* get one below the bottom */
punder = pbottom -> prventr;/* could be zero if first entry */
if(debug_flag)
	{
	fprintf(stdout,\
	"swap_position(): punder=%p\n", punder);
	}

/* get the one above the top */
pabove = ptop -> nxtentr;/* could be zero if last entry */
if(debug_flag)
	{
	fprintf(stdout,\
	"swap_position(): pabove=%p\n", pabove);
	}

/* the next pointer in punder (or frametab[0]) must now point to ptop */
if(! punder)
	{
	frametab[0] = ptop; 
	}
else
	{
	punder -> nxtentr = ptop;
	}

/* the prev pointer in in ptop must now point to punder */
ptop -> prventr = punder;/* could be zero if first entry */

/* the next pointer in ptop must now point to pbottom */
ptop -> nxtentr = pbottom;

/* the next pointer in pbottom must now point to pabove */
pbottom -> nxtentr = pabove;

/* mark last one in frametabtab */
if(! pabove)
	{
	frametab[1] = pbottom;
	}
else
	{
	/* the prev pointer in pabove must now point to pbottom */
	pabove -> prventr = pbottom;
	}

/* the prev pointer in pbottom must now point to ptop */
pbottom -> prventr = ptop;

/* return swapped */
return 1;
}/* end function swap_frame_position */


int set_text_hold_flag(int frame, int state)
{
struct frame *pa;
char temp[80];

sprintf(temp, "%d", frame);
pa = lookup_frame(temp);
if(! pa) return 0;

if(state == 1)
	{
	pa -> status |= TXT_HOLD;
	}
else
	{
	pa -> status &= 65535 - TXT_HOLD;
	}

return 1;
} /* end function set_text_hold_flag */


int get_text_hold_flag(int frame, int *state)
{
struct frame *pa;
char temp[80];

sprintf(temp, "%d", frame);
pa = lookup_frame(temp);
if(! pa) return 0;

*state = pa -> status & TXT_HOLD;

return 1;
} /* end function get_text_hold_flag */


int delete_all_empty_lines()
{
struct frame *pa, *pdel, *pnext, *pprev;

if(debug_flag)
	{
	fprintf(stdout, "delete_all_empty_lines() arg none\n");
	}

pa = frametab[0];
while(1)
	{
	if(! pa) break;
	if(strlen(pa -> text) == 0)
		{
		/* we now know which struture to delete */
		pdel = pa;

		/* get previous and next structure */
		pnext = pa -> nxtentr;
		pprev = pa -> prventr;

		/* set pointers for previous structure */
		/* if first one, modify frametab[0] */
		if(pprev == 0) frametab[0] = pnext;
		else pprev -> nxtentr = pnext;

		/* set pointers for next structure */
		/* if last one, modify frametab[1] */
		if(pnext == 0) frametab[1] = pprev;
		else pnext -> prventr = pprev;
	
		/* delete structure */	
		free(pdel -> text);
		free(pdel -> name);
		free(pdel); /* free structure */

		pa = pnext;
		} /* end if strlen text zero */
	else
		{
		pa = pa -> nxtentr;
		}
	}/* end for all frames */

return 1;
} /* end function delete_all_empty_lines */


int snap_to_frame(int *frame)
{
int dist_prev, dist_next;
int prev_frame, next_frame;
struct frame *pa, *pp;

if(debug_flag)
	{
	fprintf(stdout, "snap_to_frame(): arg *frame=%d\n", *frame);
	}

/* find the closest frame to the current position */
for(pa = frametab[0]; pa != 0; pa = pa -> nxtentr)
	{
	next_frame = atoi(pa -> name);		

	/* test if past current frame */
	if(next_frame > *frame)
		{
		/* get pointer to previous structure */
		pp = pa -> prventr;
		if(! pp)
			{
			/* return the next point */
			*frame = next_frame;

			return 1;	
			}

		/* get previous point */
		prev_frame = atoi(pp -> name);

		/* get distance to next point */
		dist_next = next_frame - *frame;
				
		/* get distance to previous one */
		dist_prev = *frame - prev_frame;

		/* select closest point */ 
		if(dist_next < dist_prev) *frame = next_frame;
		else if(dist_next == dist_prev) *frame = prev_frame; 
		else *frame = prev_frame;

		if(debug_flag)
			{
			fprintf(stdout,\
			"snap_to_frame(): returning *frame=%d\n",\
			*frame);
			}

		/* return the closest one */
		return 1;
		}/* end if condition */
	}/* end for all entries */

/* return not found (and keep the point) */
return 0;
} /* end function snap_to_frame */


int recalculate_frame_list()
{
struct frame *pa;
struct temp_frame *pt;
char temp[80];
int start_frame, end_frame;

if(debug_flag)
	{
	printf("recalculate_frame_list(): arg none\n");
	}

/* clear temp list */
delete_all_temp_frames();

/* frame_list to temp list with corrections applied */
for(pa = frametab[0]; pa != 0; pa = pa -> nxtentr)
	{
	/* get start and end frame */
	start_frame = atoi(pa -> name);
	end_frame = atoi(pa -> name);

	/* make pieces same length */
	start_frame *= dnew_old_frame_ratio;
	end_frame *= dnew_old_frame_ratio;

	/* correct shift */
	start_frame += frame_shift;
	end_frame += frame_shift;

	/* corrected values to temp list */
	sprintf(temp, "%d", start_frame);
	pt = install_temp_frame_at_end_of_list(temp);		
	if(!pt)
		{
		printf("recalculate_frame_list(): could not install in temp list\n");

		return 0;
		}

	pt -> text = strsave(pa -> text);
	if(! pt -> text)
		{
		printf(\
		"recalculate_frame_list(): could not install text in temp list\n");
    
		return 0;
		}

	pt -> end_frame = end_frame;
	pt -> status = pa -> status;
	pt -> browser_line = pa -> browser_line;
	pt -> last_updated_time = pa -> last_updated_time;

	pt -> pattern = pa -> pattern;
	pt -> background = pa -> background;
	pt -> emphasis1 = pa -> emphasis1;
	pt -> emphasis2 = pa -> emphasis2;

	pt -> pattern_contrast = pa ->  pattern_contrast;
	pt -> background_contrast = pa -> background_contrast;
	pt -> emphasis1_contrast = pa -> emphasis1_contrast;
	pt -> emphasis2_contrast = pa -> emphasis2_contrast;

	pt -> fontname = strsave(pa -> fontname);
	if(! pt -> fontname)
		{
		printf(\
		"recalculate_frame_list(): could not install fontname in temp list\n");		

		return 0;	
		}

	pt -> symbols = pa -> symbols;
	pt -> fontsize = pa -> fontsize;
	pt -> iso_extension = pa -> iso_extension;
	pt -> outline_thickness = pa -> outline_thickness;
	pt -> blur_radius = pa -> blur_radius;
	pt -> pfd = pa -> pfd;

	pt -> espace = pa -> espace;
	pt -> vespace = pa -> vespace;

	} /* end for all entries in temp list */

/* clear frame list */
delete_all_frames();

/* temp list to real list */
for(pt = temp_frametab[0]; pt != 0; pt = pt -> nxtentr)
	{
	pa = install_frame_at_end_of_list(pt -> name);		
	if(! pa)
		{
		printf(\
		"recalculate_frame_list(): could not install in frame list\n");
    
		return 0;
		}

	pa -> text = strsave(pt -> text);
	if(! pa -> text)
		{
		printf(\
		"recalculate_frame_list(): could not install text in frame list\n");

		return 0;
		}

	pa -> end_frame = pt -> end_frame;
	pa -> status = pt -> status;
	pa -> browser_line = pt -> browser_line;
	pa -> last_updated_time = pt -> last_updated_time;

	pa -> pattern = pt -> pattern;
	pa -> background = pt -> background;
	pa -> emphasis1 = pt -> emphasis1;
	pa -> emphasis2 = pt -> emphasis2;

	pa -> pattern_contrast = pt ->  pattern_contrast;
	pa -> background_contrast = pt -> background_contrast;
	pa -> emphasis1_contrast = pt -> emphasis1_contrast;
	pa -> emphasis2_contrast = pt -> emphasis2_contrast;

	pa -> fontname = strsave(pt -> fontname);
	if(! pa -> fontname)
		{
		printf(\
		"recalculate_frame_list(): could not install text in frame list\n");

		return 0;
		}

	pa -> symbols = pt -> symbols;
	pa -> fontsize = pt -> fontsize;
	pa -> iso_extension = pt -> iso_extension;
	pa -> outline_thickness = pt -> outline_thickness;
	pa -> blur_radius = pt -> blur_radius;
	pa -> pfd = pt -> pfd;
	
	pa -> espace = pt -> espace;
	pa -> vespace = pt -> vespace;

	} /* end for all entries in temp list */

/* clear temp list */
delete_all_temp_frames();

return 1;
} /* end function recalculate_frame_list */


int get_sample(int framenr, unsigned long *sample)
{
double dtime;
double da;

if(debug_flag)
	{
	fprintf(stdout,\
	"get_sample(): arg framenr=%d\n",\
	framenr);
	}

/* calculate sample from frame and audio sample speed. */
/* 
calculate from framenr using audio sample speed,
for 25.0 frames / second, 40 mS per frame
*/
da = dmilli_seconds_per_frame / 1000.0;
dtime = da * (double)framenr;

*sample = dtime * audio_sample_speed;

return 1;
}/* end function get_sample */


int get_next_frame_and_sample(unsigned long sample, int *next_frame, unsigned long *next_sample)
{
double samples_per_frame;
double da;

/* calculate frame from sound sample */

/* calculate frame per samples (40 mS per frame) */
da = dmilli_seconds_per_frame / 1000.0;
samples_per_frame = audio_sample_speed * da;

/* calculate how many frames in this many samples */
*next_frame = sample / samples_per_frame;

/* the next sample is the current sample here */
*next_sample = sample;

return 1;
}/* end function get_next_frame_and_sample */


int set_end_frame(int frame_nr, int end_frame)
{
struct frame *pa;
char temp[80];

if(debug_flag)
	{
	printf("set_end_frame(): frame_nr=%d end_frame=%d\n",\
	frame_nr, end_frame);
	}

sprintf(temp, "%d", frame_nr);
for(pa = frametab[0]; pa != 0; pa = pa -> nxtentr)
	{
	if(atoi(pa -> name) == frame_nr)
		{
		pa -> end_frame = end_frame;
			
		return 1;
		}

	} /* end for all entries */

/* not found */
return 0;
} /* end function set_end_frame */


int move_from_frame_list(int old_frame, int new_frame)
{
struct frame *pa;
struct temp_frame *pt;
char temp[80];
int start_frame, end_frame;
int shift;

if(debug_flag)
	{
	printf("move_from_frame_list(): arg old_frame=%d new_frame=%d\n", old_frame, new_frame);
	}

/* calculate shift */
shift = new_frame - old_frame;

/* clear temp list */
delete_all_temp_frames();

/* frame_list to temp list with corrections applied */
for(pa = frametab[0]; pa != 0; pa = pa -> nxtentr)
	{
	/* get start and end frame */
	start_frame = atoi(pa -> name);
	end_frame = pa -> end_frame;

	/* add shift if on or past selected line */
	if(start_frame >= old_frame)
		{
		start_frame += shift;
		end_frame += shift;
		}

	/* corrected values to temp list */
	sprintf(temp, "%d", start_frame);
	pt = install_temp_frame_at_end_of_list(temp);		
	if( !pt)
		{
		printf(\
		"move_from_frame_list(): could not install in temp list\n");

		return 0;
		}

	pt -> text = strsave(pa -> text);
	if(! pt -> text)
		{
		printf(\
		"move_from_frame_list(): could not install text in temp list\n");		

		return 0;	
		}

	pt -> end_frame = end_frame;
	pt -> status = pa -> status;
	pt -> browser_line = pa -> browser_line;
	pt -> last_updated_time = pa -> last_updated_time;

	pt -> pattern = pa -> pattern;
	pt -> background = pa -> background;
	pt -> emphasis1 = pa -> emphasis1;
	pt -> emphasis2 = pa -> emphasis2;

	pt -> pattern_contrast = pa ->  pattern_contrast;
	pt -> background_contrast = pa -> background_contrast;
	pt -> emphasis1_contrast = pa -> emphasis1_contrast;
	pt -> emphasis2_contrast = pa -> emphasis2_contrast;

	pt -> fontname = strsave(pa -> fontname);
	if(! pt -> fontname)
		{
		printf(\
		"move_from_frame_list(): could not install fontname in temp list\n");		

		return 0;	
		}

	pt -> symbols = pa -> symbols;
	pt -> fontsize = pa -> fontsize;
	pt -> iso_extension = pa -> iso_extension;
	pt -> outline_thickness = pa -> outline_thickness;
	pt -> blur_radius = pa -> blur_radius;
	pt -> pfd = pa -> pfd;

	pt -> espace = pa -> espace;
	pt -> vespace = pa -> vespace;

	} /* end for all entries in temp list */

/* clear frame list */
delete_all_frames();

/* temp list to real list */
for(pt = temp_frametab[0]; pt != 0; pt = pt -> nxtentr)
	{
	pa = install_frame_at_end_of_list(pt -> name);		
	if(!pa)
		{
		printf(\
		"move_from_frame_list(): could not install in frame list\n");

		return 0;
		}

	pa -> text = strsave(pt -> text);
	if(!pa -> text)
		{
		printf(\
		"move_from_frame_list(): could not install text in frame list\n");

		return 0;
		}

	pa -> end_frame = pt -> end_frame;
	pa -> status = pt -> status;
	pa -> browser_line = pt -> browser_line;
	pa -> last_updated_time = pt -> last_updated_time;

	pa -> pattern = pt -> pattern;
	pa -> background = pt -> background;
	pa -> emphasis1 = pt -> emphasis1;
	pa -> emphasis2 = pt -> emphasis2;

	pa -> pattern_contrast = pt ->  pattern_contrast;
	pa -> background_contrast = pt -> background_contrast;
	pa -> emphasis1_contrast = pt -> emphasis1_contrast;
	pa -> emphasis2_contrast = pt -> emphasis2_contrast;

	pa -> fontname = strsave(pt -> fontname);
	if(!pa -> fontname)
		{
		printf(\
		"move_from_frame_list(): could not install fontname in frame list\n");

		return 0;
		}

	pa -> symbols = pt -> symbols;
	pa -> fontsize = pt -> fontsize;
	pa -> iso_extension = pt -> iso_extension;
	pa -> outline_thickness = pt -> outline_thickness;
	pa -> blur_radius = pt -> blur_radius;
	pa -> pfd = pt -> pfd;

	pa -> espace = pt -> espace;
	pa -> vespace = pt -> vespace;

	} /* end for all entries in temp list */

/* clear temp list */
delete_all_temp_frames();

return 1;
} /* end function move_from_list */


struct frame *lookup_this_or_earlier_frame(int frame)
{
struct frame *pa;

if(debug_flag)
	{
	fprintf(stdout, "lookup_this_or_earlier_frame(): arg frame=%d\n", frame);
	}

/* from end of list to beginning */
for(pa = frametab[1]; pa != 0; pa = pa -> prventr)
	{
	/* skip any comment lines */
	if(pa -> text[0] == '*') continue;

	if(atoi(pa -> name) <= frame) return pa;
	}

return 0;
} /* end function lookup_this_or_earlier_frame */


struct frame *lookup_this_or_earlier_frame_that_is_no_animation_template_reference(int frame)
{
struct frame *pa;
struct animation_template *pt;

char command_part[TEXT_ARRAY_SIZE];
char text_part[TEXT_ARRAY_SIZE];

if(debug_flag)
	{
	fprintf(stdout, "lookup_this_or_earlier_frame_that_is_no_animation_template_reference(): arg frame=%d\n", frame);
	}

/* from end of list to beginning */
for(pa = frametab[1]; pa != 0; pa = pa -> prventr)
	{
	/* skip any comment lines */
	if(pa -> text[0] == '*') continue;

	if(atoi(pa -> name) <= frame)
		{
       /* test if this is a template */
        /* split of any command in the form of '[xxx x x] */
        if(! extract_command(pa -> text, text_part, command_part) ) return 0;

		pt = lookup_animation_template(command_part);
		if(! pt) return pa;
		}
	}

return 0;
} /* end function lookup_this_or_earlier_frame_that_is_no_animation_template_reference */


