/*
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "xste.h"

FD_xste *fdui;


int show_global_subtitle_palette_form()
{
if(debug_flag)
	{
	printf("show_global_subtitle_palette_form(): arg none\n");
	}

if(global_subtitle_palette_form_shown_flag) return 1;

fl_show_form(fdui -> global_subtitle_palette_form, FL_PLACE_CENTER, FL_TRANSIENT,\
	"xste: global subtitle palette");

global_subtitle_palette_form_shown_flag = 1;

return 1;
} /* end function show_fonts_form */


int global_subtitle_palette_form_set_all_color_buttons()
{
int i;

if(debug_flag)
	{
	fprintf(stdout, "global_subtitle_palette_form_set_all_color_buttons(): arg none\n");
	}

for(i = 0; i < 16; i++)
	{
	fl_mapcolor(FL_FREE_COL1 + i, xste_rgb_palette[i][0], xste_rgb_palette[i][1], xste_rgb_palette[i][2]);

	global_subtitle_palette_form_set_color_button(i);

	} /* end for al buttons */

return 1;
} /* end function global_subtitle_palette_form_set_all_color_buttons */


int global_subtitle_palette_form_set_color_button(int button)
{
FL_OBJECT *obj = NULL;

if(debug_flag)
	{
	fprintf(stdout, "global_subtitle_palette_form_set_color_button(): arg button=%d\n", button);
	}

switch(button)
	{	
	case 0:
		obj = fdui -> global_subtitle_palette_form_color0_button;
		break;
	case 1:
		obj = fdui -> global_subtitle_palette_form_color1_button;
		break;
	case 2:
		obj = fdui -> global_subtitle_palette_form_color2_button;
		break;
	case 3:
		obj = fdui -> global_subtitle_palette_form_color3_button;
		break;
	case 4:
		obj = fdui -> global_subtitle_palette_form_color4_button;
		break;
	case 5:
		obj = fdui -> global_subtitle_palette_form_color5_button;
		break;
	case 6:
		obj = fdui -> global_subtitle_palette_form_color6_button;
		break;
	case 7:
		obj = fdui -> global_subtitle_palette_form_color7_button;
		break;
	case 8:
		obj = fdui -> global_subtitle_palette_form_color8_button;
		break;
	case 9:
		obj = fdui -> global_subtitle_palette_form_color9_button;
		break;
	case 10:
		obj = fdui -> global_subtitle_palette_form_color10_button;
		break;
	case 11:
		obj = fdui -> global_subtitle_palette_form_color11_button;
		break;
	case 12:
		obj = fdui -> global_subtitle_palette_form_color12_button;
		break;
	case 13:
		obj = fdui -> global_subtitle_palette_form_color13_button;
		break;
	case 14:
		obj = fdui -> global_subtitle_palette_form_color14_button;
		break;
	case 15:
		obj = fdui -> global_subtitle_palette_form_color15_button;
		break;
	} /* end switch button */

if(! obj)
	{
	fprintf(stderr, "global_subtitle_palette_form_set_color_button(): unknown button\n");

	return 0;
	}

fl_set_object_color(obj, FL_FREE_COL1 + button, FL_FREE_COL1 + button);

fl_redraw_object(obj);

return 1;
} /* end function global_subtitle_palette_form_set_color_button */


int set_color_circle(int r, int g, int b)
{
char temp[80];
double radians, degrees;
int y, u, v;
double dvector, dsine, du, dv;

/* rgb to yuv */
y = (0.257 * (double)r) + (0.504 * (double)g) +  (0.098 * (double)b) + 16.0;

u = (0.439 * (double)r) - (0.368 * (double)g) - (0.071 * (double)b);

v = -(0.148 * (double)r) - (0.291 * (double)g) + (0.439 * (double)b);

/*
The U and V signals were intended for quadrature modulation on the 4.43
PAL carrier, lets do it:
       color vector, angle alpha sets color, amplitude sets saturation
     u /
     |/ alpha
-v -- -- v
     |
    -u
*/

/* no color no action, prevent nan */
if( (u == 0) && (v == 0) ) return 0;

/* u v to angle */

/* calculate the vector amplitude (hypotenusa) */
du = (double)u;
dv = (double)v;
//dvector = sqrt( (du * du) + ( dv * dv ) );
dvector = hypot(du, dv);

/* calculate the sine */
dsine = (du / dvector);

/* dsine must be in the range -1 to +1, else errno. */

/* get the vector angle */
errno = 0;
radians = asin(dsine);
if(errno == EDOM)
	{
	perror("set_color_circle(): rotate_color(): asin NOT A NUMBER :-)");

	/* abort */
	exit(1);
	}

/* -PI/2 to +PI/2 radians = -90 to + 90 degrees. */
degrees = radians * (180.0 / M_PI);
/* degrees now -90 to + 90 */

/*
dvector is always positive, now we can say:
if we are in the top right quadrant then u and v are positive, so dsine is positive.
in the bottom right quadrant then u is negative and v is positive, so dsine is negative
in the bottom left quadrant then u is negative and v is negative, so dsine is negative
in the top right quadrant then u is positive and v is negative. so dsien is positive.
dsine does not know about when v is negative.
If v is negative we have to add 180 degrees to teh result off asin..
*/

degrees += 90;
/* now 0-180 */

if(v < 0) degrees = 360 - degrees;

fl_set_dial_value(fdui -> global_subtitle_palette_form_color_circle_dial, degrees);

sprintf(temp, "%02f", degrees);
fl_set_input(fdui -> global_subtitle_palette_form_color_circle_input, temp);

return 1;
} /* end function set_color_circle */


