/*
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/



/*
Wrote this to be able to create output files for micro DVD player
(windows!!) from www.tiasoft.de
JP
*/

/*
Subtitles

Micro DVD Player can display subtitles. It reads them from a text file that
has a very simple structure.
For each subtitle put one line in the file that looks the following way:

{First Frame}{Last Frame}Text

The subtitle can consist of five lines max., use the character | (pipe) as
line separator.
Last Frame can be empty if the subtitle is to be displayed until the next
one is due.

The characters {, } and | can be redefined in the movie's INI file

The subtitles are - depending on the settings the user makes in the
configuration dialog - displayed in the space between the top or bottom
border of the video and the edge of the screen.
If the available space is not sufficient, or the user forces it, the
subtitles are displayed transparently over the movie.

A real-life example:

{6027}{6101}Can I have five tickets to Terrance|and Phillip's Asses of Fire, please?
{6125}{}No.
{6175}{}What do you mean, "no"?
{6211}{}Asses of Fire|has been rated "R"...
{6263}{}by the Motion Picture|Association of America.
{6314}{}You have to be accompanied|by a parent or guardian.
{6361}{}- But why?|- Because this movie|has naughty language!
{6420}{}-Next, please.|- [ Cash Register Rings ]

MDVDP supports control codes in the text part of subtitles.
Control codes may appear anywhere in the subtitle line.
Most of the codes are available in two forms: with a lower-case letter
(then the code only applies to a single line of the subtitle) or with an
upper-case letter (then all lines are affected, no matter in which line
the codes occurs).

Currently the following control codes are available:

{c:$bbggrr} 
 You use this code to make each line of a subtitle have a different color.
 This is useful e.g. for distinguishing lines of different people talking
 in a dialog.
 bb, gg and rr are 2-digit hexadecimal numbers for the blue, green and red
 component of the color.
 00 means the component is completely off, ff means it is at full intensity.
 Watch the inverted standard order! First blue, then green, then red.
 If there is no color control code in a subtitle line, then the user's
 default color (chosen in the configuration dialog) will be used.
 Note: You can also make the player automatically choose another color when
 a new person speaks in a subtitle. 

An example: {6361}{}{c:$0000FF}- But why?|{c:$00FFFF}- Because this movie|has naughty language!  Here the first line of the two lines in the subtitle would be displayed in
 full-intensity red, and the second in bright yellow.
 The same method for placing control codes can be applied to the other codes
 too! 
{C:$bbggrr}
 Same as lower-case c, but for a whole subtitle. 
{f:fontname}
 You set the font face for a single subtitle line with
 this code. Replace the fontname with a string determining the font
 name you wish to use.
{F:fontname}
 Same as lower-case f, but for a whole subtitle.
{s:size}
 This is used to set the font size for the whole subtitle.
 Replace the size with an integer number.
{S:size}
 Same as lower-case s, but for a whole subtitle.
{H:charset}
 With this you can set the charset for the font to use.
 You need this to e.g. tell the player that the chosen font has a greek,
 russian, hebrew etc. charset.
 Attention: This control code is only valid in the {DEFAULT} line
 (see below) !
{y:style}
 You use this to set the style (bold, italic, underline) for an individual
 line of a subtitle.
 Replace the style with a string containing one or more of the letters
 b, i and u, e.g. bi to make the line bold+italic, or i,u to make it
 italic-underline.
 You can use any separator you wish, all chars besides b, i and u are
 ignored.
{Y:style}
 Same as lower-case y, but for a whole subtitle.
{P:position}
 You can set the position of a subtitle using this code.
 Replace the position with either a 0 (above movie) or a 1 (below movie).
 There's logically only an upper-case version of this code.
{o:x-coordinate,y-coordinate}
 Using this code, you can place single lines of your subtitle anywhere
 freely on the movie.
 This is useful e.g. if you want to put a translation line under a foreign
 language sign that appears in the movie.
 Since the o code only affects one line, you can do this even while people
 talk, and other lines appear at their usual place, subtitling the dialog.
 Replace the x- and y-coordinate with integer numbers.
 The coordinates are relative to the upper left corner of the movie, and
 must refer to the original resolution, e.g. 720x480, not to fullscreen-
 stretched coordinates.

Important: You can set default values for a subtitle file (overriding the
settings made by the user in the configuration dialog) by adding the
following line somewhere to your subtitle file:

{DEFAULT}{}<Control Codes>

Replace the <Control Codes> with any number of control codes that are valid
in individual subtitle entries.
The control codes have to be the upper-case versions here!
E.g. to make the subtitle file by default display in Arial, 18pt,
bold+italic and below the movie, use this line:

{DEFAULT}{}{F:Arial}{S:18}{Y:b,i}{P:1}

The control code H is only valid in this DEFAULT line!

The user can override all those default and sub/line-individual settings by
activating the option "Ignore subtitle styles" in the configuration screen.

In case you have a subtitle file that shows somehow displaced subtitles,
meaning either all subs appear too early or late by the same amount of time,
or the displacement gets worse the further you look into the movie, check
this section, parameters Offset and Multiplier for a method of correcting
this.

Besides its own format, MDVDP also supports other subtitle formats.
See here for details. If you want further subtitle file formats besides the
currently known ones to be supported, contact me.
*/


#include "xste.h"

FD_xste *fd_main_form;
extern FD_xste *fdui;


/*
You can use this INI file structure if you only quickly want to display a
subtitle file together with a video file and do not need/want further stuff
like Nav Menu, Chapters, Multilanguage etc.
The following text is meant for Copy+Paste to your INI file!


[Micro DVD Ini File]

; You can remove all comments (text after a semicolon) if you want!

; Put your video and subtitle file, plus this INI file, in the same directory
; and run the player with the INI file as parameter (MDVDP.EXE TEST.INI e.g.)

; Or run the player, go to MDVD mode, go to Config screen, choose "Run Movie
; from Directory" and choose your INI file.

[MAIN]
Title=The Matrix      ; Your video title goes here

[MOVIE]
Directory=.           ; Means the video file is looked for in the same directory
                      ; as the INI file
AVIName=Filename.Ext  ; Put your video filename here

[SUBTITLES]
Directory=.           ; Means the subtitle file is looked for in the same directory
                      ; as the INI file
1=ENG English         ; Put short and long form of language here
File=Filename.Ext     ; Put your Subtitle file name here
Format=1              ; Put 0 here for MDVDP format, 1 for SubViewer format
Lines=3               ; Put maximum number of lines occuring in subtitles here
*/

int write_microdvdplayer_list()
{
/*
IMPORTANT
If you use formatted mode, a font similar to the one used must be referred
to, so p_reformat_text() and p_center_text() have a valid font descriptor,
from wich character properties are used. 
(xste.h all the way at the end hold a define DEFAULT_FONT_PATH,
this should really become an input field.
Anyways, {o:x,y}hello does NOT seem to work in mdvdplayer 1.2,
the position does NOT go to x,y, so let it format!
*/

struct frame *pa;
struct frame *pnext, *ptemp;
FILE *fptr;
char temp[4096];
int next_frame_start;
int text_end_frame;

if(debug_flag)
	{
	printf("write_microdvdplayer_list(): arg none\n");
	}

draw_mode = DRAW_MODE_BW;

/*
MDVD.INI is required to point to this filename.
*/
sprintf(temp, "%s/%s/%s/%s-mdvd.sub",\
home_dir, selected_data_directory, selected_project, selected_project);
fptr = fopen(temp, "w");
if(! fptr)
	{
	fl_show_alert(\
		"write_microdvdplayer_list", "Could not open for write", temp, 1);

	return 0;
	}

/* set defaults */
//{DEFAULT}{}{F:Arial}{S:%d}{Y:b,i}{P:1}

/* use the fontsize of the fist sub */
fprintf(fptr, "{DEFAULT}{}{F:Arial}{S:%d}{P:1}\r\n", frametab[0] -> fontsize);

for(pa = frametab[0]; pa != 0; pa = pa -> nxtentr)
	{
	/* get pointer to next structure entry */ 
	if(pa -> nxtentr) 
		{
		pnext = pa -> nxtentr;	
		
		/* find next line that is not a '*' comand line */
		while(pnext)
			{
			if(pnext -> text[0] != '*') break;

			if(pnext -> nxtentr)
				{	
				ptemp = pnext -> nxtentr;

				pnext = ptemp;
				}	
			} /* end while looking for non '*' entry */ 

		if(pnext)
			{
			/* get the start of the next frame */ 
			next_frame_start = atoi(pnext -> name);
		
			/* test if the current frame ends before the start of the next */
			if(pa -> end_frame > next_frame_start)
				{
				text_end_frame = next_frame_start;
				} /* end if frame ends before next one starts */ 
			}

		} /* end if not last entry */	

	if(pa -> text[0] == '*') continue;

	write_formatted_entry_microdvdplayer_format(pa, fptr);
	} /* end for all frames */

/* close the file */
fclose(fptr);

return 1;
}/* end function write_microdvdplayer_list */


int write_formatted_entry_microdvdplayer_format(struct frame *pa, FILE *fptr)
{
int i;
char *tptr, *cptr;
int x, y;

if(debug_flag)
	{
	printf("write_formatted_entry_microdvdplayer_format():\n\
	arg pa=%p fptr=%p\n",\
	pa, fptr);
	}

/* argument check */
if(! pa) return 0;
if(! fptr) return 0;

line_h_start = h_factor * (double)h_size;
line_h_end = (double)h_size - (double)line_h_start;

window_bottom = v_size - (v_factor * (double)v_size);

/*
Set hor_position for start text in all lines to zero,
center_text() may overrule this if center_flag.
*/
for(i = 0; i < MAX_SCREEN_LINES; i++)
	{
	screen_text[i][0] = 0;
	screen_start[i] = 0; // pixels from left used to center text
	}

/* reformat text inserting (multiple) '/n' if too long */ 
tptr = p_reformat_text(pa, line_h_end - line_h_start);
if(! tptr)
	{
	printf("write_formatted_entry_microdvdplayer_format: could not reformat text=%s\n", pa -> text);
	
	/* return error */
	return 0;
	} /* end if reformat text failed */		

if(! let_microdvdplayer_format_flag)
	{
	/* center text */
	if(pa -> status & CENTER_TEXT) p_center_text(tptr, pa);
	}

/* text to array screen_text[] */
cptr = tptr;
screen_lines = 1; /* at least one */
while(1) /* all chars in tptr */
	{
	i = 0;
	while(1) /* all chars in a line */
		{
		if(*cptr == '\n')
			{
			/* scip the '\n' */
			cptr++;
				
			/* force string termination */
			screen_text[screen_lines - 1][i] = 0;

			/* point to next screen line */
			screen_lines++;
			break;
			}

		/* copy character */
		screen_text[screen_lines - 1][i] = *cptr;	

		/* test for end of string tptr */
		if(*cptr == 0) break;

		/* point to next char in screen_lines[][] */
		i++;

		/* point to next char in tptr */
		cptr++;
		} /* end while all characters in line terminated with LF */

	/* ready if end of tptr */
	if(*cptr == 0) break;
				
	} /* end while all lines in tptr */
free(tptr);

/* some limit */
if(screen_lines > MAX_SCREEN_LINES) screen_lines = MAX_SCREEN_LINES;

line_height = pa -> pfd -> height;
window_top = window_bottom - (screen_lines * line_height);

if(debug_flag)
	{
	printf("screen_lines=%d\n", screen_lines);
	printf("line_h_start=%d line_h_end=%d\n",\
	line_h_start, line_h_end);
	printf("window_bottom=%d window_top=%d\n",\
	window_bottom, window_top);
	}

if(let_microdvdplayer_format_flag)
	{
	/* write start end end frame */
	fprintf(fptr, "{%d}{%d}", atoi(pa -> name), pa -> end_frame);
	}

/* print lines of text on screen in right position */
for(i = 0; i < screen_lines; i++)
	{
	/* convert any special characters */
/*
	j = 0;
	while(1)
		{
		c = screen_text[i][j];
		character_lookup(c, &b);
		screen_text[i][j] = b;
		if(c == 0) break;
		j++;
		}
*/
	x = screen_start[i];
	y = window_top + (i * line_height);

	if(debug_flag)
		{
		printf(\
		"screen_start[%d]=%d window_bottom=%d window_top=%d\n\
line_height=%d x=%d y=%d\n\
text=%s\n",\
		i, screen_start[i], window_bottom, window_top,\
		line_height, x, y,\
		screen_text[i]);
		}

	//{o:x-coordinate,y-coordinate}

	/*
	NOTE:
	mdvdplayer (1.2) does NOT seem to reformat long lines!
	We have to present it as text with '|' in it as a line break.
	mdvdplayer will do the centering! 
	*/

	if(let_microdvdplayer_format_flag)
		{
		if(i == 0)
			{
			fprintf(fptr, "%s", screen_text[i]);	
			}
		else
			{
			fprintf(fptr, "|%s", screen_text[i]);
			}
		} /* end if let_microdvdplayer_format_flag */
	else
		{
		fprintf(fptr, "{%d}{%d}{o:%d,%d}%s\r\n",\
		atoi(pa -> name), pa -> end_frame,\
		x, y,\
		screen_text[i]);
		} /* end if formatted text with x,y position */

	} /* end for all screen_lines */

if(let_microdvdplayer_format_flag) fprintf(fptr, "\r\n");

return 1;
} /* end function write_formatted_entry_microdvdplayer_format */


