/* 
part of xste subtitle editor
xste  is registered Copyright (C) 2001, 2002 Jan Panteltje
email: panteltje@yahoo.com

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "xste.h"

int line_number;

/* This should read "Nandub v1.0rc2 format */

int load_nandub_file(char *pathfilename)
{
int a, i;
char temp[READSIZE];
char temp2[READSIZE];
int start_frame, end_frame;
char text[READSIZE];
int text_hold_flag;
FILE *fptr;
int start_hour, start_minute, start_second, start_milli_second;
int end_hour, end_minute, end_second, end_milli_second;
double dstart_milli_seconds, dend_milli_seconds;
int line;
int nandub_record;
int dummy;

if(debug_flag)
	{
	fprintf(stdout, "load_nandub_file() arg pathfilename=%s\n", pathfilename);
	}
	
if(! pathfilename) return 0;

fptr = fopen(pathfilename, "r");
if(! fptr)
	{
	fl_show_alert("could not open file", pathfilename, "for read", 1);

	if(debug_flag)
		{
		printf("load_nandub_file(): could not load file %s\n", pathfilename);
		}
	return 0;
	} 

delete_all_frames();

/* for all lines in file */
line = 0;
nandub_record = -1;
while(1)
	{
	a = readline_msdos(fptr, temp);
	if(a == EOF) break;

	/* get rid of '\r' after linefeed */
	a = readline_msdos(fptr, temp2);
	if(a == EOF) break;

	line++;

	/* 1 */
	a = sscanf(temp, "%d:%d", &nandub_record, &dummy);
	if(a != 1)
		{
		fprintf(stderr,\
		"load_nandub_file(): line %d last nandub_record=%d format error returning error.\n",\
		line, nandub_record);

		return 0;
		}

	if(debug_flag)
		{
		fprintf(stdout, "nandub_record=%d\n", nandub_record);
		}

	a = readline_msdos(fptr, temp);
	if(a == EOF) break;

	/* get rid of '\r' after linefeed */
	a = readline_msdos(fptr, temp2);
	if(a == EOF) break;

	line++;

	/* parse first line record for time */
	/* 00:00:10,000 --> 00:00:15,000 */
	a = sscanf(temp, "%d:%d:%d,%d --> %d:%d:%d,%d",\
	&start_hour, &start_minute, &start_second, &start_milli_second,\
	&end_hour, &end_minute, &end_second, &end_milli_second);
	if(a != 8)
		{
		fprintf(stderr, "load_nandub_file(): line %d a=%d format error, returning error.\n", line, a);

		return 0;
		}

	dstart_milli_seconds =\
	3600000 * start_hour +\
	60000 * start_minute +\
	1000 * start_second +\
	start_milli_second;

	dend_milli_seconds =\
	3600000 * end_hour +\
	60000 * end_minute +\
	1000 * end_second +\
	end_milli_second;

	if(debug_flag)
		{
		printf("\n");
		printf(\
		"start_hour=%d start_minute=%d start_second=%d start_milli_second=%d\n\
		dstart_milli_seconds=%.2f\n",\
		start_hour, start_minute, start_second, start_milli_second, dstart_milli_seconds);

		printf(\
		"end_hour=%d end_minute=%d end_second=%d end_milli_second=%d\n\
		dend_milli_seconds=%.2f\n",\
		end_hour, end_minute, end_second, end_milli_second, dend_milli_seconds);
		}

	/* sanity check */
	if(dend_milli_seconds <= dstart_milli_seconds)
		{
		/*
		try to fix,
		leave on screen for 4 seconds, or until next title appears
		*/

		dend_milli_seconds = dstart_milli_seconds + 4000;	
			
		fprintf(stderr,\
		"xste: load_nandub_file(): ALERT line %d end time <= start time, fixed to + 4 seconds.\n",\
		line);
		}

	strcpy(text, "");
	for(i = 0; i < MAX_SCREEN_LINES; i++)
		{
		/* read first text line */
		a = readline_msdos(fptr, temp);
		if(a == EOF) break;

		/* get rid of '\r' after linefeed */
		a = readline_msdos(fptr, temp2);	
		if(a == EOF) break;

		line++;

		if(temp[0] == 0) break;

		if(i > 0) strcat(text, "\\");

		strcat(text, temp);

		} /* end for i */

	if( (a == EOF) && (i == 0) ) return 1; /* complete, nothing more to add */

	/* add complete record */

	/* calculate parameters */
	/* 40 mS per frame, if 25.0 frames / second */

	/* start frame */
	start_frame = dstart_milli_seconds / dmilli_seconds_per_frame;

	/* end frame */
	end_frame = dend_milli_seconds / dmilli_seconds_per_frame;

	if(debug_flag)
		{
		fprintf(stdout, "start_frame=%d end_frame=%d\n", start_frame, end_frame);
		
		fprintf(stdout, "text=%s\n", text); 
		}

	/* text_hold_flag */
	text_hold_flag = 0;

	/* add entry to structure */
	if(! add_frame(text, start_frame, end_frame, text_hold_flag) )
		{
		fprintf(stderr, "xste: load_nandub_file(): line %d could not add_frame, aborting.\n", line);

		/* close input file */
		fclose(fptr);

		exit(1);
		}

	}/* end while all lines in file */

return 1;
} /* end function load_nandub_file */


