/* 
part of xste subtitle editor
xste  is registered Copyright (C) 2001, 2002 Jan Panteltje
email: panteltje@yahoo.com

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "xste.h"

int line_number;


int write_ppml_list()
{
int a, c, i;
struct frame *pa;
struct frame *pnext;
FILE *fptr;
char temp[4096];
char temp2[4096];
int next_frame_start;
int margin;
FILE *hfptr;
char old_fontname[4096];
int old_fontsize = -1000;
int old_iso_extension = -1000;
double old_outline_thickness = -1000.0;
double old_blur_radius = -1000.0;
int old_background = -1000;
int old_pattern = -1000;
int old_emphasis1 = -1000;
int old_emphasis2 = -1000;
int old_background_contrast = -1000;
int old_pattern_contrast = -1000;
int old_emphasis1_contrast = -1000;
int old_emphasis2_contrast = -1000;
int old_espace = -1000;
int old_anti_alias_flag = -1000;
int prev_frame;
int new_font, new_contrast, new_color;
int printed_espace_flag = 0; // -Wall
char command_part[TEXT_ARRAY_SIZE];
char text_part[TEXT_ARRAY_SIZE];

if(debug_flag)
	{
	printf("write_ppml_list(): arg none\n");
	}

draw_mode = DRAW_MODE_BW;

sprintf(temp, "%s/%s/%s/%s.ppml",\
home_dir, selected_data_directory, selected_project, selected_project);
fptr = fopen(temp, "w");
if(! fptr)
	{
	fl_show_alert("write_ppml_list", "Could not open for write", temp, 1);

	return 0;
	}

/* write ID */
fprintf(fptr,\
"; subtitle file for project %s in PPML format,\n\
; generated with Linux subtitle editor %s by Jan Panteltje\n",\
selected_project, VERSION);

fprintf(fptr, "; xste settings:\n");

margin = h_factor * h_size;

fprintf(fptr, ";\n");

fprintf(fptr, "; define a name for the main movie object\n");
fprintf(fptr, "*main main_movie\n\n");

fprintf(fptr, "; define name for the subtitles\n");
fprintf(fptr, "*subtitle subtitle\n\n");

fprintf(fptr, "; set the font dir\n");
fprintf(fptr, "1 *subtitle font_dir=%s/.xste/fonts/\n\n", home_dir);

fprintf(fptr, "; set this to 1 for an X window showing output\n");
fprintf(fptr, "2 *main show_output=%d\n\n", ppml_enable_monitor_flag);

fprintf(fptr, "; center text\n"); 
fprintf(fptr, "2 *subtitle center=1\n\n");

fprintf(fptr, "; fade in subtitle :-)\n");
fprintf(fptr, "2 *subtitle color=0 sat=100 dcolor=0\n");
fprintf(fptr, "2 *subtitle contr=100 dcontr=0 transp=0 dtransp=0\n\n");

fprintf(fptr, "; set margins\n");
fprintf(fptr, "3 *subtitle hfactor=%.3f vfactor=%.3f\n\n", h_factor, v_factor);

if(ppml_use_dvd_colors_flag) fprintf(fptr, "; subtitle palette\n");
else fprintf(fptr, "; subtitle palette, uncomment this line to enable DVD type subtitles\n");

if(ppml_use_dvd_colors_flag) fprintf(fptr, "3 *subtitle palette=");
else fprintf(fptr, "; 3 *subtitle palette=");
for(i = 0; i < 16; i++)
	{
	fprintf(fptr, "%d,%d,%d", xste_rgb_palette[i][0], xste_rgb_palette[i][1], xste_rgb_palette[i][2]); 
	if(i < 15) fprintf(fptr, ",");
	}
fprintf(fptr, "\n\n");

fprintf(fptr, "; fade in main movie\n");
fprintf(fptr, "4 *main contr=100 dcontr=0 sat=100 dsat=0\n\n");

fprintf(fptr, "\n");

strcpy(old_fontname, "");
old_fontsize = 0;
old_iso_extension = 0;
old_outline_thickness = 0.0;
old_blur_radius = 0.0;

for(pa = frametab[0]; pa != 0; pa = pa -> nxtentr)
	{
	new_font = 0;
	new_contrast = 0;
	new_color = 0;

	/*
	do not print negative frames, this results in negative times,
	not sure srt likes that.
	*/
	if( atoi(pa -> name) < 0) continue;

	/* skip any command lines starting with '*' */
	if(pa -> text[0] == '*') continue;

	/* split of any command in the form of '[xxx x x] */
	if(! extract_command(pa -> text, text_part, command_part) ) return 0;

	/* fast test for special commands */
	if(strchr(text_part, '#') )
		{
		/* test for include file command */
		a = sscanf(text_part, "#include %s", temp2);
		if(a == 1)
			{
			sprintf(temp, "%s/.subtitles/%s/%s",\
			home_dir, selected_project, temp2);
			/* read from include file, write to outfile */
			hfptr = fopen(temp, "r");
			if(! hfptr)
				{
				fl_show_alert(\
				"Could not open include file", temp, "for read", 0);
				}
			else
				{
				fprintf(fptr, "\n; start included file %s\n", temp);

				while(1)
					{
					errno = 0;
					while(1)/* error re read */
						{
						c = getc(hfptr);
						if(! ferror(hfptr) ) break;
						perror(\
						"write_ppml_list(): read from include file failed ");
						}/* end while error re read */	

					if(feof(hfptr) )
						{
						fclose(hfptr);

						break;
						}

					fputc(c, fptr);
					} /* end for al chars in header file */

				fprintf(fptr, "; end included file %s\n\n", temp);
				} /* end if header file open */

			continue;
			} /* end if include file */

		} /* end if special command */

	/* io a frame back, to setup things */
	prev_frame = atoi(pa -> name) - 1;
	if(prev_frame < 2) prev_frame = 2; /* frame 0 does not exists in transcode */
	
	/* test for font change */
	if( (strcmp(pa -> fontname, old_fontname) != 0) ||\
	(pa -> fontsize != old_fontsize) ||\
	(pa -> iso_extension != old_iso_extension) ||\
	(pa -> outline_thickness != old_outline_thickness) ||\
	(pa -> blur_radius != old_blur_radius) )
		{	
		new_font = 1;

		strcpy(old_fontname, pa -> fontname);	
		old_fontsize = pa -> fontsize;
		old_iso_extension = pa -> iso_extension;
		old_outline_thickness = pa -> outline_thickness;
		old_blur_radius = pa -> blur_radius;	
		}

	/* test for color change */
	if( (pa -> background != old_background) || (pa -> pattern != old_pattern) ||\
	(pa -> emphasis1 != old_emphasis1) || (pa -> emphasis2 != old_emphasis2) )
		{
		new_color = 1;

		old_background = pa -> background;
		old_pattern = pa -> pattern;
		old_emphasis1 = pa -> emphasis1;
		old_emphasis2 = pa -> emphasis2;
		}

	/* test for contrast change */
	if( (pa -> background_contrast != old_background_contrast) ||\
	(pa -> pattern_contrast != old_pattern_contrast) ||\
	(pa -> emphasis1_contrast != old_emphasis1_contrast) ||\
	(pa -> emphasis2_contrast != old_emphasis2_contrast) )
		{
		new_contrast = 1;
		
		old_background_contrast = pa -> background_contrast;
		old_pattern_contrast = pa -> pattern_contrast;
		old_emphasis1_contrast = pa -> emphasis1_contrast;
		old_emphasis2_contrast = pa -> emphasis2_contrast;
		}

	if(new_font || new_contrast || new_color)
		{
		/* make any current sub disappear, to avoid its properies changing in screen */

		fprintf(fptr, "; make previous sub disappear, to prevent changes becoming visible.\n");
		fprintf(fptr, "%d\n", prev_frame);
		}

	if(new_font)
		{
		/* if font change send new one */
		fprintf(fptr,\
"%d *subtitle font_name=%s font_size=%d font_iso_extension=%d font_outline_thickness=%.2f font_blur_radius=%.2f\n",\
		prev_frame,\
		pa -> fontname, pa -> fontsize, pa -> iso_extension, pa -> outline_thickness, pa -> blur_radius);
		}

	if(new_color)
		{
		/* if color change send new one */
		fprintf(fptr,\
		"%d *subtitle background_color=%d pattern_color=%d emphasis1_color=%d emphasis2_color=%d\n",\
		prev_frame,\
		pa -> background, pa -> pattern, pa -> emphasis1, pa -> emphasis2);
		}

	if(new_contrast)
		{
		/* if contrast change send new one */
		fprintf(fptr,
	"%d *subtitle background_contrast=%d pattern_contrast=%d emphasis1_contrast=%d emphasis2_contrast=%d\n",\
		prev_frame,\
		pa -> background_contrast, pa -> pattern_contrast,\
		pa -> emphasis1_contrast, pa -> emphasis2_contrast); 
		}

	if( (pa -> espace != old_espace) || (!printed_espace_flag) )
		{
		fprintf(fptr, "; To get subtitle characters closer together use a negative value.\n");

		fprintf(fptr,\
		"%d *subtitle espace=%d\n", prev_frame, pa -> espace);
		printed_espace_flag = 1;
		old_espace = pa -> espace;		
		}

	if(ppml_use_dvd_colors_flag)
		{
		if(pa -> status & DVD_ANTI_ALIAS) a = 1;
		else a = 0;
		if(old_anti_alias_flag != a)
			{ 
			fprintf(fptr, "%d *subtitle anti_alias=%d\n", prev_frame, a);			

			old_anti_alias_flag = a;
			}
		}

	/* send the actual text */
	fprintf(fptr, "%d %s\n", atoi(pa -> name), text_part);

	/* the title must disappear after the specified time */

	/* first get pointer to next structure entry */ 
	if(pa -> nxtentr) 
		{
		pnext = pa -> nxtentr;	
		
		/* get the start of the next frame */ 
		next_frame_start = atoi(pnext -> name);

		/* test if the current frame ends before the start of the next */
		if(pa -> end_frame < next_frame_start)
			{
			/*
			Insert an empty line with one space here to make the current
			text disappear (ppml processes this).
			*/
			/* 
			Only make text go away if it was not a text go away line !
			set by the user.
			*/
			if(strlen(text_part) != 0)
				{
				fprintf(fptr, "%d \n", pa -> end_frame);
				}
			} /* end if frame ends before next one starts */ 
		} /* end if not last entry */	

	else /* last entry */
		{
		/*
		the last entry text should also be removed at the specified time.
		*/
		/* 
		Only make text go away if it was not a text go away line !
		set by the user.
		*/
		if(strlen(text_part) != 0)
			{
			fprintf(fptr, "%d \n", pa -> end_frame);
			}
		}

	fprintf(fptr, "\n");
	} /* end for all frames */

/* close the file */
fclose(fptr);

return 1;
}/* end function write_ppml_list */


int load_ppml_file(char *pathfilename)
{
int a;
char temp[READSIZE];
char temp2[READSIZE];
char temp3[80];
int start_frame, end_frame, previous_start_frame;
unsigned long start_sample, end_sample, previous_start_sample;
char text[READSIZE];
int text_hold_flag;
FILE *fptr;
char *ptr;
int previous_line_number;

if(debug_flag)
	{
	fprintf(stdout, "load_ppml_file() arg pathfilename=%s\n", pathfilename);
	}
	
if(! pathfilename) return 0;

start_frame = 0; // -Wall
end_frame = 0; // -Wall

fptr = fopen(pathfilename, "r");
if(! fptr)
	{
	fl_show_alert("could not open file", pathfilename, "for read", 1);

	if(debug_flag)
		{
		printf("load_ppml_file(): could not load file %s\n", pathfilename);
		}
	return 0;
	} 

delete_all_frames();

/* for all lines in file */
previous_start_sample = -1;
line_number = 0;
previous_line_number = 0;
previous_start_frame = -1;
while(1)
	{
	a = readline_ppml(fptr, temp);
	if(a == EOF) break;

	/* scip emty lines */
	if(temp[0] == 0) continue;

	/* scip lines with comments */
	if(temp[0] == ';') continue;

	/* scip object definitions */
	if(temp[0] == '*') continue;

	/*
	if line does not start with '*' and has no framenumber, syntax error
	*/
	if(! isdigit(temp[0]) )
		{
		sprintf(temp2, "syntax error in line %d", line_number);
		fl_show_alert("load ppml file:", temp2, temp, 0);  
		}

	a = sscanf(temp, "%s %s", temp3, temp2);
	if(a) start_frame = atoi(temp3);

	/* try to point to start of text */
	ptr = strstr(temp, temp2);
	if(! ptr)
		{
		text[0] = 0;
		}
	else
		{
		strcpy(text, ptr);	
		}

	if(debug_flag)
		{
		printf(\
		"load_ppml_file(): a=%d line_number=%d start_frame=%d temp=%s\n",\
		a, line_number, start_frame, temp);
		}

	/* scip object references */
	if(text[0] == '*') continue;
		
	/* 40 ms per frame */

	/* 
	now we know the end frame of the previous entry, it is this start frame.
	look the entry up and modify that end_frame
	*/
	/* test if the start_frame is lower then in the previous line */
	if( start_frame <= previous_start_frame)
		{
		sprintf(temp2,\
		"line %d start_frame=%d",\
		previous_line_number, previous_start_frame);

		sprintf(temp3,\
		"line %d start_frame=%d",\
		line_number , start_frame); 

		fl_show_alert(\
			"ppml file: CONSISTENCY ERROR FIX IT", temp2, temp3, 1);

		/*
		load the file anyways, allow user to correct with the
		'edit frame list button'
		*/
		} /* end consistency check */

	/* modify the end_frame of the previous entry */
	/* first entry has no previous */
	if(start_frame != 0)
		{		
		if(! set_end_frame(previous_start_frame, start_frame) )
			{
			sprintf(temp, "could not set end_frame=%d for frame=%d",\
			start_frame, previous_start_frame);

			fl_show_alert("load_ppml_file():", temp, "", 0);
			}
		}

	/* remember start_frame */
	previous_start_frame = start_frame;

	/* remember line number for error reporting */
	previous_line_number= line_number;

	start_sample = 0;
	end_sample = 0;
	text_hold_flag = 0;
	
	/* add the frame */
	sprintf(temp, "%d", start_frame);
	if(! add_frame(\
	text,
	start_frame, end_frame,\
	text_hold_flag) )
		{
		sprintf(temp, "line %d could not add frame", line_number);
		fl_show_alert("load ppml file:", temp, "aborting", 0);

		if(debug_flag)
			{	
			printf("load_ppml_file(): %s, aboring\n", temp);
			}

		return 0;
		}

	}/* end while all lines in file */

return 1;
}/* end function load_ppml_file */


int readline_ppml(FILE *file, char *contents)
{
int a, c, i;
int last_backslash_position;

if(debug_flag)
	{
	fprintf (stdout, "readline_ppml(): arg file=%lu\n", (long)file);
	}

/* a back-slash before a \n signals a continuation of the current line */

last_backslash_position = -2;
i = 0;
while(1)
	{
	if(i > READSIZE - 1) break;

	while(1)
		{
		c = getc(file);
		a = ferror(file);
		if(a)
			{
			perror("readline():");
			continue;
			}
		break;
		}
	if(feof(file) )
		{
		fclose(file);
		contents[i] = 0;/* EOF marker */

		line_number++;
		return(EOF);
		}

	if(c == '\\') last_backslash_position = i;

	if(c == '\n')
		{
		line_number++;
		if(last_backslash_position == i - 1)
			{
			/* continue line */
			last_backslash_position = -2;

			/* scip the back-slash AND the \n */
			if(i > 0) i--;
			continue;
			}
		else
			{
			contents[i] = 0; /* string termination */
//printf("WAS read=%s\n", contents);
			
			return 1; /* end of line */
			}
		} 

	contents[i] = c;
	i++;
	} /* end for all chars */
/*
mmm since we are here, the line must be quite long, possibly something
is wrong.
Since I do not always check for a return 0 in the use uf this function,
just to be safe, gona force a string termination.
This prevents stack overflow, and variables getting overwritten.
*/
contents[i] = 0;/* force string termination */

line_number++;

if(debug_flag)
	{
	printf(\
	"readline_ppml(): line %d to long, returning 0 contents=%s\n",\
	line_number, contents);
	}

return 0;
} /* end function readline_ppml */


int write_ppml_for_subtitler_yuv_list()
{
int a, c;
struct frame *pa;
struct frame *pnext;
FILE *fptr;
char temp[4096];
char temp2[4096];
int next_frame_start;
char *ptr;
FILE *hfptr;
font_desc_t *pfd;
char fontdir[4096];
char command_part[TEXT_ARRAY_SIZE];
char text_part[TEXT_ARRAY_SIZE];

char old_fontname[4096] = "";
int old_symbols = 0;
int old_fontsize = 0;
int old_iso_extension = 0;
double old_outline_thickness = -0.1;
double old_blur_radius = -0.1;
int have_sub_flag;

struct animation_template *pt;

if(debug_flag)
	{
	printf("write_ppml_list(): arg none\n");
	}

draw_mode = DRAW_MODE_BW;

/*
We need:
subtitle_font_name
subtitle_font_size
subtitle_iso_extention
font *subtitle_font
make_font()

Take from global defaults!!
default_subtitle_font_name
default_subtitle_font_size
default_subtitle_iso_extention
default_subtitle_outline_thickness
default_subtitle_blur_radius
*/

sprintf(temp, "%s/%s/%s/%s-subtitler-yuv.ppml",\
home_dir, selected_data_directory, selected_project, selected_project);
fptr = fopen(temp, "w");
if(! fptr)
	{
	fl_show_alert("write_ppml_list", "Could not open for write", temp, 1);

	return 0;
	}

//fptr = fopen("/dev/fd/1", "w");

/* write ID */
fprintf(fptr,\
"; subtitle file for project %s in PPML format for subtitler-yuv (use with mjpegtools),\n\
; generated with Linux subtitle editor %s by Jan Panteltje\n\n",\
selected_project, VERSION);

fprintf(fptr, "; xste settings:\n");
fprintf(fptr, "; h_size=%d v_size=%d\n", h_size, v_size);
fprintf(fptr, "; h_margin=%d v_margin=%d\n\n", h_margin, v_margin);

fprintf(fptr, "; define a name for the main movie object\n");
fprintf(fptr, "*main main_movie\n\n");

fprintf(fptr, "; define a frame counter in case we need it\n");
fprintf(fptr, "*counter frame_counter\n\n");

/* add animation template text object definitions */

if(use_animation_templates_flag)
	{
	fprintf(fptr, "; animation template text object definitions\n");
	for(pt = animation_templatetab[0]; pt != 0; pt = pt -> nxtentr)
		{
		if( strcmp(pt -> name, "subtitle") == 0) fprintf(fptr, "*subtitle subtitle\n");
		else fprintf(fptr, "*%s text\n", pt -> name);
		}
	}
else
	{
	fprintf(fptr, "; define the subtitle object\n");
	fprintf(fptr, "*subtitle subtitle\n");
	}

fprintf(fptr, "\n");

fprintf(fptr, "; set this to 1 for an X window showing output\n");
fprintf(fptr, "1 *main show_output=%d\n\n", ppml_enable_monitor_flag);

fprintf(fptr, "; fade in main movie\n");
fprintf(fptr, "1 *main contr=100 dcontr=0 sat=100 dsat=0\n\n");

fprintf(fptr, "; while skip is 1 output will be skipped, use this to edit out material\n");
fprintf(fptr, "1 *main skip=0\n\n");

fprintf(fptr, "; uncomment next line for frame counter\n");
sprintf(temp, "%s/.subtitles/%s/arial-26-iso-8859-15", home_dir, selected_project);
fprintf(fptr, ";1 *counter font_dir=%s xpos=%d ypos=10\n\n", temp, h_size / 2);

fprintf(fptr, "; center text\n"); 
fprintf(fptr, "1 *subtitle center=1\n\n");

fprintf(fptr, "; set margins\n");
fprintf(fptr, "1 *subtitle hfactor=%.3f vfactor=%.3f\n\n", h_factor, v_factor); 

have_sub_flag = 0;
for(pa = frametab[0]; pa != 0; pa = pa -> nxtentr)
	{
	/* skip any command lines starting with '*' */
	if(pa -> text[0] == '*') continue;

	/* fast test for special commands */
	if(strchr(pa -> text, '#') )
		{
		/* test for include file command */
		a = sscanf(pa -> text, "#include %s", temp2);
		if(a == 1)
			{
			sprintf(temp, "%s/.subtitles/%s/%s",\
			home_dir, selected_project, temp2);
			/* read from include file, write to outfile */
			hfptr = fopen(temp, "r");
			if(! hfptr)
				{
				fl_show_alert(\
				"Could not open include file", temp, "for read", 0);
				}
			else
				{
				fprintf(fptr, "\n; start included file %s\n", temp);

				while(1)
					{
					errno = 0;
					while(1)/* error re read */
						{
						c = getc(hfptr);
						if(! ferror(hfptr) ) break;
						perror(\
						"write_ppml_list(): read from include file failed ");
						}/* end while error re read */	

					if(feof(hfptr) )
						{
						fclose(hfptr);

						break;
						}

					fputc(c, fptr);
					} /* end for al chars in header file */

				fprintf(fptr, "; end included file %s\n\n", temp);
				} /* end if header file open */

			continue;
			} /* end if include file */

		} /* end if special command */

	/* split of any command in the form of '[xxx x x] */
	if(! extract_command(pa -> text, text_part, command_part) ) return 0;

	/* check for, and if we have possible animation templates, add these here */
	if(use_animation_templates_flag)
		{
		if(command_part[0] != 0)
			{
			a = atoi(pa -> name);
			if(a >= 0)
				{
				if(add_animation_object_to_ppml_file(a , command_part, fptr, text_part) )
					{
					/* do not use text! */ 
					continue;
					}
				}
			}

		} /* end if use_animation_templates_flag */

	/* no template reference and empy line with no text */
	if(text_part[0] == 0)
		{
		fprintf(fptr, "%d\n\n", atoi(pa -> name) );

		continue;
		}

	/* test if a subtitle template is defined, should always be */
	pt = lookup_animation_template("subtitle");

	/*
	specify font parameters and espace from frame list,
	everything else from subtitle animation template
	*/

	if(
	(strcmp(pa -> fontname, old_fontname) != 0) ||\
	(pa -> symbols != old_symbols) || \
	(pa -> fontsize != old_fontsize) || \
	(pa -> iso_extension != old_iso_extension) || \
	(pa -> outline_thickness != old_outline_thickness) || \
	(pa -> blur_radius != old_blur_radius) \
	)
		{
		sprintf(temp, "%s/.xste/fonts/%s", home_dir, pa -> fontname);

		pfd = make_old_font(
			temp,\
			pa -> fontsize,\
			pa -> iso_extension,\
			pa -> outline_thickness,\
			pa -> blur_radius);

		if(! pfd)
			{
			sprintf(temp, "size=%d iso=%d outline=%.2f blur=%.2f",\
			pa -> fontsize,\
			pa -> iso_extension,\
			pa -> outline_thickness,\
			pa -> blur_radius);

			fl_show_alert(pa -> fontname, temp, "could not create font for ppml file", 0);	

			return 0;
			}

		/* strip the .ttf */
		strcpy(temp, pa -> fontname);

		ptr = strstr(temp, ".ttf");
		if(! ptr)
			{
			fl_show_alert("Could not find .ttf in fontname", pa -> fontname, "", 0);

			return 0;
			}

		*ptr = 0;
		sprintf(fontdir,\
		"%s/.subtitles/%s/%s-%d-iso-8859-%d",\
		home_dir, selected_project,\
		temp, pa -> fontsize, pa -> iso_extension);

		if(have_sub_flag)
			{
			fprintf(fptr, "\n; remove any old sub\n");
			fprintf(fptr, "%d\n\n",  atoi(pa -> name) - 2);
			}

		fprintf(fptr, "; set the subtitle properties\n");
		fprintf(fptr,\
"%d *subtitle font_dir=%s espace=%d despace=%.2f vespace=%d contr=%.2f dcontr=%.2f color=%.2f dcolor=%.2f sat=%.2f dsat=%.2f transp=%.2f dtransp=%.2f dxpos=%.2f dypos=%.2f \n\n",\
		atoi(pa -> name) - 1, fontdir,\
		pa -> espace, pt -> despace, pa -> vespace,\
		pt -> contrast, pt -> dcontrast,\
		pt -> color, pt -> dcolor,\
		pt -> saturation, pt -> dsaturation,\
		pt -> transparency, pt -> dtransparency,\
		pt -> dxpos, pt -> dypos);

		strcpy(old_fontname, pa -> fontname);
		old_symbols = pa -> symbols;
		old_fontsize = pa -> fontsize;
		old_iso_extension = pa -> iso_extension;
		old_outline_thickness = pa -> outline_thickness;
		old_blur_radius = pa -> blur_radius;

		} /* end if font change */

	/* print the text */
	fprintf(fptr, "%d %s\n\n", atoi(pa -> name), text_part);

	have_sub_flag = 1;

	/* the title must disappear after the specified time */

	/* first get pointer to next structure entry */ 
	if(pa -> nxtentr) 
		{
		pnext = pa -> nxtentr;	
		
		/* get the start of the next frame */ 
		next_frame_start = atoi(pnext -> name);

		/* test if the current frame ends before the start of the next */
		if(pa -> end_frame < next_frame_start)
			{
			/*
			Insert an empty line with one space here to make the current
			text disappear (ppml processes this).
			*/
			/* 
			Only make text go away if it was not a text go away line !
			set by the user.
			*/
			if(strlen(pa -> text) != 0)
				{
				fprintf(fptr, "%d \n", pa -> end_frame);
				}
			} /* end if frame ends before next one starts */ 
		} /* end if not last entry */	

	else /* last entry */
		{
		/*
		the last entry text should also be removed at the specified time.
		*/
		/* 
		Only make text go away if it was not a text go away line !
		set by the user.
		*/
		if(strlen(pa -> text) != 0)
			{
			fprintf(fptr, "%d \n", pa -> end_frame);
			}
		}
	} /* end for all frames */

/* close the file */
fclose(fptr);

return 1;
} /* end function write_ppml_for_subtitler_yuv_list */



int add_animation_object_to_ppml_file(int frame, char *name, FILE *fptr, char *text)
{
struct animation_template *pt;
char *ptr;
char temp[4096];
char fontdir[4096];
font_desc_t *pfd;

if(debug_flag)
	{
	fprintf(stdout,\
	"add_animation_object_to_ppml_file(): arg frame=%d name=%s fptr=%p text=%s\n", frame, name, fptr, text); 
	}

/* argument check */
if(frame < 0) return 0;
if(! name) return 0;
if(! fptr) return 0;
if(! text) return 0;

pt = lookup_animation_template(name);
if(! pt)
	{
	fl_show_alert("add_animation_object_to_ppml_file()", "lookup failed for template", name, 0);

	return 0;
	}

sprintf(temp, "%s/.xste/fonts/%s", home_dir, pt -> fontname);

pfd = make_old_font(
	temp,\
	pt -> fontsize,\
	pt -> iso,\
	pt -> outline,\
	pt -> blur);

if(! pfd)
	{
	sprintf(temp, "size=%d iso=%d outline=%.2f blur=%.2f",\
	pt -> fontsize,\
	pt -> iso,\
	pt -> outline,\
	pt -> blur);

	fl_show_alert(pt -> fontname, temp, "could not create font for ppml file", 0);	

	return 0;
	}

/* strip the .ttf */
strcpy(temp, pt -> fontname);

ptr = strstr(temp, ".ttf");
if(! ptr)
	{
	fl_show_alert("Could not find .ttf in fontname", pt -> fontname, "", 0);

	return 0;
	}

*ptr = 0;
sprintf(fontdir,\
"%s/.subtitles/%s/%s-%d-iso-8859-%d", home_dir, selected_project,temp, pt -> fontsize, pt -> iso);

fprintf(fptr, "; set properties for animation template object\n");

fprintf(fptr, "%d *%s font_dir=%s\n", frame - 1, pt -> name, fontdir);

if( text[0] != 0)
	{
	if( pt -> status & ANIMATION_TEMPLATE_SHOW_X_Y_DELTA)
		{
		fprintf(fptr, "; using X_Y_DELTA format\n");
	
		fprintf(fptr,\
		"%d *%s xpos=%.2f ypos=%.2f zpos=%.2f dzpos=%.2f \
espace=%.2f despace=%.2f vespace=%.2f \
color=%.2f dcolor=%.2f contrast=%.2f dcontrast=%.2f sat=%.2f dsat=%.2f \
transp=%.2f dtransp=%.2f \
dxpos=%.2f dypos=%.2f\n",\
		frame - 1, pt -> name, pt -> xpos, pt -> ypos, pt -> zpos, pt -> dzpos,\
		pt -> espace, pt -> despace, pt -> vespace,\
		pt -> color, pt -> dcolor, pt -> contrast, pt -> dcontrast, pt -> saturation, pt -> dsaturation,\
		pt -> transparency, pt -> dtransparency,\
		pt -> dxpos, pt -> dypos); 
		}
	else
		{
		fprintf(fptr, "; using HEADING format\n");
	
		fprintf(fptr,\
		"%d *%s xpos=%.2f ypos=%.2f zpos=%.2f dzpos=%.2f \
espace=%.2f despace=%.2f vespace=%.2f \
color=%.2f dcolor=%.2f contrast=%.2f dcontrast=%.2f sat=%.2f dsat=%.2f \
transp=%.2f dtransp=%.2f \
heading=%.2f dheading=%.2f speed=%.2f dspeed=%.2f\n",\
		frame - 1, pt -> name, pt -> xpos, pt -> ypos, pt -> zpos, pt -> dzpos,\
		pt -> espace, pt -> despace, pt -> vespace,\
		pt -> color, pt -> dcolor, pt -> contrast, pt -> dcontrast, pt -> saturation, pt -> dsaturation,\
		pt -> transparency, pt -> dtransparency,\
		pt -> heading, pt -> dheading, pt -> speed, pt -> dspeed); 
		}
	} /* end if not empty text (not text go way) */
fprintf(fptr, "%d *%s text=%s\n\n", frame, pt -> name, text);

return 1;
} /* end function add_animation_object_to_ppml_file */


