/* 
xste subtitle editor
xste  is copyright (C) 2000 <Jan Panteltje>
email: panteltje@yahoo.com

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "xste.h"

extern FD_xste *fdui;


struct project
	{
	char *name;/* this is the project space period */
	int status;
	int browser_line;
	struct project *nxtentr;
	struct project *prventr;
	};
struct project *projecttab[2]; /* first element points to first entry,
			second element to last entry */


struct project *lookup_project(char *name)
{
struct project *pa;

/* argument check */
if(! name) return 0;

/*pa points to next entry*/
for(pa = projecttab[0]; pa != 0; pa = pa -> nxtentr)
	{
	if(strcmp(pa -> name, name) == 0) return(pa);
	}
return 0;/*not found*/
}/* end function lookup_project */


struct project *install_project_at_end_of_list(char *name)
{
struct project *plast, *pnew;
struct project *lookup_project();

if(debug_flag)
	{
	fprintf(stdout,\
	"install_project_at_end_of_list(): arg name=%s\n", name);
	}

/* argument check */
if(! name) return 0;

pnew = lookup_project(name);
if(pnew)
	{
	return(pnew);/* already there */
	}

/* create new structure */
pnew = (struct project *) calloc(1, sizeof(*pnew) );
if(! pnew) return 0;
pnew -> name = strsave(name);
if(! pnew -> name) return 0;

/* get previous structure */
plast = projecttab[1]; /* end list */

/* set new structure pointers */
pnew -> nxtentr = 0; /* new points to zero (is end) */
pnew -> prventr = plast; /* point to previous entry, or 0 if first entry */

/* set previous structure pointers */
if( !projecttab[0] ) projecttab[0] = pnew; /* first element in list */
else plast -> nxtentr = pnew;

/* set array end pointer */
projecttab[1] = pnew;

return(pnew);/* pointer to new structure */
}/* end function install_project_at_end_of_list */


int delete_project(char *name)/* delete entry from double linked list */
{
struct project *pa, *pprev, *pdel, *pnext;

if(debug_flag)
	{
	fprintf(stdout, "delete_project(): arg name=%s\n", name);
	}

/* argument check */
if(! name) return 0;

pa = projecttab[0];
while(1)
	{
	/* if end list, return not found */
	if(! pa) return 0;

	/* test for match in name */
	if(strcmp(name, pa -> name) != 0) /* no match */
		{
		/* point to next element in list */
		pa = pa -> nxtentr;

		/* loop for next element in list */
		continue;
		}

	/* we now know which struture to delete */
	pdel = pa;

	/* get previous and next structure */
	pnext = pa -> nxtentr;
	pprev = pa -> prventr;

	/* set pointers for previous structure */
	/* if first one, modify projecttab[0] */
	if(pprev == 0) projecttab[0] = pnext;
	else pprev -> nxtentr = pnext;

	/* set pointers for next structure */
	/* if last one, modify projecttab[1] */
	if(pnext == 0) projecttab[1] = pprev;
	else pnext -> prventr = pprev;
	
	/* delete structure */	
	free(pdel -> name);
	free(pdel); /* free structure */

	/* return OK deleted */
	return 1;
	}/* end for all structures */

}/* end function delete_project */


int delete_all_projects()/* delete all entries from table */
{
struct project *pa;

if(debug_flag)
	{
	fprintf(stdout, "delete_all_projects() arg none\n");
	}

while(1)
	{	
	pa = projecttab[0];
	if(! pa) break;

	projecttab[0] = pa -> nxtentr;
	free(pa -> name);

	free(pa);/* free structure */
	}/* end while all structures */

projecttab[1] = 0;
return 1;
}/* end function delete_all_projects */


int sort_projects()
/*
sorts the double linked list with as criterium that the lowest ASCII goes
on top.
doing some sort of bubble sort.
*/
{
struct project *pa;
struct project *pb;
int swap_flag;

while(1)/* go through list again and again */
	{
	if(debug_flag)
		{
		fprintf(stdout, "CHECKING LIST\n");
		}/* end if debug_flag */
		
	swap_flag = 0;
	for(pa = projecttab[0]; pa != 0; pa = pa -> nxtentr)
		{
		if(debug_flag)
			{
			fprintf(stdout, "sort_projects(): sorting %s pa=%p\n",\
			pa -> name, pa);		
			}	
	
		pb = pa -> prventr;
		if(debug_flag)
			{
			fprintf(stdout, "pb=pa->prventr=%p\n", pb);
			}
		
		if(pb)
			{
			/* compare */
			if(strcmp(pa -> name, pb -> name) < 0)
				{
				swap_flag = swap_position(pa , pb);
				/* indicate position was swapped */
				if(debug_flag)
					{
					fprintf(stdout, "swap_flag=%d\n", swap_flag);
					fprintf(stdout,\
					"AFTER SWAP pa->prventr=%p pa->nxtentr=%p\n\
					pb->prventr=%p pb-nxtentrr=%p\n",\
					pa -> prventr, pa -> nxtentr,\
					pb -> prventr, pb -> nxtentr);		
					}										
				}/* end if strcmp < 0 */
			}/* end if pb */
		}/* end for all entries */

	/* if no more swapping took place, ready, list is sorted */
	if(! swap_flag) break;
	}/* end while go through list again and again */

return 1;
}/* end function sort_projects */


int load_projects()
{
int a;
FILE *projects_dat_file;
char temp[READSIZE];
char pathfilename[TEMP_SIZE];
extern long atol();
struct project *pa;
char name[TEMP_SIZE];
int status;
int browser_line; /* used to lookup name (= id) if clicked in display,
						not saved on disk, assigned each time list
						is displayed */
if(debug_flag)
	{
	fprintf(stdout, "load_projects() arg none\n");
	}
	
delete_all_projects();

sprintf(pathfilename, "%s/.xste/projects.dat", home_dir);
projects_dat_file = fopen(pathfilename, "r");
if(! projects_dat_file)
	{
	if(debug_flag)
		{
		fprintf(stdout, "could not load file %s\n", pathfilename);
		}
	return 0;
	} 

/*
Set the global and the input field in the setup form to the selected server.
*/
a = readline(projects_dat_file, temp);
if(a == EOF) return 0;/* file format error */

if(strlen(temp) == 0) return 0;

free(selected_project);
selected_project = (char *) strsave(temp);
if(! selected_project) return 0;

fl_set_input(fdui -> project_form_input_field, "");

while(1)
	{
	a = readline(projects_dat_file, name);
	if(a == EOF) return 1;/* empty file or last line */
	
	a = readline(projects_dat_file, temp);
	if(a == EOF) break;/* last line, or file format error */
	sscanf(temp, "%d %d",\
	&status, &browser_line);

	/* create a structure entry */
	pa = install_project_at_end_of_list(name);
	if(! pa)
		{
		if(debug_flag)
			{
			fprintf(stdout,\
			"load_projects: cannot install project %s\n", temp);
			}
		break;
		}

	pa -> status = status;
	pa -> browser_line = browser_line;

	}/* end while all lines in projects.dat */

if(a == EOF)
	{
	if(debug_flag)
		{
		fprintf(stdout,\
		"load_projects(): file format error\n");
		}
	}
else
	{
	if(debug_flag)
		{
		fprintf(stdout,\
		"load_projects(): memory allocation error\n");
		}
	}	
return 0;
}/* end function load projects */


int save_projects()
{
FILE *projects_dat_file;
char pathfilename[TEMP_SIZE];
char pathfilename2[TEMP_SIZE];
struct project *pa;

if(debug_flag)
	{
	fprintf(stdout, "save_projects() arg none\n");
	}

sprintf(pathfilename, "%s/.xste/projects.tmp", home_dir);
projects_dat_file = fopen(pathfilename, "w");
if(! projects_dat_file)
	{
	if(debug_flag)
		{
		fprintf(stdout,\
		"could not open file %s for write\n", pathfilename);
		}
	return 0;
	} 

/* save selected project */
fprintf(projects_dat_file, "%s\n", selected_project);

/* save list of projects */
for(pa = projecttab[0]; pa != 0; pa = pa -> nxtentr)	
	{
	fprintf(projects_dat_file, "%s\n", pa -> name);

	fprintf(projects_dat_file, "%d %d\n",\
	pa -> status, pa -> browser_line);

	}/* end while all elements in list */

fclose(projects_dat_file);

/* set some path file names */
sprintf(pathfilename, "%s/.xste/projects.dat", home_dir);
sprintf(pathfilename2, "%s/.xste/projects.dat~", home_dir);

/* unlink the old .dat~ */
unlink(pathfilename2);

/* rename .dat to .dat~ */
if( rename(pathfilename, pathfilename2) == -1)
	{
	if(debug_flag)
		{
		fprintf(stdout, "save_projects(); rename %s into %s failed\n",\
		pathfilename, pathfilename2);
		}

	/* this will fail first time, if no projects.dat! so no exit */
//	return 0;
	}

/* rename .tmp to .dat */
sprintf(pathfilename2, "%s/.xste/projects.tmp", home_dir);
if( rename(pathfilename2, pathfilename) == -1)
	{
	if(debug_flag)
		{
		fprintf(stdout, "save_projects(); rename %s into %s failed\n",\
		pathfilename, pathfilename2);
		}
	return 0;
	}

return 1;/* ok */
}/* end function save_projects */


int delete_a_project(char *name)
{
char temp[TEMP_SIZE];
FILE *exec_filefd;
int format_error;
char *ptr;
char name_of_project[TEMP_SIZE];

if(debug_flag)
	{
	fprintf(stdout, "delete_a_project(): arg name=%s\n",\
	name);
	}

/* argument check */
if(!name) return 0;
if(strlen(name) == 0) return 0;

/* after structure is no longer allocated, name becomes invalid */
strcpy(name_of_project, name);

format_error = 0;
if(strstr(name, "/") != 0) format_error = 1;/* illegal format, and dangerous */
if(strcmp(name, "setup") == 0) format_error = 1;
if(strcmp(name, "projects.dat") == 0) format_error = 1;
if(format_error)
	{
	fl_show_alert("delete_a_project()",\
	name,\
	"format error, command cancelled", 0);
	return 0;
	}

/* it is possible that we want to delete a selected entry */
if(strcmp(name, selected_project) == 0)
	{
	fl_show_alert(\
	"This project is selected",\
	"select an other one first",\
	"command cancelled", 0);
	return 0;
	}

/* last user confirmation */
sprintf(temp, "Delete all data for %s???", name);
if(! fl_show_question(temp, 0) ) /* mouse on no */
	{
	return 0;
	}

/* user sanity test */
ptr = (char *) fl_show_input("Type: yes I want this", "No");
if(! ptr) return 0;
if(strcmp(ptr, "yes I want this") != 0)
	{
	fl_show_alert("Command cancelled", "", "", 0);
	return 0;
	}

/* delete all files in project/period directory upward */
sprintf(temp, "rm -rf %s/%s/%s/*",\
home_dir, selected_data_directory, name);
exec_filefd = popen(temp, "r");
pclose(exec_filefd);

/* delete the directory itself */
sprintf(temp, "rmdir %s/%s/%s",\
home_dir, selected_data_directory, name);
exec_filefd = popen(temp, "r");
pclose(exec_filefd);

/* delete structure entry */
if(! delete_project(name) )/* this tests for existence */
	{
	fl_show_alert(\
	"Cannot delete project", name, "command cancelled", 0);
	return 0;
	}

/*'name' no longer allocated!!!!!!!! */

/* write modified structure to disk */
save_projects();

/* delete all files in /video/vie/name_of_project/ */

sprintf(temp, "Delete recorded video in\n/video/vie/%s/ ?", name_of_project);
if(! fl_show_question(temp, 0) )/* mouse on no */
	{
	return 0;
	}

sprintf(temp, "rm /video/vie/%s/*", name_of_project);
exec_filefd = popen(temp, "r");
pclose(exec_filefd);

/* delete the directory itself */
sprintf(temp, "rmdir /video/vie/%s", name_of_project);
exec_filefd = popen(temp, "r");
pclose(exec_filefd);

return 1;
}/* end function delete_a_project */


int show_project_form()
{
show_projects(SHOW_UNREAD_AT_TOP);

fl_set_input(fdui -> project_form_input_field, "");
//fl_show_form(fdui -> project_form, FL_PLACE_CENTER, FL_NOBORDER, "");
fl_show_form(fdui -> project_form, FL_PLACE_CENTER, FL_TRANSIENT, "");

project_form_shown_flag = 1;

return 1;
}/* end function show_project_form */


int save_project_setting(char *name)
{
struct project *pa;

if(debug_flag)
	{
	fprintf(stdout, "save_project_setting(): arg name=%s\n", name);
	}

/* argument check */
if(! name) return 0;
if(strlen(name) == 0) return 0;
if(strstr(name, "/") != 0) return 0;

/* test if exists */
pa = lookup_project(name);
if(! pa)
	{
	if(debug_flag)
		{
		fprintf(stdout,\
		"save_project_setting(): project %s does not exists\n",\
		name);
		}	
	return 0;
	}

pa -> status = 0;
pa -> browser_line = 0;
	
save_projects();
			 
return 1;
}/* end function save_project_setting */


int load_project_setting(char *name)
{
struct project *pa;

if(debug_flag)
	{
	fprintf(stdout, "load_project_setting(): arg name=%s\n",\
	name);
	}

/* argument check */
if(! name) return 0;
if(strlen(name) == 0) return 0;
if(strstr(name, "/") != 0) return 0;

pa = lookup_project(name);
if(!pa)
	{
	if(debug_flag)
		{
		fprintf(stdout, "load_project_setting(): no such entry %s\n",\
		name);
		}
	return 0;
	}

return 1;
}/* end function load_project_setting */


int add_project(char *name)
{
int illegal;
struct project *pa;
char temp[TEMP_SIZE];
FILE *exec_filefd;

if(debug_flag)
	{
	fprintf(stdout, "add_project(): arg name=%s\n", name);
	}

/* argument check */
if(! name) return 0;
illegal = 0;
if(strlen(name) == 0) illegal = 1;
if(strstr(name, "/") != 0) illegal = 1;
if(strstr(name, " ") != 0) illegal = 1;
if(strstr(name, "*") != 0) illegal = 1;
if(strcmp(name, "setup") == 0) illegal = 1;
if(strcmp(name, "projects.dat") == 0) illegal = 1;
if(illegal)
	{
	fl_show_alert("Illegal name", name, "command cancelled", 0);
	return 0;
	}

pa = lookup_project(name);
if(pa)
	{
	fl_show_alert(\
	"project name exists", name, "command cancelled", 0);

	if(debug_flag)
		{
		fprintf(stdout, "add_project(): %s exists, returning\n",\
		name);
		}
	return 0;/* do not select */
	}

sprintf(temp,\
"project %s does not exist\nCreate entry?",\
name);
if(! fl_show_question(temp, 1) )/* 1 = mouse on yes */
	{
	return 0;
	}

pa = install_project_at_end_of_list(name);
if(! pa)
	{
	fl_show_alert(\
	"add_project():",\
	"cannot install project", \
	"command cancelled", 0);
	return 0;
	}

save_projects();

/* create project directory for subtitle work */
sprintf(temp, "mkdir %s/%s/%s 2> /dev/zero",\
home_dir, selected_data_directory, name);
exec_filefd = popen(temp, "r");
pclose(exec_filefd);

/* create video directory for video data and configuration files */
sprintf(temp, "mkdir /video/vie/%s 2> /dev/zero",\
name);
exec_filefd = popen(temp, "r");
pclose(exec_filefd);

/* copy the xste_synthesize script */
sprintf(temp, "cp %s/.xste/xste_synthesize  %s/%s/%s/ 2> /dev/zero",\
home_dir, home_dir, selected_data_directory, name);
exec_filefd = popen(temp, "r");
pclose(exec_filefd);

return 1;
}/* end function add_project */


int select_project(char *name)
{
if(debug_flag)
	{
	fprintf(stdout, "select_project(): arg name=%s\n",\
	name);
	}

/* argument check */
if(! name) return 0;
if(strlen(name) == 0) return 0;
if(strstr(name, "/") != 0) return 0;

/* set the global */
free(selected_project);
selected_project = strsave(name);

/* reset video error mode, maybe set from previous project */
video_error = VIDEO_OK;

/* load the variables for this project (if any so no error return) */
load_project_vars(); /* if this fails, it will set defaults */

/* load and show the selected frame list */
show_frame_form(); /* no return, project_modified_flag needs to be set */

/* projects.dat modified (first line, project) */
save_projects();

/* set the title for the main form */
set_title();

return 1;
}/* end function select_project */


int show_projects(int position)
{
char temp[TEMP_SIZE];
struct project *pa;
char formatstr[20];
int browser_line;
int browser_topline;
int first_new_project_line;
int find_first_new_project_flag;

int maxline;

if(debug_flag)
	{
	fprintf(stdout, "show_projects(): arg position=%d\n", position);
	}

load_projects();
sort_projects();

/* remember vertical slider position */
browser_topline = fl_get_browser_topline(fdui -> project_form_browser);

fl_freeze_form(fdui -> project_form);

fl_clear_browser(fdui -> project_form_browser);

first_new_project_line = -1;/* keep gcc -Wall from complaining */
find_first_new_project_flag = 1;
browser_line = 1;
for(pa = projecttab[0]; pa != 0; pa = pa -> nxtentr)	
	{
	if(position == SHOW_UNREAD_AT_TOP)
		{
		/* want to show the first unaccessed at top of display */
		if(find_first_new_project_flag)
			{
			if(pa -> status == 1)
				{
				first_new_project_line = browser_line;
				find_first_new_project_flag = 0;
				}
			}
		}
	
	strcpy(formatstr, "@f");
	if(pa -> status == 0) strcat(formatstr, "@C0");/* black */
	
	sprintf(temp, "%s %s",\
	formatstr, pa -> name);

	fl_add_browser_line(fdui -> project_form_browser, temp);
	pa -> browser_line = browser_line;/* first one is 1 */
	browser_line++;
	}/* end for all elements in list */
	
/* re adjust browser for same position of top line (vertical slider) */	
if(position == SHOW_UNREAD_AT_TOP)
	{
	fl_set_browser_topline(fdui -> project_form_browser, first_new_project_line);
	}
if(position == SHOW_BOTTOM)
	{
	/* make the status of the last project visible */
	maxline = fl_get_browser_maxline(fdui -> project_form_browser);
	fl_set_browser_topline(fdui -> project_form_browser, maxline);
	}
/*
if(position == SHOW_TOP)
	{
	fl_set_browser_topline(fdui -> project_form_browser, 1);
	}
*/
if(position == SHOW_SAME_POSITION)
	{
	fl_set_browser_topline(fdui -> project_form_browser, browser_topline);
	}	

fl_unfreeze_form(fdui -> project_form);

return 1;
}/* end function show_projects */


char *line_to_project(int line)
{
struct project *pa;
extern long atol();

if(debug_flag)
	{
	fprintf(stdout, "line_to_project(): arg line=%d\n", line);
	}

/* argument check */
if(line < 0) return 0;

for(pa = projecttab[0]; pa != 0; pa = pa -> nxtentr)	
	{
	if(pa -> browser_line == line)
		{
		return(pa -> name);
		}
	}

/* no save_projects, nothing was changed */
return 0;
}/* end function line_to_project_id */


int project_to_line(char *name, int *line)
{
struct project *pa;

if(debug_flag)
	{
	fprintf(stdout, "project_to_line(): arg name=%s\n", name);
	}

/* argument check */
if(! name) return 0;

pa = lookup_project(name);
if(! pa) return 0;

*line = pa -> browser_line;
return 1;
}/* end function project_to_line */


int set_title()
{
char temp[TEMP_SIZE];

sprintf(temp, "%s:  %s",\
VERSION, selected_project);

fl_set_form_title(fdui -> main_form, temp);

return 1;
}/* end function set_title */


int swap_position(struct project *ptop, struct project *pbottom)
{
struct project *punder;
struct project *pabove;

if(debug_flag)
	{
	fprintf(stdout,\
	"swap_position(): swapping top=%p bottom=%p\n", ptop, pbottom);
	}

/* argument check */
if(! ptop) return 0;
if(! pbottom) return 0;

/* get one below the bottom */
punder = pbottom -> prventr;/* could be zero if first entry */
if(debug_flag)
	{
	fprintf(stdout,\
	"swap_position(): punder=%p\n", punder);
	}

/* get the one above the top */
pabove = ptop -> nxtentr;/* could be zero if last entry */
if(debug_flag)
	{
	fprintf(stdout,\
	"swap_position(): pabove=%p\n", pabove);
	}

/* the next pointer in punder (or projecttab[0]) must now point to ptop */
if(! punder)
	{
	projecttab[0] = ptop; 
	}
else
	{
	punder -> nxtentr = ptop;
	}

/* the prev pointer in in ptop must now point to punder */
ptop -> prventr = punder;/* could be zero if first entry */

/* the next pointer in ptop must now point to pbottom */
ptop -> nxtentr = pbottom;

/* the next pointer in pbottom must now point to pabove */
pbottom -> nxtentr = pabove;

/* mark last one in projecttab */
if(! pabove)
	{
	projecttab[1] = pbottom;
	}
else
	{
	/* the prev pointer in pabove must now point to pbottom */
	pabove -> prventr = pbottom;
	}

/* the prev pointer in pbottom must now point to ptop */
pbottom -> prventr = ptop;

/* return swapped */
return 1;
}/* end function swap_position */


