/* 
xste subtitle editor
xste  is copyright (C) 2000 <Jan Panteltje>
email: panteltje@yahoo.com

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "xste.h"

FD_xste *fdui;

FILE *filefd;


int save_setting(char *type, char *string)
{
char pathfilename[TEMP_SIZE];
FILE *setting_file;

/* combine with home directory */
sprintf(pathfilename, "%s/.xste/setup/%s", home_dir, type);
setting_file = fopen(pathfilename, "w");
if(! setting_file)
	{
	fl_show_alert("Cannot open file for write:", pathfilename, "", 0);
	if(debug_flag)
		{
		fprintf(stdout, "\nCannot open setting file %s for write\n",\
		pathfilename);
		}
	return 0;
	}

fprintf(setting_file, "%s", string);
fclose(setting_file);
return 1;	
}/* end function save_setting */


char *load_setting(char *type)
{
int a;
char temp[1024];
char pathfilename[TEMP_SIZE];
FILE *setting_file;

/* combine with home directory */
sprintf(pathfilename, "%s/.xste/setup/%s", home_dir, type);
setting_file = fopen(pathfilename, "r");
if(! setting_file)
	{
	fl_show_alert("Cannot open file for read:", pathfilename, "", 0);
	if(debug_flag)
		{
		fprintf(stdout, "\nCannot open setting file %s for read\n",\
		pathfilename);
		}
	return 0;
	}

a = readline(setting_file, temp);/* closes file if EOF */	
if(a != EOF) fclose(setting_file);
return(strsave(temp) );
}/* end function load_setting */


int save_fontsizes()
{
char temp[TEMP_SIZE];
sprintf(temp,\
"%d %d %d %d %d",\
main_form_fontsize,\
project_form_fontsize,\
export_form_fontsize,\
fonts_form_fontsize,\
special_characters_form_fontsize\
);

if(! save_setting("fontsizes", temp) ) return 0;

return 1;
}/* end function save_fontsizes */


int load_fontsizes()
{
char *data_read;
 
data_read = (char *) load_setting("fontsizes");
if(! data_read) return 0;
sscanf(data_read,\
"%d %d %d %d %d",\
&main_form_fontsize,\
&project_form_fontsize,\
&export_form_fontsize,\
&fonts_form_fontsize,\
&special_characters_form_fontsize\
);

free(data_read);

/* for main form */
fl_set_browser_fontsize(fdui -> main_form_browser,\
integer_to_fontsize(main_form_fontsize) );

fl_set_object_lsize(fdui -> main_form_frame_input_field,\
integer_to_fontsize(main_form_fontsize) );

fl_set_object_lsize(fdui -> main_form_current_frame_input_field,\
integer_to_fontsize(main_form_fontsize) );

fl_set_object_lsize(fdui -> main_form_text_duration_input_field,\
integer_to_fontsize(main_form_fontsize) );

fl_set_object_lsize(fdui -> main_form_frames_per_second_input_field,\
integer_to_fontsize(main_form_fontsize) );

fl_set_object_lsize(fdui -> main_form_text_input_field,\
integer_to_fontsize(main_form_fontsize) );

/* for project form */
fl_set_browser_fontsize(fdui -> project_form_browser,\
integer_to_fontsize(project_form_fontsize) );

fl_set_object_lsize(fdui -> project_form_input_field,\
integer_to_fontsize(project_form_fontsize) );

/* for export form */

fl_set_object_lsize(fdui -> export_form_select_text,\
integer_to_fontsize(export_form_fontsize) );

/* for fonts form */
fl_set_browser_fontsize(fdui -> fonts_form_browser,\
integer_to_fontsize(fonts_form_fontsize) );

/*
fl_set_object_lsize(fdui -> fonts_form_title,\
integer_to_fontsize(fonts_form_fontsize) );
fl_set_object_lsize(fdui -> fonts_form_global_fonts_button,\
integer_to_fontsize(fonts_form_fontsize) );
fl_set_object_lsize(fdui -> fonts_form_default_fonts_button,\
integer_to_fontsize(fonts_form_fontsize) );
fl_set_object_lsize(fdui -> fonts_form_article_fonts_button,\
integer_to_fontsize(fonts_form_fontsize) );
fl_set_object_lsize(fdui -> fonts_form_fontsize_up_button,\
integer_to_fontsize(fonts_form_fontsize) );
fl_set_object_lsize(fdui -> fonts_form_fontsize_down_button,\
integer_to_fontsize(fonts_form_fontsize) );
fl_set_object_lsize(fdui -> fonts_form_accept_button,\
integer_to_fontsize(fonts_form_fontsize) );
*/

/* for special characters form */
fl_set_object_lsize(fdui -> special_characters_form_input,\
integer_to_fontsize(special_characters_form_fontsize) );

return 1;
}/* end function load_fontsizes */


int save_general_settings()
{
char temp[TEMP_SIZE];

sprintf(temp, "%d %d",\
spellcheck_input_flag,\
tab_size);

save_setting("global_flags", temp);

sprintf(temp, "%s", editor_command);
save_setting("editor_command", temp);

sprintf(temp, "%s", spellcheck_command);
save_setting("spellcheck_command", temp);

sprintf(temp, "%s", default_subtitle_font_name);
save_setting("subtitle_font_name", temp);

return 1;
}/* end function save_general_settings */



int load_general_settings()
{
int error;
char *ptr;

if(debug_flag)
	{
	fprintf(stdout, "load_general_settings(): arg none\n");
	}

error = 0;
ptr = load_setting("global_flags");
if(ptr)
	{
	sscanf(ptr, "%d %d",\
	&spellcheck_input_flag,\
	&tab_size);

	free(ptr);

	fl_set_button(\
		fdui -> main_form_spellcheck_input_button, spellcheck_input_flag);

	}
else error = 1;

ptr = load_setting("editor_command");
if(ptr)
	{
	free(editor_command);
	editor_command = ptr;
	}
else error = 1;

ptr = load_setting("spellcheck_command");
if(ptr)
	{
	free(spellcheck_command);
	spellcheck_command = ptr;
	}
else error = 1;

ptr = load_setting("subtitle_font_name");
if(ptr)
	{
	free(default_subtitle_font_name);
	default_subtitle_font_name = ptr;
	}
else error = 1;

/* 
create setup files from defaults if some not found,
else use values just read.
*/
if(error) save_general_settings();

return 1;
}/* end function load_general_settings */


int set_tabs(int tabsize)
{
int i;
char *space;

if(debug_flag)
	{
	fprintf(stdout, "set_tabs(): arg tabsize=%d\n", tabsize);
	}

/* argument check */
if(tabsize < 0) return 0;

/* set the global (used by expand_tabs ) */
tab_size = tabsize;

space = malloc(tabsize + 1);
if(! space) return 0;
for(i = 0; i < tabsize; i++)
	{
	space[i] = 'a';
	}
space[i] = 0;

fl_set_tabstop(space);
free(space);

return 1;
}/* end funcion set_tabs */


char *expand_tabs(char *text, int tabsize)
{
int c, i, j, k;
int tabs;
char *space;

if(debug_flag)
	{
	fprintf(stdout, "expand_tabs(): arg text=%s tabsize=%d\n",\
	text, tabsize);
	}

/* argument check */
if(! text) return 0;
if(tabsize < 0) return 0;

/* determine how many tabs, so we can calculate the required space */
i = 0;
tabs = 0;
while(1)
	{
	c = text[i];
	if(c == 0) break;
	if(c == 9) tabs++;
	i++;
	}/* end while all characters in text */

space = malloc(strlen(text) + (tabs * tabsize) - tabs + 1 + 1);
if(! space) return 0;

/* copy and expand */
i = 0;
j = 0;
while(1)
	{
	c = text[i];
	if(c == 9)
		{
		for(k = 0; k < tabsize; k++)
			{
			space[j] = ' ';
			j++;
			}
		i++;
		continue;
		}
	else
		{
		space[j] = c;
		}
	if(c == 0) break;
	i++;
	j++;
	}
return(space);
}/* end function expand_tabs */


int save_project_vars()
{
int i;
FILE *fptr;
char temp[4096];
int dummy;

if(debug_flag)
	{
	fprintf(stdout, "save_project_vars(): arg none\n");
	}

/* save to disk in project dir */
sprintf(temp,\
"%s/.subtitles/%s/vars.dat",\
home_dir, selected_project);
fptr = fopen(temp, "w");
if(! fptr)
	{
	fl_show_alert("could not open file", temp, "for write", 1);
	return 0;
	}

dummy = 0;
fprintf(fptr,\
"%d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %f %d %d %f %d %d %d %d %d %d %f %d %d %d %d %d %d %d %f %f %d %d %d %d %d %d %d %d %d %d %d %d\n",\
frame_memory_1,\
draw_mode,\
selected_start_frame,\
show_video_flag,\
selected_volume,\
text_duration_multiplier,\
output_format,\
default_subtitle_center_flag,\
ppml_use_dvd_colors_flag,\
ppml_enable_monitor_flag,\
default_subtitle_anti_alias_flag,\
default_subtitle_no_kerning_flag,\
audio_mode,\
video_in_preview_flag,\
default_subtitle_extra_character_space,\
dframes_per_second,\
default_subtitle_font_symbols,\
default_subtitle_font_size,\
h_factor,\
default_subtitle_iso_extention,\
h_size,\
default_subtitle_extra_character_vspace,\
h_margin,\
show_time_overlap_flag,\
synthesize_flag,\
v_factor,\
v_size,\
v_margin,\
enable_chapter_list_output_flag, \
use_animation_templates_flag,\
let_microdvdplayer_format_flag,\
preview_flag,\
selected_animation_template_browser_line,\
default_subtitle_blur_radius,\
default_subtitle_outline_thickness,\
dummy,\
dummy,\
dummy,\
default_subtitle_background_color,\
default_subtitle_body_color,\
default_subtitle_outline1_color,\
default_subtitle_outline2_color,\
default_subtitle_background_contrast,\
default_subtitle_body_contrast,\
default_subtitle_outline1_contrast,\
default_subtitle_outline2_contrast,\
dummy\
);

for(i = 0; i < 16; i++)
	{
	fprintf(fptr, "%d %d %d  ", xste_rgb_palette[i][0], xste_rgb_palette[i][1], xste_rgb_palette[i][2]);
	}
fprintf(fptr, "\n");

fclose(fptr);

return 1;
} /* end function save_project_vars */


int load_project_vars()
{
int a, i;
FILE *fptr;
char temp[4096];
int dummy;
int illegal_value;

if(debug_flag)
	{
	fprintf(stdout, "load_project_vars(): arg none\n");
	}

/* default CDR parameters, if file does not exists, or old version */
selected_volume = 50;

/* defaults for subtitle fonts */
default_subtitle_font_symbols = 0;
default_subtitle_font_size = 32;
default_subtitle_iso_extention = 15;

/* outline thickness */
default_subtitle_blur_radius = 2.0;

/* blur radius */
default_subtitle_outline_thickness = 1.5;

default_subtitle_extra_character_space = 2;
default_subtitle_extra_character_space = 0;

default_subtitle_background_color = 0;
default_subtitle_body_color = 9;
default_subtitle_outline1_color = 8;
default_subtitle_outline2_color = 3;

default_subtitle_background_contrast = 0; 
default_subtitle_body_contrast = 15;
default_subtitle_outline1_contrast = 15; 
default_subtitle_outline2_contrast = 0;

h_size = 720;

h_factor = SUBTITLE_H_FACTOR;
h_margin = 28;

v_size = 576;
v_factor = SUBTITLE_V_FACTOR;
v_margin = 24;

output_format = OUTPUT_FORMAT_SUBMUX_DVD;

/* center all text by default */
default_subtitle_center_flag = 1;

/* end defaults */


/* read from  disk in project dir */
sprintf(temp,\
"%s/.subtitles/%s/vars.dat",\
home_dir, selected_project);
fptr = fopen(temp, "r");
if(! fptr)
	{
	fl_show_alert("could not open file", temp, "for read", 1);

	/* set defaults for new project */
	frame_memory_1 = -1;
	sprintf(temp, "MEM\n");
	fl_set_object_label(fdui -> main_form_mem1_button, temp);

	/* set recall (rcl) button label so you can see what you recal */
	sprintf(temp, "RCL\n");
	fl_set_object_label(fdui -> main_form_rcl_button, temp);

	set_vars_values();

	return 0; /* mm ? warn anyways */
	}

a = fscanf(fptr,
"%d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %lf %d %d %lf %d %d %d %d %d %d %lf %d %d %d %d %d %d %d %lf %lf %d %d %d %d %d %d %d %d %d %d %d %d",\
&frame_memory_1,\
&draw_mode,\
&selected_start_frame,\
&show_video_flag,\
&selected_volume,\
&text_duration_multiplier,\
&output_format,\
&default_subtitle_center_flag,\
&ppml_use_dvd_colors_flag,\
&ppml_enable_monitor_flag,\
&default_subtitle_anti_alias_flag,\
&default_subtitle_no_kerning_flag,\
&audio_mode,\
&video_in_preview_flag,\
&default_subtitle_extra_character_space,\
&dframes_per_second,\
&default_subtitle_font_symbols,\
&default_subtitle_font_size,\
&h_factor,\
&default_subtitle_iso_extention,\
&h_size,\
&default_subtitle_extra_character_vspace,\
&h_margin,\
&show_time_overlap_flag,\
&synthesize_flag,\
&v_factor,\
&v_size,\
&v_margin,\
&enable_chapter_list_output_flag, \
&use_animation_templates_flag,\
&let_microdvdplayer_format_flag,\
&preview_flag,\
&selected_animation_template_browser_line,\
&default_subtitle_blur_radius,\
&default_subtitle_outline_thickness,\
&dummy,\
&dummy,\
&dummy,\
&default_subtitle_background_color,\
&default_subtitle_body_color,\
&default_subtitle_outline1_color,\
&default_subtitle_outline2_color,\
&default_subtitle_background_contrast,\
&default_subtitle_body_contrast,\
&default_subtitle_outline1_contrast,\
&default_subtitle_outline2_contrast,\
&dummy\
);

for(i = 0; i < 16; i++)
	{
	fscanf(fptr, "%d %d %d", &xste_rgb_palette[i][0], &xste_rgb_palette[i][1], &xste_rgb_palette[i][2]);
	}

fclose(fptr);

/* sanity checks */
/* prevent divide by zero and segfault in alpha() */
illegal_value = 0;
if(default_subtitle_blur_radius <= 0.0)
	{
	default_subtitle_blur_radius = 0.01;
	illegal_value = 1;
	}
if(default_subtitle_outline_thickness <= 0.0)
	{
	default_subtitle_outline_thickness = 0.01;
	illegal_value = 1;
	}
if(default_subtitle_font_symbols != 0)
	{
	default_subtitle_font_symbols = 0;
	illegal_value = 1;
	}
if(default_subtitle_font_size <= 0)
	{
	default_subtitle_font_size = 32;
	illegal_value = 1;
	}
if(default_subtitle_iso_extention <= 0)
	{
	default_subtitle_iso_extention = 15;
	illegal_value = 1;
	}
if(h_size <= 0)
	{
	h_size = 720;
	illegal_value = 1;
	}
if(v_size <= 0)
	{
	v_size = 576;
	illegal_value = 1;
	}
if(h_factor < 0)
	{
	h_factor = 0.04;
	illegal_value = 1;
	}
if(v_factor < 0.0)
	{
	v_factor = 0.04;
	illegal_value = 1;
	}
if(dframes_per_second <= 0.0)
	{
	dframes_per_second = 25.0;
	illegal_value = 1;
	}
if(text_duration_multiplier <= 0)
	{
	text_duration_multiplier = 7;
	illegal_value = 1;
	}
if( (default_subtitle_extra_character_space < -2) || (default_subtitle_extra_character_space > +10) )
	{
	default_subtitle_extra_character_space = EXTRA_CHAR_SPACE;
	illegal_value = 1;
	}

if( (default_subtitle_extra_character_vspace < -50) || (default_subtitle_extra_character_vspace > +50) )
	{
	default_subtitle_extra_character_vspace = EXTRA_CHAR_VSPACE;
	illegal_value = 1;
	}

if(illegal_value)
	{
	fl_show_alert("load_project_vars():", "strange value detected", "using defaults, check settings", 0);
	}

/* end sanity checks */

set_vars_values();

XSync(fl_get_display(), 0);

return 1;
} /* end function load_project_vars */


int select_frame_list_entry()
{
int s_line;
char temp[80];

if(debug_flag)
	{
	fprintf(stdout, "select_frame_list_entry(): arg none\n");
	}

/* get audio sample and picture number */
if(! get_sample(selected_start_frame, &selected_start_sample) )
	{
	sprintf(temp, "%d", selected_start_frame);
	fl_show_alert("No audio sample associated with frame", temp, "", 1);

	return 0;
	}

if(debug_flag)
	{
	fprintf(stdout,\
	"select_frame_list_entry():\n\
	selected_start_frame=%d,\n\
	selected_start_sample=%lu\n\
	selected_picture=%lu\n",\
	selected_start_frame,\
	selected_start_sample,\
	selected_picture);
	}

/* set the frame display */
set_frame_input(selected_start_frame);

/* set the current_frame display */
set_current_frame_input(selected_start_frame);

/* make line visible in browser */
frame_to_next_line(selected_start_frame, &s_line);

/* select the entry found (maybe be the next valid one) */
fl_select_browser_line(fdui -> main_form_browser, s_line);

/* Display some previous entries if possible */ 
s_line -= 10;
if(s_line < 0) s_line = 0;
fl_set_browser_topline(fdui -> main_form_browser, s_line);

/* just like it was selected in the browser */
clicked_picture = selected_picture;

save_project_vars();

return 1;
} /* end function select_frame_list_entry */


int set_memory_button_labels()
{
char temp[80];

if(debug_flag)
	{
	fprintf(stdout,\
	"set_memory_button_lables(): arg none using frame_memory_1=%d\n",\
	frame_memory_1);
	}

if(frame_memory_1 == -1)
	{
	/* nothing in store */
	sprintf(temp, "MEM\n");
	fl_set_object_label(fdui -> main_form_mem1_button, temp);

	/* nothing in store */
	sprintf(temp, "RCL\n");
	fl_set_object_label(fdui -> main_form_rcl_button, temp);
	}
else
	{
	sprintf(temp, "MEM\n%d", frame_memory_1);
	fl_set_object_label(fdui -> main_form_mem1_button, temp);

	/* set recal (rcl) button label so you can see what you recal */
	sprintf(temp, "RCL\n%d", frame_memory_1);
	fl_set_object_label(fdui -> main_form_rcl_button, temp);
	}

return 1;	
} /* end function set_meory_button_labels */


int set_frame_input(int framenr)
{
char temp[80];

if(debug_flag)
	{
	fprintf(stdout, "set_frame_input(): arg framenr=%d\n", framenr);
	}

frame_input_value = framenr;

sprintf(temp, "%d", framenr);
fl_set_input(fdui -> main_form_frame_input_field, temp); 		
	
return 1;
} /* end function set_frame_input */


int set_current_frame_input(int framenr)
{
char temp[80];

if(debug_flag)
	{
	fprintf(stdout, "set_current_frame_input(): arg framenr=%d\n", framenr);
	}

current_frame_input_value = framenr;

sprintf(temp, "%d", framenr);
fl_set_input(fdui -> main_form_current_frame_input_field, temp); 		

return 1;
} /* end function set_current_frame_input */


int save_selected_fonts()
{
char temp[TEMP_SIZE];

sprintf(temp,\
"%d %d %d",\
selected_articles_font, selected_global_font, fonts_mode);

if(! save_setting("selected_fonts", temp) ) return(0);

return 1;
}/* end function save_selected_fonts */


int load_selected_fonts()
{
char *ptr;
 
ptr = (char *) load_setting("selected_fonts");
if(! ptr) return 0;

sscanf(ptr,\
"%d %d %d",\
&selected_articles_font, &selected_global_font, &fonts_mode);

free(ptr);

/* set the fonts */
if(fonts_mode != DEFAULT_FONTS)
	{
	/* xforms hack */
	set_global_fonts(selected_global_font);
	set_global_fonts(selected_global_font);

	/* xforms hack */
	set_article_fonts(selected_articles_font);
	set_article_fonts(selected_articles_font);
 	}
/* else the style as specified will be used */

return 1;
}/* end function load_selected_fonts */


int set_vars_values()
{
double da;
char temp[4096];

if(debug_flag)
	{
	printf("set_vars_values(): arg none, using globals\n");
	}

/* DISABLED VIDEO */
show_video_flag = 0;

fl_set_button(\
	fdui -> main_form_video_display_button, show_video_flag);

fl_set_button(fdui -> subtitle_font_form_center_text_button, 1 - default_subtitle_center_flag);
set_subtitle_font_form_center_text_button_label(default_subtitle_center_flag);

da = selected_volume / 100.0;
fl_set_slider_value(fdui -> main_form_volume_slider, da);

set_volume(selected_volume);
 
sprintf(temp, "%d", text_duration_multiplier);
fl_set_input(fdui -> main_form_text_duration_input_field, temp);

sprintf(temp, "%.4f", dframes_per_second);
fl_set_input(fdui -> main_form_frames_per_second_input_field, temp);
set_frames_per_second_input_field_color();

dmilli_seconds_per_frame = 1000.0 / dframes_per_second;

/* set mem button labels so you can see what is in 'store' */
set_memory_button_labels();

sprintf(temp, "%d", h_size);
fl_set_input(fdui -> main_form_h_size_input_field, temp);

sprintf(temp, "%d", h_margin);
fl_set_input(fdui -> main_form_h_margin_input_field, temp);

sprintf(temp, "%.2f", h_factor);
fl_set_input(fdui -> main_form_h_factor_input_field, temp);

sprintf(temp, "%d", v_size);
fl_set_input(fdui -> main_form_v_size_input_field, temp);

sprintf(temp, "%d", v_margin);
fl_set_input(fdui -> main_form_v_margin_input_field, temp);

sprintf(temp, "%.3f", v_factor);
fl_set_input(fdui -> main_form_v_factor_input_field, temp);

sprintf(temp, "%d", default_subtitle_font_size);
fl_set_input(fdui -> subtitle_font_form_character_point_size_input_field, temp);

fl_set_button(fdui -> main_form_preview_button, preview_flag);

/* NOTE: at start up this button is released (no file selected) */
video_in_preview_flag = 0;
//fl_set_button(fdui -> main_form_video_in_preview_button, video_in_preview_flag);

fl_set_button(fdui -> subtitle_font_form_anti_alias_button, default_subtitle_anti_alias_flag);
fl_set_button(fdui -> subtitle_font_form_no_kerning_button, default_subtitle_no_kerning_flag);

sprintf(temp, "%d", default_subtitle_iso_extention);
fl_set_input(fdui -> subtitle_font_form_character_style_input_field, temp);

fl_set_button(fdui -> main_form_show_time_overlap_button, show_time_overlap_flag);

fl_set_button(fdui -> main_form_synthesize_switch, synthesize_flag);
if(synthesize_flag)
	{
	fl_set_object_label(fdui -> main_form_synthesize_switch, "synthesize");
	}
else
	{
	fl_set_object_label(fdui -> main_form_synthesize_switch, "synthesize off");
	}

/* export form */
switch(output_format)
	{
	case OUTPUT_FORMAT_PPML_FOR_SUBTITLER_YUV:
		fl_set_button(fdui -> export_form_enable_ppml_for_subtitler_yuv_output_button, 1);
		break;
	case OUTPUT_FORMAT_PPML:
		fl_set_button(fdui -> export_form_enable_ppml_output_button, 1);
		break;
	case OUTPUT_FORMAT_SUTI:
		fl_set_button(fdui -> export_form_enable_suti_output_button, 1);
		break;
	case OUTPUT_FORMAT_SSA:
		fl_set_button(fdui -> export_form_enable_ssa_output_button, 1);
		break;
	case OUTPUT_FORMAT_VPLAYER:
		fl_set_button(fdui -> export_form_enable_vplayer_output_button, 1);
		break;
	case OUTPUT_FORMAT_SUBMUX:
		fl_set_button(fdui -> export_form_enable_submux_output_button, 1);
		break;
	case OUTPUT_FORMAT_SUBMUX_DVD:
		fl_set_button(fdui -> export_form_enable_submux_dvd_output_button, 1);
		break;
	case OUTPUT_FORMAT_MICRO_DVD_PLAYER:
		fl_set_button(fdui -> export_form_enable_microdvdplayer_output_button, 1);
		break;
	case OUTPUT_FORMAT_I_AUTHOR:
		fl_set_button(fdui -> export_form_enable_iauthor_output_button, 1);
		break;
	case OUTPUT_FORMAT_EBU_STL:
		fl_set_button(fdui -> export_form_enable_ebu_stl_output_button, 1);
  		break;
	case OUTPUT_FORMAT_SRT:
		fl_set_button(fdui -> export_form_enable_srt_output_button, 1);
  		break;
	} /* end switch formats */

fl_set_button(fdui -> export_form_ppml_use_dvd_colors_button, ppml_use_dvd_colors_flag);

fl_set_button(fdui -> export_form_ppml_enable_monitor_button, ppml_enable_monitor_flag);

fl_set_button(fdui -> export_form_let_microdvdplayer_format_button, let_microdvdplayer_format_flag);

fl_set_button(fdui -> export_form_enable_chapter_list_output_button, enable_chapter_list_output_flag);

fl_set_button(fdui -> export_form_use_animation_templates_button, use_animation_templates_flag);

/* end export form */

/* signal to idle_cb to select this entry */
select_frame_list_entry_flag = 1;

/* set audio mode button state */
if(audio_mode == AUDIO_RAW)
	{
	fl_set_object_label(fdui -> main_form_audio_mode_button, "AUDIO RAW");
	fl_set_object_color(\
		fdui -> main_form_audio_mode_button, FL_GREEN, FL_GREEN);
	}
else if(audio_mode == AUDIO_WAV)
	{
	fl_set_object_label(fdui -> main_form_audio_mode_button, "AUDIO WAV");
	fl_set_object_color(\
		fdui -> main_form_audio_mode_button, FL_YELLOW, FL_YELLOW);
	}
else if(audio_mode == AUDIO_MP3)
	{
	fl_set_object_label(fdui -> main_form_audio_mode_button, "AUDIO MP3");
	fl_set_object_color(\
		fdui -> main_form_audio_mode_button, FL_DARKORANGE, FL_DARKORANGE);
	}

if(! set_default_subtitle_properties() ) return 0;

if(! global_subtitle_palette_form_set_all_color_buttons() ) return 0;

/* for animation_templates_form */
fl_set_positioner_xbounds(fdui -> animation_template_form_x_y_positioner, 0, h_size);
fl_set_positioner_ybounds(fdui -> animation_template_form_x_y_positioner, 0, v_size);

return 1;
} /* end function set_vars_values */


int set_volume(int volume)
{
FILE *pptr;
char temp[1024];

if(debug_flag)
	{
	fprintf(stderr, "set_volume(): arg volume=%d\n", volume);
	}

if(volume < 0) return 0;
if(volume > 100) return 0;

sprintf(temp, "cmix %s pcm %d\n", AUDIO_MIXER_DEVICE, volume);

if(debug_flag)
	{
	fprintf(stderr, "set_volume(): command=%s\n", temp);
	}

pptr = popen(temp, "r");
pclose(pptr);

return 1;
} /* end function setz_volume */

