/*
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "xste.h"

FD_xste *fd_main_form;
extern FD_xste *fdui;


int clean_text(char *text, char *cleaned_text)
{
int c, ii, io;

if(debug_flag)
	{
	fprintf(stdout, "clean_text(): arg text=%s cleaned_text=%p\n", text, cleaned_text);
	}

if(! text) return 0;
if(! cleaned_text) return 0;

ii = 0;
io = 0;
while(1)
	{
	c = text[ii];
	cleaned_text[io] = c;

	/* all bash characters that could cause trouble, add as required */
	if( (c == '!') || (c == '&') || (c == '\'') || (c == '/') || (c == '$') || (c == '~') || (c == '\"') )
		{
		/* stay at position for later overwrite */
		}	
	else
		{
		io++;
		}

	if(c == 0) break;
	ii++;
	}

if(debug_flag)
	{
	printf("cleaned_text(): cleaned_text=%s\n", cleaned_text);
	}

return 1;
} /* end function clean_text */

 
int extract_command(char *text, char *text_part, char *command_part)
{
int c, ii, it, ic;
int in_command_flag;

if(debug_flag)
	{
	fprintf(stdout, "extract_command(): arg text=%s text_part=%p command_part=%p\n",\
	text, text_part, command_part);
	}

if(! text) return 0;
if(! text_part) return 0;
if(! command_part) return 0;

ii = 0;
ic = 0;
it = 0;
in_command_flag = 0;
while(1)
	{
	c = text[ii];	
	if(c == '[')
		{
		in_command_flag = 1;

		ii++;
		continue;
		}

	if(in_command_flag)
		{
		if(c == ']')
			{
			in_command_flag = 0;

			command_part[ic] = 0;

			ii++;
			continue;
			}

		command_part[ic] = c;
		ic++;
		} /* end if in_command_flag */
	else
		{
		text_part[it] = c;
		it++;
		}	

	if(c == 0)
		{
		text_part[it] = 0;
		command_part[ic] = 0;

		break;
		}
	ii++;
	}

return 1;
} /* end function extract_command */


int speak_text(char *text)
{
char command_part[TEXT_ARRAY_SIZE];
char text_part[TEXT_ARRAY_SIZE];
char cleaned_text_part[TEXT_ARRAY_SIZE];
char temp[4096];
FILE *pptr;

if(debug_flag)
	{
	fprintf(stdout, "speak_text(): arg text=%s\n", text);
	}

if(! text) return 0;

/* split of any command in the form of '[female] */
if(! extract_command(text, text_part, command_part) ) return 0;

/* remove any characters that would cause problems with bash, such as '!' */
if(! clean_text(text_part, cleaned_text_part) ) return 0;

/* in background because called from idle_cb */
sprintf(temp,\
"%s/%s/%s/xste_synthesize \"%s\" \"%s\" &",\
home_dir, selected_data_directory, selected_project,\
cleaned_text_part, command_part);

if(debug_flag)
    {
    fprintf(stdout, "speak_text(): executing=%s\n", temp);
    }

pptr = popen(temp, "w");
pclose(pptr);

return 1;
} /* end function speak_text */


int text_to_wave(int frame, char *text)
{
char temp[4096];
FILE *pptr;
char command_part[TEXT_ARRAY_SIZE];
char text_part[TEXT_ARRAY_SIZE];
char cleaned_text_part[TEXT_ARRAY_SIZE];
if(debug_flag)
	{
	fprintf(stdout, "text_to_wave(): arg frame=%d text=%s\n", frame, text);
	}

if(! text) return 0;
if(frame < 0) return 0;

/* split of any command in the form of '[female] */
if(! extract_command(text, text_part, command_part) ) return 0;

/* remove any characters that would cause problems with bash, such as '!' */
if(! clean_text(text_part, cleaned_text_part) ) return 0;

/* not in background, because we want to wait for the synthesizing to finish */
sprintf(temp,\
"%s/%s/%s/xste_synthesize \"%s\" \"%s\" \"%s/%s/%s/frame%04d.wav\"",\
home_dir, selected_data_directory, selected_project,\
cleaned_text_part, command_part,\
home_dir, selected_data_directory, selected_project, frame);

if(debug_flag)
	{
	fprintf(stdout, "text_to_wave(): executing=%s\n", temp);
	}

pptr = popen(temp, "w");
pclose(pptr);

return 1;
} /* end function text_to_wave */



int write_synthesized_wave_files()
{
struct frame *pa;

if(debug_flag)
	{
	printf("write_synthesized_wave_files(): arg none\n");
	}

for(pa = frametab[0]; pa != 0; pa = pa -> nxtentr)
	{
	if(debug_flag)
		{
		printf("frame=%d pa->text=%s\n", atoi(pa -> name), pa -> text);
		}

	/* do not process negative frames, this results in negative times. */
	if( atoi(pa -> name) < 0) continue;

	/* do not process command lines in the form '*command' */
//	if(pa -> text[0] == '*') continue;

	/* do not process empty lines */
	if(pa -> text[0] == 0) continue;

	if(! text_to_wave( atoi(pa -> name), pa -> text) )
		{
		fl_show_alert("Could not convert text for frame", pa -> name, "to wave file", 0);
		}
	} /* end for all frames */

return 1;
} /* end function write_synthesized_wave_files */


int combine_synthesized_wave_files()
{
int a;
struct frame *pa;
FILE *ifptr, *ofptr;
char temp[4096];
wave_header *ifile_header, *ofile_header;
size_t length, ta, ifile_data_length, bytes_read, bytes_written, bytes_to_write, total_bytes_written;
int header_size;
int wave_file_flag;
unsigned char *buffer;
int frame;
double dframe_milli_seconds;
double dcurrent_miliseconds;
int start_hour, start_minute, start_second, start_hundreds;
long lthousands;
int fstart_hour, fstart_minute, fstart_second, fstart_hundreds;
long lfthousands;
int bytes_per_milli_second;
int underflows;
size_t sum_of_all_bytes_written;
int synthesizer_sample_frequency, synthesizer_bits_per_sample;

if(debug_flag)
	{
	fprintf(stdout, "combine_synthesized_wave_files(): arg none\n");
	}


synthesizer_sample_frequency = 0; // -Wall
synthesizer_bits_per_sample = 0; // -Wall

/*
To automatically adapt to unknown sythesizer speed and bit per sample, we
grab synthesizer_sample_frequency and synthesizer_bits_per_sample from
the first framennn.wav.
*/

/* get size of wave header */
header_size = sizeof(wave_header);

/* find first wave file and get speed and bits per sample */
for(pa = frametab[0]; pa != 0; pa = pa -> nxtentr)
	{
	/* do not process negative frames, this results in negative times. */
	if( atoi(pa -> name) < 0) continue;

	/* do not process xste command lines in the form '*command' */
//	if(pa -> text[0] == '*') continue;

	/* do not process empty lines */
	if(pa -> text[0] == 0) continue;

	sprintf(temp,\
	"%s/%s/%s/frame%04d.wav",\
	home_dir, selected_data_directory, selected_project, atoi(pa -> name) );

	ifptr = fopen(temp, "r");
	if(! ifptr)
		{
		fl_show_alert("synthesize: missing wave file", temp, "aborting", 0);

		return 0;
		}

	/* read header */	

	/* allocate space for input wave header */
	ifile_header = calloc( sizeof(wave_header), 1);
	if(! ifile_header)
		{
		fprintf(stderr, "combine_synthesized_wave_files(): get params: could not calloc space for wave header.\n");

		fclose(ifptr);

		return 0;
		}

	a = fread(ifile_header, sizeof(char), header_size, ifptr);
	if(a < header_size)
		{   
		fl_show_alert("synthesize: file", temp, "could not read wave header, aborting.", 0);    

		fclose(ifptr);
		free(ifile_header);

		return 0;
		}

	/* get speed and bits per sample */
	synthesizer_sample_frequency = ifile_header -> sample_fq;
	synthesizer_bits_per_sample = ifile_header -> bit_p_spl;

	free(ifile_header);
	fclose(ifptr);

	if(debug_flag)
		{
		printf(\
		"combine_synthesized_wave_files(): from %s found:\n\
		synthesizer_sample_frequency=%d\n\
		synthesizer_bits_per_sample=%d\n",\
		temp, synthesizer_sample_frequency, synthesizer_bits_per_sample);
		}

	break;
	} /* end for all entries in frame list */

/* open combined wave file for write */
sprintf(temp, "%s/%s/%s/synthesized.wav", home_dir, selected_data_directory, selected_project);
ofptr = fopen(temp, "w");
if(! ofptr)
	{
	fl_show_alert("Could not open file", temp, "for write, command cancelled", 0);

	return 0;
	}

if(debug_flag)
	{
	fprintf(stderr, "write_file(): wave header_size=%d\n", header_size);
	}

/* allocate space for wave header */
ofile_header = calloc( sizeof(wave_header), 1);
if(! ofile_header)
	{
	fprintf(stderr, "combine_synthesized_wave_files(): ERROR could not calloc space for output file_header.\n");

	fclose(ofptr);
	return 0;
	}

/*
Set a fake long length in the header, if we abort for any reason,
then at least we can play what we have for test this way (with only early EOF error).
*/
length = 1000000000; // 1 GB, length fields will be modified later!

strcpy(ofile_header -> main_chunk, "RIFF");
ofile_header -> length = length + header_size - 8;
strcpy(ofile_header -> chunk_type, "WAVE"); 
strcpy(ofile_header -> sub_chunk, "fmt ");		// watch out the zero termination overwrites next var
ofile_header -> length_chunk = 16;								//always 16
ofile_header -> format = 1;										//PCM
ofile_header -> modus = 1;										// 1 mono, 2 stereo
ofile_header -> sample_fq = synthesizer_sample_frequency;		// 44100, 48000, etc...
ofile_header -> byte_p_sec = synthesizer_sample_frequency * synthesizer_bits_per_sample / 8; // 32000
ofile_header -> byte_p_spl = synthesizer_bits_per_sample / 8;	// 2
ofile_header -> bit_p_spl = synthesizer_bits_per_sample;		// 16 bits
strcpy(ofile_header -> data_chunk, "data");		// watch out the zero termination overwrites next var
ofile_header -> data_length = (unsigned long)length;

bytes_per_milli_second = ofile_header -> byte_p_sec / 1000;

if(debug_flag)
	{
	printf("combine_synthesized_wave_files():\n");
	printf("ofile_header->sample_fq=%d\n",  ofile_header -> sample_fq);
	printf("ofile_header->byte_p_sec=%d\n", ofile_header -> byte_p_sec);
	printf("ofile_header->byte_p_spl=%d\n", ofile_header -> byte_p_spl);
	printf("ofile_header->bit_p_spl=%d\n", ofile_header -> bit_p_spl);

	printf("ofile_header -> data_length=%lu\n", (unsigned long)ofile_header -> data_length);
	printf("ofile_header -> length %lu\n", (unsigned long)ofile_header -> length);
	printf("bytes_per_milli_second=%d\n", bytes_per_milli_second);
	}

/* write out wave header */
ta = fwrite(ofile_header, sizeof(char), header_size, ofptr);
if(ta != header_size)
	{
	fprintf(stderr,\
	"combine_synthesized_wave_files(): could only write %lu of %d bytes of out header, aborting.\n",\
	(unsigned long) ta, header_size);   

	free(ofile_header);
	fclose(ofptr);	
	return 0;
	}

dcurrent_miliseconds = 0.0;
underflows = 0;
sum_of_all_bytes_written = 0;
for(pa = frametab[0]; pa != 0; pa = pa -> nxtentr)
	{
	/* calculate start_time */
	lthousands = (long) dcurrent_miliseconds;
	
	start_hour = lthousands / 3600000;
	lthousands -= start_hour * 3600000; 
	start_minute = lthousands / 60000;
	lthousands -= start_minute * 60000;
	start_second = lthousands / 1000;
	lthousands -= start_second * 1000;
	start_hundreds = lthousands / 10;
	
	/* calculate start_time */
	lfthousands = (long) ( atof(pa -> name) * dmilli_seconds_per_frame );
	
	fstart_hour = lfthousands / 3600000;
	lfthousands -= fstart_hour * 3600000; 
	fstart_minute = lfthousands / 60000;
	lfthousands -= fstart_minute * 60000;
	fstart_second = lfthousands / 1000;
	lfthousands -= fstart_second * 1000;
	fstart_hundreds = lfthousands / 10;
	
	if(debug_flag)
		{
		printf("\ncombine_synthesized_wave_files():\n");
		printf("frame=%d pa->text=%s\n", atoi(pa -> name), pa -> text);
		printf("dcurrent_miliseconds=%.2f\n", dcurrent_miliseconds);
		
		printf("frame time= %02d:%02d:%02d:%02d\n", fstart_hour, fstart_minute, fstart_second, fstart_hundreds);
		printf("audio time= %02d:%02d:%02d:%02d\n", start_hour, start_minute, start_second, start_hundreds);
		}

	/* do not process negative frames, this results in negative times. */
	if( atoi(pa -> name) < 0) continue;

	/* do not process xste command lines in the form '*command' */
//	if(pa -> text[0] == '*') continue;

	/* do not process empty lines */
	if(pa -> text[0] == 0) continue;

	/* first valid frame! */
	/* from current_postion up until the position of this frame insert silence in output file */

	/* frame */
	frame = atoi(pa -> name);

	/* time in milliseconds */
	dframe_milli_seconds = dmilli_seconds_per_frame * (double) frame;

	if(debug_flag)
		{
		printf(\
		"dframe_milli_seconds=%.2f dcurrent_miliseconds=%.2f\n",\
		dframe_milli_seconds, dcurrent_miliseconds);
		}

	total_bytes_written = 0;

	/*
	NOTE:
	If synthesizer speaks to slowly, we will not keep up, and lag more and more behind!
	Test after resampling to required speed and muxing?
	*/
	if(dcurrent_miliseconds > dframe_milli_seconds)
		{
		if(debug_flag)
			{
			printf(\
			"combine_synthesized_wave_files(): WARN speaker lagging by %.2f milli seconds!!!\n",\
			dframe_milli_seconds - dcurrent_miliseconds);
			}

		underflows++;
		}

	/* add silence only if space for it */
	if(dcurrent_miliseconds < dframe_milli_seconds)
		{
		/* samples, we have 16000 per second, 16 per milli second, that makes 32 bytes per milli second */
		bytes_to_write = bytes_per_milli_second * (int) (dframe_milli_seconds - dcurrent_miliseconds);

		if(debug_flag)
			{
			printf("inserting %.2f milliseconds silence\n", dframe_milli_seconds - dcurrent_miliseconds);

			printf("frame=%d dmilli_seconds_per_frame=%.2f bytes_to_write=%d\n",\
			frame, dmilli_seconds_per_frame, bytes_to_write);
			}
			
		/* write silence */
		buffer = calloc(bytes_to_write, 1);
		if(! buffer)
			{
			fprintf(stderr,\
			"xste: combine_synthesized_wave_files(): silence could not calloc %d bytes for buffer, aborting\n",\
			bytes_to_write);

			return 0;
			}

		bytes_written = fwrite(buffer, 1, bytes_to_write, ofptr);
		if(bytes_written != bytes_to_write)
			{
			fprintf(stdout,\
			"xste: combine_synthesized_wave_files(): silence: could only write %lu of %lu bytes, aborting\n",\
			(unsigned long)bytes_written, (unsigned long)bytes_to_write);

			return 0;
			}

		if(debug_flag)
			{
			printf(\
			"xste: combine_synthesized_wave_files(): after silence dcurrent_miliseconds=%.2f\n",\
			bytes_written / (double)bytes_per_milli_second);
			}

		total_bytes_written += bytes_written;
		} /* end if space for silence */

	/* open wave file */
	sprintf(temp,\
	"%s/%s/%s/frame%04d.wav",\
	home_dir, selected_data_directory, selected_project, atoi(pa -> name) );

	ifptr = fopen(temp, "r");
	if(! ifptr)
		{
		fl_show_alert("synthesize: could not open file", temp, "for read, aborting.", 0);

		fclose(ofptr);
		free(ofile_header);

		return 0;
		}

	/* allocate space for input wave header */
	ifile_header = calloc( sizeof(wave_header), 1);
	if(! ifile_header)
		{
		fprintf(stderr, "combine_synthesized_wave_files(): could not calloc space for input file_header.\n");

		fclose(ifptr);
		free(ifile_header);

		fclose(ofptr);
		free(ofile_header);

		return 0;
		}

	a = fread(ifile_header, sizeof(char), header_size, ifptr);
	if(a < header_size)
		{   
		fl_show_alert("synthesize: file", temp, "could not read wave header, aborting.", 0);    

		fclose(ifptr);
		fclose(ofptr);

		free(ifile_header);
		free(ofile_header);

		return 0;
		}

	if(ifile_header -> data_length == 0)
		{
		fl_show_alert("syntesize: file", temp, "has zero length, aborting.", 0);

		fclose(ifptr);
		free(ifile_header);

		fclose(ofptr);
		free(ofile_header);

		return 0;
		}

	if(debug_flag)
		{
		printf(\
		"combine_synthesized_wave_files(): ifile_header->sample_fq=%d\n",\
		ifile_header -> sample_fq);
		}

	/* test if a wave file */
	wave_file_flag = 1;
	//	char main_chunk[4];		/* 'RIFF' */
	if(strncmp(ifile_header -> main_chunk, "RIFF", 4) != 0) wave_file_flag = 0;

	//	char chunk_type[4];		/* 'WAVE' */
	if(strncmp(ifile_header -> chunk_type, "WAVE", 4) != 0) wave_file_flag = 0;

	//	char sub_chunk[4];		/* 'fmt ' */
	if(strncmp(ifile_header -> sub_chunk, "fmt ", 4) != 0) wave_file_flag = 0;

	//	char data_chunk[4];		/* 'data' */
	if(strncmp(ifile_header -> data_chunk, "data", 4) != 0) wave_file_flag = 0;

	if(! wave_file_flag)
		{
		fclose(ifptr);
		free(ifile_header);

		fclose(ofptr);
		free(ofile_header);

		fl_show_alert("synthesize: file", temp, "is no wave file, aborting.", 0);
		return 0;
		}

	/* get data length */
	ifile_data_length = (unsigned long) ifile_header -> data_length;

	if(debug_flag)
		{
		printf(\
		"combine_synthesized_wave_files(): file %s ifile_data_length=%lu\n",\
		temp, (unsigned long)ifile_data_length);

		printf("inserting %.2f milliseconds speech\n",\
		ifile_data_length  / (double)bytes_per_milli_second);
		}

	buffer = calloc( 2 * ifile_data_length, 1); /* 2 bytes per sample */
	if(! buffer)
		{
		printf(\
		"xste: combine_synthesized_wave_files(): could not calloc %d bytes for buffer, aborting\n",\
		2 * ifile_data_length);

		fclose(ifptr);
		free(ifile_header);

		fclose(ofptr);
		free(ofile_header);

		return 0;
		}		
	
	/* copy source to dest */
	bytes_read = fread(buffer, 1, ifile_data_length, ifptr);	

	/* close input file */
	free(ifile_header);
	fclose(ifptr);

	bytes_written = fwrite(buffer, 1, bytes_read, ofptr);
	if(bytes_written != bytes_read)
		{
		fprintf(stdout,\
		"xste: combine_synthesized_wave_files(): copy: could only write %lu of %lu bytes, aborting\n",\
		(unsigned long)bytes_written, (unsigned long)bytes_read);

		fclose(ifptr);
		free(ifile_header);

		fclose(ofptr);
		free(ofile_header);

		return 0;
		}

	free(buffer);

	total_bytes_written += bytes_written;

	if(debug_flag)
		{
		printf(\
		"combine_synthesized_wave_files(): bytes_written=%d total_bytes_written=%d\n",\
		bytes_written, total_bytes_written);
		}

	/* update current position */
	dcurrent_miliseconds += total_bytes_written / (double)bytes_per_milli_second;

	sum_of_all_bytes_written += total_bytes_written;

	} /* end for all frames */

/* modify header for length */
a = fseek(ofptr, 0, SEEK_SET);
if(a == -1)
	{
	fl_show_alert(\
	"Could not seek to start of wave file",\
	"could not set length fields in header",\
	"using 1 Gb",\
	0);
	}

/* set length fields */
ofile_header -> length = sum_of_all_bytes_written + header_size - 8;
ofile_header -> data_length = (unsigned long)sum_of_all_bytes_written;

/* re-write out wave header */
ta = fwrite(ofile_header, sizeof(char), header_size, ofptr);
if(ta != header_size)
	{
	fl_show_alert(\
	"could not write wave header",\
	"could not set length fields in header",\
	"using 1 Gb",\
	0);
	}

/* close output file */
fclose(ofptr);

free(ofile_header);

if(debug_flag)
	{
	printf("combine_synthesized_wave_files(): there were %d underflows in audio\n", underflows);
	}

if(underflows)
	{
	sprintf(temp, "%d", underflows);
	fl_show_alert("synthesize: there were", temp, "delayed entries in audio", 0);
	}

return 1;
} /* end function combine_synthesized_wave_files */


