/* 
xste subtitle editor
xste  is copyright (C) 2000 <Jan Panteltje>
email: panteltje@yahoo.com

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "xste.h"

extern FD_xste *fdui;


int load_srt_file(char *pathfilename)
{
FILE *fptr;

if(debug_flag)
	{
	fprintf(stdout,\
	"load_srt_file(): arg pathfilename=%s\n", pathfilename);
	}

/* argument check */
if(! pathfilename) return 0;

fptr = fopen(pathfilename, "r");
if(! fptr)
	{
	fl_show_alert("Could not open file", pathfilename, "for read", 1);

	return 0;
	}

/* load the .srt file into the structure list */
if(! read_in_srt_file(fptr) )
	{

	return 0;
	}

//save_project_vars();

show_frames(SHOW_TOP);

return 1;
} /* end  function load_srt_file */



int read_in_srt_file(FILE *finptr)
{
int a, b;
char temp[READSIZE];
char text[READSIZE];
int text_hold_flag;
int start_hour, start_minute, start_second, start_milli_second;
int end_hour, end_minute, end_second, end_milli_second;
double dstart_milli_seconds, dend_milli_seconds;
int start_framenr = 0; // -Wall
int end_framenr = 0; // -Wall
unsigned long start_sample, end_sample;
int line;
int mode;
int entry_number;

#define SRT_EXPECT_ENTRY_ONE		0
#define SRT_EXPECT_ENTRY_NUMBER		1
#define SRT_EXPECT_TIME				2
#define SRT_EXPECT_TEXT				3

/* clear the frame_list */
delete_all_frames();

/* start reading lines from the srt file */
line = 0;
entry_number = 1;
mode = SRT_EXPECT_ENTRY_ONE;
while(1)
	{
	/* removes any MSDOS CR */
	a = readline_msdos(finptr, temp); /* closes file if EOF */
	if(a == EOF) break;	

	line++;

	if(debug_flag)
		{
		fprintf(stdout, "load_srt_file(): line=%d mode=%d entry_number=%d temp=%s temp[0]=%d\n",\
		line, mode, entry_number, temp, temp[0]);
		}

	if(temp[0] == 0)
		{
		if(mode == SRT_EXPECT_ENTRY_ONE) continue;
		if(mode == SRT_EXPECT_ENTRY_NUMBER) continue;
		if(mode == SRT_EXPECT_TIME)	continue;
		}

	if(mode == SRT_EXPECT_ENTRY_ONE)
		{
		if(strcmp(temp, "1") != 0) continue;

		mode = SRT_EXPECT_TIME;
		continue;
		}	

	if(mode == SRT_EXPECT_ENTRY_NUMBER)
		{
		a = sscanf(temp, "%d", &b);

		if(a != 1) 
			{
			sprintf(temp, "%d", line);
			fl_show_alert("format error in srt file line", temp, "expecting entry number", 0);

			continue;

			fclose(finptr);
			return 0;
			}
		
		if(b != entry_number + 1)
			{
			if(debug_flag)
				{
				fprintf(stdout,\
				"load_srt_file(): line=%d temp=%s entry_number=%d b=%d\n", line, temp, entry_number, b); 
				}

			sprintf(temp, "line %d, expecting entry %d, but found %d", line, entry_number + 1, b);

			fl_show_alert("format error in srt file line", temp, "wrong entry number", 0);

			fclose(finptr);
			return 0;
			}	

		entry_number = b;

		mode = SRT_EXPECT_TIME;

		continue;
		}

/*
SRT example:

4M
00:01:29,019 --> 00:01:31,294M
Dieu Tout Puissant,M
nous sommes enfin libres !M
M
*/

	if(mode == SRT_EXPECT_TIME)
		{
		a = sscanf(temp, "%d:%d:%d,%d --> %d:%d:%d,%d",
		&start_hour, &start_minute, &start_second, &start_milli_second,\
		&end_hour, &end_minute, &end_second, &end_milli_second);				

		if(a != 8)
			{
			sprintf(temp, "line %d arguments=%d", line, a);
			fl_show_alert("File format error in srt file", temp, "time field expected", 0);

			fclose(finptr);
			return 0;
			}

		dstart_milli_seconds =\
		3600000 * start_hour +\
		60000 * start_minute +\
		1000 * start_second +\
		start_milli_second;

		dend_milli_seconds =\
		3600000 * end_hour +\
		60000 * end_minute +\
		1000 * end_second +\
		end_milli_second;

		if(debug_flag)
			{
			printf("load_srt_file(): \n");
			printf("temp=%s\n", temp);
			printf(\
			"start_hour=%d start_minute=%d start_second=%d start_milli_second=%d\n\
			dstart_milli_seconds=%.2f\n",\
			start_hour, start_minute, start_second, start_milli_second,\
			dstart_milli_seconds);

			printf(\
			"end_hour=%d end_minute=%d end_second=%d end_mili_second=%d\n\
			dend_milli_seconds=%.2f\n",\
			end_hour, end_minute, end_second, end_milli_second,\
			dend_milli_seconds);
			}

		/* calculate parameters */
		/* 20 mS per frame, 160 mS per frame, if 25.0 frames / second */

		/* start frame */
		start_framenr = dstart_milli_seconds / dmilli_seconds_per_frame;

		/* sanity check */
		if(dend_milli_seconds <= dstart_milli_seconds)
			{
			/*
			try to fix,
			leave on screen for 4 seconds, or until next title appears
			*/

			dend_milli_seconds = dstart_milli_seconds + 4000.0;	
			
			sprintf(temp, "line %d subtitle number %d start frame %d", line, entry_number, start_framenr); 

			fl_show_alert(temp, "end time <= start time", "fixed to + 4 seconds or next sub", 0);
			}

		/* end frame */
		end_framenr = dend_milli_seconds / dmilli_seconds_per_frame;

		if(debug_flag)
			{
			fprintf(stdout,\
			"load_srt_file(): start_frame=%d end_frame=%d\n",\
			start_framenr, end_framenr);
			}

		/* start_sample */		
		start_sample = (dstart_milli_seconds * RAW_SAMPLE_SPEED) / 1000.0;

		/* end_sample */
		end_sample = (dend_milli_seconds * RAW_SAMPLE_SPEED) / 1000.0 ;

		if(debug_flag)
			{
			fprintf(stdout,\
			"load_srt_file(): start_sample=%lu end_sample=%lu\n",\
			start_sample, end_sample);
			}

		text[0] = 0;

		mode = SRT_EXPECT_TEXT;

		continue;
		} /* end if mode is expect time */

	if(mode == SRT_EXPECT_TEXT)
		{
		if(temp[0] != 0)
			{
			if(text[0] == 0) strcpy(text, temp);
			else
				{
				strcat(text, "\\");
				strcat(text, temp);
				}

			continue;
			}		
		}

	/* an empty line signals end of text entry */
	if(temp[0] == 0) mode = SRT_EXPECT_ENTRY_NUMBER;

	/* text_hold_flag */
	text_hold_flag = 0; /* not supported in srt? */

	/* add entry to structure */
	if(! add_frame(\
	text, start_framenr, end_framenr,\
	text_hold_flag) )
		{
		fprintf(stdout, "Could not add_frame, aborting\n");

		exit(1);
		}

	} /* end while all lines in .srt file */

/* sort the double linked list with criterium time */
sort_frames();

return 1;
} /* end function read_in_srt_file */


int write_srt_list()
{
/*
purpose of this routine is to provide srt output, for for example DVD-lab PRO (windows).
srt ?? uses a time oriented format, see MPlayer-0.50/subreader.c
*/
struct frame *pa;
struct frame *pnext, *ptemp;
FILE *fptr;
char temp[4096];
int next_frame_start;
int start_hour, start_minute, start_second, end_hour, end_minute, end_second;
double dstart_second, dend_second;
long lthousands;
int start_milli_second, end_milli_second;
int text_end_frame = 0; // -Wall
int entry_number;
char text_part[TEXT_ARRAY_SIZE];
char command_part[TEXT_ARRAY_SIZE];

if(debug_flag)
	{
	printf("write_srt_list(): arg none\n");
	}

draw_mode = DRAW_MODE_BW;

/* some defaults */
/* end defaults */

/* 40 ms per frame (of 25 Hz) */
//audio_samples_per_frame = audio_sample_speed / dmilli_seconds_per_frame;

/* had to imagine some extention name 'srt' */
sprintf(temp, "%s/%s/%s/%s.srt",\
home_dir, selected_data_directory, selected_project, selected_project);
fptr = fopen(temp, "w");
if(! fptr)
	{
	fl_show_alert("write_srt_list", "Could not open for write", temp, 1);

	return 0;
	}

entry_number = 1;
for(pa = frametab[0]; pa != 0; pa = pa -> nxtentr)
	{
	/*
	do not print negative frames, this results in negative times,
	not sure srt likes that.
	*/
	if( atoi(pa -> name) < 0) continue;

	/* get pointer to next structure entry */ 
	if(pa -> nxtentr) 
		{
		pnext = pa -> nxtentr;	
		
		/* find next line that is not a '*' comand line */
		while(pnext)
			{
			if(pnext -> text[0] != '*') break;

			if(pnext -> nxtentr)
				{	
				ptemp = pnext -> nxtentr;

				pnext = ptemp;
				}	
			} /* end while looking for non '*' entry */ 

		if(pnext)
			{
			/* get the start of the next frame */ 
			next_frame_start = atoi(pnext -> name);
		
			/* test if the current frame ends before the start of the next */
			if(pa -> end_frame > next_frame_start)
				{
				text_end_frame = next_frame_start;
				} /* end if frame ends before next one starts */ 
			}

		} /* end if not last entry */	
	
	/* we have this data at our disposal */ 
	//	char *name;/* this is the start frame */
	//	char *text;
	//	int end_frame; /* this is the end frame */
	//	unsigned long start_sample;
	//	unsigned long end_sample;
 
	/*
	We can time to the sound file, at audio_sample_speed samples / second,
	(actually 11025).
	Some players sync to the audio, some to the video.
	*/

	/* calculate start_time */
	lthousands = atoi(pa -> name) * dmilli_seconds_per_frame;
	start_hour = lthousands / 3600000;
	lthousands -= start_hour * 3600000; 
	start_minute = lthousands / 60000;
	lthousands -= start_minute * 60000;
	dstart_second = lthousands / 1000.0;

	start_second = lthousands / 1000; 
	start_milli_second = lthousands - (1000 * start_second);	

	/* calculate end_time */
	lthousands = text_end_frame * dmilli_seconds_per_frame;
	end_hour = lthousands / 3600000;
	lthousands -= end_hour * 3600000; 
	end_minute = lthousands / 60000;
	lthousands -= end_minute * 60000;
	dend_second = lthousands / 1000.0;

	end_second = lthousands / 1000; 
	end_milli_second = lthousands - (1000 * end_second);	

	/*
	do not print negative frames, this results in negative times,
	not sure srt likes that.
	*/
	if(start_hour < 0) continue;
	if(start_minute < 0) continue;
	if(dstart_second < 0.0) continue;
	
	/* do not print xste command lines in the form '*command' */
	if(pa -> text[0] == '*') continue;

	/* split of any command in the form of '[female] */
	if(! extract_command(pa -> text, text_part, command_part) ) return 0;

	/* zero length subtitles (no text) we do not want in .srt format */
	if(text_part[0] == 0) continue;

/*
SRT example:

4M
00:01:29,019 --> 00:01:31,294M
Dieu Tout Puissant,M
nous sommes enfin libres !M
M
*/

	/* sequence number, starts at 1 */
	fprintf(fptr, "%d\r\n", entry_number);

	/* replace ''\' by \r\n in the text */	
	/* void replace(char *text, char *string, char *replace, char *result) */
	replace(text_part, "\\", "\r\n", temp);

	/* print entry */
	fprintf(fptr,\
	"%02d:%02d:%02d,%03d --> %02d:%02d:%02d,%03d\r\n",\
	start_hour, start_minute, start_second, start_milli_second,\
	end_hour, end_minute, end_second, end_milli_second);

	/* the text line(s) */
	fprintf(fptr, "%s\r\n", temp);

	/* empty line ends text, everything in MSDOS format */
	fprintf(fptr, "\r\n");

	entry_number++;
	} /* end for all frames */

/* close the file */
fclose(fptr);

return 1;
}/* end function write_srt_list */


