/* 
xste subtitle editor
xste  is copyright (C) 2000 <Jan Panteltje>
email: panteltje@yahoo.com

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "xste.h"

extern FD_xste *fdui;


int readline_msdos(FILE *file, char *contents)
{
int a, c, i;

for(i = 0; i < READSIZE - 1; i++)
	{
	while(1)
		{
		c = getc(file);
		a = ferror(file);
		if(a)
			{
			perror("readline():");
			continue;
			}
		break;
		}
	if(feof(file) )
		{
		fclose(file);
		contents[i] = 0;/* EOF marker */
		return(EOF);
		}

	if(c == 10)
		{
		/* get rid of possible CR 13 */		
		c = getc(file);
		if(c != 13) ungetc(c, file);

		contents[i] = 0;/* string termination */
		return 1;/* end of line */
		}

	if(c == 13)
		{
		/* get rid of possible LF 10 */
		c = getc(file);
		if(c != 10) ungetc(c, file);

		contents[i] = 0;/* string termination */
		return 1;/* end of line */
		} 
	contents[i] = c;
	}
/*
mmm since we are here, the line must be quite long, possibly something
is wrong.
Since I do not always check for a return 0 in the use of this function,
just to be safe, gona force a string termination.
This prevents stack overflow, and variables getting overwritten.
*/
contents[i] = 0;/* force string termination */
/*fclose(file);*/
if(debug_flag)
	{
	fprintf(stdout,\
	"readline(): line to long, returning 0 contents=%s\n", contents);
	}
return 0;
}/* end function readline_msdos */


int load_ssa_file(char *pathfilename)
{
FILE *fptr;

if(debug_flag)
	{
	fprintf(stdout,\
	"load_ssa_file(): arg pathfilename=%s\n", pathfilename);
	}

/* argument check */
if(! pathfilename) return 0;

fptr = fopen(pathfilename, "r");
if(! fptr)
	{
	fl_show_alert("Could not open file", pathfilename, "for read", 1);

	return 0;
	}

/* load the .ssa file into the structure list */
if(! read_in_ssa_file(fptr) )
	{

	return 0;
	}

//save_project_vars();

show_frames(SHOW_TOP);

return 1;
} /* end  function load_ssa_file */



int read_in_ssa_file(FILE *finptr)
{
int a;
char temp[READSIZE];
char start_time[1024];
char end_time[1024];
char text[8192];
char *ptr;
int start_framenr, end_framenr;
unsigned long start_sample, end_sample;
int text_hold_flag;
int start_hour, start_minute, end_hour, end_minute;
double dstart_second, dend_second;
double dstart_milli_seconds, dend_milli_seconds;
int line;

/* clear the frame_list */
delete_all_frames();

/* start reading lines from the ssa file */
line = 0;
while(1)
	{
	/* removes any MSDOS CR */
	a = readline_msdos(finptr, temp); /* closes file if EOF */
	if(a == EOF) break;	

	if(debug_flag)
		{
		fprintf(stdout, "temp=%s\n", temp);
		}

	line++;

	/* test for comment only */
	if(temp[0] == ';') continue;

	/* look for start dialog */
	if(strncasecmp(temp, "Dialogue:", 9) == 0)
		{
		/* parse the .ssa dialog line */
		/* NOTE:ASSUMING this format is used!!!! AND NO OTHER */
		/*
		Format: Marked, Start, End, Style, Name, MarginL, MarginR, \
		MarginV, Effect, Text

		Example:
		Dialogue: Marked=0,0:04:07.53,0:04:10.23,*Default,,0000,0000,0000,,\
		text
		*/

		/* point to Start */
		ptr = strchr(temp, ',') + 1;
		sscanf(ptr, "%25[^,]", start_time); 

		/* point to end_time */
		ptr = strchr(ptr, ',') + 1;
		sscanf(ptr, "%25[^,]", end_time); 

		/* point to Style */
		ptr = strchr(ptr, ',') + 1;

		/* point to Name */
		ptr = strchr(ptr, ',') + 1;

		/* point to MarginL */
		ptr = strchr(ptr, ',') + 1;

		/* point to MarginR */
		ptr = strchr(ptr, ',') + 1;

		/* point to MarginV */
		ptr = strchr(ptr, ',') + 1;

		/* point to Effect */
		ptr = strchr(ptr, ',') + 1;

		/* point to text */
		ptr = strchr(ptr, ',') + 1;

		/* replace '\n' by \ in the text */	
		/* void replace(char *text, char *string, char *replace, char *result) */
		replace(ptr, "\\n", "\\", text);

		if(debug_flag)
			{
			printf("\n");
			printf("start_time=%s\n", start_time);
			printf("end_time=%s\n", end_time);
			printf("text=%s\n", text);
			}

		/* parse start_time 0:04:07.53 */
		sscanf(start_time, "%d:%d:%lf",\
		&start_hour, &start_minute, &dstart_second);

		dstart_milli_seconds =\
		3600000 * start_hour +\
		60000 * start_minute +\
		1000 * dstart_second;

		/* parse end_time 0:04:07.53 */
		sscanf(end_time, "%d:%d:%lf",\
		&end_hour, &end_minute, &dend_second);

		dend_milli_seconds =\
		3600000 * end_hour +\
		60000 * end_minute +\
		1000 * dend_second;

		if(debug_flag)
			{
			printf("\n");
			printf(\
			"start_hour=%d start_minute=%d dstart_second=%.3f\n\
			dstart_milli_seconds=%.2f\n",\
			start_hour, start_minute, dstart_second,\
			dstart_milli_seconds);

			printf(\
			"end_hour=%d end_minute=%d dend_second=%.3f\n\
			dend_milli_seconds=%.2f\n",\
			end_hour, end_minute, dend_second,\
			dend_milli_seconds);
			}

		/* sanity check */
		if(dend_milli_seconds <= dstart_milli_seconds)
			{
			/*
			try to fix,
			leave on screen for 4 seconds, or until next title appears
			*/

			dend_milli_seconds = dstart_milli_seconds + 4000;	
			
			fprintf(stdout,\
			"ALERT line %d end time <= start time, fixed to + 4 seconds.\n",\
			 line);
			}

		/* calculate parameters */
		/* 20 mS per frame, 160 mS per frame, if 25.0 frames / second */

		/* start frame */
		start_framenr = dstart_milli_seconds / dmilli_seconds_per_frame;

		/* end frame */
		end_framenr = dend_milli_seconds / dmilli_seconds_per_frame;

		if(debug_flag)
			{
			fprintf(stdout,\
			"start_frame=%d end_frame=%d\n",\
			start_framenr, end_framenr);
			}

		/* start_sample */		
		start_sample = (dstart_milli_seconds * RAW_SAMPLE_SPEED) / 1000.0;

		/* end_sample */
		end_sample = (dend_milli_seconds * RAW_SAMPLE_SPEED) / 1000.0 ;

		if(debug_flag)
			{
			fprintf(stdout,\
			"start_sample=%lu end_sample=%lu\n",\
			start_sample, end_sample);
			}

		/* text_hold_flag */
		text_hold_flag = 0; /* not supported in ssa? */
		
		/* add entry to structure */
		if(! add_frame(\
		text, start_framenr, end_framenr,\
		text_hold_flag) )
			{
			fprintf(stdout, "Could not add_frame, aborting\n");

			/* close input file */
			fclose(finptr);

			exit(1);
			}
		} /* end while all dialog lines */

	} /* end while all lines in .ssa file */

/* sort the double linked list with criterium time */
sort_frames();

return 1;
} /* end function read_in_ssa_file */


int write_ssa_list()
{
/*
purpose of this routine is to also write a Substation Alpha subtitle format
file, so subtitles can added in MS Windows with Virtual Dub.
(After possibly conversion of pva -> mpg (with pva2mpeg.exe) -then to .avi
(with Flaskmpeg using Intel 5.x codec), then use Intel 5.1 cvodec (100% quality
setting again in virtualdub).
ssa ?? uses a time oriented format, see MPlayer-0.50/subreader.c
*/
struct frame *pa;
struct frame *pnext, *ptemp;
FILE *fptr;
char temp[4096];
int next_frame_start;
int start_hour, start_minute, end_hour, end_minute;
double dstart_second, dend_second;
long lthousands;
int text_end_frame = 0; // -Wall
double audio_samples_per_frame;
double dtimer;
int yres, depth;
int marginl, marginr, marginv;
//char style_override_code[128];
char override_string[1024];
char effect[1024];
char *ptr;
char command_part[TEXT_ARRAY_SIZE];
char text_part[TEXT_ARRAY_SIZE];

if(debug_flag)
	{
	printf("write_ssa_list(): arg none\n");
	}

draw_mode = DRAW_MODE_BW;

/* some defaults */
dtimer = 100.0;
yres = 600;
depth = 0;
marginl = 10;
marginr = 10;
marginv = 20;
strcpy(effect, "");
strcpy(override_string, "");
drop_shadow = 0; /* use 0 - 4 */
/* end defaults */

/* 40 ms per frame (of 25 Hz) */
audio_samples_per_frame = audio_sample_speed / dmilli_seconds_per_frame;

/* had to imagine some extention name 'ssa' */
sprintf(temp, "%s/%s/%s/%s.ssa",\
home_dir, selected_data_directory, selected_project, selected_project);
fptr = fopen(temp, "w");
if(! fptr)
	{
	fl_show_alert("write_ssa_list", "Could not open for write", temp, 1);

	return 0;
	}

/* write some ssa header here */

/*
[Script Info]
; This is a Sub Station Alpha v4 script.
; For Sub Station Alpha info and downloads,
; go to http://www.eswat.demon.co.uk/
; or email kotus@eswat.demon.co.uk
Title: VirtualDub 'subtitler' filter demonstration
Original Script: Avery Lee <phaeron@virtualdub.org>
ScriptType: v4.00
Collisions: Normal
PlayResY: 600
PlayDepth: 0
Timer: 100.0000
*/
fprintf(fptr, "[Script Info]\r\n");
fprintf(fptr, "; This is a Sub Station Alpha v4 script.\r\n");
fprintf(fptr, "; For Sub Station Alpha info and downloads,\r\n");
fprintf(fptr, "; go to http://www.eswat.demon.co.uk/\r\n");
fprintf(fptr, "; or email kotus@eswat.demon.co.uk\r\n");
fprintf(fptr, "Title: Panteltest from xste\r\n");
fprintf(fptr, "Original Script: Panteltje\r\n");
fprintf(fptr, "ScriptType: v4.00\r\n");
fprintf(fptr, "Collisions: Normal\r\n");
fprintf(fptr, "PlayResY: %d\r\n", yres);
fprintf(fptr, "PlayDepth: %d\r\n", depth);
fprintf(fptr, "ScriptType: v4.00\r\n");
fprintf(fptr, "Timer: %.4f\r\n", dtimer);

/*
[V4 Styles]
Format: Name, Fontname, Fontsize, PrimaryColour, SecondaryColour,\
	TertiaryColour, BackColour, Bold, Italic, BorderStyle, Outline,\
	Shadow, Alignment, MarginL, MarginR, MarginV, AlphaLevel, Encoding
*/

/*
This style override made with ssa will force 'Mainb' to xste format:
{\fs36\i0\c&HFFFFFF&\a2}
Now added Xste
*/

/*
Style: Xste,Arial,36,&HFFFFFF,&HFFFFFF,&HFFFFFF,0,0,0,1,2,4,2,25,25,30,0,0
Style: Xste,Arial,36,16777215,16777215,16777215,0,0,0,1,2,0,2,25,25,30,0,0

;Style: Foo,MS Gothic,72,65535,65280,65535,0,0,-1,1,2,4,2,30,30,30,0,128
Style: MainB,Arial,14,65535,65280,65535,0,0,-1,1,2,4,1,16,16,16,0,0
Style: MainT,Arial,14,65535,65280,65535,0,0,0,1,2,4,5,16,16,16,0,0
Style: Timer,Arial,14,65535,65280,65535,0,0,-1,1,2,4,5,16,16,40,0,0
Style: B0S0K,Arial,14,&HE0E0E0,65280,65535,&H000000,0,-1,1,0,0,5,16,16,40,0,0
Style: B2S2R,Arial,14,&HE0E0E0,65280,65535,&H004080,0,-1,1,2,2,5,16,16,40,0,0
Style: B2S6G,Arial,14,&HE0E0E0,65280,65535,&H408000,0,-1,1,2,6,5,16,16,40,0,0
Style: B4S4B,Arial,14,&HE0E0E0,65280,65535,&H804000,0,-1,1,4,4,5,16,16,40,0,0
Style: Karaoke,Arial,14,&H40FFFF,&HFF4040,65535,0,0,0,1,2,4,5,16,16,16,0,0
Style: ShiftJIS,MS Gothic,20,&HE0E0E0,65280,65535,0,0,0,1,2,4,5,16,16,16,0,128
*/

fprintf(fptr,\
"[V4 Styles]\r\n");

fprintf(fptr,\
"Format: Name, Fontname, Fontsize, PrimaryColour, SecondaryColour, TertiaryColour, BackColour, Bold, Italic, BorderStyle, Outline, Shadow, Alignment, MarginL, MarginR, MarginV, AlphaLevel, Encoding\r\n");

strcpy(temp, frametab[0] -> fontname);
ptr = strstr(temp, ".ttf");
if(! ptr) ptr = strstr(temp, ".TTF");
if(! ptr) strcpy(temp, "Arial");
/* cut of .ttf or .TTF */
*ptr = 0;

fprintf(fptr,\
"Style: Xste,%s,%d,16777215,16777215,16777215,0,0,0,1,2,%d,2,25,25,30,0,0\r\n",\
temp, frametab[0] -> fontsize, drop_shadow);
/* using fontsize first sub */


/*
[Events]
Format: Marked, Start, End, Style, Name, MarginL, MarginR, MarginV, Effect, Text
Comment: Marked=0,0:00:00.00,0:00:00.01,MainB,,0000,0000,0000,!Effect,Warning: This script contains some text in Shift-JIS encoding, which may be destroyed if edited in some text editors.

Dialogue: Marked=0,0:00:00.00,0:00:05.00,MainB,,0000,0000,0000,!Effect,{\q1}Subtitler filter for VirtualDub\NDemonstration script\N\NAvery Lee <phaeron@virtualdub.org>
Dialogue: Marked=0,0:00:05.00,0:00:10.00,MainB,,0000,0000,0000,!Effect,{\q1}Introduction\N\NThis script will demonstrate some of the simpler features of the subtitler. You will want to refer to the {\fnCourier New\b1}readme.txt{\r} file for a more complete reference.
Dialogue: Marked=0,0:00:10.00,0:00:21.00,MainB,,0000,0000,0000,!Effect,{\q1}Dialogue lines\N\NScripts are based on dialogue lines. Each dialogue line has a start time, an end time, and text to display in between.

Dialogue: Marked=0,0:01:51.33,0:01:55.06,MainB,,10,10,20,,{\fs36\i0\c&HFFFFFF&\a2}Al drie decennia heeft Maharaji mensen over de hele wereld toe gesproken,
Dialogue: Marked=0,0:01:55.06,0:02:00.27,MainB,,10,10,20,,{\fs36\i0\c&HFFFFFF&\a2}over het onderwerp van LEVEN en de mogelijkheid van een diepere voldoening.
*/

fprintf(fptr,\
"[Events]\r\n");

fprintf(fptr,\
"Format: Marked, Start, End, Style, Name, MarginL, MarginR, MarginV, Effect, Text\r\n");

//fprintf(fptr, "Comment: Marked=0,0:00:00.00,0:00:00.01,MainB,,0000,0000,0000,!Effect,Warning: This script contains some text in Shift-JIS encoding, which may be destroyed if edited in some text editors.\r\n");

/*
NOTE
it seems we have to replace the '\' with '\n' and also in our conversion
programs ssa_to_xste and vice versa.
*/

/* Note: {q1} is a style override control code (see format.zip) */

for(pa = frametab[0]; pa != 0; pa = pa -> nxtentr)
	{
	/*
	do not print negative frames, this results in negative times,
	not sure SSA likes that.
	*/
	if( atoi(pa -> name) < 0) continue;

	/* get pointer to next structure entry */ 
	if(pa -> nxtentr) 
		{
		pnext = pa -> nxtentr;	
		
		/* find next line that is not a '*' comand line */
		while(pnext)
			{
			if(pnext -> text[0] != '*') break;

			if(pnext -> nxtentr)
				{	
				ptemp = pnext -> nxtentr;

				pnext = ptemp;
				}	
			} /* end while looking for non '*' entry */ 

		if(pnext)
			{
			/* get the start of the next frame */ 
			next_frame_start = atoi(pnext -> name);
		
			/* test if the current frame ends before the start of the next */
			if(pa -> end_frame > next_frame_start)
				{
				text_end_frame = next_frame_start;
				} /* end if frame ends before next one starts */ 
			}

		} /* end if not last entry */	
	


	/* we have this data at our disposal */ 
	//	char *name;/* this is the start frame */
	//	char *text;
	//	int end_frame; /* this is the end frame */
	//	unsigned long start_sample;
	//	unsigned long end_sample;
 
	/*
	We can time to the sound file, at audio_sample_speed samples / second,
	(actually 11025).
	Some players sync to the audio, some to the video, Substation Alpha
	seems to be able to do both, trying ....
	*/

	/* calculate start_time */
	lthousands = atoi(pa -> name) * dmilli_seconds_per_frame;
	start_hour = lthousands / 3600000;
	lthousands -= start_hour * 3600000; 
	start_minute = lthousands / 60000;
	lthousands -= start_minute * 60000;
	dstart_second = lthousands / 1000.0;

	/* calculate end_time */
	lthousands = text_end_frame * dmilli_seconds_per_frame;
	end_hour = lthousands / 3600000;
	lthousands -= end_hour * 3600000; 
	end_minute = lthousands / 60000;
	lthousands -= end_minute * 60000;
	dend_second = lthousands / 1000.0;

//	sprintf(style_override_code, "%cq", '\\');

	/*
	Overrides (style, font, color, etc) look like this:
	{\fs36\i0\c&HFFFFFF&\a2}
	Not using overrides, using Xste format.
	*/

	/*
	do not print negative frames, this results in negative times,
	not sure SSA likes that.
	*/
	if(start_hour < 0) continue;
	if(start_minute < 0) continue;
	if(dstart_second < 0.0) continue;
	
	/* do not print xste command lines in the form '*command' */
	if(pa -> text[0] == '*') continue;

	/* split of any command in the form of '[female] */
	if(! extract_command(pa -> text, text_part, command_part) ) return 0;

	/* replace ''\' by \n in the text */	
	/* void replace(char *text, char *string, char *replace, char *result) */
	replace(text_part, "\\", "\\n", temp);

	/* print entry */
	fprintf(fptr,\
	"Dialogue: Marked=0,%d:%02d:%0.2f,%d:%02d:%0.2f,Xste,,\
%d,%d,%d,%s,%s%s\r\n",\
	start_hour, start_minute, dstart_second,\
	end_hour, end_minute, dend_second,\
	marginl, marginr, marginv,\
	effect,\
	override_string, temp);

	} /* end for all frames */

/* close the file */
fclose(fptr);

return 1;
}/* end function write_ssa_list */


