/*
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/


#include "xste.h"

extern FD_xste *fdui;
extern FD_xste *fd_main_form;

//#define NEW_DESC

/*
taken from mplayer, slightly modified by Jan Panteltje,
Should rewritten to put data directly in struct font_desc_t,
and also make a windows 4 bit bitmap, really.
*/

/*
Renders antialiased fonts for mplayer using freetype library.
Should work with TrueType, Type1 and any other font supported
by libfreetype.
Can generate font.desc for any encoding.

Artur Zaprzala <zybi@fanthom.irc.pl>
*/

#include <iconv.h>
//#include <math.h>
//#include <string.h>
//#include <libgen.h>


// Uncomment these 2 lines if you have compile problems and a very very old version of freetype.
#include <ft2build.h> 
#include FT_FREETYPE_H


// uncomment the next line if you have compile probels and a bit old verison of freetype
#define OLD_FREETYPE
#ifdef OLD_FREETYPE

/* freetype 2.0.1, maybe haeders are here? else comment out and uncomment lines below */
#include <freetype/freetype.h>
#include <freetype/ftglyph.h>

// some old versions of freetype had their headers in this place.....
//#include <freetype2/freetype/freetype.h> 
//#include <freetype2/freetype/ftglyph.h> 

#else

/* see freetype-2.1.4/docs/tutorial or www.freetype.org */
#include <ft2build.h>
#include FT_FREETYPE_H
    
#endif


#include "bswap.h"

/* default values */
char *encoding = "iso-8859-15";/* target encoding */
char *charmap = "ucs-4";	/* font charmap encoding, I hope ucs-4 is always big endian */
	/* gcc 2.1.3 doesn't support ucs-4le, but supports ucs-4 (==ucs-4be) */
float ppem = 22;			/* font size in pixels */
char* font_desc = "font.desc";
char *outdir = ".";

/* constants */
int const colors = 256;
int const maxcolor = 255;
unsigned const base = 256;
unsigned const first_char = 33;

#define max_charset_size 60000

unsigned charset_size = 0;

char *command;
char *font_path;
char *encoding_name;
int append_mode = 0;
int unicode_desc = 0;
unsigned char *bbuffer, *abuffer;
int	width, height;
int	padding;
static FT_ULong	charset[max_charset_size]; /* characters we want to render; Unicode */
static FT_ULong	charcodes[max_charset_size]; /* character codes in 'encoding' */
iconv_t cd;	// iconv conversion descriptor


#define eprintf(...)		fprintf(stderr, __VA_ARGS__)
//#define ERROR_(msg, ...)	(eprintf("%s: error: " msg "\n", command, __VA_ARGS__), return 0)
#define WARNING_(msg, ...)	eprintf("%s: warning: " msg "\n", command, __VA_ARGS__)
//#define ERROR(...)		ERROR_(__VA_ARGS__, NULL)
#define WARNING(...)		WARNING_(__VA_ARGS__, NULL)

#define f266ToInt(x)		(((x)+32)>>6)	// round fractional fixed point number to integer
				// coordinates are in 26.6 pixels (i.e. 1/64th of pixels)

#define f266CeilToInt(x)	(((x)+63)>>6)	// ceiling
#define f266FloorToInt(x)	((x)>>6)	// floor
#define f1616ToInt(x)		(((x)+0x8000)>>16)	// 16.16
#define floatTof266(x)		((int)((x)*(1<<6)+0.5))

#define ALIGN(x)		(((x)+7)&~7)	// 8 byte align


void paste_bitmap(FT_Bitmap *bitmap, int x, int y)
{
int drow = x + y * width;
int srow = 0;
int sp, dp, w, h;

if (bitmap -> pixel_mode == ft_pixel_mode_mono)
	for (h = bitmap -> rows; h > 0; --h, drow += width, srow += bitmap -> pitch)
	    for (w = bitmap -> width, sp = dp = 0; w > 0; --w, ++dp, ++sp)
		    bbuffer[drow + dp] = (bitmap->buffer[srow + sp / 8] & (0x80 >> (sp % 8))) ? 255 : 0;
else
	for (h = bitmap -> rows; h > 0; --h, drow += width, srow += bitmap -> pitch)
	    for (w = bitmap -> width, sp = dp = 0; w > 0; --w, ++dp, ++sp)
		    bbuffer[drow + dp] = bitmap -> buffer[srow + sp];
} /* end function paste_bitmap */


void write_header(FILE *f)
{
static unsigned char   header[800] = "mhwanh";
int i;
header[7] = 4;

if (width < 0x10000)
	{
	/* are two bytes enough for the width? */
	header[8] = width>>8;
	header[9] = (unsigned char)width;
    }
else
	{
	/* store width using 4 bytes at the end of the header */
    header[8] = header[9] = 0;
    header[28] = (width >> 030) & 0xFF;
    header[29] = (width >> 020) & 0xFF;
    header[30] = (width >> 010) & 0xFF;
    header[31] = (width       ) & 0xFF;
    }

header[10] = height>>8;	header[11] = (unsigned char)height;
header[12] = colors>>8;	header[13] = (unsigned char)colors;

for (i = 32; i<800; ++i) header[i] = (i - 32) / 3;

fwrite(header, 1, 800, f);
} /* end function write_header */


int write_bitmap(void *buffer, char type)
{
FILE *f;
int const max_name = 128;
char name[max_name];

snprintf(name, max_name, "%s/%s-%c.raw", outdir, encoding_name, type);
f = fopen(name, "wb");
if(! f)
	{
	fprintf(stderr, "xste(): write_bitmap(): could not open %s for write\n", name);

	return 0;
	}

write_header(f);

fwrite(buffer, 1, width * height, f);

fclose(f);

return 1;
} /* end function write_bitmap */


int render()
{
FT_Library	library;
FT_Face	face;
FT_Error	error;
FT_Glyph	*glyphs;
FT_BitmapGlyph glyph;
FILE	*f;
int	const	load_flags = FT_LOAD_DEFAULT | FT_LOAD_NO_HINTING;
/* this is how dvdauthor does it, now anti-aliasing no longer works! */
//int	const	load_flags = FT_LOAD_DEFAULT | FT_LOAD_NO_HINTING | FT_LOAD_MONOCHROME | FT_LOAD_RENDER;
int		pen_x = 0, pen_xa;
int		ymin = INT_MAX, ymax = INT_MIN;
int		i, uni_charmap = 1;
int		baseline, space_advance = 20;
int		glyphs_count = 0;


    /* initialize freetype */
    error = FT_Init_FreeType(&library);
    if (error)
		{
		fprintf(stderr, "xste: render(): Init_FreeType failed.");

		return 0;
		}

    error = FT_New_Face(library, font_path, 0, &face);
    if (error)
		{
		fprintf(stderr, "xste: render(): New_Face failed. Maybe the font path `%s' is wrong.", font_path);

		return 0;
		}

    /*
    if (font_metrics) {
	error = FT_Attach_File(face, font_metrics);
	if (error) fprintf(stderr, "xste: render(): FT_Attach_File failed.");
    }
    */


#if 0
    /************************************************************/
    eprintf("Font encodings:\n");
    for (i = 0; i<face->num_charmaps; ++i)
	eprintf("'%.4s'\n", (char*)&face->charmaps[i]->encoding);

    //error = FT_Select_Charmap(face, ft_encoding_unicode);
    //error = FT_Select_Charmap(face, ft_encoding_adobe_standard);
    //error = FT_Select_Charmap(face, ft_encoding_adobe_custom);
    //error = FT_Set_Charmap(face, face->charmaps[1]);
    //if (error) fprintf(stderr, "xste: render(): FT_Select_Charmap failed.");
#endif


#if 0
    /************************************************************/
    if (FT_HAS_GLYPH_NAMES(face)) {
	int const max_gname = 128;
	char gname[max_gname];
	for (i = 0; i<face->num_glyphs; ++i) {
	    FT_Get_Glyph_Name(face, i, gname, max_gname);
	    eprintf("%02x `%s'\n", i, gname);
	}

    }
#endif


    if (face->charmap==NULL || face->charmap->encoding!=ft_encoding_unicode) {
	fprintf(stderr, "xste: render(): Unicode charmap not available for this font. Very bad!");
	uni_charmap = 0;
	error = FT_Set_Charmap(face, face->charmaps[0]);
	if (error) fprintf(stderr, "xste: render(): No charmaps! Strange.");
    }



    /* set size */
    if (FT_IS_SCALABLE(face)) {
	error = FT_Set_Char_Size(face, floatTof266(ppem), 0, 0, 0);
	if (error) fprintf(stderr, "xste: render(): FT_Set_Char_Size failed.");
    } else {
	int j = 0;
	int jppem = face->available_sizes[0].height;
	/* find closest size */
	for (i = 0; i<face->num_fixed_sizes; ++i) {
	    if (fabs(face->available_sizes[i].height - ppem) < abs(face->available_sizes[i].height - jppem)) {
		j = i;
		jppem = face->available_sizes[i].height;
	    }
	}
	fprintf(stderr, "xste: render(): Selected font is not scalable. Using ppem=%i.", face->available_sizes[j].height);
	error = FT_Set_Pixel_Sizes(face, face->available_sizes[j].width, face->available_sizes[j].height);
	if (error) fprintf(stderr, "xste: render(): FT_Set_Pixel_Sizes failed.");
    }


    if (FT_IS_FIXED_WIDTH(face))
	fprintf(stderr, "xste: render(): Selected font is fixed-width.");


    /* compute space advance */
    error = FT_Load_Char(face, ' ', load_flags);
//face->glyph->advance.x = 100;

    if (error) fprintf(stderr, "xste: render(): spacewidth set to default.");
    else space_advance = f266ToInt(face->glyph->advance.x);


    /* create font.desc */

	{
    int const max_name = 128;
    char name[max_name];

    snprintf(name, max_name, "%s/%s", outdir, font_desc);
    f = fopen(name, append_mode ? "a":"w");
	if(! f)
		{
		fprintf(stderr, "xste(): render(): could not open file %s for write\n", name);

		return 0;
		}
	}

    /* print font.desc header */
    if (append_mode) {
	fprintf(f, "\n\n# ");
    } else {
	fprintf(f,  "# This file was generated with subfont for Mplayer.\n"
		    "# Subfont by Artur Zaprzala <zybi@fanthom.irc.pl>.\n\n");
	fprintf(f, "[info]\n");
    }

    fprintf(f, "name 'Subtitle font for %s %s, \"%s%s%s\" face, size: %.1f pixels.'\n",
	    encoding_name,
	    unicode_desc ? "charset, Unicode encoding":"encoding",
	    face->family_name,
	    face->style_name ? " ":"", face->style_name ? face->style_name:"",
	    ppem);

    if (!append_mode) {
#ifdef NEW_DESC
	fprintf(f, "descversion 2\n");
#else
	fprintf(f, "descversion 1\n");
#endif
	fprintf(f, "spacewidth %i\n",	2 * padding + space_advance);
#ifndef NEW_DESC
	fprintf(f, "charspace %i\n", -2 * padding);
#endif
	fprintf(f, "height %i\n", (int)((padding * 2) + f266ToInt(face->size->metrics.height)) );
#ifdef NEW_DESC
	fprintf(f, "ascender %i\n",	f266CeilToInt(face->size->metrics.ascender));
	fprintf(f, "descender %i\n",	f266FloorToInt(face->size->metrics.descender));
#endif
    }
    fprintf(f, "\n[files]\n");
    fprintf(f, "alpha %s-a.raw\n",	encoding_name);
    fprintf(f, "bitmap %s-b.raw\n",	encoding_name);
    fprintf(f, "\n[characters]\n");


    // render glyphs, compute bitmap size and [characters] section
    glyphs = (FT_Glyph*)malloc(charset_size*sizeof(FT_Glyph*));
    for (i= 0; i<charset_size; ++i) {
	FT_GlyphSlot	slot;
	FT_ULong	character, code;
	FT_UInt		glyph_index;

	character = charset[i];
	code = charcodes[i];

	// get glyph index
	if (character==0)
	    glyph_index = 0;
	else {
	    glyph_index = FT_Get_Char_Index(face, uni_charmap ? character:code);
	    if (glyph_index == 0)
			{
			if(debug_flag)
				{
				fprintf(stderr, "xste: render(): Glyph for char 0x%02x|U+%04X|%c not found.",\
				(unsigned int)code, (unsigned int)character,\
				(unsigned int)(code<' '|| code > 255 ? '.' : code) );
				}

			continue;
		    }
	}

	// load glyph
	error = FT_Load_Glyph(face, glyph_index, load_flags);
	if (error) {
	    fprintf(stderr, "xste: render(): FT_Load_Glyph 0x%02x (char 0x%02x|U+%04X) failed.",
		(unsigned int)glyph_index, (unsigned int)code, (unsigned int)character);
	    continue;
	}
	slot = face->glyph;

	// render glyph
	if (slot->format != ft_glyph_format_bitmap) {
	    error = FT_Render_Glyph(slot, ft_render_mode_normal);
	    if (error) {
		fprintf(stderr, "xste: render(): FT_Render_Glyph 0x%04x (char 0x%02x|U+%04X) failed.",\
		(unsigned int)glyph_index, (unsigned int)code, (unsigned int)character);
		continue;
	    }
	}

	// extract glyph image
	error = FT_Get_Glyph(slot, (FT_Glyph*)&glyph);
	if (error) {
	    fprintf(stderr, "xste: render(): FT_Get_Glyph 0x%04x (char 0x%02x|U+%04X) failed.",\
		(unsigned int)glyph_index, (int)code, (unsigned int)character);
	    continue;
	}
	glyphs[glyphs_count++] = (FT_Glyph)glyph;

#ifdef NEW_DESC
	// max height
	if (glyph->bitmap.rows > height) height = glyph->bitmap.rows;

	// advance pen
	pen_xa = pen_x + glyph->bitmap.width + 2 * padding;

	// font.desc
	fprintf(f, "0x%04x %i %i %i %i %i %i;\tU+%04X|%c\n", unicode_desc ? character:code,
		pen_x,						// bitmap start
		glyph->bitmap.width + 2 * padding,		// bitmap width
		glyph->bitmap.rows + 2 * padding,			// bitmap height
		glyph->left - padding,				// left bearing
		glyph->top + padding,				// top bearing
		f266ToInt(slot->advance.x),			// advance
		character, code<' '||code>255 ? '.':code);
#else
	// max height
	if (glyph->top > ymax) {
	    ymax = glyph->top;
	    //eprintf("%3i: ymax %i (%c)\n", code, ymax, code);
	}
	if (glyph->top - glyph->bitmap.rows < ymin) {
	    ymin = glyph->top - glyph->bitmap.rows;
	    //eprintf("%3i: ymin %i (%c)\n", code, ymin, code);
	}

	/* advance pen */
	pen_xa = pen_x + f266ToInt(slot->advance.x) + 2 * padding;

	/* font.desc */
	fprintf(f,\
	"0x%04x %i %i;\tU+%04X|%c\n",\
	(unsigned int)(unicode_desc ? character:code),
	(int)pen_x,						// bitmap start
	(int)pen_xa-1,					// bitmap end
	(unsigned int)character,
	(char)(code<' '||code>255 ? '.':code) );
#endif
	pen_x = ALIGN(pen_xa);
    }


    width = pen_x;
    pen_x = 0;
#ifdef NEW_DESC
    if (height<=0)
		{
		fprintf(stderr, "xste: render(): Something went wrong. Use the source!");

		return 0;		
		}

    height += 2 * padding;
#else
    if (ymax<=ymin)
		{
		fprintf(stderr, "xste: render(): Something went wrong. Use the source!");
 
		return 0;
		}

	height = ymax - ymin + 2 * padding;
    baseline = ymax + padding;
#endif

    // end of font.desc
    if (debug_flag) eprintf("bitmap size: %ix%i\n", width, height);
    fprintf(f, "# bitmap size: %ix%i\n", width, height);
    fclose(f);

    bbuffer = (unsigned char*)malloc(width*height);
    if (bbuffer==NULL)
		{
		fprintf(stderr, "xste: render(): malloc failed.");

		return 0;
		}
    memset(bbuffer, 0, width*height);


    /* paste glyphs */
    for (i= 0; i<glyphs_count; ++i) {
	glyph = (FT_BitmapGlyph)glyphs[i];
#ifdef NEW_DESC
	paste_bitmap(&glyph->bitmap,
	    pen_x + padding,
	    padding);

	/* advance pen */
	pen_x += glyph->bitmap.width + 2 * padding;
#else
	paste_bitmap(&glyph->bitmap,
	    pen_x + padding + glyph->left,
	    baseline - glyph->top);

	/* advance pen */
//printf("WAS glyph->root.advance.x = %.2f\n", (float)glyph->root.advance.x);
//glyph->root.advance.x /= 2;

	pen_x += f1616ToInt(glyph->root.advance.x) + 2 * padding;
#endif
	pen_x = ALIGN(pen_x);

	FT_Done_Glyph((FT_Glyph)glyph);
    }
    free(glyphs);

    error = FT_Done_FreeType(library);
    if (error)
		{
		fprintf(stderr, "xste: render(): FT_Done_FreeType failed.");

		return 0;
		}
return 1;
} /* end function render */


/* decode from 'encoding' to unicode */
FT_ULong decode_char(char c)
{
FT_ULong o;
char *inbuf = &c;
char *outbuf = (char*)&o;
size_t inbytesleft = 1;
size_t outbytesleft = sizeof(FT_ULong);

//size_t iconv(iconv_t cd,  char **inbuf, size_t *inbytesleft, char **outbuf, size_t *outbytesleft);
iconv(cd, &inbuf, &inbytesleft, &outbuf, &outbytesleft);

/* convert unicode BigEndian -> MachineEndian */
o = be2me_32(o);

if (outbytesleft != 0) o = 0;

/* we don't want control characters */
if (o >= 0x7f && o < 0xa0) o = 0;

return o;
} /* end function decode_char */


int prepare_charset()
{
FILE *f;
FT_ULong i;

f = fopen(encoding, "r"); // try to read custom encoding
if (f == NULL)
	{
	int count = 0;

	// check if ucs-4 is available
	cd = iconv_open(charmap, charmap);
	if (cd==(iconv_t)-1)
		{
		fprintf(stderr, "xste: prepare_charset(): iconv doesn't know %s encoding. Use the source!", charmap);

		return 0;
		}

	iconv_close(cd);

	cd = iconv_open(charmap, encoding);
	if(cd == (iconv_t) - 1)
		{
		fprintf(stderr, "xste: prepare_charset(): Unsupported encoding `%s', use iconv --list to list character sets known on your system.",\
		encoding);

		fl_show_alert(\
		"Unsupported encoding",\
		encoding, "use iconv --list\nto list character sets known\non your system.",\
		0);

		return 0;
		}	

	charset_size = 256 - first_char;
	for (i = 0; i<charset_size; ++i) 
		{
	    charcodes[count] = i+first_char;
	    charset[count] = decode_char(i+first_char);
	    //eprintf("%04X U%04X\n", charcodes[count], charset[count]);
	    if (charset[count]!=0) ++count;
		}
	charcodes[count] = charset[count] = 0; ++count;
	charset_size = count;

	iconv_close(cd);
    }
else
	{
	unsigned int character, code;
	int count;

	eprintf("Reading custom encoding from file '%s'.\n", encoding);

    while ((count = fscanf(f, "%x%*[ \t]%x", &character, &code)) != EOF)
		{
	    if (charset_size==max_charset_size)
			{
			fprintf(stderr, "xste: prepare_charset(): There is no place for  more than %i characters. Use the source!", max_charset_size);
			break;
		    }
	    if (count == 0)
			{
			fprintf(stderr, "xste: prepare_charset(): Unable to parse custom encoding file.");

			return 0;
			}

	    if (character < 32) continue;	// skip control characters
	    charset[charset_size] = character;
	    charcodes[charset_size] = count==2 ? code : character;
	    ++charset_size;
		}

	fclose(f);
//	encoding = basename(encoding);
    }

if (charset_size==0)
	{
	fprintf(stderr, "xste: prepare_charset(): No characters to render!");

	return 0;
	}

return 1;
} /* end function prepare_charset */


// general outline
void outline(unsigned char *s, unsigned char *t, int width, int height, unsigned int *m, int r, int mwidth)
{
int x, y;

for (y = 0; y < height; ++y)
	{
	for (x = 0; x < width; ++x, ++s, ++t)
		{
	    unsigned max = 0;
	    unsigned *mrow = m + r;
//		int *mrow = m + r;
	    unsigned char *srow = s -r * width;
	    int x1 = (x < r) ? -x : -r;
	    int x2 = (x + r >= width) ? (width - x - 1) : r;
	    int my;

	    for(my = -r; my <= r; ++my, srow += width, mrow += mwidth)
			{
			int mx;

			if(y + my < 0) continue;
			if(y + my >= height) break;

			for(mx = x1; mx <= x2; ++mx)
				{
		    	unsigned v = srow[mx] * mrow[mx];
		    	if(v > max) max = v;
				}
	   		}
	    *t = (max + base / 2) / base;
		}
    }
} /* end function outline */


// 1 pixel outline
void outline1(unsigned char *s, unsigned char *t, int width, int height)
{
int x, y;

for (x = 0; x<width; ++x, ++s, ++t) *t = *s;
for (y = 1; y<height-1; ++y)
	{
	*t++ = *s++;
	for (x = 1; x<width-1; ++x, ++s, ++t)
		{
	    unsigned v = \
			(
		    s[-1 - width]+
		    s[-1 + width]+
		    s[+1 - width]+
		    s[+1 + width]\
			) / 2 + \
			(
		    s[-1]+
		    s[+1]+
		    s[-width]+
		    s[+width]+
		    s[0]
			);
	    *t = v > maxcolor ? maxcolor : v;
		}
	*t++ = *s++;
    }
for (x = 0; x < width; ++x, ++s, ++t) *t = *s;

} /* end function outline1 */


// gaussian blur
void blur(
unsigned char *buffer,\
unsigned char *tmp,\
int width,\
int height,\
unsigned int *m,\
int r,\
int mwidth,\
unsigned volume\
)
{
int x, y;
unsigned char *s = buffer - r;
unsigned char *t = tmp;

for (y = 0; y < height; ++y)
	{
	for (x = 0; x < width; ++x, ++s, ++t)
		{
	    unsigned sum = 0;
	    int x1 = (x < r) ? r - x : 0;
	    int x2 = (x + r >= width) ? (r + width - x) : mwidth;
	    int mx;
	    for(mx = x1; mx < x2; ++mx) sum += s[mx] * m[mx];
	    *t = (sum + volume / 2) / volume;
	    //*t = sum;
		}
	}

tmp -= r * width;
for(x = 0; x < width; ++x, ++tmp, ++buffer)
	{
	s = tmp;
	t = buffer;
	for (y = 0; y < height; ++y, s += width, t += width)
		{
	    unsigned sum = 0;
	    int y1 = (y < r) ? r - y : 0;
	    int y2 = (y + r >= height) ? (r + height - y) : mwidth;
	    unsigned char *smy = s + y1 * width;
	    int my;
	    for (my = y1; my < y2; ++my, smy += width)
		sum += *smy * m[my];
	    *t = (sum + volume / 2) / volume;
		}
    }
} /* end function blur */


// Gaussian matrix
// Maybe for future use.
unsigned gmatrix(unsigned *m, int r, int w, double const A) 
{
unsigned volume = 0; // volume under Gaussian area is exactly -pi * base / A
int mx, my;

for (my = 0; my < w; ++my)
	{
	for (mx = 0; mx < w; ++mx)
		{
	    m[mx + my * w] = \
		(unsigned)(exp(A * ((mx - r) * (mx - r) + \
		(my - r) * (my - r))) * base + .5);
	    volume += m[mx + my * w];
	    if (debug_flag) eprintf("%3i ", m[mx + my * w]);
		}
	if (debug_flag) eprintf("\n");
    }
if (debug_flag)
	{
	eprintf("A= %f\n", A);
	eprintf("volume: %i; exact: %.0f; volume/exact: %.6f\n\n", volume, -M_PI*base/A, volume/(-M_PI*base/A));
    }
return volume;
} /* end function gmatrix */


int alpha(double outline_thickness, double blur_radius) 
{
int const g_r = ceil(blur_radius);
int const o_r = ceil(outline_thickness);
int const g_w = 2 * g_r + 1;		// matrix size
int const o_w = 2 * o_r + 1;		// matrix size
double const A = log(1.0 / base) / (blur_radius * blur_radius * 2);

int mx, my, i;
unsigned volume = 0; // volume under Gaussian area is exactly -pi * base / A

unsigned *g = (unsigned*)malloc(g_w * sizeof(unsigned));
unsigned *om = (unsigned*)malloc(o_w * o_w * sizeof(unsigned));

if(debug_flag)
	{
	fprintf(stdout, "alpha(): outline=%.2f blur=%.2f\n", outline_thickness, blur_radius); 
	}

if (g == NULL || om == NULL)
	{
	fprintf(stderr, "xste: alpha(): malloc failed.");

	return 0;
	}

if(blur_radius == 0)
	{
	fprintf(stdout, "alpha(): radius is zero, set subtitle fonts to default\n");

	return 0;
	}

// gaussian curve
for (i = 0; i < g_w; ++i)
	{
	g[i] = (unsigned)(exp(A * (i - g_r) * (i - g_r) ) * base + .5);
	volume += g[i];
	if (debug_flag) eprintf("%3i ", g[i]);
    }

//volume *= volume;
if (debug_flag) eprintf("\n");

/* outline matrix */
for (my = 0; my < o_w; ++my)
	{
	for (mx = 0; mx < o_w; ++mx)
		{
	    // antialiased circle would be perfect here, but this one is good enough
	    double d = outline_thickness + 1 - hypot(mx - o_r, my - o_r);

//		sqrt( (mx - o_r) * (mx - o_r) + (my - o_r) * (my - o_r) );

	    om[mx + my * o_w] = d >= 1 ? base : d <= 0 ? 0 : (d * base + .5);
	    if (debug_flag) eprintf("%3i ", om[mx + my * o_w]);
		}
	if (debug_flag) eprintf("\n");
    }
if (debug_flag) eprintf("\n");

if(outline_thickness == 1.0)
	outline1(bbuffer, abuffer, width, height);	// FAST solid 1 pixel outline
else
	outline(bbuffer, abuffer, width, height, om, o_r, o_w);	// solid outline

//	outline(bbuffer, abuffer, width, height, gm, g_r, g_w);	// Gaussian outline

blur(abuffer, bbuffer, width, height, g, g_r, g_w, volume);

free(g);
free(om);

return 1;
} /* end function alpha */


font_desc_t *make_font(\
	char *font_name, int font_symbols, int font_size, int iso_extention,\
	double outline_thickness, double blur_radius)
{
font_desc_t *pfontd;
char temp[4096];
FILE *pptr;
FILE *fptr;
char *ptr;
char *ptr2;

if(debug_flag)
	{	
	printf("make_font(): arg font_name=%s font_symbols=%d font_size=%d iso_extention=%d\n\
	outline_thickness=%.2f blur_radius=%.2f\n",\
	font_name, font_symbols, font_size, iso_extention, outline_thickness, blur_radius);
	}

/* argument check */
if(! font_name) return 0;
if(! font_size) return 0;
if(! iso_extention) return 0;

/* pathfilename of true type font */
if(font_path) free(font_path);
sprintf(temp, "%s/.xste/fonts/%s", home_dir, font_name);
font_path = strsave(temp);
if(! font_path) return 0;

/* test if font present in this system (rframes.dat could have been imported) */
fptr = fopen(font_path, "r");
if(! fptr)
	{
	fl_show_alert("Cannot open file", font_path, "for read", 0);

	/* ask for different font here ? */

	/* start browser */
	sprintf(temp, "%s/.xste/fonts", home_dir);

	ptr = (char *)fl_show_fselector("SELECT A FONT FILE", temp, "*.ttf", "*.ttf");
	if(! ptr) return 0;

	/*
	extract the fontname from the path-filename
	example:
	/root/.xste/fonts/arial.ttf
	
	find the last '/' and start from there
	*/

	ptr2 = strrchr(ptr, '/');
	if(! ptr2)
		{
		fl_show_alert("Cannot parse pathfilename", ptr, "no last '/' found", 0);
	
		return 0;
		}

	/* ptr2 = /arial.ttf */

	if(font_name) free(font_name);
	font_name = strsave(ptr2 + 1);
	if(! font_name)
		{
		fprintf(stdout, "make_font(): could not re-allocate space for new font_name\n");
		
		return 0;
		}

	/* create new pathfilename */
	if(font_path) free(font_path);
	sprintf(temp, "%s/.xste/fonts/%s", home_dir, font_name);
	font_path = strsave(temp);
	if(! font_path) return 0;

	} /* end if font_path not valid (font_path is actually pathfilename) */
fclose(fptr);

/* create font data directory */
sprintf(temp, "mkdir %s/.xste/data 2> /dev/zero", home_dir);
pptr = popen(temp, "w");
pclose(pptr);

/* directory where to put the temp files */
sprintf(temp, "%s/.xste/data", home_dir);
outdir = strsave(temp);
if(! outdir) return 0;

/* encoding string */
sprintf(temp, "iso-8859-%d", iso_extention); 
encoding = strsave(temp);
if(! encoding) return 0;

encoding_name = encoding;

ppem = font_size;
append_mode = 0;
unicode_desc = 0;

padding = ceil(blur_radius) + ceil(outline_thickness);

if(! prepare_charset() ) return 0;

if(! render() ) return 0;

if(! write_bitmap(bbuffer, 'b') ) return 0;

abuffer = (unsigned char*)malloc(width * height);
if(! abuffer) return 0;

if(! alpha(outline_thickness, blur_radius) ) return 0;

if(! write_bitmap(abuffer, 'a') ) return 0;

free(bbuffer);
free(abuffer);

/* reload the font ! */

/* read in font (also needed for frame counter) */
sprintf(temp, "%s/font.desc", outdir);

pfontd = read_font_desc(temp, 1, 0);
if(! pfontd)
	{
	if(debug_flag)
		{
		printf("make_font(): Could not load font %s\n", temp);
		}

	fl_show_alert("Could not load font", temp, "for read", 1);

	return 0;
	}

pfontd -> outline_thickness = outline_thickness;
pfontd -> blur_radius = blur_radius;

return pfontd;
} /* end function make_font */


font_desc_t *make_old_font(char *font_name, int font_size, int iso_extention, double outline, double blur)
{
font_desc_t *pfontd;
char temp[4096];
char temp2[4096];
FILE *pptr;
FILE *fptr;
char *ptr;
int subtitle_default_font;
double subtitle_font_factor;

if(debug_flag)
	{	
	printf("make_old_font(): arg font_name=%s font_size=%d iso_extention=%d outline=%.2f blur=%.2f\n",\
	font_name, font_size, iso_extention, outline, blur);
	}

/* argument check */
if(! font_name) return 0;
if(font_size <= 0) return 0;
if(iso_extention <= 0) return 0;
if(outline < 0.0) return 0;
if(blur < 0.0) return 0;

/* test if font file exists */
fptr = fopen(font_name, "r");
if(! fptr)
	{
	fl_show_alert("Cannot open file", font_name, "for read", 0);
	return 0;
	}
fclose(fptr);

/* pathfilename of true type font */
if(font_path) free(font_path);
font_path = strsave(font_name);
if(! font_path) return 0;

/* directory where to put the temp files */
sprintf(temp, "%s/.xste/fonts", home_dir);
outdir = strsave(temp);
if(! outdir) return 0;

/* encoding string */
sprintf(temp, "iso-8859-%d", iso_extention); 
encoding = strsave(temp);
if(! encoding) return 0;

encoding_name = encoding;

ppem = font_size;
append_mode = 0;
unicode_desc = 0;

padding = ceil(blur) + ceil(outline);

prepare_charset();

render();

write_bitmap(bbuffer, 'b');

abuffer = (unsigned char*)malloc(width * height);
if(! abuffer) return 0;

alpha(blur, outline);

write_bitmap(abuffer, 'a');

free(bbuffer);
free(abuffer);

/* reload the font ! */

/* this selects some other symbols it seems */
subtitle_default_font = 0;	// 1 = strange symbols like stars etc..

/* this sets the font outline */
subtitle_font_factor = 1; //10.75;	// outline, was .75

/* read in font (also needed for frame counter) */
sprintf(temp, "%s/font.desc", outdir);

pfontd = read_font_desc(temp, subtitle_font_factor, 0);

if(! pfontd)
	{
	if(debug_flag)
		{
		printf("make_old_font(): Could not load font %s\n", temp);
		}

	fl_show_alert("Could not load font", temp, "for read", 1);

	return 0;
	}

/* create fontname for output .ppml file */
// /root/.xste/fonts/arial.ttf
ptr = strrchr(font_name, '/');
if(! ptr)
	{
	fprintf(stderr,\
	"xste: make_old_font(): could not find '/' in font_name=%s\n",\
	font_name);

	return 0;
	}

strcpy(temp2, ptr + 1);

// arial.ttf
ptr = strchr(temp2, '.');
if(! ptr)
	{
	fprintf(stderr,\
	"xste: make_old_font(): could not find '.' in font_name=%s\n",\
	temp);

	return 0;
	}
*ptr = 0;

// arial
sprintf(temp, "%s-%d-iso-8859-%d", temp2, font_size, iso_extention);
if(subtitle_font) free(subtitle_font);
subtitle_font = strsave(temp);
if(! subtitle_font) return 0;

if(debug_flag)
	{
	fprintf(stderr,\
	"xste: make_old_font(): subtitle_font=%s\n", subtitle_font);
	}

/* create font directory and font files for subtitler */
/* no echo */
sprintf(temp, "mkdir %s/.subtitles/%s/%s 1>/dev/zero 2>/dev/zero",\
home_dir, selected_project, subtitle_font);
pptr = popen(temp, "w");
pclose(pptr);

/* move font descriptor file to font dir */
sprintf(temp,\
"mv %s/font.desc %s/.subtitles/%s/%s",\
outdir, home_dir, selected_project, subtitle_font);
pptr = popen(temp, "w");
pclose(pptr);

/* move raw files to font dir */
sprintf(temp,\
"mv %s/*.raw %s/.subtitles/%s/%s",\
outdir, home_dir, selected_project, subtitle_font);
pptr = popen(temp, "w");
pclose(pptr);

return pfontd;
} /* end function make_old_font */

