/*
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/



/*
Wrote this to be able to create output files for submux
(on the vcdimager site contributions)
Creates bitmaps, with position, size and timing info
JP
*/

/*
The subtitle image format
~~~~~~~~~~~~~~~~~~~~~~~~~

It must be in "Windows Bitmap 16 color (4bit) or 256 color (8bit)" format 
and ONLY the first four color can be used as #00 for background and #01, 
#02, #03 for anything. If other colors are used the program uses the 
AND '3' operation to calculate the color index:
subtitle_color_index = bmp_color_index AND 3. 
So #04 becomes #00, #05 becomes #01, #06 becomes #02, etc.

The subtitle script file format
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

The file format is similar to "PHILIPS SVCD DESIGNER 1.5 - 2.0 SUBTITLES FILE"
file format but it must contain ONLY the '[LIST]' section without the '[LIST]'
section identifier. Eg.:

ENG_000001.bmp 01:45:10:11 01:45:14:15 296 068 211 464

or

ENG_000001.bmp 01:45:10:11 01:45:14:15 296 068 211 464 0 255 255 255

where the columns are:

IMAGEFILE STARTTIME ENDTIME WIDTH HEIGHT X Y T0 T1 T2 T3 

IMAGEFILE: filename of the subtitle picture without path 
           (the pictures and the script file MUST BE in the same directory!)
STARTTIME, ENDTTIME: Hour:Minute:Second:Hundredthsecond
WIDTH, HEIGHT: width, height of the bitmap
X,Y: X,Y position (on the movie) of the lower upper corner of the subtitle image
T0, T1, T2, T3: is the contrast (or transparency) value of the #0, #1, #2, #3 colors
                0 means fully transparent
		255 means fully non transparent

BE CAREFUL don't make subtitles with zero or negative duration they will be skipped.
(eg: SubRip 0.97b puts '00:00:00,00' time to the last subtitle ENDTIME)
*/


#include "xste.h"

FD_xste *fd_main_form;
extern FD_xste *fdui;


int write_submux_list()
{
struct frame *pa;
struct frame *pnext, *ptemp;
FILE *fptr;
char temp[4096];
int next_frame_start;
int start_hour, start_minute, end_hour, end_minute;
int start_second, end_second;
int start_hundreds, end_hundreds;
long lthousands;
int text_end_frame;

if(debug_flag)
	{
	printf("write_submux_list(): arg none\n");
	}

/* open .sub file */
sprintf(temp, "%s/%s/%s/%s.sub",\
home_dir, selected_data_directory, selected_project, selected_project);
fptr = fopen(temp, "w");
if(! fptr)
	{
	fl_show_alert(\
		"write_submux_list", "Could not open for write", temp, 1);

	return 0;
	}

for(pa = frametab[0]; pa != 0; pa = pa -> nxtentr)
	{
	if(debug_flag)
		{
		printf("frame=%d pa->text=%s\n", atoi(pa -> name), pa -> text);
		}

	/*
	do not print negative frames, this results in negative times.
	*/
	if( atoi(pa -> name) < 0) continue;

	/* normal situation */
	text_end_frame = pa -> end_frame;

	/* get pointer to next structure entry */ 
	if(pa -> nxtentr) 
		{
		pnext = pa -> nxtentr;	
		
		/* find next line that is not a '*' comand line */
		while(pnext)
			{
			if(pnext -> text[0] != '*') break;

			if(pnext -> nxtentr)
				{	
				ptemp = pnext -> nxtentr;

				pnext = ptemp;
				}	
			} /* end while looking for non '*' entry */ 

		if(pnext)
			{
			/* get the start of the next frame */ 
			next_frame_start = atoi(pnext -> name);
		
			/* test if the current frame ends before the start of the next */
			if(pa -> end_frame > next_frame_start)
				{
				text_end_frame = next_frame_start;
				} /* end if frame ends before next one starts */ 
			}

		} /* end if not last entry */	
	
	/* we have this data at our disposal */ 
	//	char *name;/* this is the start frame */
	//	char *text;
	//	int end_frame; /* this is the end frame */
	//	unsigned long start_sample;
	//	unsigned long end_sample;
 
	/* calculate start_time */
	lthousands = atoi(pa -> name) * dmilli_seconds_per_frame;
	start_hour = lthousands / 3600000;
	lthousands -= start_hour * 3600000; 
	start_minute = lthousands / 60000;
	lthousands -= start_minute * 60000;
	start_second = lthousands / 1000;
	lthousands -= start_second * 1000;
	start_hundreds = lthousands / 10;
	
	/* calculate end_time */
	lthousands = text_end_frame * dmilli_seconds_per_frame;
	end_hour = lthousands / 3600000;
	lthousands -= end_hour * 3600000; 
	end_minute = lthousands / 60000;
	lthousands -= end_minute * 60000;
	end_second = lthousands / 1000;
	lthousands -= end_second * 1000;
	end_hundreds = lthousands / 10;

	/*
	do not print negative frames, this results in negative times,
	not sure submux likes that.
	*/
	if(start_hour < 0) continue;
	if(start_minute < 0) continue;
	if(start_second < 0.0) continue;
	if(start_hundreds < 0) continue;
	
	/* do not print xste command lines in the form '*command' */
	if(pa -> text[0] == '*') continue;

	/* 
	print entry, format:
	example:
	ENG_000001.bmp 01:45:10:11 01:45:14:15 296 068 211 464 0 255 255 255
	or:
	ENG_000001.bmp 01:45:10:11 01:45:14:15 296 068 211 464

	where the columns are:
	IMAGEFILE STARTTIME ENDTIME WIDTH HEIGHT X Y T0 T1 T2 T3 

	what it means:
	IMAGEFILE: filename of the subtitle picture without path 
           (the pictures and the script file MUST BE in the same directory!)
	STARTTIME, ENDTTIME: Hour:Minute:Second:Hundredthsecond
	WIDTH, HEIGHT: width, height of the bitmap
	X,Y: X,Y position (on the movie) of the lower upper corner of the
		subtitle image
	T0, T1, T2, T3: is the contrast (opaqueness).
		value of the #0, #1, #2, #3 colors:
			0 means fully transparent
			255 means fully non transparent

	BE CAREFUL don't make subtitles with zero or negative duration they will
	be skipped.
	*/

	draw_mode = DRAW_MODE_OGT;

	/*
	sets pa -> width, pa -> height, pa -> xpos, pa -> ypos,
	and makes a bitmap file	from pa -> text,
	and subtitle_current_font_descriptor.
	*/
	if(! create_bitmap_file(pa) ) continue;

	fprintf(fptr,\
	"frame%04d.bmp %02d:%02d:%02d:%02d %02d:%02d:%02d:%02d %d %d %d %d %d %d %d %d\n",\
	atoi(pa -> name),\
	start_hour, start_minute, start_second, start_hundreds,\
	end_hour, end_minute, end_second, end_hundreds,\
	pa -> width, pa -> height,\
	pa -> xpos, pa -> ypos,\
	pa -> background_contrast, pa -> pattern_contrast, pa -> emphasis1_contrast, pa -> emphasis2_contrast);
	} /* end for all frames */

/* close the file */
fclose(fptr);

return 1;
}/* end function write_submux_list */


