/*
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "xste.h"

FD_xste *fdui;


int show_subtitle_font_form()
{
if(debug_flag)
	{
	printf("show_subtitle_font_form(): arg none\n");
	}

if(subtitle_font_form_shown_flag) return 1;

fl_show_form(fdui -> subtitle_font_form, FL_PLACE_CENTER, FL_TRANSIENT,\
"xste: subtitle font");

subtitle_font_form_shown_flag = 1;

return 1;
} /* end function show_subtitle_font_form */


int write_ifo_color_table()
{
int a, i;
FILE *fptr;
int r, g, b;
char temp[4096];
int y, u, v;

if(debug_flag)
	{
	fprintf(stderr, "write_ifo_color_table(): arg none\n");
	}

sprintf(temp, "/video/vie/%s/xste-palette.dat", selected_project);

fptr = fopen(temp, "w");
if(! fptr)
	{
	fl_show_alert("could not open file", temp, "for write", 0);

	return 0;
	}

/* write entries in yuv */
for(i = 0; i < 16; i++)
	{
	r = xste_rgb_palette[i][0];
	g = xste_rgb_palette[i][1];
	b = xste_rgb_palette[i][2];

	y = (0.257 * (double)r) + (0.504 * (double)g) +  (0.098 * (double)b) + 16.0;
    if (y > 255) y = 255;
	if( y < 0) y = 0;

	u = (0.439 * (double)r) - (0.368 * (double)g) - (0.071 * (double)b) + 128.0;
    if (u > 255) u = 255;
    if (u < 0) u = 0;

	v = -(0.148 * (double)r) - (0.291 * (double)g) + (0.439 * (double)b) + 128.0;
    if (v > 255) v = 255;
    if (v < 0) v = 0;

	a = (y << 16) | (u << 8) | v;

	fprintf(fptr, "0x%04x\n", a); // 00yyuuvv
	} /* end for i (all defined colors) */

fclose(fptr);

return 1;
} /* end function write_ifo_color_table */


int set_default_subtitle_properties()
{
char temp[4096];

if(debug_flag)
	{
	fprintf(stdout, "set_default_subtitle_properties(): arg none\n");
	}

fl_set_object_label(fdui -> subtitle_font_form_fontname_output_field, default_subtitle_font_name);

//sprintf(temp, "%d", default_subtitle_font_size);
//fl_set_input(fdui -> subtitle_font_form_fontsize_input_field, temp);

//sprintf(temp, "%d", default_subtitle_iso_extension);
//fl_set_input(fdui -> subtitle_font_form_iso_extension_number_input_field, temp);

sprintf(temp, "%.2f", default_subtitle_outline_thickness);
fl_set_input(fdui -> subtitle_font_form_outline_thickness_input_field, temp);

sprintf(temp, "%.2f", default_subtitle_blur_radius);
fl_set_input(fdui -> subtitle_font_form_blur_radius_input_field, temp);

sprintf(temp, "%d", default_subtitle_background_contrast);
fl_set_input(fdui -> subtitle_font_form_contrast0_input_field, temp);

sprintf(temp, "%d", default_subtitle_body_contrast);
fl_set_input(fdui -> subtitle_font_form_contrast1_input_field, temp);

sprintf(temp, "%d", default_subtitle_outline1_contrast);
fl_set_input(fdui -> subtitle_font_form_contrast2_input_field, temp);

sprintf(temp, "%d", default_subtitle_outline2_contrast);
fl_set_input(fdui -> subtitle_font_form_contrast3_input_field, temp);

sprintf(temp, "%d", default_subtitle_extra_character_space);
fl_set_input(fdui -> subtitle_font_form_espace_input_field, temp);

sprintf(temp, "%d", default_subtitle_extra_character_vspace);
fl_set_input(fdui -> subtitle_font_form_vespace_input_field, temp);

fl_set_object_color(fdui -> subtitle_font_form_color0_button,\
	default_subtitle_background_color + FL_FREE_COL1, default_subtitle_background_color + FL_FREE_COL1);

fl_set_object_color(fdui -> subtitle_font_form_color1_button,\
	default_subtitle_body_color + FL_FREE_COL1, default_subtitle_body_color + FL_FREE_COL1);

fl_set_object_color(fdui -> subtitle_font_form_color2_button,\
	default_subtitle_outline1_color + FL_FREE_COL1, default_subtitle_outline1_color + FL_FREE_COL1);

fl_set_object_color(fdui -> subtitle_font_form_color3_button,\
	default_subtitle_outline2_color + FL_FREE_COL1, default_subtitle_outline2_color + FL_FREE_COL1);

fl_set_button(fdui -> subtitle_font_form_anti_alias_button, default_subtitle_anti_alias_flag);
fl_set_button(fdui -> subtitle_font_form_center_text_button, default_subtitle_center_flag);
fl_set_button(fdui -> subtitle_font_form_no_kerning_button, default_subtitle_no_kerning_flag);

return 1;
} /* end function set_default_subtitle_properties */



int force_default_font_parameters_to_all_frames()
{
struct frame *pa;
font_desc_t *pfd;
int status;

if(debug_flag)
	{
	fprintf(stdout, "force_default_font_parameters_to_all_frames(): arg none\n");
	}

if(! fl_show_question(\
"This will set all fonts in all frames to the default settings!\n\
Are you sure?",\
0) )/* mouse on no */
	{
	return 0;
	}

pfd = add_font(\
	default_subtitle_font_name,\
	default_subtitle_font_symbols,\
	default_subtitle_font_size, default_subtitle_iso_extention,\
	default_subtitle_outline_thickness, default_subtitle_blur_radius,\
	default_subtitle_extra_character_space, default_subtitle_extra_character_space);
if(! pfd)
	{
	fprintf(stderr,\
	"xste: main_form_add_button_cb(): could not add font for frame %d\n", selected_start_frame);

	return 0;
	}

status = 0;
if(default_subtitle_center_flag) status |= CENTER_TEXT;
else status &= ~CENTER_TEXT;

if(default_subtitle_anti_alias_flag) status |= DVD_ANTI_ALIAS;
else status &= ~DVD_ANTI_ALIAS;

if(default_subtitle_no_kerning_flag) status |= NO_KERNING;
else status &= ~NO_KERNING;

for(pa = frametab[0]; pa != 0; pa = pa -> nxtentr)
    {
	/* use default font */
	pa -> background = default_subtitle_background_color;
	pa -> pattern = default_subtitle_body_color;
	pa -> emphasis1 = default_subtitle_outline1_color;
	pa -> emphasis2 = default_subtitle_outline2_color;

	pa -> background_contrast = default_subtitle_background_contrast;
	pa -> pattern_contrast = default_subtitle_body_contrast;
	pa -> emphasis1_contrast = default_subtitle_outline1_contrast;
	pa -> emphasis2_contrast = default_subtitle_outline2_contrast;

	if(pa -> fontname) free(pa -> fontname);
	pa -> fontname = strsave(default_subtitle_font_name);
	if(! pa -> fontname)
		{
		fprintf(stderr, "xste: main_form_add_button_cb(): could not allocate space for pa -> fontname\n");
	
		return 0;
		}

	pa -> symbols = default_subtitle_font_symbols;
	pa -> fontsize = default_subtitle_font_size;
	pa -> iso_extension = default_subtitle_iso_extention;
	pa -> espace = default_subtitle_extra_character_space;
	pa -> vespace = default_subtitle_extra_character_vspace;

	pa -> outline_thickness = default_subtitle_outline_thickness;
	pa -> blur_radius = default_subtitle_blur_radius;
  
	pa -> pfd = pfd;

	pa -> status = status;
    } /* end for all frames */
 
if(! save_frames() ) return 0;

return 1;
} /* end function force_default_font_parameters_to_all_frames */

