/*
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "xste.h"

FD_xste *fdui;


static int subtitle_font_form_modified_flag;
static int fontsizes_modified_flag;


void subtitle_font_form_fontname_button_cb(FL_OBJECT *ob, long data)
{
char temp[4096];
char *ptr;
FILE *fptr;
char *ptr2;

/* start  browser */
sprintf(temp, "%s/.xste/fonts", home_dir);

ptr = (char *)fl_show_fselector("SELECT A FONT FILE", temp, "*.ttf", "*.ttf");
if(! ptr) return;
  
/* prevent empty entries */
if(ptr[0] == 0) return;

/* try to open this file for read */
fptr = fopen(ptr, "r");
if(! fptr)
	{
	fl_show_alert("Cannot open file", ptr, "for read", 0);
	return;
	}
fclose(fptr);

/*
extract the fontname from the path-filename
example:
/root/.xste/fonts/arial.ttf

find the last '/' and start from there
*/

ptr2 = strrchr(ptr, '/');
if(! ptr2)
	{
	fl_show_alert("Cannot parse pathfilename", ptr, "no last '/' found", 0);
	
	return;
	}

/* /arial.ttf */
default_subtitle_font_name = strsave(ptr2 + 1);
if(! default_subtitle_font_name) return;

fl_set_object_label(fdui -> subtitle_font_form_fontname_output_field, default_subtitle_font_name);

subtitle_font_form_modified_flag = 1;
} /* end function subtitle_font_form_fontname_button_cb */


void subtitle_font_form_character_point_size_input_field_cb(FL_OBJECT *ob, long data)
{
char *ptr;

if(debug_flag)
	{
	fprintf(stdout, "subtitle_font_form_character_point_size_input_field_cb(): arg none\n");
	}

ptr = (char *)fl_get_input(ob);
if(! ptr) return;

default_subtitle_font_size = atoi(ptr);

subtitle_font_form_modified_flag = 1;
}/* end function subtitle_font_form_character_point_size_input_field_cb */


void subtitle_font_form_character_style_input_field_cb(FL_OBJECT *ob, long data)
{
char *ptr;

if(debug_flag)
	{
	fprintf(stdout, "subtitle_font_form_character_style_input_field_cb(): arg none\n");
	}

ptr = (char *)fl_get_input(ob);
if(! ptr) return;

default_subtitle_iso_extention = atoi(ptr);

subtitle_font_form_modified_flag = 1;
} /* end function subtitle_font_form_character_style_input_field_cb */


void subtitle_font_form_outline_thickness_input_field_cb(\
	FL_OBJECT *ob, long data)
{
char temp[80];
char *ptr;

if(! subtitle_font_form_shown_flag) return;

ptr = (char *)fl_get_input(ob);
if(! ptr) return;

default_subtitle_outline_thickness = atof(ptr);
if(default_subtitle_outline_thickness <= 0.0)
	{
	fl_show_alert("thickness cannot be zero", "try a default button if unsure", "", 0);
	default_subtitle_outline_thickness = 0.01;

	sprintf(temp, "%0.2f", default_subtitle_outline_thickness);
	fl_set_input(ob, temp);

	return;
	}

subtitle_font_form_modified_flag = 1;
} /* end function subtitle_font_form_outline_thickness_input_field_cb */


void subtitle_font_form_outline_blur_radius_input_field_cb(\
	FL_OBJECT *ob, long data)
{
char temp[80];
char *ptr;

if(! subtitle_font_form_shown_flag) return;

ptr = (char *)fl_get_input(ob);
if(! ptr) return;

default_subtitle_blur_radius = atof(ptr);

if(default_subtitle_blur_radius <= 0.0)
	{
	fl_show_alert("blur radius cannot be zero", "try a default button if unsure", "", 0);
	default_subtitle_blur_radius = 0.01;

	sprintf(temp, "%0.2f", default_subtitle_blur_radius);
	fl_set_input(ob, temp);

	return;
	}

subtitle_font_form_modified_flag = 1;
} /* end function subtitle_font_form_outline_blur_radius_input_field_cb */


void subtitle_font_form_espace_input_field_cb(FL_OBJECT *ob, long data)
{
char temp[80];
char *ptr;

if(! subtitle_font_form_shown_flag) return;

ptr = (char *)fl_get_input(ob);
if(! ptr) return;

default_subtitle_extra_character_space = atof(ptr);

if(default_subtitle_extra_character_space < -2)
	{
	fl_show_alert("value cannot be that much negative", "try a default button if unsure", "", 0);
	default_subtitle_extra_character_space = -2;

	sprintf(temp, "%d", default_subtitle_extra_character_space);
	fl_set_input(ob, temp);

	return;
	}

subtitle_font_form_modified_flag = 1;
} /* end function void subtitle_font_form_espace_input_field_cb */


void subtitle_font_form_vespace_input_field_cb(FL_OBJECT *ob, long data)
{
char temp[80];
char *ptr;

if(! subtitle_font_form_shown_flag) return;

ptr = (char *)fl_get_input(ob);
if(! ptr) return;

default_subtitle_extra_character_vspace = atof(ptr);

if(default_subtitle_extra_character_vspace < -50)
	{
	fl_show_alert("value cannot be that much negative", "try a default button if unsure", "", 0);
	default_subtitle_extra_character_vspace = -50;

	sprintf(temp, "%d", default_subtitle_extra_character_vspace);
	fl_set_input(ob, temp);

	return;
	}

subtitle_font_form_modified_flag = 1;
} /* end function void subtitle_font_form_vespace_input_field_cb */


void subtitle_font_form_center_text_button_cb(FL_OBJECT *ob, long data)
{
default_subtitle_center_flag = 1 - fl_get_button(ob);

set_subtitle_font_form_center_text_button_label(default_subtitle_center_flag);

subtitle_font_form_modified_flag = 1;
} /* end function void main_form_center_text_button_cb */


void subtitle_font_form_dvd_default_button_cb(FL_OBJECT *ob, long data)
{
if(! fl_show_question("This will reset all setting to the DVD default OK?", 0) )
	{
	return;
	}

default_subtitle_outline_thickness = 1.5;
default_subtitle_blur_radius = 0.01;

default_subtitle_background_color = 0;
default_subtitle_body_color = 1;
default_subtitle_outline1_color = 2; 
default_subtitle_outline2_color = 15;

default_subtitle_background_contrast = 0;
default_subtitle_body_contrast = 15;
default_subtitle_outline1_contrast = 15;
default_subtitle_outline2_contrast = 0;

default_subtitle_anti_alias_flag = 0;
default_subtitle_no_kerning_flag = 0;
default_subtitle_center_flag = 1;

default_subtitle_extra_character_space = EXTRA_CHAR_SPACE;
default_subtitle_extra_character_vspace = EXTRA_CHAR_VSPACE;

set_default_subtitle_properties();

subtitle_font_form_modified_flag = 1;
} /* end function subtitle_font_form_dvd_default_button_cb */


void subtitle_font_form_cvd_default_button_cb(FL_OBJECT *ob, long data)
{
if(! fl_show_question("This will reset all setting to the CVD default OK?", 0) )
	{
	return;
	}

default_subtitle_outline_thickness = 1.5;
default_subtitle_blur_radius = 2.0;

default_subtitle_background_color = 0;
default_subtitle_body_color = 1;
default_subtitle_outline1_color = 2; 
default_subtitle_outline2_color = 15;

default_subtitle_background_contrast = 255;
default_subtitle_body_contrast = 255;
default_subtitle_outline1_contrast = 0;
default_subtitle_outline2_contrast = 0;

default_subtitle_anti_alias_flag = 0;
default_subtitle_no_kerning_flag = 0;
default_subtitle_center_flag = 1;

default_subtitle_extra_character_space = EXTRA_CHAR_SPACE;
default_subtitle_extra_character_vspace = EXTRA_CHAR_VSPACE;

set_default_subtitle_properties();

subtitle_font_form_modified_flag = 1;
} /* end function subtitle_font_form_cvd_default_button_cb */


void subtitle_font_form_svcd_default_button_cb(FL_OBJECT *ob, long data)
{
if(! fl_show_question("This will reset all setting to the SVCD default OK?", 0) )
	{
	return;
	}

default_subtitle_outline_thickness = 1.5;
default_subtitle_blur_radius = 2.0;

default_subtitle_background_color = 0;
default_subtitle_body_color = 1;
default_subtitle_outline1_color = 2; 
default_subtitle_outline2_color = 15;

default_subtitle_background_contrast = 255;
default_subtitle_body_contrast = 255;
default_subtitle_outline1_contrast = 0;
default_subtitle_outline2_contrast = 0;

default_subtitle_anti_alias_flag = 0;
default_subtitle_no_kerning_flag = 0;
default_subtitle_center_flag = 1;

default_subtitle_extra_character_space = EXTRA_CHAR_SPACE;
default_subtitle_extra_character_vspace = EXTRA_CHAR_VSPACE;

set_default_subtitle_properties();

subtitle_font_form_modified_flag = 1;
} /* end function subtitle_font_form_svcd_default_button_cb */


void subtitle_font_form_no_kerning_button_cb(FL_OBJECT *ob, long data)
{
default_subtitle_no_kerning_flag = fl_get_button(ob);

subtitle_font_form_modified_flag = 1;
} /* end function subtitle_font_form_no_kerning_button_cb */


void subtitle_font_form_anti_alias_button_cb(FL_OBJECT *ob, long data)
{
default_subtitle_anti_alias_flag = fl_get_button(ob);

if(default_subtitle_anti_alias_flag)
	{
	default_subtitle_outline2_color = 15; /* grey by default */

	fl_set_object_color(fdui -> subtitle_font_form_color3_button,\
    default_subtitle_outline2_color + FL_FREE_COL1,\
    default_subtitle_outline2_color + FL_FREE_COL1);

	fl_set_input(fdui -> subtitle_font_form_contrast3_input_field, "15");
	fl_call_object_callback(fdui -> subtitle_font_form_contrast3_input_field);
	}

subtitle_font_form_modified_flag = 1;
} /* end function subtitle_font_form_anti_alias_button_cb */


void subtitle_font_form_color0_button_cb(FL_OBJECT *ob, long data)
{
default_subtitle_background_color = show_subtitle_color_selector_form(default_subtitle_background_color);

fl_set_object_color(ob,\
	default_subtitle_background_color + FL_FREE_COL1,\
	default_subtitle_background_color + FL_FREE_COL1);

subtitle_font_form_modified_flag = 1;
} /* end function subtitle_font_form_color0_button_cb */


void subtitle_font_form_color1_button_cb(FL_OBJECT *ob, long data)
{
default_subtitle_body_color = show_subtitle_color_selector_form(default_subtitle_body_color);

fl_set_object_color(ob,\
	default_subtitle_body_color + FL_FREE_COL1,\
	default_subtitle_body_color + FL_FREE_COL1);

subtitle_font_form_modified_flag = 1;
} /* end function subtitle_font_form_color1_button_cb */


void subtitle_font_form_color2_button_cb(FL_OBJECT *ob, long data)
{
default_subtitle_outline1_color = show_subtitle_color_selector_form(default_subtitle_outline1_color);

fl_set_object_color(ob,\
	default_subtitle_outline1_color + FL_FREE_COL1,\
	default_subtitle_outline1_color + FL_FREE_COL1);

subtitle_font_form_modified_flag = 1;
} /* end function subtitle_font_form_color2_button_cb */


void subtitle_font_form_color3_button_cb(FL_OBJECT *ob, long data)
{
default_subtitle_outline2_color = show_subtitle_color_selector_form(default_subtitle_outline2_color);

fl_set_object_color(ob,\
	default_subtitle_outline2_color + FL_FREE_COL1,\
	default_subtitle_outline2_color + FL_FREE_COL1);

subtitle_font_form_modified_flag = 1;
} /* end function subtitle_font_form_color3_button_cb */


void subtitle_font_form_contrast0_input_field_cb(FL_OBJECT *ob, long data)
{
int a;
char temp[80];
char *ptr;

if(! subtitle_font_form_shown_flag) return;

ptr = (char *)fl_get_input(ob);
if(! ptr) return;

a =  atoi(ptr);

if( (a > 15) || (a < 0) )
	{
	fl_show_alert("contrast range is 0 to 15", "", "", 0);

	a = 15;
	sprintf(temp, "%d", 15);
	fl_set_input(ob, temp);	
	}

default_subtitle_background_contrast = a;

subtitle_font_form_modified_flag = 1;
} /* end function subtitle_font_form_contrast0_input_field_cb */


void subtitle_font_form_contrast1_input_field_cb(FL_OBJECT *ob, long data)
{
int a;
char temp[80];
char *ptr;

if(! subtitle_font_form_shown_flag) return;

ptr = (char *)fl_get_input(ob);
if(! ptr) return;

a =  atoi(ptr);

if( (a > 15) || (a < 0) )
	{
	fl_show_alert("contrast range is 0 to 15", "", "", 0);

	a = 15;
	sprintf(temp, "%d", 15);
	fl_set_input(ob, temp);	
	}

default_subtitle_body_contrast = a;

return;
} /* end function subtitle_font_form_contrast1_input_field_cb */


void subtitle_font_form_contrast2_input_field_cb(FL_OBJECT *ob, long data)
{
int a;
char temp[80];
char *ptr;

if(! subtitle_font_form_shown_flag) return;

ptr = (char *)fl_get_input(ob);
if(! ptr) return;

a =  atoi(ptr);

if( (a > 15) || (a < 0) )
	{
	fl_show_alert("contrast range is 0 to 15", "", "", 0);

	a = 15;
	sprintf(temp, "%d", 15);
	fl_set_input(ob, temp);	
	}

default_subtitle_outline1_contrast = a;

return;
} /* end function subtitle_font_form_contrast2_input_field_cb */


void subtitle_font_form_contrast3_input_field_cb(\
	FL_OBJECT *ob, long data)
{
int a;
char temp[80];
char *ptr;

if(! subtitle_font_form_shown_flag) return;

ptr = (char *)fl_get_input(ob);
if(! ptr) return;

a =  atoi(ptr);

if( (a > 15) || (a < 0) )
	{
	fl_show_alert("contrast range is 0 to 15", "", "", 0);

	a = 15;
	sprintf(temp, "%d", 15);
	fl_set_input(ob, temp);	
	}

default_subtitle_outline2_contrast = a;

return;
} /* end function subtitle_font_form_contrast3_input_field_cb */


void subtitle_font_form_subtitle_fontsize_up_button_cb(FL_OBJECT *ob, long data)
{
} /* end function subtitle_font_form_subtitle_fontsize_up_button_cb */


void subtitle_font_form_subtitle_fontsize_down_button_cb(FL_OBJECT *ob, long data)
{
} /* end function subtitle_font_form_subtitle_fontsize_down_button_cb */


void subtitle_font_form_cancel_button_cb(FL_OBJECT *ob, long data)
{

if(subtitle_font_form_modified_flag)
	{
	load_general_settings();
	load_project_vars();
	}

subtitle_font_form_shown_flag = 0;

fl_hide_form(fdui -> subtitle_font_form);

if(fontsizes_modified_flag)
	{
	load_fontsizes();
	fontsizes_modified_flag = 0;
	}

} /* end function subtitle_font_form_cancel_button_cb */


void subtitle_font_form_accept_button_cb(FL_OBJECT *ob, long data)
{

if(test_for_same_default_colors() ) return;

if(subtitle_font_form_modified_flag)
	{
	save_general_settings();
	save_project_vars();
	}

if(fontsizes_modified_flag)
	{
	save_fontsizes();

	fontsizes_modified_flag = 0;
	}

fl_hide_form(fdui -> subtitle_font_form);

subtitle_font_form_shown_flag = 0;
} /* end function subtitle_font_form_accept_button_cb */


int test_for_same_default_colors()
{
int a;

if(debug_flag)
	{
	fprintf(stdout,\
	"test_for_same_default_colors(): arg none, using\n\
	default_subtitle_background_color=%d\n\
	default_subtitle_body_color=%d\n\
	default_subtitle_outline1_color=%d\n\
	default_subtitle_outline2_color=%d\n",\
	default_subtitle_background_color,
	default_subtitle_body_color,\
	default_subtitle_outline1_color,\
	default_subtitle_outline2_color);
	}

a = 0;
if(default_subtitle_background_color == default_subtitle_body_color) a = 1;
if(default_subtitle_background_color == default_subtitle_outline1_color) a = 1;
if(default_subtitle_background_color == default_subtitle_outline2_color) a = 1;
if(default_subtitle_body_color == default_subtitle_outline1_color) a = 1;
if(default_subtitle_body_color == default_subtitle_outline2_color) a = 1;
if(default_subtitle_outline1_color == default_subtitle_outline2_color) a = 1;

if(a)
	{
	fl_show_alert("Same color for any of", "background, body, outline1, outline2", "not allowed", 0);

	/* return same */
	return 1;
	}

return 0;
} /* end function test_for_same_default_colors */


void set_subtitle_font_form_center_text_button_label(int state)
{
if(state)
	{
	fl_set_object_label(fdui -> subtitle_font_form_center_text_button, "CENTER\nTEXT");
	}
else
	{
	fl_set_object_label(fdui -> subtitle_font_form_center_text_button, "LEFT JUSTIFY\nTEXT");
	}

} /* end function set_subtitle_font_form_center_text_button */

