#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>

#define SUB_BUFFER_MAX      53220 
#define maxx 720
#define maxy 576

#ifndef O_BINARY
#define O_BINARY 0 //for mingw
#endif

#define optimize 0
#define until_next_sub 1 //if 0 subs without length are made when subs are overlapping
#define tbs  90
#define even_yofs 1
#define even_xd 1

extern int debug_flag;

typedef struct
    {
    unsigned short int BM;
    unsigned int size;
    unsigned int r0;
    unsigned int ofs;
    unsigned int hsize;
    unsigned int x;
    unsigned int y;
    unsigned short int bplanes;
    unsigned short int bpp;
    unsigned int comp;
    unsigned int imgsize;
    unsigned int xpm,ypm;
    unsigned int cused;
    unsigned int cimp;
    } __attribute__ ((packed)) bmpt;

typedef struct
{
unsigned char r, g, b, t;
} palt;
   
static int colors_specified_flag;
static unsigned char bb, bc;
static unsigned char *sub, *cbuf;
static unsigned char *img, *fimg;
static unsigned int x0, y0, xd, yd;
static int sd;
static int spts;
static int ofs;
static int pattern, background, emphasis1, emphasis2;
static int current_pattern, current_background, current_emphasis1, current_emphasis2;
static palt pal[4];
static palt epal[4];
static int current_background_contrast;
static int current_pattern_contrast;
static int current_emphasis1_contrast;
static int current_emphasis2_contrast;
static int current_submux_command;
static int c_start_line;
static int c_end_line;
static int c_h_start[16];
static int c_background[16];
static int c_pattern[16];
static int c_emphasis1[16];
static int c_emphasis2[16];
static int c_contrast0[16];
static int c_contrast1[16];
static int c_contrast2[16];
static int c_contrast3[16];
static int c_h_blocks;
static int current_c_start_line;
static int current_c_end_line;
static int current_c_h_start[16];
static int current_c_background[16];
static int current_c_pattern[16];
static int current_c_emphasis1[16];
static int current_c_emphasis2[16];
static int current_c_contrast0[16];
static int current_c_contrast1[16];
static int current_c_contrast2[16];
static int current_c_contrast3[16];
static int current_c_h_blocks;
static int colors_specified_flag;
static unsigned char *change_colcon_size;
static unsigned char *subtitle_image_buffer;
static unsigned char *subtitle_packet_buffer;
static unsigned char *even_buf, *odd_buf;
static int store_mode;
static unsigned char *ocptr, *ecptr;
static int rnibbles;
static int oddp;
static int max_bits_in_line;


//ssize_t read(int fd, void *buf, size_t count)
size_t cread(int fd, void *buf, size_t count)
{
size_t bytes_read;
size_t rret;

bytes_read = 0;
while(bytes_read < count)
	{
	errno = 0;
	rret = read(fd, buf + bytes_read, count - bytes_read);
	if(rret < 0)
		{
		fprintf(stderr, "WHOOPY ! read returned=%d, requested=%d\n", rret, count - bytes_read);	

		if(errno == EAGAIN) continue;
		if(errno == EINTR) continue;

		perror("read(): ");
		fprintf(stderr, "read(): error\n");

		return -1;
		}

	/* test for EOF */
	if(rret == 0)
		{
		return 0;
		}

	if(rret != count - bytes_read)
		{
		fprintf(stderr, "read(): retry count=%d bytes_read=%d **************************\n",\
		count, bytes_read);
		}
		
	bytes_read += rret;
	}

return bytes_read;
} /* end function cread */


int read_bmp(char *name)
{
bmpt bmp;
int fdr;
unsigned int a, d, i, x, y, size, pals, cst[256];
unsigned short int b;
unsigned char c;
unsigned char *limg;
palt lpal[256];
int pallette_start;

for(i = 0; i < maxx * maxy; i++)
	{
	img[i] = 0;
	}

fdr = open(name, O_RDONLY | O_BINARY);
if(fdr < 0)
	{
	fprintf(stderr,"error, unable to open file %s\n",name);
	return 0;
	} 
 
if( cread(fdr, &bmp, 54) < 54)
	{
	fprintf(stderr,"error, file too short: %s\n",name);
	close(fdr);
	return 0;
	}	
 
if(bmp.BM  !=  19778)
	{
	fprintf(stderr,"error, file %s isn't a bmp\n",name);
	close(fdr);
	return 0;
	}
 
if(debug_flag) fprintf(stderr,"xd: %d  yd: %d\n", bmp.x, bmp.y);
 
xd = bmp.x;
yd = bmp.y;

if( (bmp.x > 720) || (bmp.y > 576) )
	{
	fprintf(stderr,"error, image larger than 720x576\n");
	close(fdr);
	return 0;
	} 

if( (bmp.bpp  !=  4)  &&  (bmp.bpp  !=  8) )
	{
	fprintf(stderr,"error, only 4 & 8bpp images supported\n");
	close(fdr);
	return 0;
	}

if(bmp.comp)
	{
	fprintf(stderr,"error, only uncompressed images are supported\n");
	close(fdr);
	return 0;
	}

d = 0;
 
//lseek(fdr, bmp.hsize - 40, SEEK_CUR);
 
/*
start color table
file header size + info header size
*/

pallette_start = 14 + bmp.hsize; // hsize = BitmapInfoHeader size!

if(debug_flag)
	{
	fprintf(stderr, "name=%s pallette_start=%d\n", name, pallette_start);
	}

lseek(fdr, pallette_start, SEEK_SET);

pals = 1  <<  bmp.bpp;

for(b = 0; b < pals; b++)
	{
	d += cread(fdr, &lpal[b].b, 1); 
	d += cread(fdr, &lpal[b].g, 1); 
	d += cread(fdr, &lpal[b].r, 1); 
	d += cread(fdr, &c, 1);
	if(debug_flag)
		{
		fprintf(stderr, "read_bmp(): lpal[b].b=%d lpal[b].g=%d lpal[b].r=%d c=%d d=%d\n",\
		lpal[b].b, lpal[b].g, lpal[b].r, c, d);
		}
	}
 
if(d != pals * 4)
	{
	fprintf(stderr,"error file too short: %s\n",name);
	close(fdr);
	return 0;
	}	
 
if(bmp.bpp  ==  4)
	{
	if(xd & 7) size = (xd & (-8) ) + 8;
	else size = xd;
  
	size *= yd;
	size /= 2;

	limg = malloc(size);
  
	lseek(fdr, bmp.ofs, SEEK_SET);
  
	if( cread(fdr, limg, size)  !=  size)
		{
		fprintf(stderr,"error, file too short: %s\n",name);
		free(limg);
		close(fdr);
		return 0;
		}	
 
	d = 0;
	for(y = yd - 1; y  !=  -1; y--)
		{
		for(x = 0; x < xd; x += 2)
			{
			img[y * xd + x] = limg[d]  >>  4;
			if(x + 1  ==  xd)
				{
				/* should clear img[...] here? */
				d++;
				}
			else img[y * xd + x + 1] = limg[d++] & 15;
			}

		if(d & 3) d += 4 - (d & 3);
		} 

	/*
	ONLY if values for fg, bg, e1 anf e2 specified in .sub file,
	make a 4 bits bitmap representing 0 = bg, 1 = fg, 2 = e1, 3 = e2.
	This flags is ONLY set if all 4 color values afre specified.

	NEVER use 2 the same values for colors!!!!
	*/
	if(colors_specified_flag)
		{

		/* check if not same colors used */
		a = 0;
		if(background == pattern) a = 1;
		if(background == emphasis1) a = 1;
		if(background == emphasis2) a = 1;

		if(pattern == emphasis1) a = 1;
		if(pattern == emphasis2) a = 1;

		if(emphasis1 == emphasis2) a = 1;

		if(a)
			{
			fprintf(stderr,\
	"ERROR some identical colors used for background=%d pattern=%d emphasis1=%d emphasis2=%d\n",\
			background, pattern, emphasis1, emphasis2);

			return 0;
			}

		d = 0;
		for(y = 0; y < yd; y++)
			{
			for(x = 0; x < xd; x++)
				{
				if(img[d] == background)		img[d] = 0; // background
				else if(img[d] == pattern)		img[d] = 1; // pattern
				else if(img[d] == emphasis1)	img[d] = 2; // emphasis1
				else if(img[d] == emphasis2)	img[d] = 3; // emphasis2

				d++;
				} /* end for x */
			} /* end for y */
		} /* end if colors_specified_flag */

	} /* end if 4 bpp */
else /* 8 bits per pixel */
	{
 	if(xd & 3) size = (xd & (-4) ) + 4;
	else size = xd;
  
	size *= yd;

	limg = malloc(size);
  
	lseek(fdr, bmp.ofs, SEEK_SET);
  
	if(cread(fdr, limg, size)  !=  size)
		{
		fprintf(stderr,"error, file too short: %s\n",name);
		free (limg);
		close(fdr);

		return 0;
		}	
	d = 0;
	for(y = yd - 1; y  !=  -1; y--)
		{
		for(x = 0; x < xd; x++) img[y * xd + x] = limg[d++];

		if(d & 3) d += 4 - (d & 3);
		} 
	} /* end if 8 bpp */
 
free(limg);
close(fdr);
 
if(debug_flag)
	{
	for(i = 0; i < pals; i++)
		{
		fprintf(stderr,\
		"read_bmp(): 1 lpal[%d].b=%d lpal[%d].g=%d lpal[%d].r=%d c=%d d=%d\n",\
		i, lpal[i].b, i, lpal[i].g, i, lpal[i].r, c, d);
		}
	}

for(a = 0; a < pals; a++) cst[a] = 0;

a = 0;
for(y = 0; y < yd; y++)
	{
	for(x = 0; x < xd; x++)
		{
/*
		fprintf(stderr, "read_bmp(): a=%d img[a]=%d cst[%d]=%d\n",\
		a, img[a], img[a], cst[img[a]]);
*/

		cst[ img[a++] ]++;
		}
	}

if(debug_flag)
	{
	fprintf(stderr, "read_bmp(): name=%s pals=%d yd=%d xd=%d\n", name, pals, yd, xd);
	}

b = 0;
for(a = 0; a < pals; a++)
	{
	if(debug_flag)
		{
		fprintf(stderr, "read_bmp(): cst[%d]=%d b=%d\n", a, cst[a], b);
		}

	if(cst[a])
		{
		if(b > 3)
			{
			if(debug_flag) fprintf(stderr,\
"warning: more than 4 colors found in bitmap %s, setting color %d to 0\n",\
name, a);
			lpal[a].t = 0;
			}
		else
			{
		   	pal[b].r = lpal[a].r;
			pal[b].g = lpal[a].g;
			pal[b].b = lpal[a].b;

			if(debug_flag)
				{
				fprintf(stderr, "read_bmp(): 3 setting\n\
				b=%d a=%d\n\
				pal[b].r=%d lpal[a].r=%d\n\
				pal[b].g=%d lpal[a].g=%d\n\
				pal[b].b=%d lpal[a].b=%d\n",\
				b, a,\
				pal[b].r, lpal[a].r,\
				pal[b].g, lpal[a].g,\
				pal[b].b, lpal[a].b);
				}

			lpal[a].t = b++;

			if(debug_flag)
				{
				fprintf(stderr, "read_bmp(): 3a setting\n\
				lpal[a].t=%d\n",
				lpal[a].t);
				}
			} /* end if b <= 3 */
		} /* end if cst[a] */
	} /* end for all pallettes */ 

a = 0;
for(y = 0; y < yd; y++)
	{
	for(x = 0; x < xd; x++)
		{
//		img[a++] = lpal[ img[a] ].t; // ORIGINAL

		img[a] = lpal[ img[a] ].t;

//		img[a++] = pal[ img[a] ].t;

		a++; // -Wall
		}
	}

return 1;
} /* end function read_bmp */


int imgfix()
{
int a, b, x0_, y0_, xd_, yd_, x, y, j;
int cst[4];
unsigned char c, r;
if (!optimize)
	{
	for(c = 0; c < 4; c++)
		{
		epal[c].r = pal[c].r;
		epal[c].g = pal[c].g;
		epal[c].b = pal[c].b;
		epal[c].t = pal[c].t;
		} 
	a = 0;
	for(y = 0; (y < yd) &&  (y < yd); y++) 
		{
		for(x = 0; (x < xd)  &&  (x < xd); x++)
			{
//			fimg[a++] = img[a]; // ORIGINAL

			fimg[a] = img[a];
			a++; // -Wall	
    		}
		}

	return 1;
	}
 
//xd and yd should be multiples of 4...
a = 0;
x0_ = maxx;
xd_ = 0;
y0_ = yd_ = -1;
cst[0] = cst[1] = cst[2] = cst[3] = 0;
  
for(y = 0; y < yd; y++) 
	for(x = 0;x < xd; x++)
		{
		if (pal[img[a++]].t)
			{
			if (y0_ == -1) y0_ = y;
			if (x < x0_) x0_ = x;
			if (x > xd_) xd_ = x;
			yd_ = y;
			}
		}
  
if(yd_ != -1)
	{ 
	r = 0;

	for(a = 3; a > 0;a--) if ( !pal[a].t) r = a;
  
	if (debug_flag)
		fprintf(stderr,"colors rotated %d steps\n", r);

	for(c = 0; c < 4; c++)
		{
		epal[c].r = pal[(c + r) % 4].r;
		epal[c].g = pal[(c + r) % 4].g;
		epal[c].b = pal[(c + r) % 4].b;
		epal[c].t = pal[(c + r) % 4].t;
		}

	r = 4 - r;
   
	xd_ -= x0_ -1;
	yd_ -= y0_ -1;
  
	b = 0;
	c = 0;
	for(a = 3; a > 0; a--) if (!epal[a].t) c = a;
  
	y = 0;
  
	if( ( (y0 + y0_) & 1)  &&  (even_yofs))
		{
		if (y0_) y0_--;
		else
			{
			for (j = 0;j < xd_ + xd; j++) fimg[b++] = c;
			y++;
			yd_++;
			y0--;
			} 
		} 
  
	if (even_xd) if (xd_ & 1) xd_++;
  
	a = y0_ * xd + x0_;

	for(;(y < yd_)  &&  (y < yd); y++)
		{
		for(x = 0; (x < xd_) && (x < xd); x++)
			fimg[b++] = (img[a++] + r) % 4;

		if(xd > xd_) a += xd - xd_;
		if(xd < xd_) for (j = 0; j < xd_ - xd; j++)
			fimg[b++] = c;
		} 
 
	while (yd_ & 1)
		{
		for(x = 0; x < xd_; x++) fimg[b++] = c;
		yd_++;
		} 
  
	x0 += x0_;
	y0 += y0_;
	xd = xd_;
	yd = yd_;
 
	if (debug_flag)
		{
		fprintf(stderr,\
		"x0: %d  y0: %d  w: %d  h: %d  start: %fs  len: %fs\n",\
		x0, y0, xd, yd, (double)spts / 90000, (double)sd / 90000); 
		}

	return 1;
	} 
 
return 0;
} /* end function imgfix */


unsigned char gn()
{
if(!bc)
	{
	bb = sub[ofs++];
	bc = 1;
	return bb >> 4; 
	}
bc = 0;

if(debug_flag)
	{
	fprintf(stderr, "gn(): returning sub=%p ofs=%d bc=%d val=0x%01x\n", sub, ofs, bc, bb & 15);
	}

return bb & 15;
} /* end function gn */


int store_nibble(int val)
{
int a;
unsigned char *ptr;

if(store_mode == 1) ptr = ocptr;
else ptr = ecptr;

if(debug_flag)
	{
	fprintf(stderr,\
	"store_nibble(): arg val=0x%01x using store_mode=%d ptr=%p\n",\
	val, store_mode,  ptr);
	}

if(val > 0xf)
	{
	fprintf(stderr, "store_nibble(): val > 0xf = 0x%0x\n", val);

	return 0;
	}

a = *ptr & 0xff;
if(! oddp)
	{
	a &= 0x0f;
	a += (val << 4);
	*ptr = a;
	}
else
	{
	a &= 0xf0;
	a += val;
	*ptr = a;

	if(store_mode == 1) ptr = ocptr++;
	else ptr = ecptr++;
	}

oddp = 1 - oddp;

rnibbles++;

return 1;
} /* end function store_nibble */


int do_rle(int count, int color)
{
int a, b;

if(debug_flag)
	{
	fprintf(stderr, "do_rle(): arg count=%d color=%d\n", count, color);
	}

/* argument check */
if(count > 255)
	{
	fprintf(stderr, "do_rle(): count=%d > 255, error.\n", count);

	return 0;
	}
if(color > 3)
	{
	fprintf(stderr, "do_rle(): color=%d > 3, error.\n", color);

	return 0;
	}

/* make rle code in b */
a = count << 2;
b = a | color;

/* b now ranges from 0x4 up, because count is at least 1 */

if(count >= 64)			// 64 - 255
	{
	/* 64-255, 16 bits, 0 0 0 0  0 0 n n  n n n n  n n c c */
	if(! store_nibble(0) ) return 0;
	if(! store_nibble( (a & 0xf00) >> 8) ) return 0;
	if(! store_nibble( (a & 0xf0) >> 4) ) return 0;
	if(! store_nibble( b & 0xf) ) return 0;

	return 4;
	}
else if(count >= 16)	// 16 - 63
	{
	/* 16 - 63, 12 bits, 0 0 0 0  n n n n  n n c c */
	if(! store_nibble( 0) ) return 0;
	if(! store_nibble( (a & 0xf0) >> 4) ) return 0;
	if(! store_nibble( b & 0xf) ) return 0;;

	return 3;
	}
else if(count >= 4)		// 4 - 15
	{
	/* 4-15, 8 bits, 0 0 n n  n n c c */
	if(! store_nibble( (a & 0xf0) >> 4) ) return 0;
	if(! store_nibble( b & 0xf) ) return 0;

	return 2;
	}
else					// 1 - 3
	{
	/* 1-3, 4 bits, n n c c */
	if(! store_nibble( b) ) return 0;

	return 1;
	}

/* never here */
return 0;
} /* end function do_rle */


int send_colors(int color, int count)
{
int a, b, c, d, i;

if(debug_flag)
	{
	fprintf(stderr, "send_colors(): arg color=%d count=%d\n", color, count);
	}

c = count / 255;
d = count % 255;

b = 0;
for(i = 0; i < c; i++)
	{
	a = do_rle(255, color);
	if(! a)
		{
		fprintf(stderr, "send_colors(): error.\n");

		return -1;
		}

	b += a;
	} /* end for i */

if(d == 0) return b;

a = do_rle(d, color);
if(! a)
	{
	fprintf(stderr, "send_colors(): aborting.\n");

	return -1;
	}

b += a;

return b;
} /* end function send_colors */


int dvd_encode()
{
int a, e, i;
int subtitle_packet_size;
int xstart, xsize, xend;
int ystart, ysize, yend;
unsigned char *cptr, *next_command_ptr;
int display_duration_in_frames;
int offset0, offset1;
unsigned int delay_units;
int image_size;
int idx, new_pos;
int bits_in_odd_line, bits_in_even_line;
int x, y;
unsigned char *icptr;
int odd;
int odd_line_nibbles, even_line_nibbles;

max_bits_in_line = 0;
oddp = 0;

if(debug_flag)
	{
	fprintf(stderr,\
	"dvd_encode(): spts=%d sd=%d x=%d yd=%d x0=%d y0=%d\n",\
	spts / 90000, sd / 90000, xd, yd, x0, y0);
	}
	
xstart = x0;
xsize = xd;
ystart = y0;
ysize = yd;


i = 0; /* for gcc -Wall */
idx = 0; /* for gcc -Wall */

/*
720 x 576 = 414720 bytes, for a 2 bit bitmap = 103680 bytes, compressing MUST reduce this to less then 65536 bytes,
or a 2 byte number - overhead 65535 - 65507 = 28 bytes, as max. 65507 is OK).
This 23 bytes control buffer, 6 bytes end sequence?
*/

display_duration_in_frames = 515;
offset0 = 0;
offset1 = 0;

memset(subtitle_image_buffer, 0, maxx * maxy * 3);

// make this int check_bitmap(char *data, int x0, int y0, int xd, int yd)

/* encode the .ppm to DVD run length encoded format */
/* for all bytes in img */

/*
(X >> 2) is the number of pixels to display, and (X & 0x3)
is the color of the pixel.
*/

/* clear buffers */
memset(even_buf, 0, xd * yd);
memset(odd_buf, 0, xd * yd); // interlaced part, the odd lines go here

//memset(even_buf, 0, maxx * maxy); //xd * yd);
//memset(odd_buf, 0, maxx * maxy); //xd * yd); // interlaced part, the odd lines go here

even_line_nibbles = 0;
odd_line_nibbles = 0;
rnibbles = 0;
odd = 0;

icptr = fimg;  
//icptr = img;// use if call to imgfix() commented out

/*
Recalculate the 16 positions bitmap to a 4 position one
This is done by testing if the value equals the background, pattern,
emphasis1 or emphasis2 value, and if so, assign
0, 1, 2, 3 respectively
*/ 

/* run length encode data */
ecptr = even_buf;
ocptr = odd_buf;
for(y = 0; y < yd; y++)
	{
	bits_in_odd_line = 0;
	bits_in_even_line = 0;

	if(oddp %2)
		{
		fprintf(stderr, "dvd_encode(): oddp odd at start of line, alignment error\n");
		return 0;
		}

	if(!odd) store_mode = 0; // select ecptr for output
	else store_mode = 1; // select ocptr for output	

	new_pos = 0;
	for(x = 0; x < xd; x++)
		{
		idx = y * xd + x;
		
		/* get color from x,y */

		if(icptr[idx + 1] != icptr[idx])
			{
			e = send_colors(icptr[idx], x - new_pos + 1);
			if(e == 0) return 0;

			if(odd) odd_line_nibbles += e;
			else even_line_nibbles += e;
		
			if(odd) bits_in_odd_line += e * 4;
			else bits_in_even_line += e * 4;

			new_pos = x + 1;
			}
		} /* end for x */

	/*
	One special case,
	encoding a count of zero using the 16-bit format,
	indicates the same pixel value until the end of the line. 
	*/
	
	if( (x - new_pos) != 0)
		{
		/* send same colors to end of line */
		if(! store_nibble(0) ) return 0;
		if(! store_nibble(0) ) return 0;
		if(! store_nibble(0) ) return 0;
		if(! store_nibble(icptr[idx]) ) return 0; /* color 0-3, high 2 bits zero of this nibble */

		if(odd) odd_line_nibbles += 4;
		else even_line_nibbles += 4;


		if(odd) bits_in_odd_line += 16;
		else bits_in_even_line += 16;
		}

	/*
	If, at the end of a line, the bit count is not a multiple of 8, four fill bits of 0 are added.
	*/

	if(odd)
		{
		if(odd_line_nibbles % 2)
			{
			if(! store_nibble(0) ) return 0;
			odd_line_nibbles++;

			bits_in_odd_line += 4;
			}
		}
	else
		{
		if(even_line_nibbles % 2)
			{
			if(! store_nibble(0) ) return 0;
			even_line_nibbles++;

			bits_in_even_line += 4;
			}
		}

	if(debug_flag)
		{
		fprintf(stdout,\
		"dvd_encode(): bits_in_odd_line=%d bits_in_even_line=%d\n", bits_in_odd_line, bits_in_even_line);
		}

	if(bits_in_odd_line > max_bits_in_line) max_bits_in_line = bits_in_odd_line;
	if(bits_in_even_line > max_bits_in_line) max_bits_in_line = bits_in_even_line;

	if(debug_flag)
		{
		fprintf(stderr, "dvd_encode(): max_bits_in_line=%d\n", max_bits_in_line);
		}

	if(max_bits_in_line > 1440)
		{
		if(debug_flag)
			{
			fprintf(stderr,\
			"dvd_encode(): bitmap too wide, max_bits_in_line=%d > 1440\n",\
			max_bits_in_line);
			}

// amaroo3 frame8422.bmp font 22 v0.04 h0.02,
// bitmaps like this are dopped by standalone DVD players,
// mplayer draws a black line in the sub.
// Reformatting text is only option?

		return 0;
		}

	odd = 1 - odd;
	} /* end for y (all lines in an img) */ 

/* align on byte */
if(even_line_nibbles % 2) even_line_nibbles++;
if(odd_line_nibbles % 2) odd_line_nibbles++;

/* nibbles to bytes */
image_size = (even_line_nibbles / 2) + (odd_line_nibbles / 2); 

offset0 = 0;
offset1 = even_line_nibbles / 2;

/* max image size ! */
if(image_size > SUB_BUFFER_MAX)
	{
	fprintf(stderr, "dvd_encode(): image_size=%d > %d\n", image_size, SUB_BUFFER_MAX);

	return 0;
	}

/* copy even and odd buffers to buffer */
memcpy(subtitle_image_buffer, even_buf, even_line_nibbles / 2);
memcpy(subtitle_image_buffer + offset1, odd_buf, odd_line_nibbles / 2); 

if(debug_flag)
	{
	fprintf(stderr,
	"rnibbles=%d even_line_nibbles=%d odd_line_nibbles=%d offset1=%d image_size=%d\n",\
	rnibbles, even_line_nibbles, odd_line_nibbles, offset1, image_size);

	fprintf(stderr,\
	"img=%p xd=%d yd=%d rnibbles=%d image_size=%d\n\
	even_line_nibbles=%d odd_line_nibbles=%d even + odd line nibbles=%d\n\
	offset0=%d offset1=%d\n",\
	img, xd, yd, rnibbles, image_size,\
	even_line_nibbles, odd_line_nibbles, even_line_nibbles + odd_line_nibbles,\
	offset0, offset1);
	}

if(debug_flag)
	{
	fprintf(stderr,\
	"subtitle_image_buffer=%p image_size=%d\n", subtitle_image_buffer, image_size);
	fprintf(stderr, "subtitle_packet_buffer=%p\n", subtitle_packet_buffer);
	}

cptr = subtitle_packet_buffer;
//2 bytes packet size, to be filled in later
//2 bytes pointer to control area, to be filled in later
cptr += 4;

if(current_submux_command & 1)
	{
	/* copy image data to subtitle_packet_buffer */
	memcpy(cptr, subtitle_image_buffer, image_size);
	cptr += image_size;
	} /* end if current_submux_command & 1 */

/* start first command block */
/* 
control area starts here, with same pointer
SP_DCSQT
Sub-Picture Display Control SeQuence Table
This area contains blocks (SP_DCSQ) of commands to the decoder.
Each SP_DCSQ begins with a 2 word header

offset  name            contents
0       SP_DCSQ_STM     delay to wait before executing these commands.
                         The units are 90KHz clock (same as PTM) divided by 1024 - see below.

2       SP_NXT_DCSQ_SA  offset within the Sub-Picture Unit to the next SP_DCSQ.
                         If this is the last SP_DCSQ, it points to itself.

Converting frames and time to SP_DCSQ_STM values
The direct method of converting time to delay values is to multiply time in seconds by 90000/1024 and
truncate the value.
Rounding up will cause the display to occur one frame late.
*/

/* set pointer to this command block */
a = cptr - subtitle_packet_buffer;
subtitle_packet_buffer[2] = (a / 256) & 0xff;
subtitle_packet_buffer[3] = (a % 256) & 0xff;

if(debug_flag)
	{
	fprintf(stderr,
	"pointer to this command block=%d\n",\
	subtitle_packet_buffer[2] * 256 + subtitle_packet_buffer[3]);
	}

a = 0; // delay to wait before executing this command block
*cptr = (a / 256) & 0xff;
cptr++;
*cptr = (a % 256) & 0xff;
cptr++;

/* remember position, will set later */
next_command_ptr = cptr;
cptr++; // pointer to next command block, 2 bytes, to be filled in from next command block. 
cptr++;
if(debug_flag)
	{
	fprintf(stderr,
	"next_command_ptr=%p\n", next_command_ptr);
	}

if(current_submux_command & 8)
	{
	/* command 0, forced start display time, 1 byte. */
	*cptr = 0;
	cptr++;
	}

if(current_submux_command & 16)
	{
	/* Either FSTA_DSP or STA_DSP shall be described in the SP_DCSQ#0 */
	/* command 1, start display time, 1 byte */
	*cptr = 1;
	cptr++;
	}

if(current_submux_command & 1)
	{
	/* selected palettes for pixel value */

	/* 
	Set pointers to the color lookup table in the IFO (CLUT) for each parameter.
	Range = 0 - 15
	*/
	// now set in rns for defaults and if specified in the .sub file.

	/* 
	command 3, SET_COLOR, 3 bytes
	03 - SET_COLOR - provides four indices into the CLUT for the current PGC,
	to associate with the four pixel values.
	One nibble per pixel value for a total of 2 bytes.
	e2 e1   p b
	The CLUT is the color table in the IFO.
	*/

	if(debug_flag)
		{
		fprintf(stderr,\
		"current_background=%d current_pattern=%d current_emphasis1=%d current_emphasis2=%d\n",\
		current_background, current_pattern, current_emphasis1, current_emphasis2);
		}

	*cptr = 3;
	cptr++;
	*cptr = ( (current_emphasis2 & 0xf) << 4) | (current_emphasis1 & 0xf);
	cptr++;
	*cptr = ( (current_pattern & 0xf) << 4) | (current_background & 0xf);
	cptr++;

	if(debug_flag)
		{
		for(i = 0; i < 4; i++)
			{
			fprintf(stderr,\
			"epal[%d].r=%d epal[%d].g=%d epal[%d].b=%d epal[%d].t=%d\n",\
			i, epal[i].r, i, epal[i].g, i, epal[i].b, i, epal[i].t);
			}

		for(i = 0; i < 4; i++)
			{
			fprintf(stderr,\
			"pal[%d].r=%d pal[%d].g=%d pal[%d].b=%d pal[%d].t=%d\n",\
			i, pal[i].r, i, pal[i].g, i, pal[i].b, i, pal[i].t);
			}
		} /* end if debug_flag */

	/* command 4, alpha blend, contrast / transparency t_palette, 3 bytes  0, 15, 15, 15 */

	if(debug_flag)
		{
		fprintf(stderr, "epal[0].t=%d epal[1].t=%d epal[2].t=%d epal[3].t=%d\n",\
		epal[0].t, epal[1].t, epal[2].t, epal[3].t);
		}

	if(debug_flag)
		{
		fprintf(stdout,\
		"current_background_contrast=%d current_pattern_contrast=%d\n\
		current_emphasis1_contrast=%d current_emphasis2_contrast=%d\n",\
		current_background_contrast, current_pattern_contrast,\
		current_emphasis1_contrast, current_emphasis2_contrast);
		}

	*cptr = 4;
	cptr++;
	*cptr = ( (current_emphasis2_contrast & 0xf) << 4) | (current_emphasis1_contrast & 0xf); // 3(h) 2(l) blend1
	cptr++;
	*cptr = ( (current_pattern_contrast & 0xf) << 4) | (current_background_contrast & 0xf); // 1(h) 0(l) blend2
	cptr++;

	/* command 5, display area, 7 bytes from: startx, xsize, starty, ysize */
	xsize -= 1;
	ysize -= 1;

	/* for interlace need to start on an even line (0,2,4,..), else text jumps up and down */
	if(ystart % 2) ystart--;

	xend = xstart + xsize;
	yend = ystart + ysize;

	if(debug_flag)
		{
		fprintf(stderr, "in control buffer: xsize=%d ysize=%d  xstart=%d ystart=%d  xend=%d yend=%d\n",\
		xsize, ysize,  xstart, ystart,  xend, yend);
		}

	*cptr = 5;
	cptr++;
	*cptr = ( ( (xstart & 0xf00) >> 8) << 4)	| ( (xstart & 0xf0) >> 4);		// xstart xstart
	cptr++;
	*cptr = ( (xstart & 0xf) << 4)				| ( (xend & 0xf00) >> 8);		// xstart xend
	cptr++;
	*cptr = ( ( (xend & 0xf0) >> 4) << 4)		| (xend & 0xf);					// xend xend
	cptr++;
	*cptr = ( ( (ystart & 0xf00) >> 8) << 4)	| ( (ystart & 0xf0) >> 4);		// ystart ystart
	cptr++;
	*cptr = ( (ystart & 0xf) << 4)				| ( (yend & 0xf00) >> 8);		// ystart yend
	cptr++;
	*cptr = ( ( (yend & 0xf0) >> 4) << 4)		| (yend & 0xf);					// yend yend
	cptr++;

	/* command 6, image offsets, 5 bytes */
	*cptr = 6;
	cptr++;

	a = offset0 + 4;
	*cptr =  (a / 256) & 0xff;
	cptr++;
	*cptr = (a % 256) & 0xff;
	cptr++;

	a = offset1 + 4;
	*cptr = (a / 256) & 0xff;
	cptr++;
	*cptr = (a % 256) & 0xff;
	cptr++;
	} /* end if current_submux_command & 1 */

if(current_submux_command & 4)
	{
	if(debug_flag)
		{
		fprintf(stdout, "dvd_encode(): COLCON\n");
		}

	/*
	command 7
	07 - CHG_COLCON - allows for changing the COLor and CONtrast within one or more areas of the display.
	This command contains a series of parameters, arranged in a hierarchy.
	Following the command byte is a 2-byte value for the total size of the parameter area, including the
	size word.
	The parameter sequence begins with a LN_CTLI, which defines a vertically bounded area of the display.
	The LN_CTLI may include from one to fifteen PX_CTLI parameters, which define a starting horizontal
	position and new color and contrast value to apply from that column on towards the right to the next
	PX_CTLI or the right side of the display.

	LN_CTLI, 4 bytes, special value of 0f ff ff ff signifies the end of the parameter area (this termination
	code MUST be present as the last parameter)
	0 s   s s   n t   t t 
	sss = csln, the starting (top-most) line number for this area
	n = number of PX_CTLI to follow
	ttt = ctln, the terminating (bottom-most) line number for this area

	PX_CTLI, 6 bytes, defines a starting column and new color and contrast values
	bytes 0 and 1 - starting column number
	bytes 2 and 3 - new color values, as per SET_COLOR
	bytes 4 and 5 - new contrast values, as per SET_CONTR

	Line Control Information (LN_CTLI) Line control information LN_CTLI is
	constituted by a change start line number, the number of changes (a change
	point count), and a change termination line number (or continuous line
	count).  The number of pixel control data each constituting line control
	information LN_CTLI and pixel control information PX_CTLI is set for
	sub-picture display frames, as needed. LN_CTLI, contains 4 bytes, the
	special value of 0f ff ff ff signifies the end of the parameter area (this
	termination code MUST be present as the last parameter) 0 s s s n t t t sss
	= csln, the starting (top-most) line number for this area n = number of
	PX_CTLI to follow ttt = ctln, the terminating (bottom-most) line number for
	this area

	Offset 7 6 5 4 3 2 1 0 0 reserved (0) Change Start Line numbers (upper bits)
	1 Change Start Line numbers (lower bits) 2 Number of Changes reserved (0)   
	Change Termination Line Number (upper bits) 3 Change Termination Line Number
	(lower bits)

	Change Start Line numbers:              Number of Line from which content of
	pixel control starts to change, and described by Line number of video
	display. Number of Changes: Number of Change Points (number of PX_CTLIs)
	spead evenly on lines to be changed, could be described with numbers between
	1 and 8. Change Termination Line Number: Number of Line Describes at which  
	content of pixel control terminates the change, and described by Line number
	of video display.

	Command LN_CTLI, change line number shall be within:
	2 .. 479 (TV system with 525/60)
	2 .. 574 (TV system with 625/50)

	-        Command LN_CTLI Number_of_changes
	shall be within 1..8.  At least 8 pixels with the same content shall be
	continued on the change start pixel and the pixels which follow.
	*/

	*cptr = 7; /* CHG_COLCON */
	cptr++;

	/* remember position, will set later */
	change_colcon_size = cptr;
	/* reserve 2 bytes for change size, to be set later */
	cptr += 2; 

	/*
	LN_CTLI, 4 bytes, special value of 0f ff ff ff signifies the end of the parameter area
	(this termination code MUST be present as the last parameter)
	0 s   s s   n t   t t
	sss = csln, the starting (top-most) line number for this area
	n = number of PX_CTLI to follow
	ttt = ctln, the terminating (bottom-most) line number for this area
	*/

	*cptr = (current_c_start_line & 0x300) >> 8;
	cptr++;

	*cptr = current_c_start_line & 0xff;
	cptr++;

	*cptr = (current_c_h_blocks << 4) | ( (current_c_end_line & 0x300) >> 8);
	cptr++;

	*cptr = current_c_end_line & 0xff;
	cptr++;

	/* any number of PX_CTLI blocks */
	for(i = 0; i < current_c_h_blocks; i++)
		{
		/*
		PX_CTLI, 6 bytes, defines a starting column and new color and contrast values
		bytes 0 and 1 - starting column number
		bytes 2 and 3 - new color values, as per SET_COLOR
		bytes 4 and 5 - new contrast values, as per SET_CONTR

		Note: I assume 'column' is really horizontal pixel number.
		*/

		/* horizontal start point */
		*cptr = (current_c_h_start[i] & 0x300) >> 8;
		cptr++;
		*cptr = current_c_h_start[i] & 0xff;
		cptr++;

		/* color */
		*cptr = (current_c_emphasis2[i] << 4) | current_c_emphasis1[i];
		cptr++;
		*cptr = (current_c_pattern[i] << 4) | current_c_background[i];
		cptr++;

		/* contrast */
		*cptr = ( (current_c_contrast3[i] & 0xf) << 4) | (current_c_contrast2[i] & 0xf);
		cptr++;
		*cptr = ( (current_c_contrast1[i] & 0xf) << 4) | (current_c_contrast0[i] & 0xf);
		cptr++;

		} /* end for all PX_CTLI */

	/* termination code LN_CTLI for this CHG_COLCON command */
	*cptr = 0x0f;
	cptr++;
	*cptr = 0xff;
	cptr++;
	*cptr = 0xff;
	cptr++;
	*cptr = 0xff;
	cptr++;

	/* set the 2 parameter area size bytes at start of CHG_COLCON command */
	a = cptr - change_colcon_size;	
	change_colcon_size[0] = (a / 256) & 0xff;
	change_colcon_size[1] = (a % 256) & 0xff;

	} /* end if current_submux_command & 4 */

/* command 0xff, end command block, 1 byte, */
*cptr = 0xff;
cptr++;

/* end first command block */


/* start second command block */
 
/* set pointer in previous block to point to this position */
a = cptr - subtitle_packet_buffer;
next_command_ptr[0] = a / 256;
next_command_ptr[1] = a % 256;

if(debug_flag)
	{
	fprintf(stderr, "next_command_ptr=%d\n", (256 * next_command_ptr[0]) + next_command_ptr[1]);
	}

/* update pointer to this command block */
next_command_ptr = cptr;

/* delay to wait before executing next command */
delay_units = sd / 1024;

if(debug_flag)
	{
	fprintf(stderr, "sd=%u delay_units=%d\n", sd, delay_units);
	}

*cptr = (delay_units / 256) & 0xff;
cptr++;
*cptr = (delay_units % 256) & 0xff;
cptr++;

/* last block, point to self */
a = next_command_ptr - subtitle_packet_buffer;
*cptr = (a / 256) & 0xff;
cptr++;
*cptr = (a % 256) & 0xff;
cptr++;

if(current_submux_command & 2)
	{
	/* stop command (executed after above delay) */
	*cptr = 0x02;
	cptr++;
	} /* end if current_submux_command &2  */

/* end command block command */
*cptr = 0xff;
cptr++;

/* end second commmand block */

/* set the packet size */
subtitle_packet_size = cptr - subtitle_packet_buffer;

/* make size even if odd */
if(subtitle_packet_size % 2)
	{
	if(debug_flag)
		{
		fprintf(stderr, "dvd_encode(): ODD\n");
		}

	/* only if odd length, to make it even */
	cptr++;
	*cptr = 0xff;

	subtitle_packet_size++;
	}

/* set subtitle packet size */
a = subtitle_packet_size;
subtitle_packet_buffer[0] = (a / 256) & 0xff;
subtitle_packet_buffer[1] = (a % 256) & 0xff;

if(debug_flag)
	{
	fprintf(stderr,
	"subtitle_packet_size=%d\n",
	(subtitle_packet_buffer[0] * 256) + subtitle_packet_buffer[1]);
	}

/*
1 SPDSZ SP unit size shall be > 0. 
SP unit size shall be even !
SPU even size padding byte shall be 0xff .
SP unit size shall describe the size of a SPU in number of bytes.
SP unit size shall be <= 53.220 bytes.
The size of SP_ DCSQT in a SPU shall be equal or less than half the size of the SPU.
*/


return subtitle_packet_size;
} /* end function dvd_encode */


int test_bitmap(\
char *file_name,\
int xpos, int ypos, int xsize, int ysize,\
int bg, int fg, int emph1, int emph2,\
int bgcol, int fgcol, int emph1col, int emph2col)
{
if(debug_flag)
	{
	fprintf(stdout,\
	"test_bitmap(): arg file_name=%s\n\
xpos=%d ypos=%d xsize=%d ysize=%d bg=%d fg=%d emph1=%d emph2=%d\n\
bgcol=%d fgcol=%d emph1col=%d emph2col=%d\n",\
file_name,\
xpos, ypos, xsize, ysize,\
bg, fg, emph1, emph2,\
bgcol, fgcol, emph1col, emph2col);	
	}

x0 = xpos;
y0 = ypos;
xd = xsize;
yd = ysize;
pattern = fg;
background = bg;
emphasis1 = emph1;
emphasis2 = emph2;

c_start_line = 2;

c_end_line = 499;

c_h_start[0] = 100;
c_h_start[1] = 200;
c_h_start[2] = 300;

c_background[0] = 0;
c_background[1] = 0;
c_background[2] = 0;

c_pattern[0] = 5;
c_pattern[1] = 9;
c_pattern[2] = 13;

c_emphasis1[0] = 2;
c_emphasis1[1] = 2;
c_emphasis1[2] = 2;

c_emphasis2[0] = 3;
c_emphasis2[1] = 3;
c_emphasis2[2] = 3;

c_contrast0[0] = 0;
c_contrast0[1] = 0;
c_contrast0[2] = 0;

c_contrast1[0] = 15;
c_contrast1[1] = 15;
c_contrast1[2] = 15;

c_contrast2[0] = 15;
c_contrast2[1] = 15;
c_contrast2[2] = 15;

c_contrast3[0] = 0;
c_contrast3[1] = 0;
c_contrast3[2] = 0;

c_h_blocks = 3; 

if(! odd_buf)
	{
	odd_buf = malloc(maxx * maxy);
	if(! odd_buf)
		{
		fprintf(stderr, "test_bitmap(): could not allocate space for odd_buf.\n");

		return 0;
		}
	}

if(! even_buf)
	{
	even_buf = malloc(maxx * maxy);
	if(! even_buf)
		{
		fprintf(stderr, "test_bitmap(): could not allocate space for even_buf.\n");

		return 0;
		}
	}

if(! subtitle_image_buffer)
	{
	subtitle_image_buffer = malloc(maxx * maxy * 3);
	if(! subtitle_image_buffer)
		{
		fprintf(stderr, "test_bitmap(): could not allocate space for subtitle_image_buffer.\n");

		return 0;
		}
	}
if(! subtitle_packet_buffer)
	{
	subtitle_packet_buffer = malloc( SUB_BUFFER_MAX); //maxx * maxy * 3); //SUB_BUFFER_MAX);
	if(! subtitle_packet_buffer)
		{
		fprintf(stderr, "test_bitmap(): could not allocate space for subtitle_packet_buffer.\n");

		return 0;
		}
	}
//fprintf(stderr, "malloc subtitle_packet_buffer=%p\n", subtitle_packet_buffer);

sub = subtitle_packet_buffer;

if(! cbuf) cbuf = malloc(65536);
if(! img) img = malloc(maxx * maxy);
if(! fimg) fimg = malloc(maxx * maxy);
 
if(! read_bmp(file_name) ) return 0; //"/root/.subtitles/amaroo3-2001-dvd/frame8338.bmp");

imgfix();

if(! dvd_encode() ) return 0;

return 1;
} /* end function test_bitmap */


#ifdef TEST_BITMAP_STANDALONE
// frame8338.bmp  00:05:33:52  00:05:36:88  664 32  30 533  0 15 15 15  0 1 2 15
//                                          hs  vs  hp vp
int main(int argc, char ** argv)
{
test_bitmap(argv[1], 664, 32, 30, 533, 0, 15, 15, 15,  0, 1, 2, 15);

//int xsize, int ysize,
//int xpos, int ypos,
//int bg, int fg, int emph1, int emph2,
//int bgcol, int fgcol, int emph1col, int emph2col  

exit(0);
} /* end function main */

#endif /* TEST_BITMAP_STANDALONE */

