/*
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "xste.h"

FD_xste *fdui;


int show_text_properties_form()
{
if(debug_flag)
	{
	printf("show_text_properties_form(): arg none\n");
	}

if(text_properties_form_shown_flag) return 1;

fl_show_form(fdui -> text_properties_form, FL_PLACE_CENTER, FL_TRANSIENT, "xste: text properties");

text_properties_form_shown_flag = 1;

return 1;
} /* end function show_fonts_form */


int set_subtitle_properties(\
    int selected_start_frame,\
	int anti_alias,\
	int no_kerning,\
    int background,\
    int body,\
    int outline1,\
    int outline2,\
    int background_contrast,\
    int body_contrast,\
    int outline1_contrast,\
    int outline2_contrast,\
    double blur_radius,\
    double outline_thickness,\
	int symbols,\
    int fontsize,\
    int iso_extension,\
    char *fontname,\
	int center_flag, \
	int espace, \
	int vespace)
{
struct frame *pa;
char temp[4096];
font_desc_t *pfd;

if(debug_flag)
	{
	fprintf(stdout, "set_subitle_properties(): arg selected_start_frame=%d\n\
	anti_alias=%d,no_kerning=%d  background=%d body=%d outline1=%d outline2=%d\n\
	background_contrast=%d body_contrast=%d outline1_contrast=%d outline2_contrast=%d\n\
	blur_radius=%02f outline_thickness=%02f symbols=%d fontsize=%d iso_extension=%d\n\
	fontname=%s\n\
	center_flag=%d espace=%d vespace=%d\n",\
	selected_start_frame,\
	anti_alias, no_kerning, background, body, outline1, outline2,\
	background_contrast, body_contrast, outline1_contrast, outline2_contrast,\
	blur_radius, outline_thickness, symbols, fontsize, iso_extension,\
	fontname,\
	center_flag, espace, vespace);
	}

/* make font pointer */
pfd = add_font(fontname, symbols, fontsize, iso_extension, outline_thickness, blur_radius, espace, vespace);
if(! pfd)
	{
	fl_show_alert("could not make font", temp, "command cancelled", 0);

	return 0;	
	}

/* get pointer to frame entry */
sprintf(temp, "%d", selected_start_frame);
pa = lookup_frame(temp);
if(! pa)
	{
	fprintf(stdout, "set_subitle_properties(): could not lookup frame%d\n", selected_start_frame);

	return 0;
	}

/* data to structure */
pa -> pattern = body;
pa -> background = background;
pa -> emphasis1 = outline1;
pa -> emphasis2 = outline2;
    
pa -> pattern_contrast = body_contrast;
pa -> background_contrast = background_contrast;
pa -> emphasis1_contrast = outline1_contrast;
pa -> emphasis2_contrast = outline2_contrast;
    
pa -> symbols = symbols;

pa -> fontsize = fontsize;
pa -> iso_extension = iso_extension;

pa -> outline_thickness = outline_thickness;
pa -> blur_radius = blur_radius;

pa -> espace = espace;
pa -> vespace = vespace;

pa -> pfd = pfd;

pa -> fontname = strsave(fontname);
if(! pa -> fontname)
	{
	fprintf(stdout, "set_subitle_properties(): strsave() pa -> fontname failed\n");	

	return 0;	
	}

pa -> status &= ~CENTER_TEXT;
if(center_flag) pa -> status |= CENTER_TEXT;

if(anti_alias) pa -> status |= DVD_ANTI_ALIAS;
else pa -> status &= ~DVD_ANTI_ALIAS; 

if(no_kerning) pa -> status |= NO_KERNING;
else pa -> status &= ~NO_KERNING;

if(! save_frames() ) return 0;

return 1;
} /* end function set_subtitle_properties */


int frame_font_parameters_to_vars(int frame)
{
char temp[80];
struct frame *pa;

if(debug_flag)
	{
	fprintf(stdout, "frame_font_parameters_to_vars(): arg frame=%d\n", frame);
	}

sprintf(temp, "%d", frame);
pa = lookup_frame(temp);
if(! pa)
	{
	fl_show_alert("frame_font_parameters_to_vars():", "could not lookup frame", temp, 0);

	return 0;
	}

text_in_body = pa -> pattern;
text_in_background = pa -> background;
text_in_outline1 = pa -> emphasis1;
text_in_outline2 = pa -> emphasis2;
     
text_in_body_contrast = pa -> pattern_contrast;
text_in_background_contrast = pa -> background_contrast;
text_in_outline1_contrast = pa -> emphasis1_contrast;
text_in_outline2_contrast = pa -> emphasis2_contrast;
    
text_in_fontsize = pa -> fontsize;
text_in_iso_extension = pa -> iso_extension;

text_in_outline_thickness = pa -> outline_thickness;
text_in_blur_radius = pa -> blur_radius;

text_in_extra_character_space = pa -> espace;
text_in_extra_character_vspace = pa -> vespace;

if(text_in_fontname) free(text_in_fontname);
text_in_fontname = strsave(pa -> fontname);
if(! text_in_fontname)
	{
	fprintf(stderr, "xste(): frame_font_parameters_to_vars(): could not allocate space for fontname\n");

	return 0;
	}

if(pa -> status & CENTER_TEXT) text_in_center_flag = 1;
else text_in_center_flag = 0;

if(pa -> status & DVD_ANTI_ALIAS) text_in_anti_alias_flag = 1;
else text_in_anti_alias_flag = 0;

if(pa -> status & NO_KERNING) text_in_no_kerning_flag = 1;
else text_in_no_kerning_flag = 0;

fl_set_button(fdui -> text_properties_form_anti_alias_button, text_in_anti_alias_flag);
fl_set_button(fdui -> text_properties_form_no_kerning_button, text_in_no_kerning_flag);
set_center_text_button(text_in_center_flag);

fl_set_object_label(fdui -> text_properties_form_fontname_output_field, text_in_fontname);

sprintf(temp, "%d", text_in_fontsize);
fl_set_input(fdui -> text_properties_form_fontsize_input_field, temp);

sprintf(temp, "%d", text_in_iso_extension);
fl_set_input(fdui -> text_properties_form_iso_extension_number_input_field, temp);

sprintf(temp, "%d", text_in_extra_character_space);
fl_set_input(fdui -> text_properties_form_espace_input_field, temp);

sprintf(temp, "%d", text_in_extra_character_vspace);
fl_set_input(fdui -> text_properties_form_vespace_input_field, temp);

sprintf(temp, "%.2f", text_in_outline_thickness);
fl_set_input(fdui -> text_properties_form_outline_thickness_input_field, temp);

sprintf(temp, "%.2f", text_in_blur_radius);
fl_set_input(fdui -> text_properties_form_blur_radius_input_field, temp);

sprintf(temp, "%d", text_in_background_contrast);
fl_set_input(fdui -> text_properties_form_contrast0_input_field, temp);

sprintf(temp, "%d", text_in_body_contrast);
fl_set_input(fdui -> text_properties_form_contrast1_input_field, temp);

sprintf(temp, "%d", text_in_outline1_contrast);
fl_set_input(fdui -> text_properties_form_contrast2_input_field, temp);

sprintf(temp, "%d", text_in_outline2_contrast);
fl_set_input(fdui -> text_properties_form_contrast3_input_field, temp);

fl_set_object_color(fdui -> text_properties_form_color0_button,\
	text_in_background + FL_FREE_COL1, text_in_background + FL_FREE_COL1);

fl_set_object_color(fdui -> text_properties_form_color1_button,\
	text_in_body + FL_FREE_COL1, text_in_body + FL_FREE_COL1);

fl_set_object_color(fdui -> text_properties_form_color2_button,\
	text_in_outline1 + FL_FREE_COL1, text_in_outline1 + FL_FREE_COL1);

fl_set_object_color(fdui -> text_properties_form_color3_button,\
	text_in_outline2 + FL_FREE_COL1, text_in_outline2 + FL_FREE_COL1);


return 1;
} /* end function frame_font_parameters_to_vars */


int set_text_properties_defaults()
{
char temp[80];

if(debug_flag)
	{
	fprintf(stdout, "set_text_properties_defaults(): arg none\n");
	}

text_in_background = 0;
text_in_body = 1;
text_in_outline1 = 2;
text_in_outline2 = 15;
     
text_in_background_contrast = 0;
text_in_body_contrast = 15;
text_in_outline1_contrast = 15;
text_in_outline2_contrast = 0;

text_in_symbols = 0;

text_in_fontsize = 32;
text_in_iso_extension = 15;

text_in_extra_character_space = EXTRA_CHAR_SPACE;
text_in_extra_character_vspace = EXTRA_CHAR_VSPACE;

text_in_outline_thickness = 1.5;
text_in_blur_radius = 0.2;

text_in_anti_alias_flag = 0;
fl_set_button(fdui -> text_properties_form_anti_alias_button, text_in_anti_alias_flag);

text_in_center_flag = 1;
set_center_text_button(text_in_center_flag);

text_in_no_kerning_flag = 0;
fl_set_button(fdui -> text_properties_form_no_kerning_button, text_in_no_kerning_flag);

if(text_in_fontname) free(text_in_fontname);
text_in_fontname = strsave(default_subtitle_font_name);
if(! text_in_fontname)
	{
	fprintf(stderr, "xste(): set_text_properties_defaults(): could not allocate space for fontname\n");

	return 0;
	}

fl_set_object_label(fdui -> text_properties_form_fontname_output_field, text_in_fontname);

sprintf(temp, "%d", text_in_fontsize);
fl_set_input(fdui -> text_properties_form_fontsize_input_field, temp);

sprintf(temp, "%d", text_in_iso_extension);
fl_set_input(fdui -> text_properties_form_iso_extension_number_input_field, temp);

sprintf(temp, "%d", text_in_extra_character_space);
fl_set_input(fdui -> text_properties_form_espace_input_field, temp);

sprintf(temp, "%d", text_in_extra_character_vspace);
fl_set_input(fdui -> text_properties_form_vespace_input_field, temp);

sprintf(temp, "%.2f", text_in_outline_thickness);
fl_set_input(fdui -> text_properties_form_outline_thickness_input_field, temp);

sprintf(temp, "%.2f", text_in_blur_radius);
fl_set_input(fdui -> text_properties_form_blur_radius_input_field, temp);

sprintf(temp, "%d", text_in_background_contrast);
fl_set_input(fdui -> text_properties_form_contrast0_input_field, temp);

sprintf(temp, "%d", text_in_body_contrast);
fl_set_input(fdui -> text_properties_form_contrast1_input_field, temp);

sprintf(temp, "%d", text_in_outline1_contrast);
fl_set_input(fdui -> text_properties_form_contrast2_input_field, temp);

sprintf(temp, "%d", text_in_outline2_contrast);
fl_set_input(fdui -> text_properties_form_contrast3_input_field, temp);

fl_set_object_color(fdui -> text_properties_form_color0_button,\
	text_in_background + FL_FREE_COL1, text_in_background + FL_FREE_COL1);

fl_set_object_color(fdui -> text_properties_form_color1_button,\
	text_in_body + FL_FREE_COL1, text_in_body + FL_FREE_COL1);

fl_set_object_color(fdui -> text_properties_form_color2_button,\
	text_in_outline1 + FL_FREE_COL1, text_in_outline1 + FL_FREE_COL1);

fl_set_object_color(fdui -> text_properties_form_color3_button,\
	text_in_outline2 + FL_FREE_COL1, text_in_outline2 + FL_FREE_COL1);

return 1;
} /* end function set_set_text_properties_defaults */


void set_center_text_button(int state)
{
fl_set_button(fdui -> text_properties_form_center_text_button, 1 - state);

if(state)
	{
  	fl_set_object_color(fdui -> text_properties_form_center_text_button, FL_COL1, FL_COL1);
	fl_set_object_label(fdui -> text_properties_form_center_text_button, "CENTER\nTEXT");
	}
else
	{
	fl_set_object_color(fdui -> text_properties_form_center_text_button, FL_DARKORANGE, FL_DARKORANGE);
	fl_set_object_label(fdui -> text_properties_form_center_text_button, "LEFT JUSTIFIED\nTEXT");
	}

} /* end function set_center_text_button */


