/*
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "xste.h"

FD_xste *fdui;

static int text_properties_form_modified_flag;
static int fontsizes_modified_flag;


void text_properties_form_fontname_button_cb(FL_OBJECT *ob, long data)
{
char *ptr;
char temp[4096];
char *ptr2;

/* start  browser */
sprintf(temp, "%s/.xste/fonts", home_dir);

ptr = (char *)fl_show_fselector("SELECT A FONT FILE", temp, "*.ttf", "*.ttf");
if(! ptr) return;

/*
extract the fontname form the path-filename
example:
/root/.xste/fonts/arial.ttf

find the last '/' and start from there
*/

ptr2 = strrchr(ptr, '/');
if(! ptr2)
	{
	fl_show_alert("Cannot parse pathfilename", ptr, "no last '/' found", 0);
	
	return;
	}

/* /arial.ttf */

if(text_in_fontname) free(text_in_fontname);
text_in_fontname = strsave(ptr2 + 1);
if(! text_in_fontname) return;
	
fl_set_object_label(fdui -> text_properties_form_fontname_output_field, text_in_fontname);

text_properties_form_modified_flag = 1;
} /* end function text_properties_form_fontname_button_cb */


void text_properties_form_fontsize_input_field_cb(FL_OBJECT *ob, long data)
{
char *ptr;

ptr = (char *)fl_get_input(ob);
if(! ptr) return;

text_in_fontsize = atoi(ptr);

text_properties_form_modified_flag = 1;
} /* end function text_properties_form_fontsize_input_field_cb */


void text_properties_form_iso_extension_number_input_field_cb(FL_OBJECT *ob, long data)
{
char *ptr;

ptr = (char *)fl_get_input(ob);
if(! ptr) return;

text_in_iso_extension = atoi(ptr);

text_properties_form_modified_flag = 1;
} /* end function text_properties_form_iso_extension_number_input_field_cb */


void text_properties_form_espace_input_field_cb(FL_OBJECT *ob, long data)
{
char *ptr;
char temp[80];

ptr = (char *)fl_get_input(ob);
if(! ptr) return;

text_in_extra_character_space = atoi(ptr);

if(text_in_extra_character_space < -2)
	{
	fl_show_alert("value cannot be that much negative", "try a default button if unsure", "", 0);
	text_in_extra_character_space = -2;

	sprintf(temp, "%d", text_in_extra_character_space);
	fl_set_input(ob, temp);

	return;
	}

text_properties_form_modified_flag = 1;
} /* end function text_properties_form_vespace_input_field_cb */


void text_properties_form_vespace_input_field_cb(FL_OBJECT *ob, long data)
{
char *ptr;
char temp[80];

ptr = (char *)fl_get_input(ob);
if(! ptr) return;

text_in_extra_character_vspace = atoi(ptr);

if(text_in_extra_character_vspace < -50)
	{
	fl_show_alert("value cannot be that much negative", "try a default button if unsure", "", 0);
	text_in_extra_character_vspace = -50;

	sprintf(temp, "%d", text_in_extra_character_vspace);
	fl_set_input(ob, temp);

	return;
	}

text_properties_form_modified_flag = 1;
} /* end function text_properties_form_vespace_input_field_cb */


void text_properties_form_dvd_default_button_cb(FL_OBJECT *ob, long data)
{

if(! set_text_properties_defaults() ) return;

text_properties_form_modified_flag = 1;
} /* end function text_properties_form_dvd_default_button_cb */


void text_properties_form_no_kerning_button_cb(FL_OBJECT *ob, long data)
{
text_in_no_kerning_flag = fl_get_button(ob);

text_properties_form_modified_flag = 1;
} /* end function text_properties_form_no_kerning_button_cb */


void text_properties_form_anti_alias_button_cb(FL_OBJECT *ob, long data)
{
text_in_anti_alias_flag = fl_get_button(ob);

/* if anti alias, then also set emphasis2 to grey, and emphasis2 color to 15 */
if(text_in_anti_alias_flag)
	{
	fl_set_input(fdui -> text_properties_form_contrast3_input_field, "15");
	fl_call_object_callback(fdui -> text_properties_form_contrast3_input_field);

	text_in_outline2 =  15; /* this is grey by default */

	fl_set_object_color(fdui -> text_properties_form_color3_button,\
	text_in_outline2 + FL_FREE_COL1, text_in_outline2 + FL_FREE_COL1);
	}

text_properties_form_modified_flag = 1;
} /* end function text_properties_form_anti_alias_button_cb */


void text_properties_form_outline_thickness_input_field_cb(FL_OBJECT *ob, long data)
{
char *ptr;
char temp[80];

ptr = (char *)fl_get_input(ob);
if(! ptr) return;

text_in_outline_thickness = atof(ptr);
if(text_in_outline_thickness == 0.0)
	{
	fl_show_alert("outline thickness cannot be zero", "try default button if unsure", "", 0);
	text_in_outline_thickness = 0.01;

	sprintf(temp, "%0.2f", text_in_outline_thickness);
	fl_set_input(ob, temp);

	return;
	}

text_properties_form_modified_flag = 1;
} /* end function text_properties_form_outline_thickness_input_field_cb */


void text_properties_form_blur_radius_input_field_cb(FL_OBJECT *ob, long data)
{
char *ptr;
char temp[80];

ptr = (char *)fl_get_input(ob);
if(! ptr) return;

text_in_blur_radius = atof(ptr);
if(text_in_blur_radius == 0.0)
	{
	fl_show_alert("blur radius cannot be zero", "try default button if unsure", "", 0);
	text_in_blur_radius = 0.01;

	sprintf(temp, "%0.2f", text_in_blur_radius);
	fl_set_input(ob, temp);

	return;
	}

text_properties_form_modified_flag = 1;
} /* end function text_properties_form_blur_radius_input_field_cb */


void text_properties_form_center_text_button_cb(FL_OBJECT *ob, long data)
{
text_in_center_flag = 1 - fl_get_button(ob);

set_center_text_button(text_in_center_flag);

text_properties_form_modified_flag = 1;
} /* end text_properties_form_center_text_button_cb */


void text_properties_form_color0_button_cb(FL_OBJECT *ob, long data)
{
text_in_background = show_subtitle_color_selector_form(text_in_background);

fl_set_object_color(ob, text_in_background + FL_FREE_COL1, text_in_background + FL_FREE_COL1);

text_properties_form_modified_flag = 1;
} /* end function text_properties_form_color0_button_cb */


void text_properties_form_color1_button_cb(FL_OBJECT *ob, long data)
{
text_in_body = show_subtitle_color_selector_form(text_in_body);

fl_set_object_color(ob, text_in_body + FL_FREE_COL1, text_in_body + FL_FREE_COL1); 

text_properties_form_modified_flag = 1;
} /* end function text_properties_form_color1_button_cb */


void text_properties_form_color2_button_cb(FL_OBJECT *ob, long data)
{
text_in_outline1 = show_subtitle_color_selector_form(text_in_outline1);

fl_set_object_color(ob, text_in_outline1 + FL_FREE_COL1, text_in_outline1 + FL_FREE_COL1);

text_properties_form_modified_flag = 1;
} /* end function text_properties_form_color2_button_cb */


void text_properties_form_color3_button_cb(FL_OBJECT *ob, long data)
{
text_in_outline2 =  show_subtitle_color_selector_form(text_in_outline2);

fl_set_object_color(ob, text_in_outline2 + FL_FREE_COL1, text_in_outline2 + FL_FREE_COL1);

text_properties_form_modified_flag = 1;
} /* end function text_properties_form_color3_button_cb */


void text_properties_form_contrast0_input_field_cb(FL_OBJECT *ob, long data)
{
int a;
char *ptr;
char temp[80];

ptr = (char *)fl_get_input(ob);
if(! ptr) return;

a =  atoi(ptr);

if( (a > 15) || (a < 0) )
	{
	fl_show_alert("contrast range is 0 to 15", "", "", 0);

	a = 15;
	sprintf(temp, "%d", 15);
	fl_set_input(ob, temp);	
	}

text_in_background_contrast = a;

text_properties_form_modified_flag = 1;
} /* end function text_properties_form_contrast0_input_field_cb */


void text_properties_form_contrast1_input_field_cb(FL_OBJECT *ob, long data)
{
int a;
char *ptr;
char temp[80];

ptr = (char *)fl_get_input(ob);
if(! ptr) return;

a =  atoi(ptr);

if( (a > 15) || (a < 0) )
	{
	fl_show_alert("contrast range is 0 to 15", "", "", 0);

	a = 15;
	sprintf(temp, "%d", 15);
	fl_set_input(ob, temp);	
	}

text_in_body_contrast = a;

text_properties_form_modified_flag = 1;
} /* end function text_properties_form_contrast1_input_field_cb */


void text_properties_form_contrast2_input_field_cb(FL_OBJECT *ob, long data)
{
int a;
char *ptr;
char temp[80];

ptr = (char *)fl_get_input(ob);
if(! ptr) return;

a =  atoi(ptr);

if( (a > 15) || (a < 0) )
	{
	fl_show_alert("contrast range is 0 to 15", "", "", 0);

	a = 15;
	sprintf(temp, "%d", 15);
	fl_set_input(ob, temp);	
	}

text_in_outline1_contrast = a;

text_properties_form_modified_flag = 1;
} /* end function text_properties_form_contrast2_input_field_cb */


void text_properties_form_contrast3_input_field_cb(FL_OBJECT *ob, long data)
{
int a;
char *ptr;
char temp[80];

ptr = (char *)fl_get_input(ob);
if(! ptr) return;

a =  atoi(ptr);

if( (a > 15) || (a < 0) )
	{
	fl_show_alert("contrast range is 0 to 15", "", "", 0);

	a = 15;
	sprintf(temp, "%d", 15);
	fl_set_input(ob, temp);	
	}

text_in_outline2_contrast = a;

text_properties_form_modified_flag = 1;
} /* end function text_properties_form_contrast3_input_field_cb */


void text_properties_form_fontsize_up_button_cb(FL_OBJECT *ob, long data)
{
int size;

fontsizes_modified_flag = 1;
if(text_properties_form_fontsize < 5) text_properties_form_fontsize++;
size = integer_to_fontsize(text_properties_form_fontsize);

//fl_set_object_lsize(fdui -> text_properties_form_input, size);

save_fontsizes();

text_properties_form_modified_flag = 1;
} /* end function text_properties_form_fontsize_up_button_cb */


void text_properties_form_fontsize_down_button_cb(FL_OBJECT *ob, long data)
{
int size;

fontsizes_modified_flag = 1;
if(text_properties_form_fontsize > 0) text_properties_form_fontsize--;
size = integer_to_fontsize(text_properties_form_fontsize);

//fl_set_object_lsize(fdui -> text_properties_form_input, size);

save_fontsizes();

text_properties_form_modified_flag = 1;
} /* end function text_properties_form_fontsize_down_button_cb */


void text_properties_form_cancel_button_cb(FL_OBJECT *ob, long data)
{
if( text_properties_form_modified_flag)
	{
	/* restore from disc */
	load_general_settings();
	load_project_vars();

	text_properties_form_modified_flag = 0;
 	}

fl_hide_form(fdui -> text_properties_form);

if(fontsizes_modified_flag)
	{
	/* restore from disc */
	load_fontsizes();
	fontsizes_modified_flag = 0;
	}

text_properties_form_shown_flag = 0;

} /* end function text_properties_form_cancel_button_cb */


void text_properties_form_accept_button_cb(FL_OBJECT *ob, long data)
{
if(debug_flag)
	{
	fprintf(stdout, "text_properties_form_accept_button_cb(): arg none\n");
	}

if(test_for_same_colors() ) return;

if(text_properties_form_modified_flag)
	{
/*
XXXXXXXXXXXXXXXXX selected_start_frame will not point to real entry (frame number) if frame does
not exists (line was already entered).
*/
	/* local subtitle parameters to structure entry */
	if(! set_subtitle_properties(\
		selected_start_frame,\
		text_in_anti_alias_flag,\
		text_in_no_kerning_flag,\
		text_in_background,\
		text_in_body,\
		text_in_outline1,\
		text_in_outline2,\
		text_in_background_contrast,\
		text_in_body_contrast,\
		text_in_outline1_contrast,\
		text_in_outline2_contrast,\
		text_in_blur_radius,\
		text_in_outline_thickness,\
		text_in_symbols,\
		text_in_fontsize,\
		text_in_iso_extension,\
		text_in_fontname,\
		text_in_center_flag,\
		text_in_extra_character_space,\
		text_in_extra_character_vspace) ) return;
	
	save_general_settings();
	save_project_vars();

	text_properties_form_modified_flag = 0;
	}

fl_hide_form(fdui -> text_properties_form);

if(fontsizes_modified_flag)
	{
	save_fontsizes();

	save_general_settings();
	save_project_vars();

	fontsizes_modified_flag = 0;
	}

/* force redraw preview display */
new_preview_flag = 1;

text_properties_form_shown_flag = 0;
} /* end function text_properties_form_accept_button_cb */


int test_for_same_colors()
{
int a;

if(debug_flag)
	{
	fprintf(stdout,\
	"test_for_same_colors(): arg none, using\n\
	text_in_background=%d\n\
	text_in_body=%d\n\
	text_in_outline1=%d\n\
	text_in_outline2=%d\n",\
	text_in_background,
	text_in_body,\
	text_in_outline1,\
	text_in_outline2);
	}

a = 0;
if(text_in_background == text_in_body) a = 1;
if(text_in_background == text_in_outline1) a = 1;
if(text_in_background == text_in_outline2) a = 1;
if(text_in_body == text_in_outline1) a = 1;
if(text_in_body == text_in_outline2) a = 1;
if(text_in_outline1 == text_in_outline2) a = 1;

if(a)
	{
	fl_show_alert("Same color for any of", "background, body, outline1, outline2", "not allowed", 0);

	/* return same */
	return 1;
	}

return 0;
} /* end function test_for_same_colors */

