/*
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "xste.h"

FD_xste *fdui;

FILE *filefd;


char *strsave(char *s) /* save char array s somewhere */
{
char *p;

p = malloc( strlen(s) +  1);
if(! p) return 0;

strcpy(p, s);
return p;
} /* end function strsave */


int readline(FILE *file, char *contents)
{
int a, c, i;

if(debug_flag)
	{
	fprintf (stdout, "readline(): arg file=%lu\n", (long)file);
	}

for(i = 0; i < READSIZE - 1; i++)
	{
	while(1)
		{
		c = getc(file);
		a = ferror(file);
		if(a)
			{
			perror("readline():");
			continue;
			}
		break;
		}
	if(feof(file) )
		{
		fclose(file);
		contents[i] = 0;/* EOF marker */
		return EOF;
		}
	if(c == '\n')
		{
		contents[i] = 0;/* string termination */
		return 1;/* end of line */
		} 
	contents[i] = c;
	}
/*
mmm since we are here, the line must be quite long, possibly something
is wrong.
Since I do not always check for a return 0 in the use uf this function,
just to be safe, gona force a string termination.
This prevents stack overflow, and variables getting overwritten.
*/
contents[i] = 0;/* force string termination */
/*fclose(file);*/
if(debug_flag)
	{
	fprintf(stdout,\
	"readline(): line to long, returning 0 contents=%s\n", contents);
	}
return 0;
}/* end function readline */


int integer_to_fontsize(int fontsize)
{
int size;

size = 0;/* only for clean compilation with -Wall */

/* tiny snall normal medium large huge */
if(fontsize == 0) size = FL_TINY_SIZE;
if(fontsize == 1) size = FL_SMALL_SIZE;
if(fontsize == 2) size = FL_NORMAL_SIZE;
if(fontsize == 3) size = FL_MEDIUM_SIZE;
if(fontsize == 4) size = FL_LARGE_SIZE;
if(fontsize == 5) size = FL_HUGE_SIZE;

return size;
} /* end function integer to fontsize */


int ogettime(
char *weekday,
char *month,
int *monthday,
int *hour,
int *minute,
int *second,
int *year
)
{
time_t now;
char *datestring;

time(&now); /* sets unsigned long int I to seconds since start 1970 */
datestring = ctime(&now); /*char * converts now to character string date and time */

sscanf(datestring,"%s %s %d %d:%d:%d %d",\
weekday,month,monthday,hour,minute,second,year); /* no &, already pointers */

return 1;
} /* end function ogettime */


char *get_universal_time_as_string()
{
time_t now;
static char temp[TEMP_SIZE];
size_t strftime();
struct tm *universal_time, *gmtime();
char *ptr;
char *strsave();
char *old_locale, *saved_locale;

if(debug_flag)
	{
	fprintf(stdout, "get_universal_time_as_string(): arg none\n");
	}

/* 
set the locale to english language even if env shows otherwise
*/
/* Get the name of the current locale */
old_locale = setlocale (LC_ALL, NULL);
     
/* Copy the name so it won't be clobbered by `setlocale' */
saved_locale = strdup (old_locale);
if (saved_locale == NULL)
	{
	fprintf(stdout, "get_universal_time_as_string(): Out of memory\n");

	return 0;
	}     

/* 
Now change the locale to US english for 'Wed and not for example 'wo'
server may not be able to parse something like 'wo'.
*/
ptr = setlocale(LC_ALL, "en_US");
if(! ptr)
	{
	fprintf(stdout, "get_universal_time_as_string(); could not setlocale\n");

	return 0;
	}

/* 
decided to adapt to universal time (UT == GMT).
New format:
from rfc1036.html:
Wdy, DD Mon YY HH:MM:SS TIMEZONE 
*/

now = time(0);
universal_time = gmtime(&now);
strftime (temp, 511, "%a, %d %b %Y %H:%M:%S GMT", universal_time);

ptr = strsave(temp);

if(debug_flag)
	{
	fprintf(stdout, "get_universal_time_as_string(): ptr=%s\n", ptr);
	}

/* Restore the original locale */
setlocale (LC_ALL, saved_locale);
free (saved_locale);

return ptr;
} /* end function get_universal_time_as_string */

