/*

xste is Copyright 2004 and later Jan Panteltje

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/


#include "xste.h"

extern FD_xste *fdui;
extern FD_xste *fd_main_form;


extern int mpeg3_read_yuvframe_ptr(mpeg3_t *file,
                char **y_output,
                char **u_output,
                char **v_output,
               int stream);


FILE *filefd;

pthread_t play_thread;
pthread_attr_t *attributes;

static int window_open_flag;
static uint8_t *xbuffer;
static int old_current_frame;
static int last_frame = 0;

int init_video_file()
{
int a;

if(debug_flag)
	{
	fprintf(stdout, "init_video_file(): arg none, using video_file=%p video_filename=%s\n",\
	video_file, video_filename);
	}

video_error = VIDEO_OK;

/* argument check */
if(! video_filename)
	{
//	video_error = VIDEO_NO_FILENAME;

	return 0;
	}

if(video_file)
	{
	mpeg3_close(video_file);
	}

if(! mpeg3_check_sig(video_filename) )
	{
	if(debug_flag)
		{
		fprintf(stdout, "mpeg3_check_sig(): returned zero\n");
		}
	}

#ifdef LIBMPEG3_VERSION_1_5_3
video_file = mpeg3_open(video_filename);
#endif

#ifdef LIBMPEG3_VERSION_1_7
video_file = mpeg3_open(video_filename, &a);
#endif

if(! video_file)
	{
	fprintf(stderr, "xste: mpeg3_open(): %s failed\n", video_filename);

	video_error = VIDEO_NO_OPEN;

	return 0;
	}

/* set number of CPUs */
//mpeg3_set_cpus(mpeg3_t *file, int cpus)
mpeg3_set_cpus(video_file, CPUS_USED_BY_LIBMPEG3);

#ifdef LIBMPEG3_VERSION_1_5_3
/* no longer in libmpeg3-1.7 */
//mpeg3_set_mmx(mpeg3_t *file, int use_mmx)
mpeg3_set_mmx(video_file, USE_MMX_IN_LIBMPEG3); // mmx, libmpeg3 mmx may corrupt data, see docs
#endif

//int mpeg3_has_video(mpeg3_t *file);
if(! mpeg3_has_video(video_file) )
	{
	fprintf(stderr, "xste: no video streams found in %s\n", video_filename);

	video_error = VIDEO_NO_VIDEO;
	return 0;
	}

//int mpeg3_total_vstreams(mpeg3_t *file);            // Number of multiplexed video streams
//int mpeg3_video_width(mpeg3_t *file, int stream);
video_file_width = mpeg3_video_width(video_file, 0);

//int mpeg3_video_height(mpeg3_t *file, int stream);
video_file_height = mpeg3_video_height(video_file, 0);

//float mpeg3_frame_rate(mpeg3_t *file, int stream);  // Frames/sec
//long mpeg3_video_frames(mpeg3_t *file, int stream); // Total length
video_file_frames = mpeg3_video_frames(video_file, 0);

//int mpeg3_colormodel(mpeg3_t *file, int stream);
//video_file_color_model = mpeg3_colormodel(video_file, 0);

if(debug_flag)
	{
	fprintf(stdout, "xste: video_file_video_streams=%d\n", video_file_video_streams);
	fprintf(stdout, "xste: video_file_width=%d\n", video_file_width);
	fprintf(stdout, "xste: video_file_height=%d\n", video_file_height);
	fprintf(stdout, "xste: video_file_fps=%.3f\n", video_file_fps);
	fprintf(stdout, "xste: video_file_frames=%ld\n", video_file_frames);
	}

/*
fprintf(stdout, "xste: file_color_model=%d\n", file_color_model);

if(file_color_model == MPEG3_YUV420P)
	{
	fprintf(stdout, "xste: mpeg3_colormodel(): video_file_color_model=MPEG3_YUV420P\n");
	}
else if (file_color_model == MPEG3_YUV422P)
	{
	fprintf(stdout, "xste: mpeg3_colormodel(): video_file_color_model=MPEG3_YUV422P\n");
	}
*/
	
video_file_video_streams = mpeg3_total_vstreams(video_file); 
if(video_file_video_streams == 0)
	{
	video_error = VIDEO_NO_STREAMS;

	return 0;
	}

if( (video_file_width == 0) || (video_file_height == 0) )
	{
	fprintf(stderr, "streams=%d width=%d height=%d",\
	video_file_video_streams, video_file_width, video_file_height);

	video_error = VIDEO_NO_SIZE;

	return 0;
	}

video_file_fps = mpeg3_frame_rate(video_file, 0);

if ( fabs(video_file_fps - dframes_per_second) >= 0.001 )
	{
	video_error = VIDEO_FPS;

	return 0;
	}

a = video_file_height * video_file_width;

return 1;
} /* end function init_video_file */


void *video_routine()
{
if(debug_flag)
	{
	fprintf(stdout, "video_routine(): arg none\n");
	}

while(1)
	{
	if(preview_flag)
		{
		if(new_preview_flag || (current_frame != old_current_frame) )
			{
			if(! video_file_init_flag)
				{
				if(! init_video_file() ) 
					{
					video_file = 0;
					}

				video_file_init_flag = 1;
				}
			else
				{
				//if(
				preview_display(); // )
					{
					old_current_frame = current_frame;
		
					new_preview_flag = 0;
					}
				}
			}
		} /* end if preview_flag */

	/* take it easy, more apps are runing */

	/* if displaying video get maximum speed if play_button pressed, to avoid dropping frames */
	if(play_button && video_in_preview_flag)
		{
		}
	else
		{
		usleep(10000);
		}
	} /* end while look for new picture */

return 0;
} /* end function video_routine */


int video_init()
{
if(debug_flag)
	{
	fprintf(stdout, "video_init(): arg none\n");
	}

/* this is read by idle_cb */
video_error = VIDEO_OK;

pthread_create(&play_thread, NULL , &video_routine, NULL);

return 1;
} /* end function video_init */


int preview_display()
{
int a, c, i, j, x, y;
char *tptr, *cptr;
int width, height;
static int window_size;
unsigned char *ucptrd, *ucptrs, *ucptr;
struct frame *pa;
int text_start, max_width, line_len;
int bg_x_start, bg_x_end, bg_width, bg_y_start, bg_y_end, bg_height;
double da, dm, dr, dg, db, dir, dig, dib;
char *f_y_optr, *f_u_optr, *f_v_optr;
int cy, cu, cv, cr, cg, cb;
int is_subtitle_flag;
struct animation_template *pt;
char command_part[TEXT_ARRAY_SIZE];
char text_part[TEXT_ARRAY_SIZE];
int use_dvd_colors_flag;
double radians, degrees;
double dvector, dsine, du, dv;
double dy;
int frames;
double espace, contrast, saturation, transparency;

if(debug_flag)
	{
	fprintf(stdout,\
	"preview_display(): arg none using: selected_picture=%lu window_open_flag=%d h_size=%d v_size=%d\n",\
	selected_picture, window_open_flag, h_size, v_size);

	fprintf(stdout, "preview_display(): using current_frame=%d\n", current_frame);

	fprintf(stdout, "preview_display(): video_filename=%s video_file=%p\n", video_filename, video_file); 
	}

use_dvd_colors_flag = 0; // -Wall
width = 0; // -Wall
height = 0; // -Wall
frames = 0; // -Wall

/* parameter check */
if( (h_size > 720) || (v_size > 576) )
	{
	fprintf(stderr, "xste: preview_display(): illegal size: h_size=%d v_size=%d\n", h_size, v_size);

	video_error = VIDEO_SIZE_TOO_BIG;

	return 0;	
	}

if( (h_size == 0) || (v_size == 0) )
	{
	fprintf(stdout,\
	"preview_display(): window_open_flag=%d h_size=%d v_size=%d, cannot display\n",\
	window_open_flag, h_size, v_size);

	video_error = VIDEO_SIZE_ZERO;

	return 0;
	}
	
if(video_in_preview_flag && video_file)
	{
	if( (h_size != video_file_width) || (v_size != video_file_height) )
		{
		fprintf(stderr, "xste: preview_display(): video size %d x %d not equal selected display size %d x %d\n",\
		video_file_width, video_file_height, h_size, v_size);

		video_error = VIDEO_SIZE_DIFFERENT;

		return 0;
		}
	}

switch(output_format)
	{
	case OUTPUT_FORMAT_PPML_FOR_SUBTITLER_YUV:
		use_dvd_colors_flag = 0;
		break;
	case OUTPUT_FORMAT_PPML:
		if(ppml_use_dvd_colors_flag)
			{
			draw_mode = DRAW_MODE_OGT;
			use_dvd_colors_flag = 1;
			}
		else
			{
			draw_mode = DRAW_MODE_BW;
			use_dvd_colors_flag = 0;
			}
		break;
	case OUTPUT_FORMAT_SUTI:
		use_dvd_colors_flag = 0;
		draw_mode = DRAW_MODE_BW;
		break;
	case OUTPUT_FORMAT_SSA:
		use_dvd_colors_flag = 0;
		draw_mode = DRAW_MODE_BW;
		break;
	case OUTPUT_FORMAT_VPLAYER:
		use_dvd_colors_flag = 0;
		draw_mode = DRAW_MODE_BW;
		break;
	case OUTPUT_FORMAT_SUBMUX:
		use_dvd_colors_flag = 1;
		draw_mode = DRAW_MODE_OGT;
		break;
	case OUTPUT_FORMAT_SUBMUX_DVD:
		use_dvd_colors_flag = 1;
		draw_mode = DRAW_MODE_OGT;
		break;
	case OUTPUT_FORMAT_MICRO_DVD_PLAYER:
		use_dvd_colors_flag = 0;
		draw_mode = DRAW_MODE_BW;
		break;
	case OUTPUT_FORMAT_I_AUTHOR:
		use_dvd_colors_flag = 0;
		draw_mode = DRAW_MODE_BW;
		break;
	case OUTPUT_FORMAT_EBU_STL:
		use_dvd_colors_flag = 0;
		draw_mode = DRAW_MODE_BW;
  		break;
	case OUTPUT_FORMAT_SRT:
		use_dvd_colors_flag = 0;
		draw_mode = DRAW_MODE_BW;
  		break;
	} /* end switch formats */

width = h_size;
height = v_size;

/* open a window for the size we now know */
if(! window_open_flag)
	{
//	openwin(argc, argv, width, height);
 	openwin(0, NULL, width, height);

	window_size = width * height;
	window_open_flag = 1;
	}
else /* have window */
	{
	if( (width * height) != window_size)
		{
		/* close window and open a new one */
//		closewin(); //crashes
//		resize_window(xsize, ysize); // problem ?
//		no problem, now we have 2 windows, use window manager to kill one

//		openwin(argc, argv, xsize, ysize);
		openwin(0, NULL, width, height);

		window_size = width * height;
		} /* end if different window size */

	} /* end if window already open */

/* get color_depth */
if(color_depth == 0) color_depth = get_x11_bpp();

if(debug_flag)
	{
	fprintf(stdout, "preview_display(): colordepth=%d\n", color_depth);
	}

/* get X11 buffer */
xbuffer = (unsigned char *)getbuf();
if(! xbuffer)
	{
	if(debug_flag)
		{
		printf("preview_display(): could not get xbuffer\n");
		}

	video_error = VIDEO_NO_XBUFFER;
	}

/* clear image_buffer */
ucptrd = image_buffer;

if(! video_in_preview_flag)
	{
	a = 255;
	for(y = 0; y < height; y++)
		{
		/* leave some letterbox like area black, the rest white, to see where we are with the second line */
		if(y > (height - 76) ) a = 0;
		for(x = 0; x < width; x++)
			{
			*ucptrd++ = a;
			*ucptrd++ = a;
			*ucptrd++ = a;
			}
		}
	} /* end if not video */

if(debug_flag)
	{
	fprintf(stderr, "preview_display(): current_frame=%d\n", current_frame);
	}

if(video_in_preview_flag && video_file)
	{
	/* seek to current frame */

	//int mpeg3_set_frame(mpeg3_t *file, long frame, int stream); // Seek
	/* NOTE: this will return 0 some times, but still work, why? */

	a = mpeg3_set_frame(video_file, current_frame, 0); // Seek

	if( (! a)  && (! video_have_warning_flag) )
		{
		fprintf(stderr, "xste: preview_display(): mpeg3_set_frame(%d): returned 0\n",\
		current_frame);

		video_have_warning_flag = 1;

//		video_error = VIDEO_SET_FRAME;
	
//		return 0;
		}

//int mpeg3_read_yuvframe_ptr(mpeg3_t *file,
//                char **y_output,
//                char **u_output,
//                char **v_output,
//               int stream);

	a = mpeg3_read_yuvframe_ptr(video_file, &f_y_optr, &f_u_optr, &f_v_optr, 0);

	/* picture to image buffer in RGB */
	ucptrd = image_buffer;
	for(y = 0; y < height; y++)
		{
		for(x = 0; x < width; x++)
			{
			/* yuv to rgb */
			cy = (*f_y_optr & 0xff) - 16;
			cu = (*f_u_optr & 0xff) - 128;
			cv = (*f_v_optr & 0xff) - 128;

			cy  *= 76310;

			cr = 104635 * cv;
			cg = -25690 * cu + -53294 * cv;
			cb = 132278 * cu;

			*ucptrd++ = LIMIT(cb + cy);
			*ucptrd++ = LIMIT(cg + cy);
			*ucptrd++ = LIMIT(cr + cy);

			f_y_optr++;

			/* increment U and V vector pointers */
			if(x % 2)
				{
				f_u_optr++;
				f_v_optr++;
				}

			} /* end for x */

		/* 
		2 x 2 color pixels on screen for each Y value,
		repeat each line twice.					

		Orientation on screen Y (*) and U V (o)
		* o
		o o 
		drop shadow :-) color less area below char looks better.
		sink a line.
		*/
		if( (y + 1) % 2)
			{
			f_u_optr -= width / 2;
			f_v_optr -= width / 2;
			}
		} /* end for y */

	} /* end if file and preview */

/* black white area, and possible picture now in image buffer */


/* in case subtitler-yuv templates used */
if( (output_format == OUTPUT_FORMAT_PPML_FOR_SUBTITLER_YUV) && (use_animation_templates_flag) )
	{ 
	/* and any animation template objects */
	add_objects(image_buffer, current_frame);
	}
		
/* seek to subtitle from current frame */

if( (output_format == OUTPUT_FORMAT_PPML_FOR_SUBTITLER_YUV) && (use_animation_templates_flag) )
	{
	pa = lookup_this_or_earlier_frame_that_is_no_animation_template_reference(current_frame);
	}
else
	{
	pa = lookup_this_or_earlier_frame(current_frame);
	}

/*	if pa == 0, then list at or before this point is empty, no subtitles */

if(debug_flag)
	{
	fprintf(stdout,\
	"preview_display(): pa=%p\n", pa);
	}

/* if any subtitle */
if(pa)
	{
	if(debug_flag)
		{
		fprintf(stdout,\
		"processing subtitle for pa->name=%s pa->end_frame=%d pa->text=%s\n",\
		pa -> name, pa -> end_frame, pa -> text);
		}

	is_subtitle_flag = 1;

	if(output_format == OUTPUT_FORMAT_PPML_FOR_SUBTITLER_YUV)
		{
		/* split of any command in the form of '[xxx x x]' */
		if(! extract_command(pa -> text, text_part, command_part) ) return 0;

		if(! use_animation_templates_flag)
			{
			/* treat anything as a subtitle, but get parameters from subtitle template */

			pt = lookup_animation_template("subtitle");

			/* pt cannot be zero here, as subtitle template is always there */
			}
		else
			{
			/* any templates have already been displayed */

			/* filter out the subtitle only */

			/* lookup command part in template list */
			pt = lookup_animation_template(command_part);
	
			/* if pt is zero it is not in the template list, and must be a subtitle */
			if(! pt)
				{
				/* set pt to point to the subtitle template, that is always present in the list */
				pt = lookup_animation_template("subtitle");

				/* pt cannot be zero here, as subtitle template is always there */
				}
			else
				{
				/*
				test if it was the 'subtitle' template (command part was 'subtitle'), 
				we normally do not specify a subtitle with [subtitle], but if we do, it is.
				*/

				if(strcmp(pt -> name, "subtitle") != 0)
					{
					is_subtitle_flag = 0;
					}
				}

			/* pt now points to the 'subtitle' animation template entry */

			} /* end if use_animation_templates_flag */

		/*
		template could be 'subtitle', if so and use_animation_templates_flag is set, then
		we use pt  font and color for all subtitles, else we use pa vars. 
		*/

		/* add if subtitle and in time range */
		if(	(is_subtitle_flag) && (current_frame >= atoi(pa -> name) ) && (current_frame < pa -> end_frame) )
			{
			if(debug_flag)
				{
				fprintf(stdout, "preview_display(): in range\n");
				}

			line_h_start = h_factor * (double)h_size;
			line_h_end = (double)h_size - (double)line_h_start;

			window_bottom = v_size - (v_factor * (double)v_size);

			/*
			Set hor_position for start text in all lines to zero,
			center_text() may overrule this if center_flag.
			*/
			for(i = 0; i < MAX_SCREEN_LINES; i++)
				{
				screen_text[i][0] = 0;
				screen_start[i] = 0; // pixels from left used to center text
				}

			/* first part subtitle dynamics */
			espace = pa -> espace + (frames * pt -> despace);
//			vespace = pt -> vespace + (frames * pt -> dvespace);
			degrees = pt -> color + (frames * pt -> dcolor);

			/* reformat and center need other espace, save old one */
			/* NOTE: actually we should freeze main form here */
			da = pa -> espace;

			pa -> espace = (int)espace;

			/* reformat text inserting (multiple) '/n' if too long */ 
			tptr = p_reformat_text(pa, line_h_end - line_h_start);
			if(! tptr)
				{
				printf("subtitler(): could not reformat text=%s\n", pa -> text);
		
				/* return error */
				return 0;
				} /* end if reformat text failed */		

			if(debug_flag)
				{
				fprintf(stdout, "preview_display(): reformatted text=%s\n", tptr);
				}

			/* center text */
			if(pa -> status & CENTER_TEXT) p_center_text(tptr, pa);

			/* restore original esapce */
			pa -> espace = da;

			/* text now in tptr */

			line_height = pa -> pfd -> height + pa -> vespace;

			/* text to array screen_text[] */
			cptr = tptr;

			if(debug_flag)
				{
				fprintf(stdout, "preview_display(): reformatted and centered text=%s\n", cptr);
				}

			screen_lines = 1; /* at least one */
			while(1) /* all chars in tptr */
				{
				i = 0;
				while(1) /* all chars in a line */
					{
					if(*cptr == '\n')
						{
						/* scip the '\n' */
						cptr++;
					
						/* force string termination */
						screen_text[screen_lines - 1][i] = 0;
	
						/* point to next screen line */
						screen_lines++;
						break;
						}
	
					/* copy character */
					screen_text[screen_lines - 1][i] = *cptr;	
	
					/* test for end of string tptr */
					if(*cptr == 0) break;

					/* point to next char in screen_lines[][] */
					i++;
	
					/* point to next char in tptr */
					cptr++;
					} /* end while all characters in line terminated with LF */
	
				a = 0;
				/* some limit */
				if(screen_lines > MAX_SCREEN_LINES)
					{
					screen_lines = MAX_SCREEN_LINES;
					a = 1;
					}

				window_top = window_bottom - (screen_lines * line_height) + pa -> vespace;
	
				/* prevent what does not fit vertically */
				if(window_top < 0) // - line_height)
					{
					screen_lines--;
					window_top = window_bottom - (screen_lines * line_height);
					a = 1;
					}

				/* actually we should notify user here */
				if(a)
					{
					/* trigger fl_show alert() in idle_cb, we are in a thread, cannot call it here */
					size_error_flag = 1;
					break;
					}

				/* ready if end of tptr */
				if(*cptr == 0) break;
				} /* end while all lines in tptr */
	
			free(tptr);
	
			if(debug_flag)
				{
				printf("screen_lines=%d\n", screen_lines);
				printf("line_h_start=%d line_h_end=%d\n",\
				line_h_start, line_h_end);
				printf("window_bottom=%d window_top=%d\n",\
				window_bottom, window_top);
				}
	
			/* centered and formatted text now in array screen_lines[] */

			/* print lines of text on screen in right position */
			for(i = 0; i < screen_lines; i++)
				{
				/* convert any special characters */
				/*
				j = 0;
				while(1)
					{
					c = screen_text[i][j];
					character_lookup(c, &b);
					screen_text[i][j] = b;
					if(c == 0) break;
					j++;
					}
				*/
				x = screen_start[i];

				y = window_top + (i * line_height);
	
				if(debug_flag)
					{
					printf(\
					"screen_start[%d]=%d window_bottom=%d window_top=%d\n\
line_height=%d x=%d y=%d\n\
text=%s\n",\
					i, screen_start[i], window_bottom, window_top,\
					line_height, x, y,\
					screen_text[i]);
					}
	
				if(y > 576) continue;
				if(x > 720) continue;

				/* use font from framelist, other settings from subtitle template */

				/* use frame colors if not in animation mode */
				if(! use_animation_templates_flag)
					{
					/* white part */

					dm = 100.0 * (double) pa -> pattern_contrast / 15.0;

					dr = (double)xste_rgb_palette[pa -> pattern][0];
					dg = (double)xste_rgb_palette[pa -> pattern][1];
					db = (double)xste_rgb_palette[pa -> pattern][2];

					/* rgb to yuv */
					dy = (0.257 * dr) + (0.504 * dg) +  (0.098 * db) + 16.0;
					du = (0.439 * dr) - (0.368 * dg) - (0.071 * db);
					dv = -(0.148 * dr) - (0.291 * dg) + (0.439 * db);

					/* no color no action, prevent nan */
					if( (du != 0) && (dv != 0) )
						{
						/* u v to angle */

						/* calculate the vector amplitude (hypotenusa) */
						dvector = hypot(du, dv);

						/* calculate the sine */
						dsine = (du / dvector);

						/* dsine must be in the range -1 to +1, else errno. */

						/* get the vector angle */
						errno = 0;
						radians = asin(dsine);
						if(errno == EDOM)
							{
							perror("set_color_circle(): rotate_color(): asin NOT A NUMBER :-)");
							}
	
						/* -PI/2 to +PI/2 radians = -90 to + 90 degrees. */
						degrees = radians * (180.0 / M_PI);
						/* degrees now -90 to + 90 */

						if(debug_flag)
							{
							fprintf(stdout,\
							"dr=%.2f dg=%.2f db=%.2f dy=%.2f du=%.2f dv=%.2f degrees=%.2f\n",\
							dr, dg, db, dy, du, dv, degrees); 
							}

						/* 4 quadrants */
						if( (du > 0) && (dv > 0) ) degrees += 0.0;
						if( (du < 0) && (dv > 0) ) degrees = 90.0 - degrees;
						if( (du > 0) && (dv < 0) ) degrees += 270.0;
						if( (du < 0) && (dv < 0) ) degrees = 180.0 - degrees;

						if(debug_flag)
							{
							fprintf(stdout, "new degrees=%.2f\n", degrees);
							}
						} /* end if any color */
					} /* end if new_font_flag */

				/* second part subtitle dynamics */

				/* no heading, this is only for scrolling titles */

				frames = current_frame - atoi(pa -> name);
				y += frames * pt -> dypos;
				x += frames * pt -> dxpos;
				saturation = pt -> saturation + (frames * pt -> dsaturation);
				contrast = pt -> contrast + (frames * pt -> dcontrast);
				transparency = pt -> transparency + (frames * pt -> dtransparency);

				/* limit ranges */
	
				if(x < -100000.0) x = -100000.0;
				if(x >  100000.0) x =  100000.0;

				if(y < -100000.0) y = -100000.0;
				if(y >  100000.0) y =  100000.0;
				
				if(contrast <   0.0) contrast =   0.0;
				if(contrast > 100.0) contrast = 100.0;

				if(transparency <   0.0) transparency =   0.0;
				if(transparency > 100.0) transparency = 100.0;

				if(saturation <   0.0) saturation =   0.0;
				if(saturation > 100.0) saturation = 100.0;

				if(espace <  -20.0) espace =  -20.0;
				if(espace > 1000.0) espace = 1000.0;
					
				/* output text */

				add_text_yuv(\
				image_buffer,\
				x, y,\
				degrees, saturation,\
				contrast, transparency,\
				espace,\
				screen_text[i],\
				pa -> pfd);

				} /* end for all screen_lines */

			} /* end if subtitle and in range */

		} /* end if ppml-yuv */
	else /* all other sub types */
		{
		/* everything is a subtitle, add if in time range */
		if( (current_frame >= atoi(pa -> name) ) && (current_frame < pa -> end_frame) )
			{
			if(debug_flag)
				{
				fprintf(stdout, "preview_display(): in range\n");
				}
	
			line_h_start = h_factor * (double)h_size;
			line_h_end = (double)h_size - (double)line_h_start;

			window_bottom = v_size - (v_factor * (double)v_size);

			/*
			Set hor_position for start text in all lines to zero,
			center_text() may overrule this if center_flag.
			*/
			for(i = 0; i < MAX_SCREEN_LINES; i++)
				{
				screen_text[i][0] = 0;
				screen_start[i] = 0; // pixels from left used to center text
				}
	
			/* reformat text inserting (multiple) '/n' if too long */ 
			tptr = p_reformat_text(pa, line_h_end - line_h_start);
			if(! tptr)
				{
				printf("subtitler(): could not reformat text=%s\n", pa -> text);
		
				/* return error */
				return 0;
				} /* end if reformat text failed */		

			if(debug_flag)
				{
				fprintf(stdout, "preview_display(): reformatted text=%s\n", tptr);
				}

			/* center text */
			if(pa -> status & CENTER_TEXT) p_center_text(tptr, pa);

			/* text now in tptr */

			line_height = pa -> pfd -> height + pa -> vespace;

			/* text to array screen_text[] */
			cptr = tptr;
	
			if(debug_flag)
				{
				fprintf(stdout, "preview_display(): reformatted and centered text=%s\n", cptr);
				}

			screen_lines = 1; /* at least one */
			while(1) /* all chars in tptr */
				{
				i = 0;
				while(1) /* all chars in a line */
					{
					if(*cptr == '\n')
						{
						/* scip the '\n' */
						cptr++;
					
						/* force string termination */
						screen_text[screen_lines - 1][i] = 0;
	
						/* point to next screen line */
						screen_lines++;
						break;
						}
	
					/* copy character */
					screen_text[screen_lines - 1][i] = *cptr;	
	
					/* test for end of string tptr */
					if(*cptr == 0) break;
	
					/* point to next char in screen_lines[][] */
					i++;
	
					/* point to next char in tptr */
					cptr++;
					} /* end while all characters in line terminated with LF */
	
				a = 0;
				/* some limit */
				if(screen_lines > MAX_SCREEN_LINES)
					{
					screen_lines = MAX_SCREEN_LINES;
					a = 1;
					}

				window_top = window_bottom - (screen_lines * line_height) + pa -> vespace;
	
				/* prevent what does not fit vertically */
				if(window_top < 0) // - line_height)
					{
					screen_lines--;
					window_top = window_bottom - (screen_lines * line_height);
					a = 1;
					}

				/* actually we should notify user here */
				if(a)
					{
					/* trigger fl_show alert() in idle_cb, we are in a thread, cannot call it here */
					size_error_flag = 1;
					break;
					}

				/* ready if end of tptr */
				if(*cptr == 0) break;
				} /* end while all lines in tptr */

			free(tptr);
	
			if(debug_flag)
				{
				printf("screen_lines=%d\n", screen_lines);
				printf("line_h_start=%d line_h_end=%d\n",\
				line_h_start, line_h_end);
				printf("window_bottom=%d window_top=%d\n",\
				window_bottom, window_top);
				}
	
			/* calculate overlay size */
			text_start = INT_MAX;
			max_width = 0;
			for(i = 0; i < screen_lines; i++)
				{
				/* get text length */
				line_len = 0;
				j = 0;
				while(1)
					{
					c = screen_text[i][j];
					line_len += get_h_pixels(c, pa);
	
					if(c == 0) break;
					j++;
					}
	
				x = screen_start[i];
	
				/* get size of bitmap as in bitmap.c */
				if(x < text_start) text_start = x;
				if(line_len > max_width) max_width = line_len;
	
				y = window_top + (i * line_height);

				} /* end for all screen_lines */

			/* fill in background */
			if(use_dvd_colors_flag)
				{
				bg_height = screen_lines * line_height - pa -> vespace;
				a = bg_height % 4;   
				if(a) bg_height += 4 - a;

				bg_width = max_width + 6;	
				a = bg_width % 8;
				if(a) bg_width += 8 - a;

				bg_y_start = window_top;
				bg_y_end = bg_y_start + bg_height;

				/* we have text_start and max_width */
				bg_x_start = text_start;	
				bg_x_end = bg_x_start + bg_width;
	
				if(debug_flag)
					{
					fprintf(stdout,\
					"bg_y_start=%d bg_height=%d bg_x_start=%d bg_width=%d h_size=%d\n",\
					bg_y_start, bg_height, bg_x_start, bg_width, h_size);
					}

				for(y = bg_y_start; y < bg_y_end; y++)
					{
					for(x = bg_x_start; x < bg_x_end; x++)
						{
						ucptr = image_buffer + (3 * (y * h_size + x) );

						/* get original */
						dib = ucptr[0];
						dig = ucptr[1];
						dir = ucptr[2];

						/* calculate multiplier */
						dm = (double) pa -> background_contrast / 15.0;

						/* get new part */
						dr = (double)xste_rgb_palette[pa -> background][0];
						dg = (double)xste_rgb_palette[pa -> background][1];
						db = (double)xste_rgb_palette[pa -> background][2];
	
						/* calculate value new part in output */
						dr *= dm;
						dg *= dm;
						db *= dm;

						/* calculate value old part in output */
						dib *= 1.0 - dm;
						dig *= 1.0 - dm;
						dir *= 1.0 - dm;
					
						/* combine old and new parts in output */
						ucptr[0] = (int) (db + dib);
						ucptr[1] = (int) (dg + dig);
						ucptr[2] = (int) (dr + dir);
		
						} /* end for y */
					} /* end for y */
				} /* end if use_dvd_colors_flag */

			/* print lines of text on screen in right position */
			for(i = 0; i < screen_lines; i++)
				{
				/* convert any special characters */
				/*
				j = 0;
				while(1)
					{
					c = screen_text[i][j];
					character_lookup(c, &b);
					screen_text[i][j] = b;
					if(c == 0) break;
					j++;
					}
				*/
				x = screen_start[i];
	
				y = window_top + (i * line_height);
	
				if(debug_flag)
					{
					printf(\
					"screen_start[%d]=%d window_bottom=%d window_top=%d\n\
line_height=%d x=%d y=%d\n\
text=%s\n",\
					i, screen_start[i], window_bottom, window_top,\
					line_height, x, y,\
					screen_text[i]);
					}
	
				if(y >= 576) continue;
				if(x >= 720) continue;
	
/*
				int add_text_x(\
				int x, int y, char *text, struct frame *pa, int u, int v, double contrast, double transparency)
*/

				add_text_x(x, y, screen_text[i], pa, 0, 0, 100.0, 0.0);

				} /* end for all screen_lines */

			} /* end if subtitle and in range */

		} /* end all other sub types */

	} /* end if pa */

/* copy picture to X11 buffer */

/* get X11 buffer */
ucptrd = (unsigned char *)getbuf();

/* copy data to X11 buffer */
ucptrs = image_buffer;

/* need vertical flip, but not horizontal flip */
if(color_depth == 32)
	{
	for(y = 0; y < height; y++)
		{
		/*
		get a line from buffer start, copy to xbuffer end
		*/
		for(x = 0; x < width; x++)
			{
			*ucptrd++ = *ucptrs++;
			*ucptrd++ = *ucptrs++;
			*ucptrd++ = *ucptrs++;

			ucptrd++; /* nothing in byte 4 */
			}
		} /* end for y (all lines) */
	} /* end if color_depth 32 */
else if(color_depth == 24) // NOT TESTED!!!!!!!!
	{
	/* ucptrs points to start buffer, ucptrd to X buffer */
	for(y = 0; y < height; y++)
		{
		/*
		get a line from buffer start, copy to xbuffer end
		*/
		for(x = 0; x < width; x++)
			{
			*ucptrd++ = *ucptrs++;
			*ucptrd++ = *ucptrs++;
			*ucptrd++ = *ucptrs++;
			}
		} /* end for y (all lines) */
	} /* end if color_depth 32 */

/* display the image */
putimage(width, height);

return 1;
} /* end function preview_display */


int add_text_x(\
int x, int y, char *text, struct frame *pa, int u, int v,\
double contrast, double transparency)
{
int a;
char *ptr;

if(debug_flag)
	{
	printf("add_text_x(): x=%d y=%d text=%s\n\
	pa=%p u=%d v=%d contrast=%.2f transparency=%.2f\n",\
	x, y, text, pa, u, v, contrast, transparency);
	} 

ptr = text;
while(*ptr)
	{
	/* convert to signed */
	a = *ptr;
	if(*ptr < 0) a += 256;

	if(a == ' ')
		{
		/* want to print background only here, not '_' */
		print_char_x(x, y, a, pa, u, v, contrast, transparency, pa -> pfd, 1);

		}
	else
		{
		print_char_x(x, y, a, pa , u, v, contrast, transparency, pa -> pfd, 0);
		}

	x += pa -> pfd -> width[a] + pa -> pfd -> charspace;

	x += pa -> espace;

	ptr++;
	}

return 1;
} /* end function add_text_x */


int print_char_x(\
int x, int y, int c,\
struct frame *pa,\
int u, int v,\
double contrast, double transparency, font_desc_t *pfd, int is_space)
{
if(debug_flag)
	{
	printf("print_char_x(): arg\n\
	x=%d y=%d c=%d pa =%p u=%d v=%d contrast=%.2f transparency=%.2f pfd=%p is_space=%d\n",\
	x, y, c, pa,  u, v, contrast, transparency, pfd, is_space);
	}

draw_alpha_x(\
	x,\
	y,\
	pa,\
	pfd->width[c],\
	pfd->pic_a[pa -> symbols]->h,\
	pfd->pic_b[pa -> symbols]->bmp + pfd->start[c],\
	pfd->pic_a[pa -> symbols]->bmp + pfd->start[c],\
	pfd->pic_a[pa -> symbols]->w,\
	u, v, contrast, transparency, is_space);

return 1;
} /* end function print_char_x */


void draw_alpha_x(\
	int x0, int y0,\
	struct frame *pa,\
	int w, int h,\
	uint8_t *src, uint8_t *srca, int stride,\
	int u, int v, double contrast, double transparency, int is_space)
{
if(debug_flag)
	{
	printf(\
	"draw_alpha_x(): x0=%d y0=%d pa=%p w=%d h=%d\n\
	src=%p srca=%p stride=%d\n",\
	x0, y0, pa, w, h,\
	src, srca, stride);
	}

draw_alpha_rgb24_x(\
	w, h, pa, src, srca, stride,\
	image_buffer + 3 * (y0 * h_size + x0),\
	3 * h_size, is_space);
} /* end function draw_alpha_x */


void draw_alpha_rgb24_x(\
int w, int h,\
struct frame *pa,\
unsigned char *src, unsigned char *srca, int srcstride,\
unsigned char *dstbase, int dststride, int is_space)
{
int y;
int fg_red, fg_green, fg_blue;
int bg_red, bg_green, bg_blue;
double dm, dr, dg, db, dir, dig, dib;
double dmulto, dmulti;

if(debug_flag)
	{
	printf("draw_alpha_rgb24_x(): dstbase=%p pa=%p using draw_mode=%d\n",\
	dstbase, pa, draw_mode);
	}

fg_red = 255;
fg_green = 255;
fg_blue = 255;

bg_red = 0;
bg_green = 0;
bg_blue = 0;

for(y = 0; y < h; y++)
	{
	register unsigned char *dst = dstbase;
	register int x;

	/* clip top */
	if(y >= v_size) continue;

	for(x = 0; x < w; x++)
		{
		/* clip right side */
		if(x >= h_size) continue;

//fprintf(stderr, "WAS draw_mode=%d\n", draw_mode);

		/*
		srca outline
		src white part of character.
		*/
		switch(draw_mode)
			{
			case DRAW_MODE_COLOR:
				if(srca[x] && !is_space)
					{
					/* BGR */
					dst[0] = ((dst[0] * fg_blue) >> 8) + src[x];
					dst[1] = ((dst[1] * fg_green) >> 8) + src[x];
					dst[2] = ((dst[2] * fg_red) >> 8) + src[x];
					}
				else
					{
					dst[0] = ((dst[0] * bg_blue) >> 8) + src[x];
					dst[1] = ((dst[1] * bg_green) >> 8) + src[x];
					dst[2] = ((dst[2] * bg_red) >> 8) + src[x];
					}
				break;
			case DRAW_MODE_BW_FAST:
				if(srca[x] && !is_space)
					{
					dst[0] = dst[1] = dst[2] = src[x];
					}
				break;
			case DRAW_MODE_BW:	
				if(srca[x] && !is_space)
					{
					dst[0] = ((dst[0] * srca[x]) >> 8) + src[x];
					dst[1] = ((dst[1] * srca[x]) >> 8) + src[x];
					dst[2] = ((dst[2] * srca[x]) >> 8) + src[x];
					}
				break;
			case DRAW_MODE_OGT:
				/* character present */

				/* get original */
				dib = dst[0];
				dig = dst[1];
				dir = dst[2];

				if(srca[x] && !is_space) // character body
					{
					/* get multiplier as double */
					dmulto = (double) srca[x] / 256.0;

					if(pa -> status & DVD_ANTI_ALIAS)
						{
						/* multiplier to range 0 to 1.0 */
						dmulti = (double) src[x] / 256.0;
						}
					else
						{
						if(src[x]) dmulti = 1.0;
						else dmulti = 0.0;
						}

					if(dmulti > .5)
						{
						/* white part */
						dm = (double) pa -> pattern_contrast / 15.0;

						dr = (double)xste_rgb_palette[pa -> pattern][0];
						dg = (double)xste_rgb_palette[pa -> pattern][1];
						db = (double)xste_rgb_palette[pa -> pattern][2];

						/* calculate value new part in output */
						dr *= dm;
						dg *= dm;
						db *= dm;

						/* calculate value old part in output */
						dib *= 1.0 - dm;
						dig *= 1.0 - dm;
						dir *= 1.0 - dm;

						/* combine old and new parts in output */
						dst[0] = (int) (db + dib);
						dst[1] = (int) (dg + dig);
						dst[2] = (int) (dr + dir);
						} /* end body */
					else
						{
						if(pa -> status & DVD_ANTI_ALIAS) /* use emphasis2 for anti-alias */
							{
							/* use outline2 for anti aliasing, default is grey 50 % */
							if( (dmulti > 0) && (dmulti < .5) )
								{
								dm = (double) pa -> emphasis2_contrast / 15.0;

								dr = (double)xste_rgb_palette[pa -> emphasis2][0];
								dg = (double)xste_rgb_palette[pa -> emphasis2][1];
								db = (double)xste_rgb_palette[pa -> emphasis2][2];

								/* calculate value new part in output */
								dr *= dm;
								dg *= dm;
								db *= dm;

								/* calculate value old part in output */
								dib *= 1.0 - dm;
								dig *= 1.0 - dm;
								dir *= 1.0 - dm;

								/* combine old and new parts in output */
								dst[0] = (int) (db + dib);
								dst[1] = (int) (dg + dig);
								dst[2] = (int) (dr + dir);
								}
							else
								{
								/* border */
								dm = (double) pa -> emphasis1_contrast / 15.0;

								dr = (double)xste_rgb_palette[pa -> emphasis1][0];
								dg = (double)xste_rgb_palette[pa -> emphasis1][1];
								db = (double)xste_rgb_palette[pa -> emphasis1][2];

								/* calculate value new part in output */
								dr *= dm;
								dg *= dm;
								db *= dm;
	
								/* calculate value old part in output */
								dib *= 1.0 - dm;
								dig *= 1.0 - dm;
								dir *= 1.0 - dm;
						
								/* combine old and new parts in output */
								dst[0] = (int) (db + dib);
								dst[1] = (int) (dg + dig);
								dst[2] = (int) (dr + dir);

								}
							} /* end DVD anti-alias */
						else
							{
							/* border */
							dm = (double) pa -> emphasis1_contrast / 15.0;

							dr = (double)xste_rgb_palette[pa -> emphasis1][0];
							dg = (double)xste_rgb_palette[pa -> emphasis1][1];
							db = (double)xste_rgb_palette[pa -> emphasis1][2];

							/* calculate value new part in output */
							dr *= dm;
							dg *= dm;
							db *= dm;
	
							/* calculate value old part in output */
							dib *= 1.0 - dm;
							dig *= 1.0 - dm;
							dir *= 1.0 - dm;
						
							/* combine old and new parts in output */
							dst[0] = (int) (db + dib);
							dst[1] = (int) (dg + dig);
							dst[2] = (int) (dr + dir);
							
							} /* end no alias */
						} /* end dmulti < 0.5 */

					} /* end if srca[x] */
				else // not character body 
					{
					if(! (pa -> status & DVD_ANTI_ALIAS) ) /* use emphasis2 for  character space */
						{
						/* calculate multiplier */
						dm = (double) pa -> emphasis2_contrast / 15.0;

						/* get new part */
						dr = (double)xste_rgb_palette[pa -> emphasis2][0];
						dg = (double)xste_rgb_palette[pa -> emphasis2][1];
						db = (double)xste_rgb_palette[pa -> emphasis2][2];

						/* calculate value new part in output */
						dr *= dm;
						dg *= dm;
						db *= dm;

						/* calculate value old part in output */
						dib *= 1.0 - dm;
						dig *= 1.0 - dm;
						dir *= 1.0 - dm;
					
						/* combine old and new parts in output */
						dst[0] = (int) (db + dib);
						dst[1] = (int) (dg + dig);
						dst[2] = (int) (dr + dir);
						} /* end if ! DVD_ANTI_ALIAS */
					} /* end if ! srca[x] */
				break;
			default:
				return;
				break;
			} /* end switch draw_mode */

		dst += 3; // 24bpp
		} /* end for all x */

	src += srcstride;
	srca += srcstride;
	dstbase += dststride; // flipped ??
	} /* end for all y */

return;
} /* end function draw_alpha_rgb24_x */


int add_text_yuv(uint8_t *buffer, int xpos, int ypos,\
	double color, double saturation, double contrast, double transparency,\
	int espace, char *text, font_desc_t *pfd)
{
char *tptr;

if(debug_flag)
	{
	fprintf(stdout,\
	"add_text_yuv(): arg buffer=%p xpos=%d ypos=%d\n\
	color=%.2f saturation=%.2f contrast=%.2f transparency=%.2f\n\
	espace=%d text=%s pfd=%p\n",\
	buffer, xpos, ypos,\
	color, saturation, contrast, transparency,\
	espace, text, pfd);	
	}

tptr = text;
while(1)
	{
	if(*tptr == 0) break;

	if(*tptr != ' ')
		{
		print_char_col(xpos, ypos, *tptr, color, contrast, transparency, saturation, pfd);
		}

	/* calculate position */
	xpos += pfd -> width[ (int)*tptr] + pfd -> charspace;;

	xpos += espace;

	tptr++;
	} /* end for all characters in text */

return 1;
} /* end function add_text_yuv */


int print_char_col(int xpos, int ypos, int c,\
	double color, double contrast, double transparency, double saturation, font_desc_t *pfd)
{
int cy, cr, cg, cb;
double du, dv;
int red, green, blue;
int y, u, v;
int default_font = 0;

if(debug_flag)
	{
	fprintf(stderr,\
	"print_char_col(): arg\n\
	xpos=%d ypos=%d c=%d color=%.2f contrast=%.2f transparency=%.2f saturation=%.2f pfd=%p\n",\
	xpos, ypos, c, color, contrast, transparency, saturation, pfd);
	}

if(! pfd) 
	{
	fprintf(stderr, "print_char_col(): no font specified, aborting\n");

	return 0;
	}

saturation /= 100.0;

du = sin(color * M_PI / 180.0);
dv = cos(color * M_PI / 180.0);

u = du * 255.0 * saturation;
v = dv * 255.0 * saturation;

/* yuv to rgb */
/* lot of white */
y = 200.0 * contrast / 100.0;
cy = y - 16;
cy  *= 76310;

cr = 104635 * v;
cg = -25690 * u + -53294 * v;
cb = 132278 * u;

red = LIMIT(cr + cy);
green = LIMIT(cg + cy);
blue =  LIMIT(cb + cy);

if(debug_flag)
	{
	fprintf(stdout, "print_char_col(): red=%d green=%d blue=%d\n", red, green, blue);
	}

draw_alpha_rgb24_r(\
	xpos,\
	ypos,\
	pfd->width[c],\
	pfd->pic_a[default_font]->h,\
	pfd->pic_b[default_font]->bmp + pfd->start[c],\
	pfd->pic_a[default_font]->bmp + pfd->start[c],\
	pfd->pic_a[default_font]->w,\
	red, green, blue, contrast, transparency);

return 1;
} /* end function print_char_col */


void draw_alpha_rgb24_r(\
	int x0,\
	int y0,\
	int w,\
	int h,\
	uint8_t *src,\
	uint8_t *srca,\
	int stride,\
	int red, int green, int blue, double contrast, double transparency)
{
int b, y;
uint8_t *pr, *pg, *pb, *pbstart, *pbend;
double dm, di, dci;
uint8_t ua, uc;
uint8_t ur, ug, ub;
double dmr, dmg, dmb;

if(debug_flag)
	{
	fprintf(stderr, \
	"draw_alpha_rgb24_r(): x0=%d y0=%d w=%d h=%d\n\
	src=%p srca=%p stride=%d red=%d green=%d blue=%d\n\
	contrast=%.2f transparency=%.2f\n",\
	x0, y0, w, h,\
	src, srca, stride, red, green, blue,\
	contrast, transparency);

	fprintf(stderr, "using: image_buffer=%p\n", image_buffer);
	}

pbstart = image_buffer;
pbend = image_buffer + (3 * (h_size * v_size) );

/* calculate multiplier for transparency outside loops */
dm = transparency / 100.0; // main, 1 for 100 % transparent
di = 1.0 - dm; // insert (text here) 0 for 100% transparent

/*
do not multiply color (saturation) with contrast,
saturation could be done in adjust color, but done here for speed
*/
dci = di;

di *= (contrast / 100.0); // adjust contrast insert (text)

/* sequence in buffer is b, g, r */

dmr = red / 255.0;
dmg = green / 255.0;
dmb = blue / 255.0;

//a = 3 * (y0 * h_size + x0);
b = h_size * 3;

for(y = 0; y < h; y++)
	{
//	register unsigned char *dst = srca;
	register int x;

	/* clip top scroll */
	if( (y + y0) >= v_size) continue;
	
	/* clip bottom scroll */
	if( (y + y0) < - h) continue;

	for(x = 0; x < w; x++)
		{
		/* clip right scroll */
		if( (x + x0) >= h_size) continue;

		/* clip left scroll */
		if( (x + x0) < 0) continue;

		pb = image_buffer + (3 * ( ( (y + y0) * h_size) + (x + x0) ) ); 
		
		/* prevent segfault when out of buffer area, this happens for some negative values of x0 and y0 */
		if(pb < pbstart) continue;

		pg = pb + 1;
		pr = pb + 2;

		if(pr > pbend) continue;

		if(srca[x])
			{
			/* some temp vars */
			ur = *pr;
			ug = *pg;
			ub = *pb;

			ua = srca[x];
			uc = src[x];

			/* get decision factor before we change anything */
//			cd = ( (pr[x * 3] * srca[x]) >> 8) < 5;

			/* calculate value insert (character) */
			ur = ( (ur * ua) >> 8) + uc;
			ug = ( (ug * ua) >> 8) + uc;
			ub = ( (ub * ua) >> 8) + uc;

			/* attenuate insert (character) the opposite way */
			ur *= di; // di is 0 for 100 % transparent
			ug *= di; // di is 0 for 100 % transparent
			ub *= di; // di is 0 for 100 % transparent

			/* attenuate main */
			*pr *= dm; // dm is 1 for 100% transp
			*pg *= dm; // dm is 1 for 100% transp
			*pb *= dm; // dm is 1 for 100% transp
							
//			if(cd)
			if(src[x]) /* character body */
				{
				ur *= dmr;
				ug *= dmg;
				ub *= dmb;
				}
			else /* outline, no color */
				{

				}

			/* add what is left of the insert (character) */
			*pr += ur;
			*pg += ug;
			*pb += ub;

			} /* end if sa[x] */

		} /* end for all x */

	src += stride;
	srca += stride;
	} /* end for all y */

} /* end function draw_alpha_rgb24_r */


int add_objects(uint8_t *buffer, int frame)
{
struct animation_template *pt;
struct frame *pf;
char command_part[TEXT_ARRAY_SIZE];
char text_part[TEXT_ARRAY_SIZE];
int display_object;
int define_frame = 0; // - Wall
int frames;
char temp[4096];
font_desc_t *pfd;
int a, i;
double da;
int sort_flag;

if(debug_flag)
	{
	fprintf(stdout, "add_objects(): arg buffer=%p frame=%d\n", buffer, frame);
	}

/* for each object get definition frame, then calculate current parameters in pt -> p..... */
for(pt = animation_templatetab[0]; pt != 0; pt = pt -> nxtentr)
	{
	/* if it is subtitle leave it for now, video display will handle this separately */
	if( strcmp(pt -> name, "subtitle") == 0) continue;

	display_object = 0;
	/* find last reference at or before this frame in frame list */
	for(pf = frametab[1]; pf != 0; pf = pf -> prventr)
		{
	    /* skip any comment lines */
		if(pf -> text[0] == '*') continue;

		if(atoi(pf -> name) > frame) continue;

		/* split of any command in the form of '[xxx x x] */
		if(! extract_command(pf -> text, text_part, command_part) ) return 0;

		pt -> ptext = pf -> text + strlen(command_part) + 2;

		if(strcmp(pt -> name, command_part) != 0) continue; 

		if(text_part[0] != 0) display_object = 1;

		define_frame = atoi(pf -> name);

		break;
		} /* end for all entries in frame list */

//fprintf(stdout, "WAS pf=%p\n", pf);

	/* check if object still needs to be displayed */
	if(! display_object) 
		{
		pt -> status &= ~ANIMATION_TEMPLATE_DISPLAY;
		continue;
		}
	else
		{
		pt -> status |= ANIMATION_TEMPLATE_DISPLAY;
		}

	/* calculate current position */
	frames = frame - define_frame;
	
	/* pt points to template */

	pt -> pzpos = pt -> zpos + (frames * pt -> dzpos);
	pt -> pespace = pt -> espace + (frames * pt -> despace);
	pt -> pcontrast = pt -> contrast + (frames * pt -> dcontrast);
	pt -> pcolor = pt -> color + (frames * pt -> dcolor);
	pt -> psaturation = pt -> saturation + (frames * pt -> dsaturation);
	pt -> ptransparency = pt -> transparency + (frames * pt -> dtransparency);

	if(pt -> status & ANIMATION_TEMPLATE_SHOW_X_Y_DELTA)
		{
		pt -> pxpos = pt -> xpos + (frames * pt -> dxpos);
		pt -> pypos = pt -> ypos + (frames * pt -> dypos);
		} /* end if x y delata */
	else
		{
		if(pt -> speed == 0.0)
			{
			pt -> pxpos = pt -> xpos;
			pt -> pypos = pt -> ypos;
			}
		else
			{
			/* calculate current heading */
//			heading = pt -> heading + (frames * pt -> dheading);

			/* some other vars */
			da = M_PI / 180.0;
//			daspect = (double)h_size / (double)v_size;

			/* calculate current speed */
			if(play_start_flag)
				{
//fprintf(stdout, "WAS INIT\n");

				/* these will be altered */
				pt -> pspeed = pt -> speed;
				pt -> pheading = pt -> heading;				

				pt -> pdspeed = pt -> dspeed;
				pt -> pdheading = pt -> dheading;				

				pt -> pxpos = pt -> xpos;
				pt -> pypos = pt -> ypos;
				pt -> pzpos = pt -> zpos;

				/* for all frames up to start of play */
				a = frames;

				}
			else/* in play */
				{	
//fprintf(stdout, "WAS IN PLAY\n");

				a = current_frame - last_frame;
				} /* end if in play */

			/*
			do this for all frames until here when play button pressed,
			when aleardy in play just update one frame by deltas.
			result stored in structure in p... vars
			*/
			for(i = 0; i < a; i++)

				{				
/*
fprintf(stdout,\
"WAS IN FOR current_frame=%d last_frame=%d name=%s a=%d i=%d, pt->heading=%.2f pt->pheading=%.2f\n",\
current_frame, last_frame, pt -> name, a, i, pt -> heading, pt -> pheading);

fprintf(stdout,\
"WAS pt->name=%s pt->xpos=%.2f pt->ypos=%.2f pt->zpos=%.2f pt->dzpos=%.2f\n",\
pt -> name, pt -> xpos, pt -> ypos, pt -> zpos, pt -> dzpos);
*/
 
				/* calculate x, y */
				pt -> dxpos =  sin(pt -> pheading * da );
				pt -> dypos = -cos(pt -> pheading * da );
					
				pt -> dxpos *= pt -> pspeed;
				pt -> dypos *= pt -> pspeed;

				/* add deltas */
				pt -> pxpos += pt -> dxpos;
				pt -> pypos += pt -> dypos;
				pt -> pzpos += pt -> dzpos;

				pt -> pspeed += pt -> pdspeed;
				pt -> pheading += pt -> pdheading;

				/* limit vars */

				/* no limit on heading and dheading, it is circular */

				if(pt -> pspeed < -100000.0) pt -> pspeed = -100000.0;
				if(pt -> pspeed >  100000.0) pt -> pspeed =  100000.0;

				if(pt -> pdspeed < -100000.0) pt -> pdspeed = -100000.0;
				if(pt -> pdspeed >  100000.0) pt -> pdspeed =  100000.0;

				if(pt -> pxpos < -100000.0) pt -> pxpos = -100000.0;
				if(pt -> pxpos >  100000.0) pt -> pxpos =  100000.0;
				if(pt -> pypos < -100000.0) pt -> pypos = -100000.0;
				if(pt -> pypos >  100000.0) pt -> pypos =  100000.0;
				if(pt -> pzpos < -100000.0) pt -> pzpos = -100000.0;
				if(pt -> pzpos >  100000.0) pt -> pzpos =  100000.0;
				if(pt -> dxpos < -100000.0) pt -> dxpos = -100000.0;
				if(pt -> dxpos >  100000.0) pt -> dxpos =  100000.0;
				if(pt -> dypos < -100000.0) pt -> dypos = -100000.0;
				if(pt -> dypos >  100000.0) pt -> dypos =  100000.0;
				if(pt -> dzpos < -100000.0) pt -> dzpos = -100000.0;
				if(pt -> dzpos >  100000.0) pt -> dzpos =  100000.0;

				} /* end for all required frames */

			pt -> pxpos = pt -> pxpos;
			pt -> pypos = pt -> pypos;
			pt -> pzpos = pt -> pzpos;

			} /* end if speed */

		} /* end if use heading */

	/* get font */

	sprintf(temp, "%s/.xste/fonts/%s", home_dir, pt -> fontname);
 
	if(! pt -> pfd)
		{		
		pfd = make_old_font(temp, pt -> fontsize, pt -> iso, pt -> outline, pt -> blur);
 
		if(! pfd) 
			{
			fprintf(stderr,\
			"preview_display(): could not create font %s size=%d iso=%d outline=%.2f blur=%.2f",\
			pt -> fontname,\
			pt -> fontsize,\
			pt -> iso,\
			pt -> outline,\
			pt -> blur);

			return 0;	
			}

		pt -> pfd = pfd;
		} /* end if not pt -> pfd */

	if(pt -> psaturation < 0.0) pt -> psaturation = 0.0;
	if(pt -> psaturation > 100.0) pt -> psaturation = 100.0;

	if(pt -> pcontrast < 0.0) pt -> pcontrast = 0.0;
	if(pt -> pcontrast > 100.0) pt -> pcontrast = 100.0;

	if(pt -> ptransparency < 0.0) pt -> ptransparency = 0.0;
	if(pt -> ptransparency > 100.0) pt -> ptransparency = 100.0;

	if(pt -> pxpos < -10000.0) pt -> pxpos = -10000.0;
	if(pt -> pxpos > 10000.0) pt -> pxpos = 10000.0;

	if(pt -> pypos < -10000.0) pt -> pypos = -10000.0;
	if(pt -> pypos > 10000.0) pt -> pypos = 10000.0;

	if(pt -> pzpos < -10000.0) pt -> pzpos = -10000.0;
	if(pt -> pzpos > 10000.0) pt -> pzpos = 10000.0;

	} /* end for each object */

/* sort objects by zaxis */
/* if any delta z, sort list for zpos */
sort_flag = 0;
for(pt = animation_templatetab[0]; pt != 0; pt = pt -> nxtentr)
	{
	if(pt -> dzpos != 0.0)
		{
		sort_flag = 1;
		break;
		}
	}
if(sort_flag) sort_animation_templates_for_pzpos();

/* display each object */
for(pt = animation_templatetab[0]; pt != 0; pt = pt -> nxtentr)
	{
	if( (pt -> status & ANIMATION_TEMPLATE_DISPLAY) == 0) continue;

	/* add to display */
	add_text_yuv(\
	buffer, (int)pt -> pxpos, (int)pt -> pypos,\
	pt -> pcolor, pt -> psaturation,\
	pt -> pcontrast, pt -> ptransparency,\
	(int)pt -> pespace, pt -> ptext, pt -> pfd); 
	}

play_start_flag = 0;
last_frame = current_frame;

return 1;
} /* end function add_objects */


