/*
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "xste.h"



int main(int argc, char **argv)
{
/*
the arguments are:
filename		argv[1]
device			argv[2]
start_sample	argv[3]
end_sample		argv[4]
speed			argv[5]
*/

if(argc < 5)
	{
	fprintf(stdout,\
	"Usage: xste_audio filename device start_sample end_sample speed\n");

	exit(1);
	}

if( !play_file(\
	argv[1], argv[2], atol(argv[3]), atol(argv[4]), atoi(argv[5]) ) )
		{
		exit(1);
		}

exit(0);
}/* end function main */


int play_file(\
	char *filename, char *device,\
	unsigned long start_sample, unsigned long end_sample,\
	int speed\
	)
{
int a, c;
int tmp;
int length;
int total;
unsigned long bytes;
int audiofd;
int filefd;
int timelimit;
int dsp_speed;
int dsp_stereo;
int sample_size;
int abuf_size;
char *audiobuf;
char *ptr;
wave_header *pw;
int multiplier = 0; // -Wall
int header_size;

if(debug_flag)
	{
	fprintf(stdout, "play_file(): arg\n\
	filename=%s device=%s start_sample=%lu end_sample=%lu speed=%d\n",\
	filename, device, start_sample, end_sample, speed);
	}

header_size = sizeof(wave_header);
pw = malloc(header_size);
if(! pw)
	{
	printf("play_file(): could not allocate memory for wave_header\n");

	return 0;
	}

timelimit = 0;
dsp_speed = speed;
dsp_stereo = 0;
sample_size = 8;

bytes = end_sample - start_sample;
if(bytes < 0)
	{
	fprintf(stdout, "play_file(): end < start aborted\n");

	return 0;
	}

audiofd = open (device, O_WRONLY, 0);
if(audiofd < 0)
	{
	fprintf(stdout,\
	"play_file(): could not open device %s for read, aborted\n",\
	device);

	return 0;
	}

if ((filefd = open (filename, O_RDONLY, 0)) == -1)
	{
	close(audiofd);

	fprintf(stdout,\
	"play_file(): could not open file %s for read, aborted\n",\
	filename);

	return 0;
	}

/* test if wave file */
/* get start extention */
ptr = strrchr(filename, '.');
if(ptr)
	{
	if(strcmp(ptr, ".wav") == 0)
		{
		/* read in header_size (44 bytes) wave header */
		c = header_size;
		while(1)
			{
			a = read(filefd, pw, c);

			if(a < 0)
				{
				perror("W");

				return 0;
				}

			if(a == 0) break;
			c -= a;	
			} /* end while read header */

		/* look in header for paramaters we need */
		//struct
		//	{						/* header for WAV-Files */
		//	char main_chunk[4];		/* 'RIFF' */
		//	DWORD length;			/* length of file */
		//	char chunk_type[4];		/* 'WAVE' */
		//	char sub_chunk[4];		/* 'fmt' */
		//	DWORD length_chunk;		/* length sub_chunk, always 16 bytes */
		//	WORD format;			/* always 1 = PCM-Code */

		//	WORD modus;				/* 1 = Mono, 2 = Stereo */
		//	DWORD sample_fq;		/* Sample Freq */
		//	DWORD byte_p_sec;		/* Data per sec */
		//	WORD byte_p_spl;		/* bytes per sample, 1=8 bit, 2=16 bit (mono)
		//						     2=8 bit, 4=16 bit (stereo) */
		//	WORD bit_p_spl;			/* bits per sample, 8, 12, 16 */
		//	char data_chunk[4];		/* 'data' */
		//	DWORD data_length;		/* length of data */
		//	} wave_header;

		dsp_speed = pw -> sample_fq;
		dsp_stereo = pw -> modus - 1;

		sample_size = (pw -> byte_p_spl * 8) / pw -> modus;

		multiplier = pw -> byte_p_spl;
		multiplier *= pw -> sample_fq / RAW_SAMPLE_SPEED;

		start_sample *= multiplier;
		end_sample *= multiplier;
		start_sample += header_size;
		end_sample += header_size;

		bytes *= multiplier;

		} /* end if wave file detected */
	} /* end if file extention specified */

if(debug_flag)
	{
	printf(\
	"play_file():\n\
	dsp_speed=%d dsp_stereo=%d sample_size=%d multiplier=%d\n\
	start_sample=%lu end_sample=%lu bytes=%lu\n",\
	dsp_speed, dsp_stereo, sample_size, multiplier,\
	start_sample, end_sample, bytes);
	}

tmp = sample_size;
ioctl(audiofd, SNDCTL_DSP_SAMPLESIZE, &tmp);
if (tmp != sample_size)
	{
	close(audiofd);

	fprintf(stdout,\
	"play_file(): could not set sample_size %d for %s aborted\n",\
	sample_size, device);

	return 0;
	}

if (ioctl (audiofd, SNDCTL_DSP_STEREO, &dsp_stereo) == -1)
	{
	close(audiofd);

	fprintf(stdout,\
	"play_file(): Could not set stereo / mono mode for %s aborted\n",\
	device);

	return 0;
	}

tmp = dsp_speed;
if (ioctl (audiofd, SNDCTL_DSP_SPEED, &tmp) == -1)
	{
	close(audiofd);

	fprintf(stdout,\
	"play_file(): could not set sample speed %d for %s aborted\n",\
	dsp_speed, device);

	return 0;
	}

ioctl (audiofd, SNDCTL_DSP_GETBLKSIZE, &abuf_size);
if (abuf_size < 1)
	{
	close(audiofd);

	fprintf(stdout,\
	"play_file(): could not get blocksize for %s aborted\n",\
	device);

	return 0;
	}

if ((audiobuf = malloc (abuf_size)) == NULL)
	{
	close(audiofd);

	fprintf(stdout,\
	"play_file(): could not allocate buffer, aborted\n");

	return 0;
	}

a = lseek(filefd, start_sample, SEEK_SET);
if(a == -1)
	{
	close(audiofd);

	fprintf(stdout,\
	"play_file(): could not seek to sample %lu, aborted\n",\
	start_sample);

	return 0;
	}

while(1)
	{
	if(bytes <= 0) break;

	c = bytes;
	if (bytes > abuf_size) c = abuf_size;

	errno = 0;
	length = read (filefd, audiobuf, c);
	if(length < 0)
		{
		if(errno == EAGAIN) continue;
		else if(errno == EINTR) continue;	
		else
			{
			perror("play_file(): read():");
			fprintf(stdout, "play_file(): read failed from %s aborted\n",\
			filename);
			

			close(audiofd);
			close(filefd);

			return 0;
			}
		}
	if(length == 0)
		{
		close(audiofd);
		close(filefd);

		return 1;
		}
	total = length;
	while(1)
		{
		if(total <= 0) break;
		errno = 0;
		length = write (audiofd, audiobuf, total);
		if(length == -1)
			{
			if(errno == EAGAIN) continue;
			else if(errno == EINTR) continue;	
			else
				{
				fprintf(stdout,\
				"play_file(): write failed to %s, aborted\n",\
				 device);
	
				close(audiofd);
				close(filefd);

				return 0;
				}
			}
		else
			{
			total -= length;
			continue;
			}
		} /* end while write */

	bytes -= length;
	} /* end while bytes */

close(audiofd);
close(filefd);

return 1;
}/* end function play_file */


