/*
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "xste.h"

FD_xste *fd_main_form;
extern FD_xste *fdui;


int main(int argc, char *argv[])
{
char temp[1024];
struct passwd *userinfo;
Pixmap p, mask;
unsigned w,h; 

FL_IOPT fl_iopt;
    
/* some defaults */
debug_flag = 0;
data_file_name = strsave(DATA_FILE_NAME);
if(! data_file_name)
	{
	printf("xste(): main(): strsave() data_file_name failed, aborting\n");

	exit(1);
	}

frame_mode = 0;
dnew_old_frame_ratio = 1.0;
text_duration_multiplier = 7;
audio_mode = AUDIO_RAW;
audio_sample_speed = RAW_SAMPLE_SPEED;

dframes_per_second = PAL_FRAMES_PER_SECOND;
dmilli_seconds_per_frame = 1000.0 / dframes_per_second;

/* end defaults */

if(argc > 1) debug_flag = 1;

/* against flashing */
/* set double buffering */
fl_iopt.doubleBuffer = 1;
fl_set_defaults(FL_PDDouble, &fl_iopt);
                   
/* get user info */
userinfo = getpwuid(getuid() );

/* get home directory */ 
home_dir = strsave(userinfo -> pw_dir);
if(! home_dir)
	{
	fprintf(stdout,\
	"main(): could allocte space for home_dir\n");
	exit(1);
	}

user_name = strsave(userinfo -> pw_name);
if(! user_name)
	{
	fprintf(stdout,\
	"main(): could allocate space for user_name\n");
	exit(1);
	}

selected_data_directory = strsave(DATA_DIRECTORY);
if(! selected_data_directory)
	{
	fprintf(stdout,\
	"main(): could allocate space for selected_data_directory\n");
	exit(1);
	}

sprintf(temp, "%s", AUDIO_DEVICE);
selected_device = strsave(temp);
if(! selected_device) 
	{
	fprintf(stdout, "main(): could not allocate space for selected_device\n");
	exit(1);
	} 

/* set default volume for sound playback */
selected_volume = 50;

/* set default use of spell checker on input field */
spellcheck_input_flag = 0;

/* set default editor command */
//editor_command = strsave("rxvt -geometry 80x28  -sl 500 -fn 10x20  -e joe");
editor_command = strsave(DEFAULT_EDIT_COMMAND);
//"rxvt -geometry 100x28  -sl 500 -e joe");
if(! editor_command)
	{
	fprintf(stdout, "Could not allocate space for editor_command\n");

	exit(1);
	}

/* set default spell check command */
spellcheck_command = strsave(DEFAULT_SPELL_CHECK_COMMAND);
//"rxvt -geometry 80x28  -sl 500 -fn 10x20 -e ispell -d nederlands");
if(! spellcheck_command)
	{
	fprintf(stdout, "Could not allocate space for spellcheck_command\n");

	exit(1);
	} 

fl_initialize(&argc, argv, 0, 0, 0);
fd_main_form = create_form_main_form();

/* read the icon */
sprintf(temp, "%s/.xste/xste.xpm", home_dir);
p = fl_read_pixmapfile(fl_root, temp, &w, &h, &mask, 0, 0, 0);
fl_set_form_icon(fdui -> main_form, p, mask);

/* set default fontsizes for forms */
main_form_fontsize = 3;
project_form_fontsize = 3;
load_fontsizes();

default_subtitle_font_name = strsave(DEFAULT_SUBTITLE_FONT);
if(! default_subtitle_font_name)
	{
	printf(\
	"main(): Could not allocate space for subtitle_font_name, aborting\n");

	exit(1);
	}

/* set default settings */
load_general_settings();

load_projects(); /* sets selected_project */

/* load variables for the selected project */
load_project_vars(); /* uses selected_project */

load_animation_templates();
create_subtitle_animation_template();

if(! show_frame_form() ) exit(1);

add_system_fonts();

/* show the main form */
fl_show_form(\
fd_main_form -> main_form, FL_PLACE_FREE, FL_FULLBORDER, VERSION);

/* print the (default) in the input fields */

/* set the title of the main form to the version and the selected project */
set_title();

/*
sets fonts for buttons and articles,
this must happen after fontsizes it seems
*/
load_selected_fonts();


/* FORCED COLOR DEPTH */
color_depth = 32;

/* this is changed depending on the selected output format. */
draw_mode = DRAW_MODE_BW;

pebu_gsi = malloc(sizeof(struct ebu_gsi) );
if(! pebu_gsi)
    {   
    fprintf(stdout, "main(): could not allocate memory for ebu_gsi data block\n");

    return 0;
    }

/* 
start a thread that will start the video display, and update it if
selected_picture changes
*/
if(! video_init() )
	{
	fprintf(stdout, "Could not start video display thread\n");		

	exit(1);
	}

start_timeval = malloc(sizeof(struct timeval) );
if(! start_timeval)
	{
	fprintf(stdout, "could not allocate space for start_timeval\n");

	exit(1);
	}

current_timeval = malloc(sizeof(struct timeval) );
if(! current_timeval)
	{
	fprintf(stdout, "could not allocate space for current_timeval\n");

	exit(1);
	}

/* allocate space for 16 extra colors for the xste global color palette  (will go to CLUT in IFO */

/* setup has loaded xste_rgb_palette[][] using load_project_vars() */
global_subtitle_palette_form_set_all_color_buttons();

allow_idle_cb = 1;

fl_do_forms();

return 0;
} /* end function main */

