
/*
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/


#include "xste.h"

extern FD_xste *fdui;
extern FD_xste *fd_main_form;
static int fontsizes_modified_flag;

FL_APPEVENT_CB optimize_idle_cb;

struct timeval timeout;
time_t idletimer; /* idle time for user */
char *present_time();
int have_start_difference_flag;


void main_form_show_time_overlap_button_cb(FL_OBJECT *ob, long data)
{
show_time_overlap_flag = fl_get_button(ob);

show_frames(SHOW_SAME_POSITION);

save_project_vars();
} /* end function main_form_show_time_overlap_button_cb */


void main_form_preview_snapshot_button_cb(FL_OBJECT *ob, long data)
{
take_preview_snapshot(current_frame);

} /* end function main_form_preview_snapshot_button_cb */


int test_for_open_forms(int *some_form_open_flag)
{
*some_form_open_flag = 0;
if(project_form_shown_flag) 
	{
	fl_show_alert("Close project form first (use ACCEPT)", "", "", 1);
	*some_form_open_flag = 1;
	}

if(export_form_shown_flag) 
	{
	fl_show_alert("Close export form first (use ACCEPT)", "", "", 1);
	*some_form_open_flag = 1;
	}

if(subtitle_font_form_shown_flag) 
	{
	fl_show_alert("Close subtitle font form first (use ACCEPT)", "", "", 1);
	*some_form_open_flag = 1;
	}

if(ebu_general_subtitle_information_form_shown_flag)
	{
	fl_show_alert("Close ebu general subtitle information form first (use ACCEPT)", "", "", 1);
	*some_form_open_flag = 1;
	}

if(text_properties_form_shown_flag)
	{
	fl_show_alert("Close text properties form first (use ACCEPT)", "", "", 1);
	*some_form_open_flag = 1;
	}
if(global_subtitle_palette_form_shown_flag)
	{
	fl_show_alert("Close global subtitle palette form first (use ACCEPT)", "", "", 1);
	*some_form_open_flag = 1;
	}
if(subtitle_color_selector_form_shown_flag)
	{
	fl_show_alert("Close subtitle color selector form (select a color or press CANCEL)", "", "", 1);
	*some_form_open_flag = 1;
	}
if(animation_template_form_shown_flag)
	{
	fl_show_alert("Close animation template form first", "Use ACCEPT to save any changes", "", 1);
	*some_form_open_flag = 1;
	}

return 1;
}/* end function test_for_open_forms */


void main_form_play_button_cb(FL_OBJECT *ob, long data)
{

return;
} /* end function main_form_play_button_cb */


void main_form_align_start_button_cb(FL_OBJECT *ob, long data)
{
/* alert user, mouse on NO */
if(! fl_show_question("This will alter ALL your frame numbers! OK?", 0) )
	{
	return;
	}

new_start_frame = selected_start_frame;
org_start_frame = frame_list_browser_clicked_frame;

have_start_difference_flag = 1;

return;
} /* end main_form_align_start_button_cb */


void main_form_align_end_button_cb(FL_OBJECT *ob, long data)
{
int a, b;

/* alert user, mouse on NO */
if(! fl_show_question("This will alter ALL your frame numbers! OK?", 0) )
	{
	return;
	}

if(! have_start_difference_flag)
	{
	fl_show_alert("press start difference A E button first", "", "", 1);

	return;
	}

new_end_frame = selected_start_frame;
org_end_frame = frame_list_browser_clicked_frame;

/* the ratio is the second length / first length */
a = new_end_frame - new_start_frame;
b = org_end_frame - org_start_frame;
dnew_old_frame_ratio = (double)a / (double)b;

/* the shift is the new start - (correction * old start) */ 
a = org_start_frame * dnew_old_frame_ratio;
frame_shift = new_start_frame - a;

recalculate_frame_list();

recalculate_end_frames();

show_frames(SHOW_SAME_POSITION);

have_start_difference_flag = 0;

if(! fl_show_question(\
"Press NO to exit xste, else changes are permanent", 0) )
	{
	/*
	prevent user from pressing ANY more buttons, as that will save frame
	list (with the new numbers).
	Restarting xste will reload the old numbers.	
	*/
	exit(1);
	}

/* changes permanent, only one time backup available */
save_frames();

return;
} /* end function main_form_align_end_button_cb */


void main_form_browser_cb(FL_OBJECT *ob, long data)
{
int selected_line;
int mouse_button;
int seconds;

if(debug_flag)
	{
	fprintf(stdout, "main_form_browser_cb():\n");
	}

mouse_button = fl_mouse_button();

selected_line = fl_get_browser(ob);
if(! selected_line) return;

/* set selected frame */
if(! line_to_frame(abs(selected_line),\
	&selected_start_frame, &selected_end_frame) ) return;

/* maintain visible indication that line is selected with single click */
fl_select_browser_line(fdui -> main_form_browser, abs(selected_line) );

/* set text input field */
set_text_input(selected_start_frame);

/* look up the corresponding audio_sample */
if(! get_sample(selected_start_frame, &selected_start_sample) ) return;
	
set_frame_input(selected_start_frame);

set_current_frame_input(selected_start_frame);

/* for the MEM button */
current_frame = selected_start_frame;

set_force_duration_button(selected_start_frame);

/* for the Substation Alpha export re-align and skew calculation */
frame_list_browser_clicked_frame = selected_start_frame;

/* font parameters for this text from structure to text_properties_form and variables */
if(! frame_font_parameters_to_vars(selected_start_frame) ) return;

/* if preview enabled, show previev */
new_preview_flag = 1;

/* also play some if right mouse */
if(mouse_button == FL_RIGHT_MOUSE)
	{
	/* allow for 10 seconds sound */
	/* calculate end sample */
	seconds = RIGHT_MOUSE_PLAY_SECONDS;
	selected_end_sample =\
		selected_start_sample + (seconds * audio_sample_speed);

	play_right_mouse_flag = 1;

	if(audio_mode == AUDIO_RAW)
		{
		if(! play_audio_raw(selected_start_sample, selected_end_sample) ) return;
		}
	else if(audio_mode == AUDIO_WAV)
		{
		if(! play_audio_wav(selected_start_sample, selected_end_sample) ) return;
		}
	else if(audio_mode == AUDIO_MP3)
		{
		if(! play_audio_mp3(selected_start_frame) ) return;
		}

	return;
	}

if(mouse_button == FL_MIDDLE_MOUSE)
	{
	if(! show_text_properties_form() ) return;

	return;
	}

/* save o.a. selected_start_frame */
save_project_vars();

}/* end function main_form_browser_cb */


void main_form_browser_double_cb(FL_OBJECT *ob, long data)
{
int selected_line;
int status;
int mouse_button;

if(debug_flag)
	{
	fprintf(stdout, "main_form_browser_double_cb():\n");
	}

mouse_button = fl_mouse_button();

status = 0;/* keep gcc -Wall happy */

selected_line = fl_get_browser(ob);
if(! selected_line) return;

/* moved to single click, xform does not see double click on middle mouse */

//fl_select_browser_line(fdui -> main_form_browser, abs(selected_line) );

/* form was modified */
main_form_modified_flag = 1;

}/* end function main_form_browser_double_cb */


void main_form_text_input_field_cb(FL_OBJECT *ob, long data)
{
char * ptr;
int text_lines;

ptr = (char *)fl_get_input(ob);
if(! ptr) return;

if( (output_format == OUTPUT_FORMAT_EBU_STL) || \
	(output_format == OUTPUT_FORMAT_SUTI) || \
	(OUTPUT_FORMAT_PPML_FOR_SUBTITLER_YUV)
	)
	{
	/*
	ring bell if no fit in 2 teletext lines,
	each line may be 40 minus 2 box control chars and minus one size control
	character, so 37 chars per line, 74 chars total.
	If any more characters, these will be cut off.
	*/

	if( strlen(ptr) > 74) fl_ringbell(100);

	return;
	}
else if(\
	(output_format ==  OUTPUT_FORMAT_SSA) || \
	(output_format ==  OUTPUT_FORMAT_SUTI) || \
	(output_format ==  OUTPUT_FORMAT_VPLAYER) || \
	(output_format ==  OUTPUT_FORMAT_I_AUTHOR) || \
	(output_format ==  OUTPUT_FORMAT_MICRO_DVD_PLAYER ) || \
	(output_format ==  OUTPUT_FORMAT_PPML)
	)
	{	
	/* test if shorter then 2 lines */
	test_length(ptr, &text_lines);

	if(text_lines >= 2)
		{
		fl_ringbell(100);
		}

	return;
	}

/* all other bitmap based cases are handled upon pressing ADD */

}/* end function main_form_text_input_field_cb */


void main_form_text_duration_input_field_cb(FL_OBJECT *ob, long data)
{
char *ptr;
int s_line;

ptr = (char *)fl_get_input(ob);
if(! ptr) return;

if(debug_flag)
	{
	fprintf(stdout,\
	"main_form_text_duration_input_field_cb(): ptr=%s\n",\
	ptr);
	}

text_duration_multiplier = atoi(ptr);

if(! recalculate_end_frames() ) return;

show_frames(SHOW_SAME_POSITION);

/* mark new entry in browser */
frame_to_line(selected_start_frame, &s_line);

fl_select_browser_line(fdui -> main_form_browser, s_line);

save_frames();

save_project_vars();

}/* end function main_form_text_duration_input_field_cb */


void main_form_frames_per_second_input_field_cb(FL_OBJECT *ob, long data)
{
char *ptr;

ptr = (char *)fl_get_input(ob);
if(! ptr) return;

if(debug_flag)
	{
	fprintf(stdout,\
	"main_form_frames_per_second_input_field_cb(): ptr=%s\n",\
	ptr);
	}

dframes_per_second = atof(ptr);
dmilli_seconds_per_frame = 1000.0 / dframes_per_second;

set_frames_per_second_input_field_color();

save_project_vars();

}/* end function main_form_frames_per_second_input_field_cb */


void main_form_hold_text_properties_button_cb(FL_OBJECT *ob, long data)
{
hold_text_properties_flag = fl_get_button(ob);

if(hold_text_properties_flag) fl_set_object_label(ob, "keep this font");
else fl_set_object_label(ob, "default font");

} /* end function main_form_hold_text_properties_button_cb */



void main_form_set_PAL_fps_button_cb(FL_OBJECT *ob, long data)
{
char temp[80];

if(debug_flag)
	{
	fprintf(stdout,\
	"main_form_set_PAL_fps_button_cb():\n");
	}

dframes_per_second = PAL_FRAMES_PER_SECOND;
dmilli_seconds_per_frame = 1000.0 / dframes_per_second;

sprintf(temp, "%.4f", dframes_per_second);
fl_set_input(fdui -> main_form_frames_per_second_input_field, temp);

set_frames_per_second_input_field_color();

save_project_vars();

}/* end function main_form_set_PAL_fps_button_cb */


void main_form_set_NTSC_fps_button_cb(FL_OBJECT *ob, long data)
{
char temp[80];

if(debug_flag)
	{
	fprintf(stdout,\
	"main_form_set_NTSC_fps_button_cb():\n");
	}

dframes_per_second = NTSC_FRAMES_PER_SECOND;
dmilli_seconds_per_frame = 1000.0 / dframes_per_second;

sprintf(temp, "%.4f", dframes_per_second);
fl_set_input(fdui -> main_form_frames_per_second_input_field, temp);

set_frames_per_second_input_field_color();

save_project_vars();

}/* end function main_form_set_NTSC_fps_button_cb */


void main_form_h_size_input_field_cb(FL_OBJECT *ob, long data)
{
char *ptr;

if(debug_flag)
	{
	fprintf(stdout,\
	"main_form_h_size_input_field_cb(): arg none\n");
	}

ptr = (char *)fl_get_input(ob);
if(! ptr) return;

h_size = atoi(ptr);

if( (h_size < 1) || (h_size > 800) )
	{
	fl_show_alert("Illegal h_size", "use 100-720", "", 0);

	h_size = 720;
	fl_set_input(ob, "720");
	}

calculate_and_set_h_margin();

save_project_vars();

/* for animation_templates_form */
fl_set_positioner_xbounds(fdui -> animation_template_form_x_y_positioner, 0, h_size);

}/* end function main_form_h_size_input_field_cb */


void main_form_h_margin_input_field_cb(FL_OBJECT *ob, long data)
{
char *ptr;

if(debug_flag)
	{
	fprintf(stdout,\
	"main_form_h_margin_input_field_cb(): arg none\n");
	}

ptr = (char *)fl_get_input(ob);
if(! ptr) return;

h_margin = atoi(ptr);

calculate_and_set_h_factor();

save_project_vars();

}/* end function main_form_h_margin_input_field_cb */


void main_form_h_factor_input_field_cb(FL_OBJECT *ob, long data)
{
char *ptr;

if(debug_flag)
	{
	fprintf(stdout,\
	"main_form_h_factor_input_field_cb(): arg none\n");
	}

ptr = (char *)fl_get_input(ob);
if(! ptr) return;

h_factor = atof(ptr);

calculate_and_set_h_margin();

save_project_vars();

}/* end function main_form_h_factor_input_field_cb */


void main_form_v_size_input_field_cb(FL_OBJECT *ob, long data)
{
char *ptr;

if(debug_flag)
	{
	fprintf(stdout,\
	"main_form_v_size_input_field_cb(): arg none\n");
	}

ptr = (char *)fl_get_input(ob);
if(! ptr) return;

v_size = atoi(ptr);

if( (v_size < 1) || (v_size > 600) )
	{
	fl_show_alert("Illegal v_size", "use 100-576", "", 0);

	v_size = 576;
	fl_set_input(ob, "576");
	}

calculate_and_set_v_margin();

save_project_vars();

/* for animation_templates_form */
fl_set_positioner_ybounds(fdui -> animation_template_form_x_y_positioner, 0, v_size);

}/* end function main_form_v_size_input_field_cb */


void main_form_v_margin_input_field_cb(FL_OBJECT *ob, long data)
{
char *ptr;

if(debug_flag)
	{
	fprintf(stdout,\
	"main_form_v_margin_input_field_cb(): arg none\n");
	}

ptr = (char *)fl_get_input(ob);
if(! ptr) return;

v_margin = atoi(ptr);

calculate_and_set_v_factor();

save_project_vars();

}/* end function main_form_v_margin_input_field_cb */


void main_form_v_factor_input_field_cb(FL_OBJECT *ob, long data)
{
char *ptr;

if(debug_flag)
	{
	fprintf(stdout,\
	"main_form_v_factor_input_field_cb(): arg none\n");
	}

ptr = (char *)fl_get_input(ob);
if(! ptr) return;

v_factor = atof(ptr);

calculate_and_set_v_margin();

save_project_vars();

}/* end function main_form_v_factor_input_field_cb */


void main_form_text_hold_button_cb(FL_OBJECT *ob, long data)
{
int hold_flag;
int s_line;

if(debug_flag)
	{
	fprintf(stdout,\
	"main_form_text_hold_button(): arg none\n");
	}

hold_flag = fl_get_button(ob);

set_text_hold_flag(selected_start_frame, hold_flag);

if(! recalculate_end_frames() ) return;

show_frames(SHOW_SAME_POSITION);

/* mark new entry in browser */
frame_to_line(selected_start_frame, &s_line);

fl_select_browser_line(fdui -> main_form_browser, s_line);

save_frames();

save_project_vars();

}/* end function main_form_text_hold_button_cb */


void main_form_frame_input_field_cb(FL_OBJECT *ob, long data)
{
char *ptr;
char temp[80];

ptr = (char *)fl_get_input(ob);
if(! ptr) return;

if(debug_flag)
	{
	fprintf(stdout, "main_form_frame_input_field_cb(): ptr=%s\n", ptr);
	}

selected_start_frame = atoi(ptr);

if(! get_sample(selected_start_frame, &selected_start_sample) )
	{
	sprintf(temp, "%d", selected_start_frame);
	fl_show_alert("No audio sample associated with frame", temp, "", 1);

	return;
	}

}/* end function main_form_frame_input_field_cb */


void main_form_current_frame_input_field_cb(FL_OBJECT *ob, long data)
{
char *ptr;

ptr = (char *)fl_get_input(ob);
if(! ptr) return;

if(debug_flag)
	{
	fprintf(stdout, "main_form_current_frame_input_field_cb(): ptr=%s\n", ptr);
	}

}/* end function main_form_current_frame_input_field_cb */


void main_form_frame_now_button_cb(FL_OBJECT *ob, long data)
{
char temp[80];

/* selected_start_frame becomes current_frame */
selected_start_frame = current_frame;

/* get the other stuff that comes with it */
if(! get_sample(selected_start_frame, &selected_start_sample) )
	{
	sprintf(temp, "%d", selected_start_frame);
	fl_show_alert("No audio sample associated with frame", temp, "", 1);

	return;
	}

/* set the frame display */
set_frame_input(selected_start_frame);

}/* end main_form_frame_now_button_cb */


void main_form_audio_mode_button_cb(FL_OBJECT *ob, long data)
{
audio_mode++;

/*
mp3 not implemented yet, the decoding delays are too big,
and not sure about the relation audio frame versus video frame
*/
//if(audio_mode >= AUDIO_MP3 + 1) audio_mode = AUDIO_RAW;
if(audio_mode >= AUDIO_MP3) audio_mode = AUDIO_RAW;

/* set audio mode button state */
if(audio_mode == AUDIO_RAW)
	{
	fl_set_object_label(fdui -> main_form_audio_mode_button, "AUDIO RAW");
	fl_set_object_color(\
		fdui -> main_form_audio_mode_button, FL_COL1, FL_COL1);
	}
else if(audio_mode == AUDIO_WAV)
	{
	fl_set_object_label(fdui -> main_form_audio_mode_button, "AUDIO WAV");
	fl_set_object_color(\
		fdui -> main_form_audio_mode_button, FL_YELLOW, FL_YELLOW);
	}
else if(audio_mode == AUDIO_MP3)
	{
	fl_set_object_label(fdui -> main_form_audio_mode_button, "AUDIO MP3");
	fl_set_object_color(\
		fdui -> main_form_audio_mode_button, FL_DARKORANGE, FL_DARKORANGE);
	}

save_project_vars();
}/* end function main_form_audio_mode_button_cb */


void main_form_preview_button_cb(FL_OBJECT *ob, long data)
{
preview_flag = fl_get_button(ob);

save_project_vars();
}/* end function main_form_preview_button_cb */


void main_form_video_in_preview_button_cb(FL_OBJECT *ob, long data)
{
int a, button;
char temp[4096];
char temp2[4096];
char temp3[4096];
FILE *fptr, *pptr;
char *ptr;

button = fl_get_button(ob);

if(button)
	{
	/* set preview mode */
	fl_set_button(fdui -> main_form_preview_button, 1);
	preview_flag = 1;

	/* start file browser */
	sprintf(temp, "/video/vie/%s/", selected_project);
	strcpy(temp2, "*.[mt][op][cv]");
	sprintf(temp3, "%s.mpv", selected_project); 
	ptr = (char *)fl_show_fselector("SELECT A FILE", temp, temp2, temp3);
	if(! ptr)
		{
		fl_set_button(ob, 0);
		video_in_preview_flag = 0;
		return;
		}

	/* test if file exists (nothing selected), or directory changed, selected no longer there */
	fptr = fopen(ptr, "r");
	if(fptr) fclose(fptr);
	else
		{
		fl_show_alert("No such file", ptr, "default may not exist, did you select an entry?", 0); 

		fl_set_button(ob, 0);
		video_in_preview_flag = 0;
		return;
		}

	/* test if a .toc file was selected */
	a = strlen(ptr);
	strcpy(temp, ptr + a - 4);

	if(strcmp(temp, ".toc") == 0)
		{
		/* use .toc file */
		/* ptr = ptr; */
		}
	else /* test if toc file exists, if not ask user to create it */
		{
		/* append .toc to filename */
		sprintf(temp, "%s.toc", ptr);	
		fptr = fopen(temp, "r");
		
		if(fptr) /* exists */
			{
			fclose(fptr);
			ptr = temp;
			}
		else
			{
			if(! fl_show_question(\
			"No toc file found\n\
video will only work when a table of contents file is present\n\
Create and use toc? (will take a long time)", 1) ) /* mouse on yes */
				{
				fl_set_button(ob, 0);
				video_in_preview_flag = 0;
				return;
				}	

			/* create .toc file */
			sprintf(temp2, "mpeg3toc %s %s", ptr, temp);

			XSync(fl_get_display(), 0);

			sprintf(temp3, "Executing: %s, please wait.", temp2);

			fl_show_oneliner(temp3, 25, 150);

			XSync(fl_get_display(), 0);

			if(debug_flag)
				{
				fprintf(stdout, "executing %s\n", temp2);
				}

			pptr = popen(temp2, "w");
			pclose(pptr);

			fl_hide_oneliner();
	
			ptr = temp;
			}
		} /* end if not .toc */

	if(debug_flag)
		{
		fprintf(stdout, "main_form_video_in_preview_button_cb(): using %s\n", ptr);
		}

	if(video_filename) free(video_filename);
	video_filename = strsave(ptr);
	if(! video_filename)
		{
		fprintf(stderr, "main_form_video_in_preview_button_cb(): strsave(ptr) failed\n");	

		fl_set_button(ob, 0);
		video_in_preview_flag = 0;
		return;
		}

	if(debug_flag)
		{
		fprintf(stdout,\
		"xste_main_form_cb.c main_form_video_in_preview_button_cb(): selected=%s\n", ptr);
		}

	video_have_warning_flag = 0;

	/* force video thread to re-init */
	video_file_init_flag = 0;
	} /* end if video_in_preview_flag */

video_in_preview_flag = button;

save_project_vars();

}/* end function main_form_video_in_preview_button_cb */


void main_form_search_input_field_cb(FL_OBJECT *ob, long data)
{
main_form_reset_last_found_line();
}/* end function main_form_search_input_field_cb */


void main_form_search_button_cb(FL_OBJECT *ob, long data)
{
search_and_select_next_frame();
}/* end function main_form_search_button_cb */


void main_form_search_case_button_cb(FL_OBJECT *ob, long data)
{
main_form_reset_last_found_line();
}/* end function main_form_search_case_button_cb */


void main_form_video_display_button_cb(FL_OBJECT *ob, long data)
{
show_video_flag = fl_get_button(ob);

save_project_vars();
}/* end function void_main_form_video_display_button_cb */


void main_form_next_red_button_cb(FL_OBJECT *ob, long data)
{
find_next_too_long_line();

//show_frames(SHOW_SAME_POSITION);
}/* end function main_form_next_red_button_cb */


void main_form_volume_slider_cb(FL_OBJECT *ob, long data)
{
selected_volume = 100 * fl_get_slider_value(ob);

set_volume(selected_volume);

save_project_vars();
}/* end function main_form_volume_slider_cb */


void main_form_erase_all_empty_lines_button_cb(FL_OBJECT *ob, long data)
{
int s_line;

if(! fl_show_question("ARE YOU VERY SURE?", 0) ) /* mouse on NO */
	{
	return;
	}
if(! fl_show_question("A G A I N  ARE YOU VERY SURE?", 0) ) /* mouse on NO */
	{
	return;
	}
	
delete_all_empty_lines();

show_frames(SHOW_SAME_POSITION);

/* mark new entry in browser */
frame_to_line(selected_start_frame, &s_line);

fl_select_browser_line(fdui -> main_form_browser, s_line);

save_frames();

save_project_vars();

}/* end function main_form_erase_all_empty_lines_button_cb */


void main_form_edit_frame_list_button_cb(FL_OBJECT *ob, long data)
{

edit_frame_list();
}/* end function main_form_edit_frame_list_button_cb */


void main_form_spellcheck_input_button_cb(FL_OBJECT *ob, long data)
{
spellcheck_input_flag = fl_get_button(ob);

save_general_settings();
}/* end function main_form_spellcheck_input_button_cb */


void main_form_delete_button_cb(FL_OBJECT *ob, long data)
{
int total_lines;
int s_line;
int start_frame;
int end_frame;

total_lines = fl_get_browser_maxline(fdui -> main_form_browser);
for(s_line = 1; s_line <= total_lines; s_line++)
	{
	if(fl_isselected_browser_line(fdui -> main_form_browser, s_line) )
		{
		main_form_modified_flag = 1;

		line_to_frame(s_line, &start_frame, &end_frame);

		/* finally delete structure entry */
		delete_frame(start_frame);

		/* write modified structure list to disk */
		save_frames();

		break;
		}/* end selected line */
	}/* end for all lines */

show_frames(SHOW_SAME_POSITION);
}/* end function main_form_delete_button_cb */


void main_form_clear_button_cb(FL_OBJECT *ob, long data)
{
if(debug_flag)
	{
	fprintf(stdout, "main_form_clear_button_cb(): arg none\n");
	}

fl_set_input(fdui -> main_form_text_input_field, "");

XSync(fl_get_display(), 0);

/*main_form_modified_flag = 1;*/
}/* end function main_form_clear_button_cb */


void main_form_add_button_cb(FL_OBJECT *ob, long data)
{
int a;
int s_line;
char *ptr;
int force_duration;
char temp[4096];
font_desc_t *pfd;
struct frame *pa;
int new_frame_flag;
 
/* test if spellchecker requested on input field */
if(spellcheck_input_flag)
	{
	if(! spellcheck_input_field() ) return;
	}

/* get text from input field */
ptr = (char *)fl_get_input(fdui -> main_form_text_input_field);
if(! ptr) return;

if( strlen(ptr) >= TEXT_ARRAY_SIZE - 1)
	{
	sprintf(temp, "%d", TEXT_ARRAY_SIZE - 1);

	fl_show_alert("Clipping line, max text size is", temp, "bytes", 0);
	}

/* limit input line length, max browser line size is 1024 */
ptr[TEXT_ARRAY_SIZE - 1] = 0;

/* prevent empty entries */
if(ptr[0] == 0)
	{
	/* mouse on NO */
	if(! fl_show_question("Trying to add an empty line! Are you sure?", 0) )
		{
		return;
		}
	}

/* calculate the new end_frame */
/* get length frame depending on text */
if(! text_to_frames(ptr, &a) ) return;
selected_end_frame = selected_start_frame + a;

/* lookup selected_start_sample for selected_start_frame in database */
if(! get_sample(selected_start_frame, &selected_start_sample) ) return;

/* lookup selected_end_sample for selected_end_frame in database */
if(! get_sample(selected_end_frame, &selected_end_sample) ) return;

/* get force duration from input field */
force_duration = fl_get_button(fdui -> main_form_text_hold_button);

/* add the entry to the list */
pa = add_frame_in_position(\
	ptr, selected_start_frame, selected_end_frame, force_duration, &new_frame_flag);
if(! pa)
	{
	fprintf(stderr, "xste: main_form_add_button_cb(): could not lookup frame %s", temp);

	fl_show_alert("main_form_add_button_cb()", "could not lookup frame", temp, 0);

	return;
	} /* end if frame already exists */

//pa -> status &= ~RLE_ERROR;

//if(! new_frame_flag)
//	{
	/* leave font parameters as they are */
//	fprintf(stdout, "WAS NO CHANGE (no new_frame_flag)\n");

//	}
//else
//	{
	/* frame did not yet exist, a new structure entry was created */
	if(hold_text_properties_flag)
		{
//fprintf(stdout, "WAS TEXT HOLD\n");

		/*
		use last font,
		we leave the font parameters as they are, just changed the text.
		*/

		/* these hold the parameters of the last clicked entry in the browser */ 
		pa -> pattern = text_in_body;
		pa -> background = text_in_background;
		pa -> emphasis1 = text_in_outline1;
		pa -> emphasis2 = text_in_outline2;
     
		pa -> pattern_contrast = text_in_body_contrast;
		pa -> background_contrast = text_in_background_contrast;
		pa -> emphasis1_contrast = text_in_outline1_contrast;
		pa -> emphasis2_contrast = text_in_outline2_contrast;
    
		pa -> symbols = text_in_symbols;
		pa -> fontsize = text_in_fontsize;
		pa -> iso_extension = text_in_iso_extension;

		pa -> espace = text_in_extra_character_space;
		pa -> vespace = text_in_extra_character_vspace;

		pa -> outline_thickness = text_in_outline_thickness;
		pa -> blur_radius = text_in_blur_radius;

		pa -> fontname = strsave(text_in_fontname);
		if(! pa -> fontname)		
			{
			fprintf(stderr, "xste: main_form_add_button_cb(): could not allocate space for pa -> fontname\n");		

			return;
			}

		pfd = add_font(\
			text_in_fontname,\
			text_in_symbols,\
			text_in_fontsize, text_in_iso_extension,\
			text_in_outline_thickness, text_in_blur_radius, text_in_extra_character_space,
			text_in_extra_character_vspace);	
		if(! pfd)
			{
			fprintf(stderr,\
			"xste: main_form_add_button_cb(): could not add font for frame %d\n", selected_start_frame);

			return;
			}

		pa -> pfd = pfd;

		if(text_in_center_flag)	pa -> status |= CENTER_TEXT;
		else pa -> status &= ~CENTER_TEXT;

		if(text_in_anti_alias_flag) pa -> status |= DVD_ANTI_ALIAS;
		else pa -> status &= ~DVD_ANTI_ALIAS;
	
		if(text_in_no_kerning_flag) pa -> status |= NO_KERNING;
		else pa -> status &= ~NO_KERNING;

		}
	else /* text properties hold flag not set */
		{	
		/* use default font */
//fprintf(stdout, "WAS USING DEFAULT FONT PARAMETERS\n");

		pa -> background = default_subtitle_background_color;
		pa -> pattern = default_subtitle_body_color;
		pa -> emphasis1 = default_subtitle_outline1_color;
		pa -> emphasis2 = default_subtitle_outline2_color;

		pa -> background_contrast = default_subtitle_background_contrast;
		pa -> pattern_contrast = default_subtitle_body_contrast;
		pa -> emphasis1_contrast = default_subtitle_outline1_contrast;
		pa -> emphasis2_contrast = default_subtitle_outline2_contrast;

		if(pa -> fontname) free(pa -> fontname);
		pa -> fontname = strsave(default_subtitle_font_name);
		if(! pa -> fontname)
			{
			fprintf(stderr, "xste: main_form_add_button_cb(): could not allocate space for pa -> fontname\n");
	
			return;
			}

		pa -> symbols = default_subtitle_font_symbols;
		pa -> fontsize = default_subtitle_font_size;
		pa -> iso_extension = default_subtitle_iso_extention;
		pa -> outline_thickness = default_subtitle_outline_thickness;
		pa -> blur_radius = default_subtitle_blur_radius;
		pa -> espace = default_subtitle_extra_character_space;
		pa -> vespace = default_subtitle_extra_character_vspace;
  
		pfd = add_font(\
			default_subtitle_font_name,\
			default_subtitle_font_symbols,\
			default_subtitle_font_size, default_subtitle_iso_extention,\
			default_subtitle_outline_thickness, default_subtitle_blur_radius,\
			default_subtitle_extra_character_space, default_subtitle_extra_character_vspace);
		if(! pfd)
			{
			fprintf(stderr,\
			"xste: main_form_add_button_cb(): could not add font for frame %d\n", selected_start_frame);

			return;
			}

		pa -> pfd = pfd;

		/* center text if global request */
		if(default_subtitle_center_flag) pa -> status |= CENTER_TEXT;
		else pa -> status &= ~CENTER_TEXT;

		if(default_subtitle_anti_alias_flag) pa -> status |= DVD_ANTI_ALIAS;
		else pa -> status &= ~DVD_ANTI_ALIAS;

		if(default_subtitle_no_kerning_flag) pa -> status |= NO_KERNING;
		else pa -> status &= ~NO_KERNING;

		} /* end if ! hold_text_properties_flag */	

//	} /* end if new_frame_flag */

/* in case submux-dvd output format selected in export form, test if bitmap rle to spec */
if(output_format == OUTPUT_FORMAT_SUBMUX_DVD)
	{
	/* only test if a real text */
	if(strlen(pa -> text) != 0)
		{				
		/* enter create_bitmap_file test mode, in this mode test.bmp is written */
		pa -> status |= TEST_BITMAP;

		/*
		sets pa -> width, pa -> height, pa -> xpos, pa -> ypos,
		and makes a bitmap file	from pa -> text,
		and subtitle_current_font_descriptor.
		*/
		if(! create_bitmap_file(pa) )
			{
			fl_show_alert(\
			"main_form_add_button_cb():", "could not create test bitmap", "text may not be DVD compatible", 0);

			return;
			}

		/* leave create_bitmap_file test mode */
		pa -> status &=	~TEST_BITMAP;

		/* test the bmp file */
		sprintf(temp, "%s/%s/%s/test.bmp", home_dir, selected_data_directory, selected_project);

/*
		int test_bitmap(\
		char *file_name,\
		int xpos, int ypos, int xsize, int ysize,\
		int bg, int fg, int emph1, int emph2,\
		int bgcol, int fgcol, int emph1col, int emph2col)
*/
		if(! test_bitmap(temp, pa -> xpos, pa -> ypos, pa -> width, pa -> height,\
		pa -> background_contrast, pa -> pattern_contrast, pa -> emphasis1_contrast, pa -> emphasis2_contrast,\
		pa -> background, pa -> pattern, pa -> emphasis1, pa -> emphasis2) )
			{
			fl_show_alert("bitmap", pa -> name, "not DVD compatible, try reformatting text.", 0); 

			/* select browser line here to make it easier to find */

			pa -> status |= RLE_ERROR;
			}
		else
			{
			pa -> status &= ~RLE_ERROR;
			}

		/* remove the test.bmp file */
		unlink(temp);

		/*
		maintain visible indication that line is selected with single
		click
		*/
		fl_select_browser_line(fdui -> main_form_browser, pa -> browser_line);
		} /* end if strlen text */

	} /* end if OUTPUT_FORMAT_SUBMUX_DVD */

/* show X11 window with formatted text if preview enabled */
if(preview_flag)
	{
	new_preview_flag = 1;
	}

save_frames();

/* this must happen last, as pa and pa -> pfd must be set for calculate_text_lines() !! */
show_frames(SHOW_SAME_POSITION);

/* mark new entry in browser */
frame_to_line(selected_start_frame, &s_line);

fl_select_browser_line(fdui -> main_form_browser, s_line);

main_form_modified_flag = 1;
}/* end function main_form_add_button_cb */


void main_form_play_part_cb(FL_OBJECT *ob, long data)
{
int a, b;

if(debug_flag)
	{
	fprintf(stdout, "main_form_play_part_cb(): data=%ld\n", data);
	}

/*
save selected frame in case we do not have associated audio samples
we restore selected frame from this.
*/
b = selected_start_frame;

a = (int)data;
selected_start_frame += a;

if(selected_start_frame < 0) selected_start_frame = 0;
if(selected_start_frame > INT_MAX) selected_start_frame = INT_MAX;

if(! get_sample(selected_start_frame, &selected_start_sample) )
	{
	fl_show_alert(\
		"No more audio samples", "associated with this frame", "", 1); 

	/* restore old value */
	selected_start_frame = b;
	return;
	}

/* set the frame display */
set_frame_input(selected_start_frame);

}/* end function main_form_play_part_cb */


void main_form_mem1_button_cb(FL_OBJECT *ob, long data)
{
if(debug_flag)
	{
	fprintf(stdout, "main_form_mem1_button_cb():\n");
	}

/* set the global */
frame_memory_1 = current_frame;

/* set the labels in the buttons */
set_memory_button_labels();

/* save frame_memory_1 to disk in project dir */
if(! save_project_vars() ) return;

/* NOTE: if save fails, use frame_memory_1 */

}/* end function main_form_mem1_button_cb */


void main_form_rcl_button_cb(FL_OBJECT *ob, long data)
{
if(debug_flag)
	{
	fprintf(stdout, "main_form_rcl_button_cb():\n");
	}

/* get frame_memory_1 from project directory */
//if(! load_project_vars() ) {}; /* sets defaults if error (new project) */

/* nothing was stored */
if(frame_memory_1 == -1) return;

/* selected_start_frame becomes current_frame */
selected_start_frame = frame_memory_1;

select_frame_list_entry();

}/* end function main_form_rcl_button_cb */


void main_form_snap_button_cb(FL_OBJECT *ob, long data)
{
if(debug_flag)
	{
	fprintf(stdout, "main_form_snap_button_cb():\n");
	}

/* copy selected_start_frame to current_frame */
selected_start_frame = current_frame;

/* find the closest existing entry that was automatically detected */
snap_to_frame(&selected_start_frame);

select_frame_list_entry();

}/* end function main_form_snap_button_cb */


void main_form_synthesize_switch_cb(FL_OBJECT *ob, long data)
{
FILE *fptr, *pptr;
char temp[4096];

synthesize_flag = fl_get_button(ob);

if(synthesize_flag)
	{
	/* test if synthesize script present */
	sprintf(temp, "%s/%s/%s/xste_synthesize", home_dir, selected_data_directory, selected_project);
	fptr = fopen(temp, "r");
	if(fptr)
		{
		fclose(fptr);
		}
	else /* no script found */
		{
		/* test if xste_synthesize script installed in ~/.xste/ */
		sprintf(temp, "%s/.xste/xste_synthesize", home_dir);
		fptr = fopen(temp, "r");
		if(fptr)
			{	 
			fclose(fptr);
	
			sprintf(temp,\
			"Script %s/%s/%s/xste_synthesize not found,\ncopy it from %s/.xste/xste_synthesize ?",\
			home_dir, selected_data_directory, selected_project, home_dir);

			if(fl_show_question(temp, 1) )
				{
				sprintf(temp, "cp %s/.xste/xste_synthesize %s/%s/%s/ 2>/dev/zero",\
				home_dir, home_dir, selected_data_directory, selected_project);

				pptr = popen(temp, "r");
				pclose(pptr);
				}
			else
				{
				fl_set_button(ob, 0);

				synthesize_flag = 0;
				}
			}
		else
			{
			sprintf(temp, "%s/.xste/", home_dir);

			fl_show_alert(\
			"You need to get xste_synthesize from the distribution and\n\
modify it for path filenames and languages then copy it to",\
			temp,\
			"",\
			0); 

			fl_set_button(ob, 0);

			synthesize_flag = 0;
			}
		}

	} /* end if synthesize_flag */

if(synthesize_flag)
	{
	fl_set_object_label(fdui -> main_form_synthesize_switch, "synthesize");
	}
else
	{
	fl_set_object_label(fdui -> main_form_synthesize_switch, "synthesize off");
	}

save_project_vars();
} /* end function main_form_synthesize_switch_cb */


void main_form_output_switch_cb(FL_OBJECT *ob, long data)
{

fl_set_object_color(ob, FL_YELLOW, FL_YELLOW);
fl_set_object_label(ob, "WAIT");
XSync(fl_get_display(), 0);

/* 
write new-timed.txt output file for DVB-s subtitler viplay / suti.
*/
/* signal to bitmap.c what color tables to use */
switch(output_format)
	{
	case OUTPUT_FORMAT_PPML_FOR_SUBTITLER_YUV:
		/* 
		write .ppml output file (Panteltje's Picture Manipulation language)
		for use with subtitler-yuv with mjpegtools.
		*/
		/* error message in function */
		if(! write_ppml_for_subtitler_yuv_list() )
			{
			fl_set_object_color(ob, FL_COL1, FL_YELLOW);
			fl_set_object_label(ob, "OUTPUT");

			return;
			}
		break;
	case OUTPUT_FORMAT_PPML:
		/* 
		write .ppml output file (Panteltje's Picture Manipulation language)
		for use with subtitler in Linux transcode.
		*/
		if(! write_ppml_list() )
			{
			fl_set_object_color(ob, FL_COL1, FL_YELLOW);
			fl_set_object_label(ob, "OUTPUT");

			return;
			}
		break;
	case OUTPUT_FORMAT_SUTI:
		if(! write_suti_list() )
			{
			fl_set_object_color(ob, FL_COL1, FL_YELLOW);
			fl_set_object_label(ob, "OUTPUT");

			return;
			}
		break;
	case OUTPUT_FORMAT_SSA:
		/* write .ssa Substation Alpha output file if requested */
		if(! write_ssa_list() )
			{
			fl_set_object_color(ob, FL_COL1, FL_YELLOW);
			fl_set_object_label(ob, "OUTPUT");

			return;
			}
		break;
	case OUTPUT_FORMAT_VPLAYER:
		/* write a vplayer format list if requested */
		if(! write_vplayer_list() )
			{
			fl_set_object_color(ob, FL_COL1, FL_YELLOW);
			fl_set_object_label(ob, "OUTPUT");

			return;
			}
		break;
	case OUTPUT_FORMAT_SUBMUX: /* CVD, SVCD */
		if(! write_submux_list() ) return;
		break;
	case OUTPUT_FORMAT_SUBMUX_DVD: /* DVD */
		if(! write_submux_dvd_list() )
			{
			fl_set_object_color(ob, FL_COL1, FL_YELLOW);
			fl_set_object_label(ob, "OUTPUT");

			return;
			}
		if(! write_ifo_color_table() )
			{
			fl_set_object_color(ob, FL_COL1, FL_YELLOW);
			fl_set_object_label(ob, "OUTPUT");

			return;
			}
		break;
	case OUTPUT_FORMAT_MICRO_DVD_PLAYER:
		if(! write_microdvdplayer_list() )
			{
			fl_set_object_color(ob, FL_COL1, FL_YELLOW);
			fl_set_object_label(ob, "OUTPUT");

			return;
			}
		break;
	case OUTPUT_FORMAT_I_AUTHOR:
		if(! write_iauthor_list() )
			{
			fl_set_object_color(ob, FL_COL1, FL_YELLOW);
			fl_set_object_label(ob, "OUTPUT");

			return;
			}
		break;
	case OUTPUT_FORMAT_EBU_STL:
		/* load settings */
		if(! load_ebu_gsi() )
			{
			fl_show_alert(\
			"No EBU information was set",\
			"Use FILE->EXPORT SETTINGS->EBU STL: SETTINGS BUTTON",
			"and fill in the form", 0); 

			fl_set_object_color(ob, FL_COL1, FL_YELLOW);
			fl_set_object_label(ob, "OUTPUT");

			return;
			}

		/* calculated values need to be set */
		if(! xste_frames_to_ebu_gsi_tti() )
			{
			fl_set_object_color(ob, FL_COL1, FL_YELLOW);
			fl_set_object_label(ob, "OUTPUT");

			return;
			}

		/* write the .STL file */
		if(! write_ebu_stl_list() )
			{
			fl_set_object_color(ob, FL_COL1, FL_YELLOW);
			fl_set_object_label(ob, "OUTPUT");

			return;
			}
  		break;
	case OUTPUT_FORMAT_SRT:
		if(! write_srt_list() )
			{
			fl_set_object_color(ob, FL_COL1, FL_YELLOW);
			fl_set_object_label(ob, "OUTPUT");

			return;
			}
		break;
	} /* end switch formats */

if(enable_chapter_list_output_flag)
	{
	if(!write_chapter_list() )
		{
//		fl_show_alert("Could not write chapter list", "", "", 0);
		}
	}

if(synthesize_flag)
	{
	write_synthesized_wave_files();

	combine_synthesized_wave_files();
	}


fl_set_object_color(ob, FL_COL1, FL_YELLOW);
fl_set_object_label(ob, "OUTPUT");

} /* end function main_form_output_switch_cb */


void main_form_move_from_switch_cb(FL_OBJECT *ob, long data)
{

/* alert user, mouse on NO */
if(! fl_show_question("This will alter ALL your frame numbers\nfrom this point upwards!\n\nOK?", 0) )
	{
	return;
	}

new_start_frame = selected_start_frame;
org_start_frame = frame_list_browser_clicked_frame;

/* reset to no length manipulation */
dnew_old_frame_ratio = 1.0;

if(! move_from_frame_list(org_start_frame, new_start_frame) ) return;

//recalculate_end_frames();

show_frames(SHOW_SAME_POSITION);

if(! fl_show_question(\
"Press NO to exit xste, else changes are permanent", 0) )
	{
	/*
	prevent user from pressing ANY more buttons, as that will save frame
	list (with the new numbers).
	Restarting xste will reload the old numbers.	
	*/
	exit(1);
	}

/* changes permanent, only one time backup available */
save_frames();

return;
} /* end function main_form_main_form_move_from_switch_cb */


void main_form_auto_switch_cb(FL_OBJECT *ob, long data)
{
char pathfilename[4096];
char temp[4096];
FILE *fptr;

/* construct pathfilename */
sprintf(pathfilename, "%s/%s/%s/%s",\
home_dir, selected_data_directory, selected_project, data_file_name);

/* 
test if data_file_name already exists, doing this will irreversibly modify it
*/
fptr = fopen(pathfilename, "r");
if(fptr)
	{
	fclose(fptr);

	sprintf(temp, "File %s exists, lose all your work?", pathfilename);
	if(! fl_show_question(temp, 0) )/* mouse on NO */ return;

	if(! fl_show_question("ARE YOU VERY SURE?", 0) )/* mouse on NO */ return;
	}

/* construct pathfilename sound file */
if(audio_mode == AUDIO_RAW)
	{
	sprintf(pathfilename, "%s/%s/%s/sound.raw",\
	home_dir, selected_data_directory, selected_project);

	/* use for low signal in audio */
/*
	find_start_points_in_raw_file(\
	pathfilename, selected_device, 20, 500); // window, milli seconds
*/

	/* find start points in raw sound file, and update data_file_name */
	find_start_points_in_raw_file(\
	pathfilename, selected_device, 60, 500); // window, milli seconds
	}
else if(audio_mode == AUDIO_WAV)
	{
	sprintf(pathfilename, "%s/%s/%s/sound.wav",\
	home_dir, selected_data_directory, selected_project);

	/* use for low signal in audio */
/*
	find_start_points_in_wave_file(\
	pathfilename, selected_device, 20, 500); // window, milli seconds
*/

	/* find start points in raw sound file, and update data_file_name */
	find_start_points_in_wave_file(\
	pathfilename, selected_device, 60, 500); // window, milli seconds
	}
else if(audio_mode == AUDIO_MP3)
	{
	/* not yet */
	return;
	}

/* save updated frame list  */
save_frames();

/* show the frame list */
show_frames(SHOW_BOTTOM);

} /* end function main_form_auto_switch_cb */


void main_form_file_menu_cb(FL_OBJECT *ob, long data)
{
int some_form_open_flag;
char *ptr;
char temp[4096];
char temp2[4096];

main_form_file_menu_selection = fl_get_menu(ob);

/* if nothing selected, but mouse in menu, no action */
if(main_form_file_menu_selection <= 0) return;

if(main_form_file_menu_selection == PROJECTS) 
	{
	show_project_form();

	return;
	}
else if(main_form_file_menu_selection == IMPORT_PPML_FILE) 
	{
	/* alert user, mouse on NO */
	if(! fl_show_question("This will overwrite all your work, OK?", 0) ) return;

	if(! fl_show_question("ARE YOU VERY SURE?", 0) )/* mouse on NO */ return;

	/* start file browser */
	sprintf(temp, "%s/.subtitles/%s/", home_dir, selected_project);
	sprintf(temp2, "%s.ppml", selected_project);
	ptr = \
	(char *)fl_show_fselector("SELECT A FILE", temp, "*.ppml", temp2);
	if(! ptr) return;

	if(debug_flag)
		{
		fprintf(stdout,\
		"xste_main_form_cb.c IMPORT_PPML_FILE(): selected=%s\n", ptr);
		}

	if(! load_ppml_file(ptr) ) return;

	recalculate_frame_list();

	recalculate_end_frames();

	if(! fl_show_question(\
	"Press NO to exit xste, else changes are permanent", 0) )
		{
		/*
		prevent user from pressing ANY more buttons, as that will save frame
		list (with the new numbers).
		Restarting xste will reload the old numbers.	
		*/
		exit(1);
		}

	save_frames();

	show_frames(SHOW_SAME_POSITION);

	return;
	}
else if(main_form_file_menu_selection == IMPORT_SUTI_FILE) 
	{
	/* alert user, mouse on NO */
	if(! fl_show_question("This will overwrite all your work, OK?", 0) ) return;

	if(! fl_show_question("ARE YOU VERY SURE?", 0) )/* mouse on NO */ return;

	/* start file browser */
	sprintf(temp, "%s/.subtitles/%s/", home_dir, selected_project);
	ptr = \
	(char *)fl_show_fselector("SELECT A FILE", temp, "*.txt", "new-timed.txt");
	if(! ptr) return;

	if(debug_flag)
		{
		fprintf(stdout,\
		"xste_main_form_cb.c IMPORT_SUTI_FILE(): selected=%s\n", ptr);
		}

	if(! load_suti_file(ptr) ) return;

	recalculate_frame_list();

	recalculate_end_frames();

	if(! fl_show_question(\
	"Press NO to exit xste, else changes are permanent", 0) )
		{
		/*
		prevent user from pressing ANY more buttons, as that will save frame
		list (with the new numbers).
		Restarting xste will reload the old numbers.	
		*/
		exit(1);
		}

	save_frames();

	show_frames(SHOW_SAME_POSITION);

	return;
	}
else if(main_form_file_menu_selection == IMPORT_SSA_FILE) 
	{
	/* alert user, mouse on NO */
	if(! fl_show_question("This will overwrite all your work, OK?", 0) ) return;

	if(! fl_show_question("ARE YOU VERY SURE?", 0) )/* mouse on NO */ return;

	/* start file browser */
	sprintf(temp, "%s/.subtitles/%s/", home_dir, selected_project);
	sprintf(temp2, "%s.ssa", selected_project);
	ptr = \
	(char *)fl_show_fselector("SELECT A FILE", temp, "*.ssa", temp2);
	if(! ptr) return;

	if(debug_flag)
		{
		fprintf(stdout,\
		"xste_main_form_cb.c IMPORT_SSA_FILE(): selected=%s\n", ptr);
		}

	if(! load_ssa_file(ptr) ) return;

	recalculate_frame_list();

	recalculate_end_frames();

	if(! fl_show_question(\
	"Press NO to exit xste, else changes are permanent", 0) )
		{
		/*
		prevent user from pressing ANY more buttons, as that will save frame
		list (with the new numbers).
		Restarting xste will reload the old numbers.	
		*/
		exit(1);
		}

	save_frames();

	show_frames(SHOW_SAME_POSITION);

	return;
	}
else if(main_form_file_menu_selection == IMPORT_NANDUB_FILE) 
	{
	/* alert user, mouse on NO */
	if(! fl_show_question("This will overwrite all your work, OK?", 0) ) return;

	if(! fl_show_question("ARE YOU VERY SURE?", 0) )/* mouse on NO */ return;

	/* start file browser */
	sprintf(temp, "%s/.subtitles/%s/", home_dir, selected_project);
	sprintf(temp2, "%s.sub", selected_project);
	ptr = \
	(char *)fl_show_fselector("SELECT A FILE", temp, "*.sub", temp2);
	if(! ptr) return;

	if(debug_flag)
		{
		fprintf(stdout,\
		"xste_main_form_cb.c IMPORT_SUBVIEWER_FILE(): selected=%s\n", ptr);
		}

	if(! load_nandub_file(ptr) ) return;

	recalculate_frame_list();

	recalculate_end_frames();

	if(! fl_show_question(\
	"Press NO to exit xste, else changes are permanent", 0) )
		{
		/*
		prevent user from pressing ANY more buttons, as that will save frame
		list (with the new numbers).
		Restarting xste will reload the old numbers.	
		*/
		exit(1);
		}

	save_frames();

	show_frames(SHOW_SAME_POSITION);

	return;
	}
else if(main_form_file_menu_selection == IMPORT_SRT_FILE) 
	{
	/* alert user, mouse on NO */
	if(! fl_show_question("This will overwrite all your work, OK?", 0) ) return;

	if(! fl_show_question("ARE YOU VERY SURE?", 0) )/* mouse on NO */ return;

	/* start file browser */
	sprintf(temp, "%s/.subtitles/%s/", home_dir, selected_project);
	sprintf(temp2, "%s.srt", selected_project);
	ptr = \
	(char *)fl_show_fselector("SELECT A FILE", temp, "*.srt", temp2);
	if(! ptr) return;

	if(debug_flag)
		{
		fprintf(stdout,\
		"xste_main_form_cb.c IMPORT_SRT_FILE(): selected=%s\n", ptr);
		}

	if(! load_srt_file(ptr) ) return;

	recalculate_frame_list();

	recalculate_end_frames();

	if(! fl_show_question(\
	"Press NO to exit xste, else changes are permanent", 0) )
		{
		/*
		prevent user from pressing ANY more buttons, as that will save frame
		list (with the new numbers).
		Restarting xste will reload the old numbers.	
		*/
		exit(1);
		}

	save_frames();

	show_frames(SHOW_SAME_POSITION);

	return;
	}
else if(main_form_file_menu_selection == EXPORT_SETTINGS) 
	{
	show_export_form();

	return;
	}
else if(main_form_file_menu_selection == ANIMATION_TEMPLATES) 
	{
	show_animation_template_form();

	return;
	}
else if(main_form_file_menu_selection == XSTE_FONTS) 
	{
	show_fonts_form();

	return;
	}
else if(main_form_file_menu_selection == GLOBAL_SUBTITLE_PALETTE)
	{
	show_global_subtitle_palette_form();

	return;
	}
else if(main_form_file_menu_selection == DEFAULT_SUBTITLE_FONT_SETTINGS) 
	{
	show_subtitle_font_form();

	return;
	}
else if(main_form_file_menu_selection == FORCE_DEFAULT_TO_ALL_SUBTITLES) 
	{
	force_default_font_parameters_to_all_frames();

	return;
	}
else if(main_form_file_menu_selection == SPECIAL_CHARACTERS) 
	{
	show_special_characters_form();

	return;
	}
else if(main_form_file_menu_selection == EXIT) 
	{
	/* test if any forms are open */
	if(! test_for_open_forms(&some_form_open_flag) ) return;
	if(some_form_open_flag) return;

	exit(0);
	}

} /* end function main_form_file_menu_cb */


void main_form_fontsize_up_button_cb(FL_OBJECT *ob, long data)
{
int size;

fontsizes_modified_flag = 1;
if(main_form_fontsize < 5) main_form_fontsize++;
size = integer_to_fontsize(main_form_fontsize);

fl_set_browser_fontsize(fdui -> main_form_browser, size);
fl_set_object_lsize(fdui -> main_form_text_input_field, size);
//fl_set_object_lsize(fdui -> main_form_frame_input_field, size);
//fl_set_object_lsize(fdui -> main_form_current_frame_input_field, size);
//fl_set_object_lsize(fdui -> main_form_text_duration_input_field, size);
//fl_set_object_lsize(fdui -> main_form_frames_per_second_input_field, size);

save_fontsizes();
}/* end function main_form_fontsize_up_button_cb */


void main_form_fontsize_down_button_cb(FL_OBJECT *ob, long data)
{
int size;

fontsizes_modified_flag = 1;
if(main_form_fontsize > 0) main_form_fontsize--;
size = integer_to_fontsize(main_form_fontsize);

fl_set_browser_fontsize(fdui -> main_form_browser, size);
fl_set_object_lsize(fdui -> main_form_text_input_field, size);
//fl_set_object_lsize(fdui -> main_form_frame_input_field, size);
//fl_set_object_lsize(fdui -> main_form_current_frame_input_field, size);
//fl_set_object_lsize(fdui -> main_form_text_duration_input_field, size);
//fl_set_object_lsize(fdui -> main_form_frames_per_second_input_field, size);

save_fontsizes();
}/* end function main_form_fontsize_down_button_cb */


void main_form_exit_switch_cb(FL_OBJECT *ob, long data)
{
int some_form_open_flag;

/* test if any forms are open */
if(! test_for_open_forms(&some_form_open_flag) ) return;
if(some_form_open_flag) return;

exit(1);
} /* end function main_form_exit_switch_cb */


int set_force_duration_button(int frame)
{
int hold_flag;

if(debug_flag)
	{
	fprintf(stdout, "set_force_duration_input(): arg frame=%d\n", frame);
	}

get_text_hold_flag(frame, &hold_flag);

fl_set_button(fdui -> main_form_text_hold_button, hold_flag);

return 1;
} /* end function set_force_duration_button */


void set_frames_per_second_input_field_color()
{
if(debug_flag)
	{
	printf("void set_frames_per_second_input_field_color(): arg none\n");
	}

if(dframes_per_second == PAL_FRAMES_PER_SECOND)
	{
	fl_set_object_color(\
	fdui -> main_form_frames_per_second_input_field,\
	FL_COL1, FL_COL1);
	}
else if(dframes_per_second == NTSC_FRAMES_PER_SECOND)
	{
	fl_set_object_color(\
	fdui -> main_form_frames_per_second_input_field,\
	FL_DARKORANGE, FL_DARKORANGE);
	}
else 
	{
	fl_set_object_color(\
	fdui -> main_form_frames_per_second_input_field,\
	FL_RED, FL_RED);
	}

return;
} /* end function set_frames_per_second_input_field_color */


int calculate_and_set_h_factor()
{
char temp[80];

if(debug_flag)
	{
	printf("calculate_and_set_h_factor(): arg none\n");
	}

h_factor = (double)h_margin / (double)h_size;

sprintf(temp, "%.2f", h_factor);
fl_set_input(fdui -> main_form_h_factor_input_field, temp);

return 1;
} /* end function calculate_and_set_h_factor */


int calculate_and_set_h_margin()
{
char temp[80];

if(debug_flag)
	{
	printf("calculate_and_set_h_margin(): arg none\n");
	}

h_margin = h_factor * h_size;

sprintf(temp, "%d", h_margin);
fl_set_input(fdui -> main_form_h_margin_input_field, temp);

return 1;
} /* end function calculate_and_set_h_margin */


int calculate_and_set_v_factor()
{
char temp[80];

if(debug_flag)
	{
	printf("calculate_and_set_v_factor(): arg none\n");
	}

v_factor = (double)v_margin / (double)v_size;

sprintf(temp, "%.2f", v_factor);
fl_set_input(fdui -> main_form_v_factor_input_field, temp);

return 1;
} /* end function calculate_and_set_v_factor */


int calculate_and_set_v_margin()
{
char temp[80];

if(debug_flag)
	{
	printf("calculate_and_set_v_margin(): arg none\n");
	}

v_margin = v_factor * v_size;

sprintf(temp, "%d", v_margin);
fl_set_input(fdui -> main_form_v_margin_input_field, temp);

return 1;
} /* end function calculate_and_set_v_margin */


int test_length(char *text, int *mode)
{
int width_in_pixels;
int space;
int used_character_style;

if(debug_flag)
	{
	printf("test_length(): arg text=%s\n", text);
	}

used_character_style = 10;

/* get the length of the text in pixels */
width_in_pixels =\
fl_get_string_width(\
	used_character_style, default_subtitle_font_size, text, strlen(text) );

if(debug_flag)
	{
	printf("test_length(): width_in_pixels=%d\n", width_in_pixels);
	}

/* test if >= horizontal space available */

/* calculate available space */
space = h_size - (2 * h_margin);

*mode = width_in_pixels / space;

return 1;
} /* end function test_length */


